package com.caimei365.order.components;

import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.mapper.OrderCommonMapper;
import com.caimei365.order.model.enums.OrderStatus;
import com.caimei365.order.model.enums.ReceivablesType;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.utils.ImageUtil;
import com.caimei365.order.utils.MathUtil;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/26
 */
@Slf4j
@Service
public class OrderCommonService {
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private OrderCommonMapper orderCommonMapper;
    @Resource
    private BaseMapper baseMapper;

    /**
     * 设置订单状态
     */
    public void setOrderStatus(OrderVo order) {
        // 111, 待付待收待发
        if (OrderStatus.UNRECEIVED_AND_UNSHIPPED.getCode() == order.getStatus() && 1 == order.getPayStatus()) {
            order.setStatus(111);
        }
        // 判断交易全退情况下,是否发过货,77,交易全退可以查看物流
        int logisticsCount = orderCommonMapper.countLogisticsBatch(order.getOrderId());
        if (OrderStatus.FULL_RETURNED.getCode() == order.getStatus() && logisticsCount > 0) {
            order.setStatus(77);
        }
        // 判断二手订单情况下，若部分付款和已付款，排除退货/款的情况，且未确认打款供应商，10,添加确认打款供应商按钮
        if (1 == order.getSecondHandOrderFlag() && 0 == order.getRefundType() && 0 == order.getAffirmPaymentFlag()) {
            if (order.getStatus().toString().startsWith("2") || order.getStatus().toString().startsWith("3")) {
                // 之前是 “00”, 现在int 10, 前端显示按钮用
                order.setAffirmPaymentFlag(10);
            }
        }
    }

    /**
     * 设置子订单数据
     */
    public void getShopOrderData(OrderVo order) {
        // 子订单
        if (StringUtils.isEmpty(order.getShopOrderIds())) {
            return;
        }
        Integer organizeId = orderCommonMapper.getOrganizeId(order.getOrderId());
        String[] shopOrderIdArr = order.getShopOrderIds().split(",");
        List<String> shopOrderIds = Arrays.asList(shopOrderIdArr);
        List<ShopOrderVo> shopOrderList = orderCommonMapper.getShopOrderList(shopOrderIds);
        shopOrderList.forEach(shopOrder -> {
            // 冷链商品类型
            List<Integer> arrList = new ArrayList<>();
            // 店铺促销活动
            PromotionsVo shopPromotion = null;
            if (null != shopOrder.getOrderPromotionsId() && shopOrder.getOrderPromotionsId() > 0) {
                shopPromotion = orderCommonMapper.getOrderPromotionsById(shopOrder.getOrderPromotionsId());
                shopOrder.setShopPromotion(shopPromotion);
            }
            List<OrderProductVo> orderProductList = orderCommonMapper.getShopOrderProduct(shopOrder.getShopOrderId());
            orderProductList.removeIf(Objects::isNull);
            // 冷链商品判定
            List<Integer> collect = orderProductList.stream().map(OrderProductVo::getProductId).collect(Collectors.toList());
            collect.removeIf(c -> c == 999);
            if (collect.stream().allMatch(c -> c == 7881)) {
                arrList.add(1);
            } else if (!collect.contains(7881)) {
                arrList.add(2);
            } else {
                arrList.add(3);
            }
            orderProductList.forEach(orderProduct -> {
                // 不含税可开票商品,单价/折后单价在原基础上加上税费
                boolean taxFlag = false;
                if (0 == organizeId) {
                    taxFlag = (Integer.valueOf(0).equals(orderProduct.getIncludedTax()) && (Integer.valueOf(1).equals(orderProduct.getInvoiceType()) || Integer.valueOf(2).equals(orderProduct.getInvoiceType())));
                }
                if (taxFlag) {
                    Double valueTax = MathUtil.div(MathUtil.mul(orderProduct.getPrice(), orderProduct.getTaxRate()), 100).doubleValue();
                    orderProduct.setPrice(MathUtil.add(orderProduct.getPrice(), valueTax).doubleValue());
                    orderProduct.setDiscountPrice(MathUtil.add(orderProduct.getDiscountPrice(), orderProduct.getAddedValueTax()).doubleValue());
                }
                orderProduct.setImage(ImageUtil.getImageUrl("product", orderProduct.getImage(), domain));
                // 查询订单下商品的促销活动
                if (null != orderProduct.getOrderPromotionsId() && orderProduct.getOrderPromotionsId() > 0) {
                    PromotionsVo promotions = orderCommonMapper.getOrderPromotionsById(orderProduct.getOrderPromotionsId());
                    if (null != promotions) {
                        if (1 == promotions.getType() && 1 == promotions.getMode()) {
                            // 单品优惠价取sku的优惠价
                            Double touchPrice = baseMapper.getTouchPriceBySku(orderProduct.getSkuId(),promotions.getId());
                            promotions.setTouchPrice(touchPrice);
                        }
                        if (taxFlag && Integer.valueOf(1).equals(promotions.getType()) && Integer.valueOf(1).equals(promotions.getMode())) {
                            promotions.setTouchPrice(MathUtil.add(promotions.getTouchPrice(), MathUtil.div(MathUtil.mul(promotions.getTouchPrice(), orderProduct.getTaxRate()), 100)).doubleValue());
                        }
                        orderProduct.setProductPromotion(promotions);
                    }
                }
                // 超级会员优惠商品设置优惠标签
                if (null != orderProduct.getSvipPriceFlag() && 1 == orderProduct.getSvipPriceFlag()) {
                    if (1 == orderProduct.getSvipPriceType()) {
                        orderProduct.setSvipPriceTag(MathUtil.div(orderProduct.getSvipDiscount(), 10, 1) + "折");
                    } else if (2 == orderProduct.getSvipPriceType()) {
                        orderProduct.setSvipPriceTag("￥" + orderProduct.getDiscountPrice());
                    }
                }
            });
            // 冷链运输费 -- 联合丽格
            if (null != shopOrder.getIsColdChina() && 1 == shopOrder.getIsColdChina()) {
                shopOrder.setColdChina(700.00d);
            } else {
                shopOrder.setColdChina(0.00d);
            }
            // 拼接运费信息--联合丽格
            if (null != shopOrder.getColdChina()) {
                String open = "";
                String close = "";
                if (shopOrder.getIsColdChina()==1) {
                    open = "(";
                    close = ")";
                } else if (!arrList.stream().allMatch(c -> c == 1)) {
                    open = "(";
                    close = ")";
                } else {
                    open = "";
                    close = "";
                }
                if (-1 == shopOrder.getShopPostFlag()) {
                    // 到付
                    shopOrder.setPostageInfo("￥" + shopOrder.getShopPostFee().doubleValue() + open +(shopOrder.getIsColdChina()==1?"冷链费: ￥" + shopOrder.getColdChina().doubleValue():"") + (arrList.stream().allMatch(c -> c == 1)?"":" 其他: 到付")+close);
                } else if (0 == shopOrder.getShopPostFlag()) {
                    // 包邮
                    shopOrder.setPostageInfo("￥" + shopOrder.getShopPostFee().doubleValue() + open +(shopOrder.getIsColdChina()==1?"冷链费: ￥" + shopOrder.getColdChina().doubleValue():"") + (arrList.stream().allMatch(c -> c == 1)?"":" 其他: 包邮")+close);
                } else {
                    // 有运费
                    shopOrder.setPostageInfo("￥" + shopOrder.getShopPostFee().doubleValue() + open +(shopOrder.getIsColdChina()==1?"冷链费: ￥" + shopOrder.getColdChina().doubleValue():"") + (!arrList.stream().allMatch(c -> c == 1)?" 其他: ￥" + MathUtil.sub(order.getPostage(), order.getColdChina()).doubleValue():"") + close);
                }
            } else {
                shopOrder.setPostageInfo(shopOrder.getShopPostFlag() == -1?"到付":shopOrder.getShopPostFlag() == 0?"包邮":"￥" + shopOrder.getShopPostFee().doubleValue());
            }
            shopOrder.setOrderProductList(orderProductList);
            shopOrder.setShopLogo(ImageUtil.getImageUrl("shopLogo", shopOrder.getShopLogo(), domain));
            if (0 == order.getOnlinePayFlag()) {
                shopOrder.setObligation(MathUtil.sub(shopOrder.getRealPay(), shopOrder.getReceiptAmount()).doubleValue());
            }

        });
        // 过滤运费商品
        shopOrderList.removeIf(shopOrder -> shopOrder.getShopId() == 998);
        order.setShopOrderList(shopOrderList);
    }


    /**
     * 设置付款金额
     *
     * @param order OrderVo
     */
    public List<DiscernReceiptVo> getDiscernReceiptAndSetOrder(OrderVo order) {
        // 支付记录
        List<DiscernReceiptVo> discernReceiptList = orderCommonMapper.getDiscernReceipt(order.getOrderId(), order.getShopOrderIds());
        Double receiptAmount = 0d;
        // 订单款
        if (!discernReceiptList.isEmpty()) {
            AtomicDouble finalReceiptAmount = new AtomicDouble(0d);
            discernReceiptList.forEach(discernReceipt -> {
                finalReceiptAmount.set(MathUtil.add(finalReceiptAmount.get(), discernReceipt.getAssociateAmount()).doubleValue());
                if (null != discernReceipt.getPayType()) {
                    discernReceipt.setPayTypeStr(ReceivablesType.getReceivablesType(discernReceipt.getPayType()));
                }
            });
            receiptAmount = finalReceiptAmount.get();
        } else {
            //返佣款
            if (!order.getShopOrderIds().contains(",")) {
                Double tempAmount = orderCommonMapper.getRebateAmountByShopOrder(Integer.parseInt(order.getShopOrderIds()));
                if (null != tempAmount) {
                    order.setOnlinePayFlag(1);
                }
            }
        }
        // 判断是否可以走线上支付
        int offlineCount = orderCommonMapper.countOfflinePayment(order.getOrderId());
        if (offlineCount > 0) {
            order.setOnlinePayFlag(1);
        }

//        //付供应商总金额 + 默认手续费 > 订单总金额
//        Double payTotalFee = order.getPayTotalFee();
//        Double handlingFee = MathUtil.mul(payTotalFee, 0.0038, 2).doubleValue();
//        if (MathUtil.compare(handlingFee, 8) < 0) {
//            handlingFee = 8d;
//        }
//        List<Double> shouldPayShopAmount = orderCommonMapper.getShouldPayShopAmountList(order.getOrderId());
//        if (!shouldPayShopAmount.isEmpty()) {
//            AtomicDouble finalHandlingFee = new AtomicDouble(handlingFee);
//            shouldPayShopAmount.forEach(amount -> {
//                finalHandlingFee.set(MathUtil.add(finalHandlingFee.get(), amount).doubleValue());
//            });
//            handlingFee = finalHandlingFee.get();
//        }
//        if (MathUtil.compare(payTotalFee, handlingFee) < 0) {
//            order.setPayButton(true);
//        } else {
//            order.setPayButton(false);
//        }
        //待付总金额
        order.setPendingPayments(MathUtil.sub(order.getPayableAmount(), receiptAmount).doubleValue());
        //支付总金额
        order.setReceiptAmount(receiptAmount);
        return discernReceiptList;
    }

    /**
     * 设置子订单状态
     * shopStatus  0：待确认 1：已确认 2：交易完成 3：订单完成 4：已关闭 5：交易全退
     *
     * payStatus (付款供应商)付款状态：1待付款、2部分付款、3已付款
     * sendOutStatus  发货状态：1待发货、2部分发货、3已发货
     * receiptStatus 收款状态：1待收款、2部分收款、3已收款
     */
    public void setShopOrderStatus(ShopOrderVo shopOrderVo) {
        // 11, 待付待收待发
        if (1 == shopOrderVo.getStatus() && 1 == shopOrderVo.getOrderStatusFlag() && 1 == shopOrderVo.getRefundStatus()) {
            if ( 1 == shopOrderVo.getReceiptStatus() && 1 == shopOrderVo.getSendOutStatus()){
                shopOrderVo.setStatus(11);
            } else if ( 1 == shopOrderVo.getReceiptStatus() && 2 == shopOrderVo.getSendOutStatus()) {
                shopOrderVo.setStatus(12);
            } else if ( 1 == shopOrderVo.getReceiptStatus() && 3 == shopOrderVo.getSendOutStatus()) {
                shopOrderVo.setStatus(13);
            } else if ( 2 == shopOrderVo.getReceiptStatus() && 1 == shopOrderVo.getSendOutStatus()) {
                shopOrderVo.setStatus(21);
            } else if ( 2 == shopOrderVo.getReceiptStatus() && 2 == shopOrderVo.getSendOutStatus()) {
                shopOrderVo.setStatus(22);
            } else if ( 2 == shopOrderVo.getReceiptStatus() && 3 == shopOrderVo.getSendOutStatus()) {
                shopOrderVo.setStatus(23);
            } else if ( 3 == shopOrderVo.getReceiptStatus() && 1 == shopOrderVo.getSendOutStatus()) {
                shopOrderVo.setStatus(31);
            } else if ( 3 == shopOrderVo.getReceiptStatus() && 2 == shopOrderVo.getSendOutStatus()) {
                shopOrderVo.setStatus(32);
            } else if ( 3 == shopOrderVo.getReceiptStatus() && 3 == shopOrderVo.getSendOutStatus()) {
                shopOrderVo.setStatus(33);
            }
        }
        // 判断交易全退情况下,是否发过货,6,交易全退可以查看物流
        int logisticsCount = orderCommonMapper.countLogisticsBatchShop(shopOrderVo.getShopOrderId());
        if (5 == shopOrderVo.getStatus() && logisticsCount > 0) {
            shopOrderVo.setStatus(6);
        }
        if (1 != shopOrderVo.getOrderStatusFlag() || 1 != shopOrderVo.getRefundStatus()) {
            shopOrderVo.setStatus(5);
        }
    }

    /**
     * 子订单信息
     * @param shopOrder
     */
    public void setShopOrderInfo(ShopOrderVo shopOrder) {
        if (null != shopOrder) {
            Integer organizeId = orderCommonMapper.getShopOrganizeId(shopOrder.getShopOrderId());
            // 冷链商品类型
            List<Integer> arrList = new ArrayList<>();
            // 店铺促销活动
            PromotionsVo shopPromotion = null;
            if (null != shopOrder.getOrderPromotionsId() && shopOrder.getOrderPromotionsId() > 0) {
                shopPromotion = orderCommonMapper.getOrderPromotionsById(shopOrder.getOrderPromotionsId());
                shopOrder.setShopPromotion(shopPromotion);
            }
            List<OrderProductVo> orderProductList = orderCommonMapper.getShopOrderProduct(shopOrder.getShopOrderId());
            orderProductList.removeIf(Objects::isNull);
            // 冷链商品判定
            List<Integer> collect = orderProductList.stream().map(OrderProductVo::getProductId).collect(Collectors.toList());
            collect.removeIf(c -> c == 999);
            if (collect.stream().allMatch(c -> c == 7881)) {
                arrList.add(1);
            } else if (!collect.contains(7881)) {
                arrList.add(2);
            } else {
                arrList.add(3);
            }
            // 是否有分帐号
            AtomicBoolean isSplit = new AtomicBoolean(true);
            orderProductList.forEach(orderProduct -> {
                // 不含税可开票商品,单价/折后单价在原基础上加上税费
                boolean taxFlag = false;
                if (0 == organizeId) {
                    taxFlag = (Integer.valueOf(0).equals(orderProduct.getIncludedTax()) && (Integer.valueOf(1).equals(orderProduct.getInvoiceType()) || Integer.valueOf(2).equals(orderProduct.getInvoiceType())));
                }
                /*if (taxFlag) {
                    Double valueTax = MathUtil.div(MathUtil.mul(orderProduct.getPrice(), orderProduct.getTaxRate()), 100).doubleValue();
                    orderProduct.setPrice(MathUtil.add(orderProduct.getPrice(), valueTax).doubleValue());
                    orderProduct.setDiscountPrice(MathUtil.add(orderProduct.getDiscountPrice(), orderProduct.getAddedValueTax()).doubleValue());
                }*/
                orderProduct.setImage(ImageUtil.getImageUrl("product", orderProduct.getImage(), domain));
                // 查询订单下商品的促销活动
                if (null != orderProduct.getOrderPromotionsId() && orderProduct.getOrderPromotionsId() > 0) {
                    PromotionsVo promotions = orderCommonMapper.getOrderPromotionsById(orderProduct.getOrderPromotionsId());
                    if (null != promotions) {
                        if (1 == promotions.getType() && 1 == promotions.getMode()) {
                            // 单品优惠价取sku的优惠价
                            Double touchPrice = baseMapper.getTouchPriceBySku(orderProduct.getSkuId(),promotions.getPromotionsId());
                            promotions.setTouchPrice(touchPrice);
                        }
                        if (taxFlag && Integer.valueOf(1).equals(promotions.getType()) && Integer.valueOf(1).equals(promotions.getMode())) {
                            promotions.setTouchPrice(MathUtil.add(promotions.getTouchPrice(), MathUtil.div(MathUtil.mul(promotions.getTouchPrice(), orderProduct.getTaxRate()), 100)).doubleValue());
                        }
                        orderProduct.setProductPromotion(promotions);
                    }
                }
                // 超级会员优惠商品设置优惠标签
                if (null != orderProduct.getSvipPriceFlag() && 1 == orderProduct.getSvipPriceFlag()) {
                    if (1 == orderProduct.getSvipPriceType()) {
                        orderProduct.setSvipPriceTag(MathUtil.div(orderProduct.getSvipDiscount(), 10, 1) + "折");
                    } else if (2 == orderProduct.getSvipPriceType()) {
                        orderProduct.setSvipPriceTag("￥" + orderProduct.getDiscountPrice());
                    }
                }
                if (StringUtils.isEmpty(orderProduct.getSplitCode())) {
                    isSplit.set(false);
                }
            });
            // 拼接运费信息
            shopOrder.setPostageInfo(shopOrder.getShopPostFlag() == 2 ?"到付":shopOrder.getShopPostFlag() == 0?"包邮":"￥" + shopOrder.getShopPostFee());

            shopOrder.setOrderProductList(orderProductList);
            shopOrder.setShopLogo(ImageUtil.getImageUrl("shopLogo", shopOrder.getShopLogo(), domain));
            //if (0 == order.getOnlinePayFlag()) {
            if (null == shopOrder.getRealPay()) {
                shopOrder.setRealPay(shopOrder.getNeedPayAmount());
            }
            shopOrder.setObligation(MathUtil.sub(shopOrder.getRealPay(), shopOrder.getReceiptAmount()).doubleValue());
            //}
        }

    }

    /**
     * 设置子订单付款金额
     * @param shopOrderVo
     * @return
     */
    public List<DiscernReceiptVo> getDiscernReceiptAndSetShoporder(ShopOrderVo shopOrderVo) {
        // 判断是否可以走线上支付
        if (0 == shopOrderVo.getPayway()) {
            shopOrderVo.setOnlinePay(0);
        } else if (1 == shopOrderVo.getPayway()) {
            shopOrderVo.setOnlinePay(1);
        } else {
            shopOrderVo.setOnlinePay(2);
        }
        Integer orderVoucher = orderCommonMapper.getOrderVoucher(shopOrderVo.getShopOrderId());
        if (null != orderVoucher && orderVoucher > 0) {
            shopOrderVo.setOnlinePay(2);
        }
        // 余额抵扣线下
        if (1 == shopOrderVo.getUseBalanceFlag()) {
            shopOrderVo.setOnlinePay(2);
        }
        // 支付记录
        List<DiscernReceiptVo> discernReceiptList = orderCommonMapper.getDiscernReceipt(shopOrderVo.getOrderId(), shopOrderVo.getShopOrderId().toString());
        Double receiptAmount = 0d;
        // 订单款
        if (!discernReceiptList.isEmpty()) {
            AtomicDouble finalReceiptAmount = new AtomicDouble(0d);
            discernReceiptList.forEach(discernReceipt -> {
                finalReceiptAmount.set(MathUtil.add(finalReceiptAmount.get(), discernReceipt.getAssociateAmount()).doubleValue());
                if (null != discernReceipt.getPayType()) {
                    discernReceipt.setPayTypeStr(ReceivablesType.getReceivablesType(discernReceipt.getPayType()));
                }
                if (1 == discernReceipt.getPayWay()) {
                    shopOrderVo.setOnlinePay(1);
                } else {
                    shopOrderVo.setOnlinePay(2);
                }
            });
            receiptAmount = finalReceiptAmount.get();
        } else {
            //返佣款
            Double tempAmount = orderCommonMapper.getRebateAmountByShopOrder(shopOrderVo.getShopOrderId());
            if (null != tempAmount) {
                shopOrderVo.setOnlinePay(2);
            }
        }
        //待付总金额
        if (null == shopOrderVo.getRealPay()) {
            shopOrderVo.setRealPay(shopOrderVo.getNeedPayAmount());
        }
        AtomicDouble receipt = new AtomicDouble(0);
        if (MathUtil.compare(shopOrderVo.getAccountAmount(), 0) > 0 && shopOrderVo.getStatus() != 0) {
            receipt.set(MathUtil.add(receipt.get(), shopOrderVo.getAccountAmount()).doubleValue());
        }
        // 支付总金额 = 用户结算金额 + 付款金额
        if(!discernReceiptList.isEmpty()) {
            shopOrderVo.setReceiptAmount(MathUtil.add(receipt.get(), receiptAmount).doubleValue());
        }
        // 待付金额
        shopOrderVo.setObligation(MathUtil.sub(shopOrderVo.getRealPay(), shopOrderVo.getReceiptAmount()).doubleValue());
        return discernReceiptList;
    }

    public void setPostageInfo(ShopOrderVo shopOrder) {
        // 冷链商品类型
        List<Integer> arrList = new ArrayList<>();
        List<OrderProductVo> orderProductList = orderCommonMapper.getShopOrderProduct(shopOrder.getShopOrderId());
        orderProductList.removeIf(Objects::isNull);
        // 冷链商品判定
        List<Integer> collect = orderProductList.stream().map(OrderProductVo::getProductId).collect(Collectors.toList());
        collect.removeIf(c -> c == 999);
        if (collect.stream().allMatch(c -> c == 7881)) {
            arrList.add(1);
        } else if (!collect.contains(7881)) {
            arrList.add(2);
        } else {
            arrList.add(3);
        }
        shopOrder.setPostageInfo(shopOrder.getShopPostFlag() == 2?"到付":shopOrder.getShopPostFlag() == 0?"包邮":"￥" + shopOrder.getShopPostFee());
    }

    /**
     * 设置搜索关键词历史记录
     *
     * @param userId     用户Id
     * @param searchWord 搜索关键词
     */
    public void setHistoryRecord(Integer userId, String searchWord) {
        // 初始化一个搜索关键词历史记录
        SearchHistoryVo historyRecord = new SearchHistoryVo();
        historyRecord.setUserId(userId);
        historyRecord.setSearchDate(new Date());
        historyRecord.setSearchWord(searchWord);
        historyRecord.setDelFlag(0);
        // 查询搜索关键词历史记录是否存在
        Integer recordId = orderCommonMapper.getSearchHistoryIdByWord(searchWord);
        // 保存搜索关键词历史记录
        if (null != recordId && recordId > 0) {
            historyRecord.setId(recordId);
            orderCommonMapper.updateSearchHistory(historyRecord);
        } else {
            orderCommonMapper.insertSearchHistory(historyRecord);
        }
        // 查询关键字历史记录条数
        int count = orderCommonMapper.countSearchHistory(userId);
        // 只保留10条的搜索关键词历史记录
        if (count > 10) {
            orderCommonMapper.deleteSearchHistoryLimit(userId);
        }
    }
}
