package com.caimei365.order.controller;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.BankListDto;
import com.caimei365.order.model.dto.HeliDto;
import com.caimei365.order.model.dto.PayDto;
import com.caimei365.order.model.dto.PayLinkDto;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.HeliPayService;
import com.caimei365.order.service.PayOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.*;


import javax.servlet.http.HttpServletResponse;
import java.beans.IntrospectionException;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Map;


/**
 * Description
 *
 * @author : zzj
 * @date : 2021/11/11
 */
@Api(tags = "合利宝支付API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/order/pay")
public class HeliPayApi {

    private final HeliPayService heliPayService;
    private final PayOrderService payOrderService;

    @ApiOperation("快捷支付解绑银行卡")
    @ApiImplicitParam(required = true, name = "cardId", value = "银行卡绑定表Id")
    @GetMapping("/quick/unbound/card")
    public ResponseJson unboundCard(Integer cardId) {
        if (null == cardId) {
            return ResponseJson.error("绑定卡Id不能为空！");
        }
        return heliPayService.unBoundCard(cardId);
    }

    @ApiOperation("快捷支付银行信息")
    @ApiImplicitParam(required = false, name = "cardType", value = "银行卡类型")
    @GetMapping("/quick/bank/infos")
    public ResponseJson<List<BankListDto>> quickBankCode(Integer cardType) {
        return heliPayService.bankInfos(cardType);
    }

    @ApiOperation("免密绑卡银行信息")
    @GetMapping("/quick/uncode/infos")
    public ResponseJson<List<QuickBankVo>> unCodeInfos() {
        return heliPayService.unCodeInfos();
    }

    @ApiOperation("免密一键绑卡")
    @ApiImplicitParam(required = false, name = "cardType", value = "银行卡类型")
    @PostMapping("/quick/bind/nocode")
    public ResponseJson<String> quickBankCode(HeliDto heliDto) {
        if (StringUtils.isBlank(heliDto.getBankCode())) {
            return ResponseJson.error("银行编码不能为空！");
        }
        if (StringUtils.isBlank(heliDto.getQuickPayUserName())) {
            return ResponseJson.error("用户姓名不能为空！");
        }
        if (StringUtils.isBlank(heliDto.getIdCard())) {
            return ResponseJson.error("身份证号不能为空！");
        }
        if (StringUtils.isBlank(heliDto.getQuickPayMobile())) {
            return ResponseJson.error("手机号码不能为空！");
        }
        if (null == heliDto.getCardType()) {
            return ResponseJson.error("银行卡类型不能为空！");
        }
        return heliPayService.bindCardNoCode(heliDto);
    }


    @ApiOperation("绑卡验证码校验")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "flag", value = "验证码类型"),
            @ApiImplicitParam(required = true, name = "bindCode", value = "验证码"),
            @ApiImplicitParam(required = true, name = "orderId", value = "绑定单号"),
            @ApiImplicitParam(required = false, name = "infoId", value = "记录Id"),
            @ApiImplicitParam(required = false, name = "customer", value = "商户号")
    })
    @GetMapping("/quick/bind/code")
    public ResponseJson bindCardCheck(Integer flag, String bindCode, String orderId, String infoId, String customer) {
        if (null == flag) {
            return ResponseJson.error("验证码类型不能为空！");
        }
        if (StringUtils.isBlank(bindCode)) {
            return ResponseJson.error("验证码不能为空！");
        }
        if (StringUtils.isBlank(orderId)) {
            return ResponseJson.error("订单Id不能为空！");
        }
        return heliPayService.bindByCode(flag, bindCode, orderId, infoId, customer);
    }


    @ApiOperation("快捷支付预绑卡")
    @PostMapping("/quick/bind/card")
    public ResponseJson<Map<String, String>> bindBankCard(HeliDto heliDto) {
        if (null == heliDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！");
        }
        if (StringUtils.isBlank(heliDto.getQuickPayBankNumber())) {
            return ResponseJson.error("绑定卡号不能为空！");
        }
        if (StringUtils.isBlank(heliDto.getQuickPayUserName())) {
            return ResponseJson.error("用户名不能为空！");
        }
        if (StringUtils.isBlank(heliDto.getIdCard())) {
            return ResponseJson.error("身份证号不能为空！");
        }
        if (StringUtils.isBlank(heliDto.getQuickPayMobile())) {
            return ResponseJson.error("银行预留手机号码不能为空！");
        }
        return heliPayService.bindBankCard(heliDto);
    }

    @ApiOperation("获取子订单信息")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "orderId", value = "订单Id"),
            @ApiImplicitParam(required = true, name = "shopOrderId", value = "子订单Id")
    })

    @GetMapping("/shoporders")
    public ResponseJson<List<ShopOrderVo>> getShopOrders(Integer orderId, Integer shopOrderId) {
        if (null == orderId) {
            if (null == shopOrderId) {
                return ResponseJson.error("子订单Id不能为空！", null);
            }
        }
        if (null == shopOrderId) {
            if (null == orderId) {
                return ResponseJson.error("订单Id不能为空！", null);
            }
        }
        return heliPayService.getShopOrders(orderId, shopOrderId);
    }

    /**
     * 订单支付前效验付款规则
     */
    @ApiOperation("订单支付前效验付款规则（旧：/order/paymentValidation）")
    @ApiImplicitParam(required = false, name = "shopOrderId", value = "订单Id")
    @GetMapping("/check")
    public ResponseJson<Map<String, Object>> paymentValidation(Integer shopOrderId) {
        if (null == shopOrderId) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return payOrderService.payBeforeCheck(shopOrderId);
    }

    /**
     * 订单支付,余额抵扣
     *
     * @param payDto {
     *               orderId      订单ID
     *               }
     */
    @ApiOperation("订单支付,余额抵扣（旧：/order/balanceDeduction）")
    @PostMapping("/balance/deduction")
    public ResponseJson<Map<String, Object>> balanceDeduction(PayDto payDto) {
        if (null == payDto.getShopOrderId()) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return payOrderService.balanceDeduction(payDto.getShopOrderId());
    }

    /**
     * 获取线上支付全局开关状态
     */
    @ApiOperation("获取线上支付全局开关状态（旧：/PayOrder/onLineSwitch）")
    @GetMapping("/online/switch")
    public ResponseJson<Integer> getPayOnLineSwitch() {
        return payOrderService.getPayOnLineSwitch();
    }

    /**
     * 收银台数据显示
     */
    @ApiOperation("收银台数据（旧：/PayOrder/checkoutCounter）")
    @ApiImplicitParam(required = true, name = "shopOrderId", value = "子订单Id")
    @GetMapping("/checkout/counter")
    public ResponseJson<Map<String, Object>> getCheckoutCounter(Integer shopOrderId) {
        if (null == shopOrderId) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return payOrderService.getCheckoutCounter(shopOrderId);
    }

    /**
     * 收银台数据显示
     */
    @ApiOperation("收银台数据（旧：/PayOrder/checkoutCounter）")
    @ApiImplicitParam(required = true, name = "orderId", value = "订单Id")
    @GetMapping("/checkout/shoporders")
    public ResponseJson<Map<String, Object>> getCheckoutShopOrders(Integer shopOrderId) {
        if (null == shopOrderId) {
            return ResponseJson.error("子订单Id不能为空！", null);
        }
        return payOrderService.getCheckoutShopOrders(shopOrderId);
    }


    /**
     * 生成网银支付链接
     *
     * @param payLinkDto {
     *                   orderId       订单ID
     *                   vipRecordId   会员购买记录Id
     *                   }
     */
    @ApiOperation("生成网银支付链接（旧：/PayOrder/payLink）")
    @PostMapping("/link")
    public ResponseJson<String> getPayLink(PayLinkDto payLinkDto) {
        return payOrderService.getPayLink(payLinkDto);
    }

    /**
     * 支付链接重定向到页面
     *
     * @param linkLogo 链接标识
     */
    @ApiOperation("支付链接重定向到页面（旧：/PayOrder/jumpPage）")
    @ApiImplicitParam(required = false, name = "linkLogo", value = "链接标识")
    @GetMapping("/link/jump")
    public void jumpPayPage(String linkLogo, HttpServletResponse response) throws IOException {
        payOrderService.jumpPayPage(linkLogo, response);
    }

    /**
     * 重定向页面的支付数据
     */
    @ApiOperation("重定向页面的支付数据（旧：/PayOrder/linkData）")
    @GetMapping("/link/data")
    public ResponseJson<Map<String, Object>> linkPayData(String linkLogo) {
        if (StringUtils.isBlank(linkLogo)) {
            return ResponseJson.error("链接标识不能为空！", null);
        }
        return payOrderService.linkPayData(linkLogo);
    }

    /**
     * 判断此次支付是否完成
     *
     * @param shopOrderId       子订单id
     * @param paySuccessCounter 付款次数
     */
    @ApiOperation("判断此次支付是否完成（旧：/PayOrder/payWhetherSuccess）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "shopOrderId", value = "子订单id"),
            @ApiImplicitParam(required = false, name = "paySuccessCounter", value = "付款次数")
    })
    @GetMapping("/result/check")
    public ResponseJson<String> payWhetherSuccess(Integer shopOrderId, Integer paySuccessCounter) {
        if (null == shopOrderId) {
            return ResponseJson.error("子订单Id不能为空！", null);
        }
        if (null == paySuccessCounter) {
            return ResponseJson.error("付款次数不能为空！", null);
        }
        return payOrderService.payWhetherSuccess(shopOrderId, paySuccessCounter);
    }

    /**
     * 查询本次支付订单结果
     *
     * @param mbOrderId 平台唯一流水号
     */
    @ApiOperation("查询本次支付订单结果（旧：/PayOrder/findOrderStatus）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "mbOrderId", value = "平台唯一流水号"),
            @ApiImplicitParam(required = true, name = "orderFlag", value = "订单标识：0非订单，1订单"),
            @ApiImplicitParam(required = false, name = "shopOrderId", value = "子订单Id")
    })
    @GetMapping("/result/json")
    public ResponseJson<JSONObject> getPayOrderResult(String mbOrderId, Integer orderFlag, Integer shopOrderId) throws IntrospectionException, InvocationTargetException, IllegalAccessException {
        if (null == mbOrderId) {
            return ResponseJson.error("平台唯一流水号不能为空！", null);
        }
        if (null == orderFlag) {
            return ResponseJson.error("订单标识不能为空！", null);
        }
        return heliPayService.getPayOrderResult(mbOrderId, orderFlag, shopOrderId);
    }

    /**
     * 查询网银本次支付订单结果
     *
     * @param mbOrderId 平台唯一流水号
     */
    @ApiOperation("查询网银本次支付订单结果（旧：/PayOrder/findOrderStatus）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "mbOrderId", value = "平台唯一流水号"),
            @ApiImplicitParam(required = true, name = "orderFlag", value = "订单标识：0非订单，1订单"),
            @ApiImplicitParam(required = false, name = "shopOrderId", value = "子订单Id")
    })
    @GetMapping("/result/union")
    public ResponseJson<JSONObject> getUnionResult(String mbOrderId, Integer orderFlag, Integer shopOrderId) {
        if (null == mbOrderId) {
            return ResponseJson.error("平台唯一流水号不能为空！", null);
        }
        if (null == orderFlag) {
            return ResponseJson.error("订单标识不能为空！", null);
        }
        return heliPayService.getUnionResult(mbOrderId, orderFlag, shopOrderId);
    }

    @ApiOperation("合利宝快捷支付")
    @PostMapping("/quick/pay")
    public ResponseJson<Map<String, String>> quickPay(HeliDto heliDto, @RequestHeader HttpHeaders headers) {
        if (null == heliDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！");
        }
        if (null == heliDto.getQuickPayFlag()) {
            return ResponseJson.error("支付类型不能为空！");
        }
        if (StringUtils.isBlank(heliDto.getQuickPayBankNumber())) {
            return ResponseJson.error("绑定卡号不能为空！");
        }
        if (StringUtils.isBlank(heliDto.getQuickPayUserName())) {
            return ResponseJson.error("用户名不能为空！");
        }
        if (StringUtils.isBlank(heliDto.getIdCard())) {
            return ResponseJson.error("身份证号不能为空！");
        }
        if (StringUtils.isBlank(heliDto.getQuickPayMobile())) {
            return ResponseJson.error("银行预留手机号码不能为空！");
        }
        return heliPayService.quickPay(heliDto, headers);
    }

    /**
     * 合利宝pc支付宝二维码
     */
    @ApiOperation("合利宝支付宝扫码支付")
    @PostMapping("/scan/pay")
    public ResponseJson<JSONObject> payByWeChat(HeliDto heliDto, @RequestHeader HttpHeaders headers) throws IntrospectionException, InvocationTargetException, IllegalAccessException {
        if (StringUtils.isBlank(heliDto.getPayType())) {
            return ResponseJson.error("支付类型不能为空！", null);
        }
        if (null == heliDto.getShopOrderId()) {
            return ResponseJson.error("子订单Id不能为空！", null);
        }
        return heliPayService.payByWeChat(heliDto, headers);
    }

    /**
     * 合利宝公众号/小程序支付
     */
    @ApiOperation("合利宝微信小程序/公众号支付")
    @PostMapping("/online")
    public ResponseJson<JSONObject> payOnline(HeliDto heliDto, @RequestHeader HttpHeaders headers) throws IntrospectionException, InvocationTargetException, IllegalAccessException {
        if (StringUtils.isBlank(heliDto.getPayType())) {
            return ResponseJson.error("支付类型不能为空！", null);
        }
        if (null == heliDto.getShopOrderId()) {
            return ResponseJson.error("子订单Id不能为空！", null);
        }
        return heliPayService.payOnline(heliDto, headers);
    }

    /**
     * 合利宝网银支付
     */
    @ApiOperation("合利宝网银支付")
    @PostMapping("/union")
    public ResponseJson<String> link(HeliDto heliDto, @RequestHeader HttpHeaders headers) {
        if (StringUtils.isBlank(heliDto.getPayType())) {
            return ResponseJson.error("支付类型不能为空！", null);
        }
        if (null == heliDto.getShopOrderId()) {
            return ResponseJson.error("子订单Id不能为空！", null);
        }
        if (null == heliDto.getPayAmount() || Double.parseDouble(heliDto.getPayAmount()) <= 0.1) {
            return ResponseJson.error("支付金额不正确！", null);
        }
        if (StringUtils.isEmpty(heliDto.getReturnUrl())) {
            return ResponseJson.error("回调地址不能为空！", null);
        }
        if (StringUtils.isEmpty(heliDto.getBankCode())) {
            return ResponseJson.error("银行编码不能为空！", null);
        }
        if (StringUtils.isEmpty(heliDto.getUserType())) {
            return ResponseJson.error("银行用户类型不能为空！", null);
        }
        return heliPayService.link(heliDto, headers);
    }

    /**
     * 支付异步通知回调
     */
    @ApiOperation("支付异步通知回调（旧：/PayOrder/paymentCallback）")
    @PostMapping("/callback")
    public String paymentCallback(NotifyResponseVo res) throws IntrospectionException, InvocationTargetException, IllegalAccessException {
        if (res == null) {
            return "回调参数失败";
        }
        return heliPayService.paymentCallback(res);
    }

    @ApiOperation("快捷支付异步通知回调")
    @PostMapping("/quick/callback")
    public String quickPayCallback(QuickPayConfirmPayResponseVo res) {
        if (res == null) {
            return "回调参数失败";
        }
        return heliPayService.quickPayCallback(res);
    }

    @ApiOperation("网银支付异步通知回调")
    @PostMapping("/union/callback")
    public String unionCallback(UnionResVo res) throws IntrospectionException, InvocationTargetException, IllegalAccessException {
        if (res == null) {
            return "回调参数失败";
        }
        return heliPayService.unionCallback(res);
    }

    /**
     * 延时分账异步通知回调
     */
    @ApiOperation("延时分账异步通知回调（旧：/PayOrder/delayedSplittingCallback）")
    @PostMapping("/delay/split/callback")
    public String delayedSplittingCallback(AccountResVo data) {
        if (null == data) {
            return "回调参数失败";
        }
        return heliPayService.delayedSplittingCallback(data);
    }

    /**
     * 网银退款手续费转回回调
     */
    @ApiOperation("网银退款手续费转回回调")
    @PostMapping("/refund/split/callback")
    public String refundSplittingCallback(AccountResVo data) {
        if (null == data) {
            return "回调参数失败";
        }
        return heliPayService.refundSplittingCallback(data);
    }

    @ApiOperation("银行通道码获取")
    @GetMapping("/bankcode")
    public ResponseJson<Map<String, Object>> bankcode() {
        return heliPayService.bankCode();
    }

}
