package com.caimei365.order.controller;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.vo.OrderVo;
import com.caimei365.order.model.vo.ShopOrderVo;
import com.caimei365.order.service.OrderSellerService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
 * 协销订单API
 *
 * @author : Charles
 * @date : 2021/7/19
 */
@Api(tags = "协销订单API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/order/seller")
public class OrderSellerApi {
    private final OrderSellerService orderSellerService;

    /**
     * 获取协销各状态订单数量
     */
    @ApiOperation("获取协销各状态订单数量")
    @ApiImplicitParam(required = true, name = "serviceProviderId", value = "协销Id")
    @GetMapping("/count")
    public ResponseJson<Map<String, Object>> getOrderCount(Integer serviceProviderId) {
        if (null == serviceProviderId) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        return orderSellerService.getSellerOrderCount(serviceProviderId);
    }

    /**
     * 协销订单列表
     */
    @ApiOperation("协销订单列表（旧：/seller/order/list）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "serviceProviderId", value = "协销Id"),
            @ApiImplicitParam(required = false, name = "listType", value = "1:待确认,2:待付款,3:待发货,4:已发货,5:退货款"),
            @ApiImplicitParam(required = false, name = "type", value = "null:协销,2:分销人员"),
            @ApiImplicitParam(required = false, name = "shopOrderNo", value = "订单编号"),
            @ApiImplicitParam(required = false, name = "orderSubmitType", value = "1:自主订单, 2:协销或后台订单,3:组员订单"),
            @ApiImplicitParam(required = false, name = "groupServiceId", value = "组员协销id"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/list")
    public ResponseJson<PageInfo<ShopOrderVo>> getOrderList(Integer serviceProviderId, Integer listType, Integer orderSubmitType,
                                                            String shopOrderNo, Integer groupServiceId,
                                                            @RequestParam(required = false) Integer type,
                                                            @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (null == serviceProviderId) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        return orderSellerService.getOrderList(serviceProviderId,type, listType, orderSubmitType, shopOrderNo, groupServiceId, pageNum, pageSize);
    }

    /**
     * 协销下机构总订单列表
     */
    @ApiOperation("协销下机构总订单列表（旧：/seller/clubOrder）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "serviceProviderId", value = "协销Id"),
            @ApiImplicitParam(required = false, name = "clubId", value = "机构Id"),
            @ApiImplicitParam(required = false, name = "listType", value = "1:待确认,2:待付款,3:待发货,4:已发货,5:退货款"),
            @ApiImplicitParam(required = false, name = "orderSubmitType", value = "1:自主订单, 2:协销或后台订单"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/list/club")
    public ResponseJson<PageInfo<ShopOrderVo>> getOrderListClub(Integer serviceProviderId, Integer clubId, Integer listType, Integer orderSubmitType,
                                                                @RequestParam(required = false) Integer type,
                                                            @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (null == serviceProviderId) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        if (null == clubId) {
            return ResponseJson.error("机构Id不能为空！", null);
        }
        return orderSellerService.getOrderListClub(serviceProviderId, clubId, listType, orderSubmitType,type, pageNum, pageSize);
    }

    /**
     * 协销根据关键词搜索订单
     */
    @ApiOperation("协销根据关键词搜索订单（旧：/seller/searchOrder）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "serviceProviderId", value = "协销Id"),
            @ApiImplicitParam(required = false, name = "searchWord", value = "搜索关键词"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/search")
    public ResponseJson<PageInfo<ShopOrderVo>> getOrderListByKeyword(Integer serviceProviderId, String searchWord,
                                                                 @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                                 @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (null == serviceProviderId) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        if (StringUtils.isEmpty(searchWord)) {
            return ResponseJson.error("请输入搜索关键词！", null);
        }
        return orderSellerService.getOrderListByKeyword(serviceProviderId, searchWord, pageNum, pageSize);
    }
}
