package com.caimei365.order.mapper;

import com.caimei365.order.model.po.AddressPo;
import com.caimei365.order.model.vo.AddressSelectVo;
import com.caimei365.order.model.vo.AddressVo;
import com.caimei365.order.model.vo.OrderUserinfoVo;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/2
 */
@Mapper
public interface AddressMapper {
    /**
     * 收货地址列表
     * @param userId 用户Id
     */
    List<AddressVo> getUserAddressList(Integer userId);
    /**
     * 获取所有省份列表
     */
    List<AddressSelectVo> getAllProvinceList();
    /**
     * 获取当前省份下的所有城市列表
     * @param provinceId 省份Id
     */
    List<AddressSelectVo> getCityListByProvinceId(Integer provinceId);
    /**
     * 获取当前城市下的所有地区列表
     * @param cityId
     */
    List<AddressSelectVo> getTownListByCityId(Integer cityId);

    /**
     * 根据townId查询
     * @param townId
     * @return
     */
    AddressVo getAddressByTownId(Integer townId);
    /**
     * 地址Id获取收货地址
     * @param addressId 地址Id
     */
    AddressPo getAddressById(Integer addressId);
    /**
     * 获取默认地址Id
     * @param userId 用户Id
     */
    Integer getDefaultAddressId(Integer userId);
    /**
     * 修改地址默认状态
     * @param addressId 地址Id
     * @param defaultFlag 默认状态
     */
    void updateAddressDefault(Integer addressId, int defaultFlag);
    /**
     * 新增地址
     */
    void insertAddress(AddressPo address);
    /**
     * 更新地址
     */
    void updateAddress(AddressPo address);
    /**
     * 删除地址
     * @param addressId 地址Id
     * @param userId 用户Id
     */
    void deleteAddress(Integer addressId, Integer userId);
    /**
     * 订单中收货地址
     * @param orderId 订单Id
     */
    OrderUserinfoVo getOrderUserinfo(Integer orderId);
}
