package com.caimei365.order.mapper;

import com.caimei365.order.model.dto.VipSkuPriceDto;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;


/**
 * Description
 *
 * @author : Charles
 * @date : 2021/4/9
 */
@Mapper
public interface BaseMapper {
    /**
     * 获取用户组织Id -- userId
     *
     * @param userId
     * @return
     */
    Integer getOrganizeId(@Param("userId") Integer userId);

    /**
     * 获取用户组织Id -- shopOrderId
     *
     * @param shopOrderId
     * @return
     */
    Integer getOrganizeIdShop(@Param("shopOrderId") Integer shopOrderId);

    /**
     * 供应商名称
     *
     * @param shopId 供应商Id
     */
    String getShopNameById(Integer shopId);

    /**
     * 供应商名称
     *
     * @param clubId 供应商Id
     */
    String getClubNameById(Integer clubId);

    /**
     * 获取商品供应商Id
     *
     * @param skuId skuId
     */
    Integer getShopIdByproductId(Integer skuId);

    /**
     * 获取机构一级分类
     *
     * @param clubId
     * @return
     */
    String firstClubTypeById(Integer clubId);

    /**
     * 商品对应械字号类型
     *
     * @param skuIds
     * @return
     */
    List<String> machineTypeByID(List<String> skuIds);

    /**
     * 获取机构用户Id
     *
     * @param clubId 机构Id
     */
    Integer getUserIdByClubId(Integer clubId);

    /**
     * 根据用户Id查询用户身份
     */
    Integer getIdentityByUserId(Integer userId);

    /**
     * 根据用户Id查询用户身份
     */
    Integer getUserIdentity(@Param("userId") Integer userId,@Param("orderId")Integer orderId,@Param("shopOrderId")Integer shopOrderId);

    /**
     * 根据用户Id查询公司名
     */
    String getUserNameByUserId(Integer userId);

    /**
     * 根据用户Id查询名称
     * @param userId
     * @return
     */
    String getNameByUserId(Integer userId);

    /**
     * 根据用户Id查询手机号
     */
    String getBindMobileByUserId(Integer userId);

    /**
     * 根据商品ID查询阶梯价列表
     */
    List<LadderPriceVo> getLadderPriceList(Integer skuId);

    /**
     * 根据skuId和用户ID 查询复购价
     */
    Double getRepurchasePrice(@Param("skuId") Integer skuId, @Param("userId") Integer userId);

    /**
     * 供应商促销优惠活动
     *
     * @param shopId 供应商Id
     */
    PromotionsVo getPromotionByShopId(Integer shopId);

    /**
     * 商品促销优惠活动
     *
     * @param productId 商品Id
     */
    PromotionsVo getPromotionByProductId(Integer productId);

    /**
     * 促销优惠活动赠品列表
     *
     * @param promotionsId 促销Id
     */
    List<CartItemVo> getPromotionGifts(Integer promotionsId);

    /**
     * 根据商品Id获取供应商
     *
     * @param productId 商品Id
     */
    CartShopVo getShopByProductId(Integer productId);

    /**
     * 发票信息
     *
     * @param userId 用户Id
     */
    InvoiceVo getUserInvoice(Integer userId);

    /**
     * 根据用户Id获取用户余额
     *
     * @param userId 用户Id
     */
    Double getUserMoney(Integer userId);

    /**
     * 可用余额
     *
     * @param userId 用户Id
     */
    Double getAbleUserMoney(Integer userId);

    /**
     * 可用采美豆
     *
     * @param userId 用户Id
     */
    Integer getUserBeans(Integer userId);

    /**
     * 获取商品运费 （0包邮 1到付 2默认（遵循运费规则））
     *
     * @param skuIds skuId列表
     */
    List<ProductPostageVo> getPostageFlagList(@Param("skuIds") List<String> skuIds);

    /**
     * 获取商品运费 （0包邮 1到付 2默认（遵循运费规则）） -- 组织
     *
     * @param skuIds skuId列表
     */
    List<ProductPostageVo> getPostageFlagOrganizeList(@Param("skuIds") List<String> skuIds);

    /**
     * 获取省Id
     * @param cityId
     * @return
     */
    Integer getProvince(Integer cityId);

    /**
     * 指定商品的运费
     * @param productId
     * @return
     */
    Double getFreightAmount(@Param("productId") Integer productId);

    /**
     * 获取模板信息
     * @param freightId
     * @return
     */
    Integer getFreightTemplate(@Param("freightId") Integer freightId);
    /**
     * 获取运费规则
     * @param freightId
     * @param cityId
     * @return
     */
    FreightRuleVo getFreightRule(@Param("freightId") Integer freightId, @Param("cityId") Integer cityId);
    /**
     * 获取购物车商品数据
     *
     * @param userId
     * @param skuIds
     * @return
     */
    List<CartPo> getCart(Integer userId, @Param("skuIds") List<String> skuIds);

    /**
     * 统计用户订单数
     *
     * @param userId 用户Id
     */
    Integer countUserOrder(Integer userId);

    /**
     * 获取根据地区Id获取省市地址信息
     *
     * @param townId 地区Id
     */
    AddressVo getProvinceIdAndCityId(Integer townId);

    /**
     * 获取详细地址信息
     *
     * @param addressId 地址Id
     */
    Integer getTownIdByAddressId(Integer addressId);

    /**
     * 更新用户增值税发票
     */
    void updateUserInvoiceByUserId(InvoiceVo invoice);

    /**
     * 新增用户增值税发票
     */
    void insertUserInvoiceByUserId(InvoiceVo userInvoice);

    /**
     * 查询当前商品复购价信息
     *
     * @param userId 机构用户Id
     * @param skuId  skuId
     */
    PurchasePricePo getPurchasePricePo(Integer userId, Integer skuId);

    /**
     * 更新复购价格库
     */
    void updatePurchasePrice(PurchasePricePo purchase);

    /**
     * 新增复购价格库
     */
    void insertPurchasePrice(PurchasePricePo purchase);

    /**
     * 新增历史复购价记录
     */
    void insertPurchaseHistory(PurchaseHistoryPo purchaseHistory);

    /**
     * 保存 收款记录
     */
    void insertDiscernReceipt(DiscernReceiptPo discernReceipt);

    /**
     * 保存 收款项和订单关系
     */
    void insertOrderReceiptRelation(OrderReceiptRelationPo relation);

    /**
     * 保存 采美豆使用记录
     */
    void insertBeansHistory(UserBeansHistoryPo beansHistory);

    /**
     * 更新用户剩余采美豆数量
     *
     * @param userBeans 采美豆数量
     * @param userId    机构用户Id
     */
    void updateUserBeans(Integer userId, int userBeans);

    /**
     * 抵扣后更新用户余额
     *
     * @param userMoney     账户余额
     * @param ableUserMoney 账户实际可用余额
     * @param userId        机构用户Id
     */
    void updateUserMoney(Double userMoney, Double ableUserMoney, Integer userId);

    /**
     * 保存余额到余额收支记录
     */
    void insertBalanceRecord(BalanceRecordPo balanceRecord);

    /**
     * 获取条款列表
     */
    List<ClauseVo> getClauseList();

    /**
     * 查询超级会员用户id
     *
     * @param userId 用户id
     * @return
     */
    Integer getSvipUserIdByUserId(Integer userId);

    /**
     * 查询超级会员用户id
     *
     * @param clubId 机构id
     * @return
     */
    Integer getSvipUserIdByClubId(Integer clubId);

    /**
     * 查询机构状态
     *
     * @param userId 用户id
     * @return
     */
    Integer getClubStatus(Integer userId);

    /**
     * 查询最后的确认时间
     *
     * @param userId 用户id
     * @return
     */
    Date getClubConfirmTime(Integer userId);

    /**
     * 更新user机构状态
     *
     * @param clubStatus 机构状态
     * @param userId     用户id
     */
    void updateUserClubStatus(@Param("clubStatus") int clubStatus, @Param("userId") Integer userId);

    /**
     * 更新club机构状态
     *
     * @param clubStatus 机构状态
     * @param userId     用户id
     */
    void updateClubStatus(@Param("clubStatus") int clubStatus, @Param("userId") Integer userId);

    UserBeansHistoryPo findLowOrder(Integer orderId);

    Integer findBeans(Integer userId);

    /**
     * 查询认证通用户名称
     *
     * @param userId
     * @return
     */
    AuthUserVo getAuthUser(Integer userId);

    String findType(String mbOrderId);

    String findShopOrderIds(Integer orderId);

    void updateOrderOnlineFlag(Integer orderId);

    CouponVo findCoupon(Integer couponId);

    VipSkuPriceDto getSvipSku(Integer skuId);

    Double getTouchPriceBySku(Integer skuId, Integer promotionId);

    List<BalanceRecordPo> findShopOrderBalanceRecord(Integer shopOrderId);

    Integer findShopIdByProduct(Integer couponId);
}
