package com.caimei365.order.mapper;

import com.caimei365.order.model.dto.CartDto;
import com.caimei365.order.model.dto.InvoiceDto;
import com.caimei365.order.model.dto.InvoiceOrganizeDto;
import com.caimei365.order.model.po.CartPo;
import com.caimei365.order.model.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/25
 */
@Mapper
public interface CartClubMapper {
    /**
     * 购物车供应商列表
     * @param userId 用户Id
     */
    List<CartShopVo> getCartShops(Integer userId);
    /**
     * 购物车供应商下商品列表
     * @param shopId 供应商Id
     * @param userId 用户Id
     */
    List<CartItemVo> getCartProductsByShopId(@Param("shopId") Integer shopId, @Param("userId") Integer userId);

    /**
     * 外部商城 购物车供应商下商品列表
     * @param organizeId
     * @param shopId
     * @param userId
     * @return
     */
    List<CartItemVo> getCartOrganizeProductsByShopId(@Param("organizeId") Integer organizeId, @Param("shopId") Integer shopId, @Param("userId") Integer userId);
    /**
     * 获取购物车商品列表（不区分供应商）
     * @param userId 用户Id
     */
    List<CartItemVo> getCartProductList(Integer userId);

    /**
     * 获取购物车商品列表（不区分供应商） - 组织
     * @param userId
     * @return
     */
    List<CartItemVo> getCartOrganizeProductList(Integer userId);
    /**
     * 获取购物车
     * @param cartDto userId,productId
     */
    CartPo getCartPo(CartDto cartDto);
    /**
     * 更新购物车
     * @param cart CartPo
     */
    void updateCart(CartPo cart);
    /**
     * 新增购物车
     * @param cart CartPo
     */
    void insertCart(CartPo cart);
    /**
     * 删除购物车
     * @param userId     用户Id
     * @param skuIds     skuIds
     */
    void deleteCartByProductIds(@Param("userId") Integer userId, @Param("skuIds") List<String> skuIds);
    /**
     * 根据商品Ids获取购物车供应商列表
     * @param userId 用户Id
     */
    List<CartShopVo> getCartShopsByProductIds(@Param("userId") Integer userId, @Param("skuIds") List<String> skuIds,@Param("status")Integer status);

    /**
     * 商品能否退货
     * @param skuIds
     * @return
     */
    List<CartShopVo> getProductReturnGoodsStutas(@Param("skuIds") List<String> skuIds);
    /**
     * 根据商品ids获取供应商列表
     * @param productIds
     * @return
     */
    List<CartShopVo> getShopsByProductIds(@Param("productIds") List<String> productIds);
    /**
     * 根据商品Ids获取购物车已上架商品列表
     * @param userId 用户Id
     */
    List<CartItemVo> getCartProductsByShopIdAndProductIds(@Param("userId") Integer userId,@Param("shopId")  Integer shopId, @Param("skuIds") List<String> skuIds,@Param("splitCode")String splitCode);

    /**
     * 根据商品Ids获取购物车已上架商品列表 -- 组织
     * @param userId
     * @param shopId
     * @param organizeId
     * @param skuIds
     * @param splitCode
     * @return
     */
    List<CartItemVo> getCartProductsOrganize(@Param("userId") Integer userId,@Param("shopId")  Integer shopId,@Param("organizeId") Integer organizeId, @Param("skuIds") List<String> skuIds,@Param("splitCode")String splitCode);

    /**
     * 根据商品id获取已上架商品列表
     * @param shopId
     * @param skuIdList
     * @return
     */
    List<CartItemVo> getProductsByShopIdAndProductIds(@Param("shopId")  Integer shopId, @Param("skuIdList") List<Integer> skuIdList);
    /**
     * 删除用户发票信息
     */
    void deleteUserInvoice(Integer userId);
    /**
     * 新增用户发票信息
     */
    void insertUserInvoice(InvoiceDto invoiceDto);
    /**
     * 新增用户发票信息 -- 组织
     */
    void insertOrganizeUserInvoice(InvoiceOrganizeDto invoiceDto);
    /**
     * 更新用户发票信息
     */
    void updateUserInvoice(InvoiceDto invoiceDto);
    /**
     * 更新用户发票信息 -- 组织
     */
    void updateOrganizeUserInvoice(InvoiceOrganizeDto invoiceDto);
    /**
     * 获取用户发票信息
     */
    InvoiceVo getUserInvoice(Integer userId);
    /**
     * 根据skuId获取下单商品信息
     * @param skuId
     */
    CartItemVo getCartItemByProductId(Integer skuId);
    /**
     * 根据skuId获取下单商品信息 -- 组织
     * @param skuId
     * @return
     */
    CartItemVo getCartOrganizeProductId(Integer skuId);
    /**
     * 根据skuId获取下单商品信息 -- 分销人员
     * @param skuId
     * @return
     */
    CartItemVo getDistributionProductId(Integer skuId);

    List<Integer> findVipCoupon();
    /**
     * 会员机构类型：1医美，2生美
     */
    Integer getClubTypeById(Integer userId);

    void deleteCartSku(@Param("userId") Integer userId, @Param("oldSkuId") Integer oldSkuId);


    String getHelpContent(Integer helpId);

}
