package com.caimei365.order.mapper;

import java.math.BigDecimal;
import java.util.List;

import com.caimei365.order.model.po.CmReportingClub;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

/**
 * 机构报备Mapper接口
 *
 * @author Kaick
 * @date 2023-10-11
 */
@Mapper
public interface CmReportingClubMapper
{
    /**
     * 通过对象查询机构报备列表
     *
     * @param cmReportingClub 机构报备
     * @return 机构报备集合
     */
    public List<CmReportingClub> getCmReportingClubList(CmReportingClub cmReportingClub);

    /**
     * 通过Id查询机构报备对象
     *
     * @param id 机构报备主键
     * @return 机构报备
     */
    public CmReportingClub getCmReportingClubById(String id);

    /**
     * 通过对象查询机构报备对象
     *
     * @param cmReportingClub 机构报备
     * @return 机构报备
     */
    public CmReportingClub getByCmReportingClub(CmReportingClub cmReportingClub);

    /**
     * 通过对象查询机构报备Id
     *
     * @param cmReportingClub 机构报备
     * @return String
     */
    public String getById(CmReportingClub cmReportingClub);


    /**
     * 通过对象查询机构报备记录总数
     *
     * @param cmReportingClub 机构报备
     * @return 机构报备Integer
     */
    public Integer getCmReportingClubCount(CmReportingClub cmReportingClub);

    /**
     * 新增机构报备
     *
     * @param cmReportingClub 机构报备
     * @return 结果
     */
    public int addCmReportingClub(CmReportingClub cmReportingClub);

    /**
     * 修改机构报备
     *
     * @param cmReportingClub 机构报备
     * @return 结果
     */
    public int updateCmReportingClub(CmReportingClub cmReportingClub);

    /**
     * 删除机构报备
     *
     * @param id 机构报备主键
     * @return 结果
     */
    public int delCmReportingClubById(String id);

    /**
     * 批量删除机构报备
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int delCmReportingClub(CmReportingClub cmReportingClub);

    /**
     * 机构报备商品佣金
     */
    public BigDecimal getDistributionProductCommission(Integer productId);

    /**
     * 分销团队分账号
     */
    public String getDistributionSplitCode(Integer id);

}
