package com.caimei365.order.mapper;

import java.util.List;

import com.caimei365.order.model.po.CmReportingMember;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

/**
 * 分销报备成员Mapper接口
 *
 * @author Kaick
 * @date 2023-10-11
 */
@Mapper
public interface CmReportingMemberMapper
{
    /**
     * 通过对象查询分销报备成员列表
     *
     * @param cmReportingMember 分销报备成员
     * @return 分销报备成员集合
     */
    public List<CmReportingMember> getCmReportingMemberList(CmReportingMember cmReportingMember);

    /**
     * 通过Id查询分销报备成员对象
     *
     * @param id 分销报备成员主键
     * @return 分销报备成员
     */
    public CmReportingMember getCmReportingMemberById(String id);

    /**
     * 通过对象查询分销报备成员对象
     *
     * @param cmReportingMember 分销报备成员
     * @return 分销报备成员
     */
    public CmReportingMember getByCmReportingMember(CmReportingMember cmReportingMember);

    /**
     * 通过对象查询分销报备成员Id
     *
     * @param cmReportingMember 分销报备成员
     * @return String
     */
    public String getById(CmReportingMember cmReportingMember);


    /**
     * 通过对象查询分销报备成员记录总数
     *
     * @param cmReportingMember 分销报备成员
     * @return 分销报备成员Integer
     */
    public Integer getCmReportingMemberCount(CmReportingMember cmReportingMember);

    /**
     * 新增分销报备成员
     *
     * @param cmReportingMember 分销报备成员
     * @return 结果
     */
    public int addCmReportingMember(CmReportingMember cmReportingMember);

    /**
     * 修改分销报备成员
     *
     * @param cmReportingMember 分销报备成员
     * @return 结果
     */
    public int updateCmReportingMember(CmReportingMember cmReportingMember);

    /**
     * 删除分销报备成员
     *
     * @param id 分销报备成员主键
     * @return 结果
     */
    public int delCmReportingMemberById(String id);

    /**
     * 批量删除分销报备成员
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int delCmReportingMember(CmReportingMember cmReportingMember);

}
