package com.caimei365.order.mapper;

import com.caimei365.order.model.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/26
 */
@Mapper
public interface OrderCommonMapper {
    /**
     * 查询订单发货次数
     * @param orderId 订单Id
     */
    int countLogisticsBatch(Integer orderId);

    /**
     * 查询订单发货次数
     * @param shopOrderId 子订单Id
     */
    int countLogisticsBatchShop(Integer shopOrderId);
    /**
     * 组织
     * @param orderId
     * @return
     */
    Integer getOrganizeId(@Param("orderId") Integer orderId);

    /**
     * 组织
     * @param shopOrderId
     * @return
     */
    Integer getShopOrganizeId(@Param("shopOrderId") Integer shopOrderId);
    /**
     * 根据子订单Id查询子订单列表 -- 不包括已付款
     * @param shopOrderIds 子订单Ids
     */
    List<ShopOrderVo> getShopOrderList(@Param("shopOrderIds") List<String> shopOrderIds);

    /**
     * 根据子订单Id查询子订单列表
     * @param shopOrderIds
     * @return
     */
    List<ShopOrderVo> getPayShopOrderList(@Param("shopOrderIds") List<String> shopOrderIds);

    /**
     * 子订单是否支付
     * @param shopOrderId
     * @return
     */
    List<DiscernReceiptVo> getShopOrderReceipt(@Param("shopOrderId") Integer shopOrderId);
    /**
     * 根据Id获取订单促销
     * @param orderPromotionsId 订单促销Id
     */
    PromotionsVo getOrderPromotionsById(Integer orderPromotionsId);
    /**
     * 获取子订单订单商品
     * @param shopOrderId 子订单Id
     */
    List<OrderProductVo> getShopOrderProduct(Integer shopOrderId);
    /**
     * 更具订单Id获取订单商品列表
     * @param orderId 订单Id
     */
    List<OrderProductVo> getOrderProductByOrderId(Integer orderId);

    /**
     * 根据订单Id获取订单商品列表
     * @param shopOrderId
     * @return
     */
    List<OrderProductVo> getshopOrderProductByOrderId(Integer shopOrderId);
    /**
     * 支付记录(收款记录)
     * @param orderId 订单Id
     */
    List<DiscernReceiptVo> getDiscernReceipt(@Param("orderId") Integer orderId, @Param("shopOrderId") String shopOrderId);
    /**
     * 超级会员支付记录
     * @param vipRecordId 购买记录Id
     */
    DiscernReceiptVo getVipDiscernReceipt(Integer vipRecordId);
    /**
     * 获取返佣款
     * @param shopOrderId 子订单Id
     */
    Double getRebateAmountByShopOrder(Integer shopOrderId);
    /**
     * 是否可以走线上支付
     * @param orderId 订单Id
     */
    int countOfflinePayment(Integer orderId);

    /**
     * 是否线上
     * @param shopOrderId
     * @return
     */
    int countOfflinePaymentshopOrder(Integer shopOrderId);

    /**
     * 线下支付凭证
     * @param shopOrderId
     * @return
     */
    Integer getOrderVoucher(Integer shopOrderId);
    /**
     * 付供应商费用列表
     * @param orderId 订单Id
     */
    List<Double> getShouldPayShopAmountList(Integer orderId);
    /**
     * 查询关键字历史记录是否存在
     * @param searchWord 关键字
     */
    Integer getSearchHistoryIdByWord(String searchWord);
    /**
     * 更新搜索关键词历史记录
     * @param historyRecord SearchHistoryVo
     */
    void updateSearchHistory(SearchHistoryVo historyRecord);
    /**
     * 新增搜索关键词历史记录
     * @param historyRecord SearchHistoryVo
     */
    void insertSearchHistory(SearchHistoryVo historyRecord);
    /**
     * 统计搜索关键词数量
     * @param userId 用户Id
     */
    int countSearchHistory(Integer userId);
    /**
     * 只保留10条的搜索关键词历史记录
     * @param userId 用户Id
     */
    void deleteSearchHistoryLimit(Integer userId);
    /**
     * 已退货数量
     * @param shopOrderId 子订单Id
     * @param productId   商品Id
     */
    Integer countReturnedNum(Integer shopOrderId, Integer productId);
    /**
     * 已取消发货数量
     * @param shopOrderId 子订单Id
     * @param productId   商品Id
     */
    Integer countActualCancelNum(Integer shopOrderId, Integer productId);

    /**
     * 子订单退款状态
     * @param shopOrderId
     * @return
     */
    Integer getRefundStatus(@Param("shopOrderId") Integer shopOrderId );
    /**
     * 根据订单Id获取订单
     * @param orderId 订单Id
     */
    OrderVo getOrderByOrderId(Integer orderId);

    /**
     * 根据子订单Id获取子订单数据
     * @param shopOrderId
     * @return
     */
    ShopOrderVo getShopOrderById(Integer shopOrderId);
    /**
     * 根据子订单Id获取订单
     * @param shopOrderId 订单Id
     */
    OrderVo getOrderByShopOrderId(Integer shopOrderId);
    /**
     * 根据订单Id获取子订单列表
     */
    List<ShopOrderVo> getShopOrderListByOrderId(Integer orderId);
    /**
     * 获取物流批次列表
     * @param orderId 订单Id
     */
    List<LogisticsBatchVo> getLogisticsBatchList(Integer orderId);
    /**
     * 获取物流批次列表
     * @param shopOrderId 子订单Id
     */
    List<LogisticsBatchVo> getShopLogisticsBatchList(Integer shopOrderId);
    /**
     * 获取发货物流记录
     * @param shopOrderId      子订单Id
     * @param logisticsBatchId 物流批次Id
     */
    List<LogisticsRecordVo> getLogisticsRecord(Integer shopOrderId, Integer logisticsBatchId);
    /**
     * 获取物流信息
     * @param logisticsBatchId 物流批次Id
     */
    List<LogisticsInformationVo> getLogisticsInfoList(Integer logisticsBatchId);
    /**
     * 获取用户可用优惠券列表
     * @param userId 用户Id
     */
    List<CouponVo> getClubCouponList(Integer userId);
    /**
     * 未领取的优惠券
     * @param userId 用户Id
     * @param registerTime
     */
    List<CouponVo> getPrevCouponList(@Param("userId") Integer userId, @Param("registerTime") Date registerTime);
    /**
     * 活动券商品Ids
     */
    List<Integer> getCouponProductIds(Integer couponId, Integer source);
    /**
     * 获取订单发票信息
     */
    InvoiceVo getOrderInvoice(Integer orderId);
    /**
     * 根据用户id查询注册时间
     */
    Date getUserRegisterTime(Integer userId);

    Integer finduserBean(Integer orderId);

    Integer findSplitCode(Integer orderId);

    List<DiscernReceiptVo> getShopOrderDiscernReceipt(Integer shopOrderId);

    ShopOrderVo getShopOrderByOrderId(Integer shopOrderId);

    List<OrderProductVo> getOrderProductByShopOrderId(Integer shopOrderId);

    Double findShopFee(Integer shopOrderId);

    Double findEachDiscount(Integer shopOrderId);

    String findWeChatFlag(String splitCode,Integer shopId);

    Integer findPayB2BRecord(Integer shopOrderId);

    Integer findMaxStock(Integer productId);

    String findOnlinePayWays(Integer shopOrderId);

    String getHelpContent(Integer helpId);

    void updateShopOrderBalance(ShopOrderVo s);
}

