package com.caimei365.order.mapper;

import com.caimei365.order.model.bo.VipRecordBo;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/29
 */
@Mapper
public interface PayOrderMapper {
    /**
     * 获取线上支付全局开关状态
     */
    Integer getPayOnLineSwitch();

    /**
     * 供应商分帐账号
     * @param shopId
     * @return
     */
    String getShopCode(@Param("shopId") Integer shopId);
    /**
     * 查询审核中的退款退货数量
     * @param orderId 订单Id
     */
    int getRefundCount(Integer orderId);

    /**
     * 查询审核中的退款退货数量
     * @param shopOrderId
     * @return
     */
    int getShopRefundCount(Integer shopOrderId);

    /**
     * 主订单下子订单收款状态
     * @param orderId
     * @return
     */
    List<Integer> getReceiptStatus(Integer orderId);
    /**
     * 更新订单状态
     * @param order 主订单
     */
    void updateOrderStatus(OrderVo order);

    /**
     * 更新子订单状态
     * @param shopOrderVo 子订单
     */
    void updateShopOrderStatus(ShopOrderVo shopOrderVo);
    /**
     * 根据用户Id更新余额
     * @param userMoney      用户余额
     * @param ableUserMoney 用户可用余额
     * @param userId 用户Id
     */
    void updateMoneyByUserId(Double userMoney, Double ableUserMoney,Integer userId);

    void updateAbleMoney( Double ableUserMoney,Integer userId);
    /**
     * 订单支付链接
     * @param orderId 订单Id
     * @param amount  本次待付款金额
     */
    OrderPayLinkVo getOrderPayLink(Integer orderId, Double amount);
    /**
     * svip支付链接
     * @param vipRecordId   会员购买记录Id
     * @param amount  本次待付款金额
     */
    OrderPayLinkVo getVipPayLink(Integer vipRecordId, Double amount);
    /**
     * 支付链接
     * @param linkLogo  链接标识
     */
    OrderPayLinkVo getPayLinkByLinkLogo(String linkLogo);
    /**
     * 新增支付链接
     */
    void insertOrderPayLink(OrderPayLinkVo orderPayLink);
    /**
     * 更新支付链接
     */
    void updateOrderPayLink(OrderPayLinkVo orderPayLink);
    /**
     * 修改支付链接状态
     */
    void updateOrderPayLinkStatus(OrderPayLinkVo orderPayLink);
    /**
     * 订单促销Id
     * @param orderId 订单Id
     */
    Integer getOrderPromotionsId(Integer orderId);
    /**
     * 付款次数
     * @param shopOrderId 订单Id
     */
    Integer getDbPaySuccessCounter(Integer shopOrderId);

    /**
     * 根据用户id获取用户openid列表(微信公众号openid,下单)
     */
    List<String> getOpenidListByPermission(String unionid);

    /**
     * 保存分账详情
     */
    void insertSplitAccount(SplitAccountPo splitAccount);
    /**
     * 更新二手详情
     */
    void updateSeconHandDetail(Integer productId, String payType, double payAmount, String payFormData);
    /**
     * 修改分账付款状态
     */
    void updateSplitAccountByPay(String mbOrderId);
    /**
     * 修改收款分账状态
     */
    void updateBySplitStatus(String mbOrderId);
    /**
     * 查询分账给子商户号
     */
    List<SplitAccountPo> getSplitAccountList(String mbOrderId);
    /**
     * 查询已付供应商金额
     */
    Double getPaidShopAmount(Integer shopOrderId);
    /**
     * 修改子订单付款状态及付款金额
     */
    void updateShopOrderByPayStatus(Integer shopOrderId, Double paidShop, Integer payStatus);
    /**
     * 修改主订单付款状态
     */
    void updateOrderByPayStatus(Integer orderId, Integer payStatus);
    /**
     * 保存付款单表
     */
    void insertPayShop(PayShopPo payShop);
    /**
     * 保存付供应商记录
     */
    void insertPayShopRecord(PayShopRecordPo shopRecord);
    /**
     * 查询未分账已支付收款
     */
    List<OrderReceiptRelationPo> getUndividedPaidReceipt(String currentTime);
    /**
     * 查询已支付金额
     * @param orderProductId 订单商品Id
     */
    Double getOrderProductPaidAmount(Integer orderProductId);
    /**
     * 查询子商户商编
     * @param shopId 供应商Id
     */
    String getShopCommercialCode(Integer shopId);
    /**
     * 查询已支付运费
     */
    Double getPaidShipping(Integer orderId, Integer shopId);
    /**
     * 获取会员套餐价格
     * @param id 会员套餐Id
     */
    Double getVipPackagePriceById(Integer id);
    /**
     * 获取会员套餐月份
     * @param id 会员套餐Id
     */
    Integer getVipPackageDurationById(Integer id);
    /**
     * 获取会员套餐记录
     * @param id
     */
    VipRecordBo getVipPackageRecord(Integer id);
    /**
     * 获取会员信息
     * @param userId
     */
    UserVipPo getUserVipInfo(Integer userId);
    /**
     * 新增会员信息
     */
    void insertUserVipInfo(UserVipPo userVip);
    /**
     * 更新会员信息
     */
    void updateUserVipInfo(UserVipPo userVip);
    /**
     * 更新会员支付历史记录
     */
    void updateVipPackageRecord(VipRecordBo record);

    List<BankCodeVo> findBankCode();
    /**
     * 查询认证通会员开通历史记录
     * @param authVipRecordId
     * @return
     */
    VipRecordBo getAuthVipRecord(Integer authVipRecordId);

    /**
     * 查询认证通会员套餐价格
     * @param packageId
     * @return
     */
    Double getAuthVipPackagePrice(Integer packageId);

    /**
     * 查询认证通会员信息
     * @param userId
     * @return
     */
    UserVipPo getAuthVipInfo(int userId);

    /**
     * 查询认证通会员套餐信息
     * @param vipId
     * @return
     */
    AuthVipPackagePo getAuthVipPackageById(int vipId);

    /**
     * 更新认证通会员信息
     */
    void updateAuthVipInfo(UserVipPo authVip);
    /**
     * 新增认证通会员信息
     */
    void insertAuthVipInfo(UserVipPo authVip);
    /**
     * 更新认证通会员支付历史记录
     */
    void updateAuthVipRecord(VipRecordBo record);

    /**
     * 认证通会员支付链接
     * @param recordId   会员购买记录Id
     * @param amount  本次待付款金额
     */
    OrderPayLinkVo getAuthVipPayLink(@Param("recordId") int recordId, @Param("amount") Double amount);

    Double getCouponPrice(Integer recordId);

    void insertCouponRecord(CouponRecordPo cr);

    void updateCouponRecord(Integer recordId);

    Integer findPayStatus(Integer recordId);

    Integer findClubType(Integer userId);

    List<CouponCheckVo> findCouponOrder();

    void deleteAuthVipRoleRelation(int userId);

    void insertAuthVipRoleRelation(Integer userId, Integer roleId);

    void updateSplitAccount(String mbOrderId, String orderRequestNo);

    String findShopOrderSplitCode(Integer shopOrderId);

    void updateShopOrder(ShopOrderVo shopOrder);

    Integer getOrderId(Integer shopOrderId);

    List<BankCodeVo> findB2BBankCode();

    List<BankCodeVo> findB2CBankCode();

    String getSplitCodeByShopOrderId(Integer shopOrderId);

    void updateBySplitStatusByNo(String shopOrderId);

    QuickInfoVo findInfoByBankNumber(String quickPayBankCode);

    void insertQuickPayInfo(QuickInfoVo quickInfoVo);

    void updateQuickInfo(String bankCode, Integer bankType, Integer infoId);

    void delQuickInfo(String infoId);

    void insertBindId(String bindId, String customerNumber, String userId, Integer infoId);

    List<QuickBankVo> findQuickBank(Integer cardType,String initial);

    QuickInfoVo getQuickInfoById(Integer cardId);

    void delQuickBindId(String quickBindId);

    List<String> findQuickBankInitial(Integer cardType);

    void updateQuickPayInfo(QuickInfoVo quickInfoVo);

    List<QuickBankVo> findUnCodeBanks();

    QuickInfoVo findInfoById(Integer infoId);

    QuickInfoVo findInfoNotDel(String infoId);

    void updateShopOrderBalancePay(ShopOrderVo shopOrder);
}
