package com.caimei365.order.mapper;

import com.caimei365.order.model.bo.ReceiptParamsBo;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2022/2/11
 */
@Mapper
public interface ReceiptMapper {
    /**
     * 保存收款记录
     */
    void insertReceipt(ReceiptPo receiptPo);

    /**
     * 更新收款记录
     */
    void updateReceipt(ReceiptPo receiptPo);

    /**
     * 根据openid获取收款用户
     */
    ReceiptUserVo getReceiptUserByOpenid(String openId);

    /**
     * 根据收款用户Id获取姓名
     */
    String getReceiptUserNameById(Integer id);

    /**
     * 根据mobile获取收款用户
     */
    ReceiptUserVo getReceiptUserByMobile(String mobile);

    /**
     * 根据mobile更新openid
     */
    void updateOpenidByMobile(String openid, String unionId, String mobile);

    /**
     * 获取用户权限: 1录入收款,2收款列表操作,3关联订单款,4关联返佣款,5口头返佣,6关联供应商退款,7审核订单款,8收款详情分享,
     * 9非订单款查看(全部)，10订单款查看(全部)，11返佣款查看(全部)，12供应商退款查看(全部)，13订单款查看(协销自己机构)，14返佣款查看(协销自己机构)，15供应商退款查看(协销自己机构)
     */
    List<Integer> getPermissionsByUserId(Integer userId);

    /**
     * 获取收款类型列表
     */
    List<cmOfflineCollectionVo> getReceiptType();

    /**
     * 获取收款类型
     */
    ReceiptTypeVo getReceiptTypeById(Integer id);

    String getReceiptTypeStrById(Integer id);

    /**
     * 短信内容Md5加密串 获取收款记录
     */
    List<ReceiptVo> getReceiptBySmsMd5Code(String smsMd5Code);

    /**
     * 获取收款类型
     */
    Integer getReceivablesTypeId(String type);

    /**
     * 获取订单已支付金额(待审核状态的收款也算)
     */
    Double getPaidAmountByOrderId(Integer shopOrderId);

    /**
     * 获取订单已支付金额(待审核状态的收款也算)
     * @param orderId
     * @return
     */
    Double getAmountByOrderId(Integer orderId);

    /**
     * 确认订单类型
     * @param shopOrderId
     * @return
     */
    List<Integer> getDiscernConfirmType(Integer shopOrderId);

    /**
     * 获取订单待审核金额
     */
    Double getUncheckAmount(Integer shopOrderId);

    /**
     * 获取订单已退金额
     */
    Double getReturnedAmount(Integer shopOrderId);

    /**
     * 获取主订单下面的所有的确认收货商品数量
     */
    Integer getReceiptNumByOrderId(Integer shopOrderId);

    /**
     * 获取该主订单下面所有已完成状态退货退款商品数量
     */
    Integer getReturnAndCancelNum(Integer shopOrderId);

    /**
     * 获取协销Id
     */
    Integer getServiceProviderId(String unionId);

    /**
     * 获取协销下的机构userIds
     */
    List<Integer> getClubUserIdBySpId(Integer spId);

    /**
     * 获取收款列表
     */
    List<ReceiptVo> getReceiptList(ReceiptParamsBo receiptParamsBo);

    /**
     * 获取收款详情
     */
    ReceiptVo getReceiptDetail(Integer id);

    /**
     * 获取收款关联订单 shopOrderId
     */
    List<ShopOrderVo> getReceiptShopOrders(Integer receiptId);

    /**
     * 获取收款关联订单 orderId
     * @param receiptId
     * @return
     */
    List<ShopOrderVo> getReceiptOrders(Integer receiptId);

    List<ShopOrderVo> getReceiptOrderlist(Integer receiptId);

    List<OrderVo> getReceiptOrderlists(Integer receiptId);

    /**
     * 获取收款商品订单列表
     */
    List<ShopOrderVo> getOrderListByParams(ReceiptParamsBo receiptParamsBo);

    List<ShopOrderVo> getOrderListByOrderIds(@Param("orderIdS") String[] orderIdS);

    /**
     * 存在退款中的记录
     */
    Integer countHaveReturning(Integer shopOrderId);

    /**
     * 存在收款中的记录
     */
    Integer countHaveReceipting(Integer shopOrderId);

    /**
     * 线上支付收款Id
     */
    Integer getOnlineReceiptId(Integer shopOrderId);

    /**
     * 获取已退款金额
     */
    List<ReturnedPurchaseVo> getRefundFeeByOrderId(Integer shopOrderId);

    /**
     * 已关联子订单
     */
    List<ShopOrderVo> getShopOrderListByReceiptId(Integer receiptId);

    /**
     * 搜索子订单进行关联
     */
    List<ShopOrderVo> getRefundShopOrders(Integer confirmedType, String shopName, String userName, Integer orderId);

    /**
     * 返佣款-子订单列表
     */
    List<ShopOrderVo> getRebateShopOrders(String shopName);

    /**
     * 获取子订单信息
     */
    ShopOrderVo getShopOrderById(Integer shopOrderId);

    /**
     * 返佣订单付款总金额
     */
    double countAllPayAssociateAmount(Integer orderId);

    /**
     * 需要支付金额
     */
    double countNeedPayAmount(Integer orderId);

    /**
     * 删除之前与这个收款所有的关系
     */
    void deleteReceiptRelation(Integer receiptId);

    /**
     * 子订单付款类型列表
     */
    List<Integer> getRebatePayTypeList(Integer shopOrderId);

    /**
     * 根据用户类型获取用户openid列表(微信公众号openid,非收款小程序)
     */
    List<String> getOpenidListByPermission(List<Integer> permissions);

    /**
     * 统计收款金额
     */
    double countAssociateAmountById(Integer receiptId);

    /**
     * 总退款金额
     */
    double getRefundShopAmount(Integer shopOrderId);

    /**
     * 保存供应商退款
     */
    void insertRefundShop(RefundShopPo refundShopPo);

    /**
     * 保存供应商退款记录
     */
    void insertRefundShopRecord(RefundShopRecordPo refundShopRecord);

    /**
     * 供应商Id
     */
    Integer getShopIdByShopOrderId(Integer shopOrderId);

    /**
     * 查询这个子订单的主订单下面的所有子订单ID
     */
    List<Integer> getSubShopOrderList(Integer shopOrderId);

    /**
     * 查询当前子订单列表中已经返佣了的子订单
     */
    List<Integer> getSubShopOrderRebate(@Param("shopOrderIdList") List<Integer> shopOrderIdList);

    /**
     * 修改(付款供应商)付款状态
     *
     * @param shopOrderId 子订单Id
     * @param payStatus   付款状态
     * @param receiptStatus   收款状态：1待收款、2部分收款、3已收款
     */
    void updateShopOrderPayStatus(Integer shopOrderId, Integer payStatus, Integer receiptStatus);

    /**
     * 收款项和订单关系
     */
    List<OrderReceiptRelationPo> getOrderReceiptRelationList(Integer receiptId);

    /**
     * 查询待审核数量
     */
    int getPendingAuditCount(Integer shopOrderId);

    /**
     * 添加 返佣欠款记录
     */
    void insertRebateRecord(RebateRecordPo crr);

    /**
     * 更新供应商欠款金额
     */
    void updateShopRebateAmount(Integer shopId, Double amount);

    List<ShopOrderVo> ReceiptDetailVo(String keyWord);

    List<ShopOrderVo> ReceiptDetail(String keyWord);

    List<ShopOrderVo> ReceiptDetails(String keyWord);

    List<Integer> getTeamClubUserIdBySpId(Integer spId);

    PutAwayFeePo findPutAwayFee(Integer shopId);

    void insertPutAwayFeeRecordForNew(PutAwayFeePo history);

    void insertPutAwayFeeRecordForOld(PutAwayFeePo history);

    Double findCmCost(Integer shopOrderId);

    void updateReceiptStatus(Integer shopOrderId);

    List<ShopOrderVo> getPlatformOrderByReceiptId(Integer id);

    List<ShopOrderVo> getPlatformOrderByName(String shopName);
}
