package com.caimei365.order.model.dto;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/6
 */
@Data
public class SubmitDto implements Serializable {
    private static final long serialVersionUID = 1L;
    /**
     * orderSeen:订单对机构可见度，1可见，2不可见
     */
    @ApiModelProperty("订单对机构可见度，1可见，2不可见")
    private Integer orderSeen;
    /**
     * 机构ID
     */
    @ApiModelProperty("机构Id")
    private Integer clubId;
    /**
     * 微信unionId
     */
    @ApiModelProperty("机构运营人员微信unionId")
    private String unionId;
    /**
     * 收货地址Id
     */
    @ApiModelProperty("收货地址Id")
    private Integer addressId;
    /**
     * 订单来源：1WWW、2CRM、4客服[适用后台下单]、5外单[适用后台下单]、6小程序[采美,星范]、7呵呵商城小程序、8维沙小程序
     */
    @ApiModelProperty("订单来源：1WWW、2CRM、4客服[适用后台下单]、5外单[适用后台下单]、6小程序[采美,星范]、7呵呵商城小程序、8维沙小程序")
    private Integer orderSource;
    /**
     * 购买类型:（1自主下单(购物车),2自主下单(立即购买)，3协销下单）
     */
    @ApiModelProperty("购买类型:（1自主下单(购物车),2自主下单(立即购买)，3协销下单）")
    private Integer cartType;
    /**
     * 协销Id
     */
    @ApiModelProperty("协销Id")
    private Integer serviceProviderId;
    /**
     * 报备机构ID
     */
    @ApiModelProperty("报备机构ID")
    private String clubReportingId;
    /**
     * 优惠券Id
     */
    @ApiModelProperty("优惠券Id")
    private Integer clubCouponId;
    /**
     * 订单商品数据：[
     *            { "shopId":1001,    // 供应商ID
     *              "note":备注,
     *              "productInfo":[   // 商品id，数量，赠品数,商品类型
     *                  {"productId": 2789, "productNum": 1, "presentNum":0,"productType":2},
     *                  {"productId": 2789, "productNum": 1, "presentNum":0,"productType":0}
     *               ]
     *            },
     *            {多个供应商商品数据结构同上}
     *         ]
     */
    @ApiModelProperty("订单商品数据")
    private String orderInfo;
    /**
     * 订单金额数据：{
     *         "orderShouldPayFee": 609.11,
     *         "balancePayFlag": 0,   余额支付标识，0不使用，1使用
     *         "clauseId": "2",
     *         "postage": "15",
     *         "postageFlag": 1, // 运费标志：0包邮 -1到付 1遵循运费规则
     *         "userBeans": 100,//抵扣采美豆数量
     *         "rebateFlag":0
     *        }
     */
    @ApiModelProperty("订单金额数据")
    private String payInfo;
    /**
     * 发票信息：{"type": 0}// 不开发票
     *      或：{ // 普通发票
     *              "type": 1,
     *              "invoiceTitle": "企业抬头",
     *              "invoiceTitleType": 1,
     *              "corporationTaxNum": "XXX4156465465",
     *              "invoiceContent": "明细 "
     *
     *          }
     *      或：{ // 增值税发票
     *              "type": 2,
     *              "invoiceTitle": "单位名称",
     *              "corporationTaxNum": "NSRSBM97897",
     *              "registeredAddress": "注册地址",
     *              "registeredPhone": "15814011616",
     *              "openBank": "开户银行",
     *              "bankAccountNo": "987987465465464"
     *          }
     */
    @ApiModelProperty("发票信息")
    private String orderInvoice;

    /**
     * 订单提交状态 0初始提交 1 继续提交
     */
    @ApiModelProperty("订单提交状态")
    private Integer orderMiniType;

    @Override
    public String toString() {
        return "SubmitDto{" +
                "orderSeen=" + orderSeen +
                ", clubId=" + clubId +
                ", unionId='" + unionId + '\'' +
                ", addressId=" + addressId +
                ", orderSource=" + orderSource +
                ", cartType=" + cartType +
                ", serviceProviderId=" + serviceProviderId +
                ", clubCouponId=" + clubCouponId +
                ", orderInfo='" + orderInfo + '\'' +
                ", payInfo='" + payInfo + '\'' +
                ", orderInvoice='" + orderInvoice + '\'' +
                ", orderMiniType=" + orderMiniType +
                '}';
    }
}
