package com.caimei365.order.model.enums;

/**
 * 订单状态
 *
 * @author : Charles
 * @date : 2021/8/17
 */
public enum OrderStatus {
    /**
     * 0待确认,11待收待发,12待收部发,13待收全发,21部收待发,22部收部发,23部收全发,31已收待发,32已收部发,33已收全发,4交易完成,5订单完成,6已关闭,7交易全退
     */
    UNCONFIRMED(0, "待确认"),
    UNRECEIVED_AND_UNSHIPPED(11, "待收待发"),
    UNRECEIVED_AND_PART_SHIPPED(12, "待收部发"),
    UNRECEIVED_AND_FULL_SHIPPED(13, "待收全发"),
    PART_RECEIVED_AND_UNSHIPPED(21, "部收待发"),
    PART_RECEIVED_AND_PART_SHIPPED(22, "部收部发"),
    PART_RECEIVED_AND_FULL_SHIPPED(23, "部收全发"),
    RECEIVED_AND_UNSHIPPED(31, "已收待发"),
    RECEIVED_AND_PART_SHIPPED(32, "已收部发"),
    RECEIVED_AND_FULL_SHIPPED(33, "已收全发"),
    TRANSACTION_COMPLETED(4, "交易完成"),
    ORDER_COMPLETED(5, "订单完成"),
    FULL_RETURNED(7, "交易全退"),
    CLOSED(6, "已关闭"),
    // ---数据库不存在的状态
    WAIT_PAY(1,"待支付"),
    TRANSACTION(2,"交易中"),
    REFUND_RETURN(8,"退货退款");

    private final int code;
    private final String desc;

    public int getCode() {
        return code;
    }

    public String getDesc() {
        return desc;
    }

    OrderStatus(int code, String desc) {
        this.code = code;
        this.desc = desc;
    }

    public static String getOrderStatusText(int code){
        for(OrderStatus type : OrderStatus.values()){
            if(type.getCode() == code){
                return type.desc;
            }
        }
        return "";
    }

}
