package com.caimei365.order.service;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.HeliDto;
import com.caimei365.order.model.dto.PayCouponDto;
import com.caimei365.order.model.dto.PaySecondDto;
import com.caimei365.order.model.dto.PayVipDto;
import com.caimei365.order.model.vo.NotifyResponseVo;
import com.caimei365.order.model.vo.QuickPayConfirmPayResponseVo;
import com.caimei365.order.model.vo.UnionResVo;
import org.springframework.http.HttpHeaders;

import java.beans.IntrospectionException;
import java.lang.reflect.InvocationTargetException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;

public interface HeliPayNonOrderService {
    ResponseJson<PayCouponDto> getCouponRecord(PayCouponDto payCouponDto, HttpHeaders headers);

    ResponseJson<JSONObject> payCouponByWeChat(HeliDto heliDto, HttpHeaders headers) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    String couponCallback(NotifyResponseVo res) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    /**
     * 二手发布微信线上支付
     *
     * @param heliDto {
     *                productId      二手发布商品id
     *                returnUrl     页面回调地址
     *                code          微信小程序code
     *                state         微信公众号state参数
     *                }
     */
    ResponseJson<JSONObject> paySecondByWeChat(HeliDto heliDto, HttpHeaders headers) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    /**
     * 二手发布支付宝线上支付
     *
     * @param heliDto {
     *                productId      二手发布商品id
     *                returnUrl     页面回调地址
     *                }
     */
    ResponseJson<JSONObject> paySecondByAlipay(HeliDto heliDto, HttpHeaders headers) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    /**
     * 二手发布支付回调
     */
    String paymentSecondCallback(NotifyResponseVo res) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    /**
     * 升级超级会员-微信线上支付
     *
     * @param heliDto {
     *                vipRecordId   会员购买记录Id
     *                returnUrl     页面回调地址
     *                code          微信小程序code
     *                state         微信公众号state参数
     *                }
     */
    ResponseJson<JSONObject> paySuperVipByWeChat(HeliDto heliDto, HttpHeaders headers) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    /**
     * 升级超级会员-支付宝线上支付
     *
     * @param heliDto {
     *                vipRecordId   会员购买记录Id
     *                returnUrl     页面回调地址
     *                }
     */
    ResponseJson<JSONObject> paySuperVipByAlipay(HeliDto heliDto, HttpHeaders headers) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    /**
     * 升级超级会员-支付回调
     */
    String paymentSuperVipCallback(NotifyResponseVo data) throws NoSuchAlgorithmException, InvalidKeySpecException, IntrospectionException, InvocationTargetException, IllegalAccessException;

    ResponseJson<String> unionPay(HeliDto heliDto, HttpHeaders headers) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    ResponseJson<String> unionPaySecond(HeliDto heliDto, HttpHeaders headers) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    /**
     * 升级超级会员-网银支付回调
     */
    String unionVipCallback(UnionResVo data) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    /**
     * 二手发布-网银支付回调
     */
    String unionSecondCallback(UnionResVo res) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    ResponseJson<JSONObject> payCouponByAlipay(HeliDto heliDto, HttpHeaders headers) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    ResponseJson<String> couponUnionPay(HeliDto heliDto, HttpHeaders headers) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    String unionCouponCallback(UnionResVo data) throws IntrospectionException, InvocationTargetException, IllegalAccessException;

    ResponseJson couponCheck(Integer couponRecordId);

    /**
     * 认证通会员网银支付
     */
    ResponseJson<String> authVipUnionPay(HeliDto heliDto, HttpHeaders headers);

    /**
     * 认证通会员微信支付
     */
    ResponseJson<JSONObject> payAuthVipByWeChat(HeliDto heliDto, HttpHeaders headers) throws IllegalAccessException, IntrospectionException, InvocationTargetException;

    /**
     * 认证通会员支付宝支付
     */
    ResponseJson<JSONObject> payAuthVipByAlipay(HeliDto heliDto, HttpHeaders headers);

    /**
     * 认证通会员支付回调
     */
    String paymentAuthVipCallback(NotifyResponseVo data) throws IllegalAccessException, IntrospectionException, InvocationTargetException;

    /**
     * 认证通会员网银支付回调
     */
    String unionAuthVipCallback(UnionResVo data) throws IllegalAccessException, IntrospectionException, InvocationTargetException;

    /**
     * 认证通会员购买确认
     */
    ResponseJson authVipCheck(Integer vipRecordId);

    ResponseJson vipCheck(Integer recordId);

    ResponseJson<JSONObject> payCouponByBeans(HeliDto heliDto, HttpHeaders headers);

    String quickCallback(QuickPayConfirmPayResponseVo res);
}
