package com.caimei365.order.service;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.VoucherDto;
import com.caimei365.order.model.po.ShopPo;
import com.caimei365.order.model.vo.*;
import com.github.pagehelper.PageInfo;
import org.springframework.http.HttpHeaders;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/19
 */
public interface OrderClubService {
    /**
     * 获取各状态订单数量
     * @param userId 用户id
     */
    ResponseJson<Map<String, Object>> getOrderCount(Integer userId);
    /**
     * 机构订单列表
     *
     * @param userId     用户Id
     * @param orderState 1:待确认,2:待付款,3:待发货,4:已发货,5:退货款
     * @param orderNo    订单编号
     * @param beginTime  开始时间
     * @param endTime    截止时间
     * @param pageNum    页码
     * @param pageSize   每页数量
     */
    ResponseJson<PageInfo<ShopOrderVo>> getOrderList(Integer userId, Integer orderState, String orderNo, String beginTime, String endTime, int pageNum, int pageSize);
    /**
     * 根据关键词搜索订单
     * @param userId     用户Id
     * @param searchWord 搜索关键词
     * @param pageNum    页码
     * @param pageSize   每页数量
     */
    ResponseJson<PageInfo<ShopOrderVo>> getOrderListByKeyword(Integer userId, String searchWord, int pageNum, int pageSize);
    /**
     * 机构订单详情
     * @param userId     用户Id
     * @param shopOrderId    订单Id
     */
    ResponseJson<Map<String, Object>> getOrderDetail(Integer userId, Integer shopOrderId);

    /**
     * 上传支付凭证
     * @param voucherDto
     * @return
     */
    ResponseJson voucher(VoucherDto voucherDto);
    /**
     * 机构最新订单
     * @param userId     用户Id
     */
    ResponseJson<OrderVo> getOrderLatest(Integer userId);
    /**
     * 机构确认订单
     * @param shopOrderId  订单Id
     */
    ResponseJson<Void> confirmMainOrder(Integer shopOrderId);
    /**
     * 机构取消订单
     * @param shopOrderId  订单Id
     * @param userIdentity    用户身份 1协销 其他用户
     * @param remark    备注（订单取消原因）
     */
    ResponseJson<Void> cancelMainOrder(Integer shopOrderId,Integer userIdentity, String remark);
    /**
     * 机构删除订单
     * @param shopOrderId  订单Id
     */
    ResponseJson<Void> deleteMainOrder(Integer shopOrderId);
    /**
     * 机构确认收货
     * @param shopOrderId 子订单Id
     */
    ResponseJson<Void> receiveGoods(Integer shopOrderId);
    /**
     * 根据用户Id查找订单搜索历史记录
     * @param userId  用户Id
     */
    ResponseJson<List<SearchHistoryVo>> getOrderSearchHistory(Integer userId);
    /**
     * 根据用户Id删除订单搜索历史记录
     * @param userId  用户Id
     */
    ResponseJson<Void> deleteOrderSearchHistory(Integer userId);
    /**
     * 物流详情
     * @param shopOrderId 子订单Id
     */
    ResponseJson<List<LogisticsBatchVo>> getOrderLogistics(Integer shopOrderId);
    /**
     * 分享订单，获取分享码
     * @param shopOrderId 订单Id
     */
    ResponseJson<String> getOrderShareCode(Integer shopOrderId);
    /**
     * 分享订单，用户身份
     * @param shopOrderId 订单Id
     * @param userId 用户Id
     * @param serviceProviderId 协销Id
     * @param code 微信code
     * @param encryptedData 微信加密数据
     * @param iv 微信加密算法的初始向量
     * @param headers HttpHeaders
     */
    ResponseJson<String> identityOrderShareCode(Integer shopOrderId, Integer userId, Integer serviceProviderId, String code, String encryptedData, String iv, HttpHeaders headers);
    /**
     * 分享订单，校验分享码
     * @param userId 用户Id
     * @param shopOrderId 订单Id
     * @param shareCode 分享码
     * @param code 微信code
     * @param encryptedData 微信加密数据
     * @param iv 微信加密算法的初始向量
     * @param headers HttpHeaders
     */
    ResponseJson<String> checkOrderShareCode(Integer userId, Integer shopOrderId, String shareCode, String code, String encryptedData, String iv, HttpHeaders headers);
    ResponseJson<String> checkOrderShareCode(Integer userId, Integer shopOrderId, String shareCode);
    /**
     * 分享订单，初始商品数据
     * @param shopOrderId 订单Id
     */
    ResponseJson<Map<String,Object>> getOrderShareInitProduct(Integer shopOrderId);
    /**
     * 二手订单 确认打款供应商
     * @param orderId 订单Id
     * @param userId 用户Id
     */
    ResponseJson<Void> secondPaymentConfirm(Integer orderId, Integer userId);
    /**
     * 根据售后条款ID查看售后条款内容
     * @param clauseId 售后条款Id
     */
    ResponseJson<ClauseVo> getClauseById(Integer clauseId);

    /**
     * 查询供应商银行卡信息
     * @param shopId
     * @return
     */
    ResponseJson<ShopPo> getShopBankAccount(Integer shopId);

}
