package com.caimei365.order.service;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.PayCouponDto;
import com.caimei365.order.model.dto.PaySecondDto;
import com.caimei365.order.model.dto.PayVipDto;
import org.springframework.http.HttpHeaders;

import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/9/27
 */
public interface PayNonOrderService {
    /**
     * 二手发布微信线上支付
     * @param paySecondDto {
     *               productId      二手发布商品id
     *               returnUrl     页面回调地址
     *               code          微信小程序code
     *               state         微信公众号state参数
     * }
     */
    ResponseJson<JSONObject> paySecondByWeChat(PaySecondDto paySecondDto, HttpHeaders headers);
    /**
     * 二手发布支付宝线上支付
     * @param paySecondDto {
     *               productId      二手发布商品id
     *               returnUrl     页面回调地址
     * }
     */
    ResponseJson<JSONObject> paySecondByAlipay(PaySecondDto paySecondDto, HttpHeaders headers);
    /**
     * 二手发布银联线上支付
     * @param paySecondDto {
     *               productId       二手发布商品id
     *               returnUrl     页面回调地址
     *               bankCode      银行编码(银联支付使用)
     *               userType      用户类型(银联支付使用)企业:ENTERPRISE，个人:USER
     * }
     */
    ResponseJson<JSONObject> paySecondByUnionPay(PaySecondDto paySecondDto, HttpHeaders headers);
    /**
     * 二手发布支付回调
     */
    String paymentSecondCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException;

    /**
     * 升级超级会员-微信线上支付
     * @param payVipDto {
     *               vipRecordId   会员购买记录Id
     *               returnUrl     页面回调地址
     *               code          微信小程序code
     *               state         微信公众号state参数
     * }
     */
    ResponseJson<JSONObject> paySuperVipByWeChat(PayVipDto payVipDto, HttpHeaders headers);
    /**
     * 升级超级会员-支付宝线上支付
     * @param payVipDto {
     *               vipRecordId   会员购买记录Id
     *               returnUrl     页面回调地址
     * }
     */
    ResponseJson<JSONObject> paySuperVipByAlipay(PayVipDto payVipDto, HttpHeaders headers);
    /**
     * 升级超级会员-银联线上支付
     * @param payVipDto {
     *               vipRecordId   会员购买记录Id
     *               returnUrl     页面回调地址
     *               bankCode      银行编码(银联支付使用)
     *               userType      用户类型(银联支付使用)企业:ENTERPRISE，个人:USER
     * }
     */
    ResponseJson<JSONObject> paySuperVipByUnionPay(PayVipDto payVipDto, HttpHeaders headers);
    /**
     * 升级超级会员-支付回调
     */
    String paymentSuperVipCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException;
    /**
     * 升级超级会员-线上退款
     * @param payVipDto {
     *               vipRecordId   会员购买记录Id
     *               code          退款口令
     * }
     */
    ResponseJson<JSONObject> superVipOnlineRefund(PayVipDto payVipDto) throws NoSuchAlgorithmException, InvalidKeySpecException;

    /**
     * 微信购买优惠券
     * @param payCouponDto
     * @param headers
     * @return
     */
    ResponseJson<JSONObject> payCouponByWeChat(PayCouponDto payCouponDto, HttpHeaders headers);

    String couponCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException;

    ResponseJson<JSONObject> payCouponByAlipay(PayCouponDto payCouponDto, HttpHeaders headers);

    ResponseJson<JSONObject> payCouponUnionPay(PayCouponDto payCouponDto, HttpHeaders headers);

    ResponseJson<PayCouponDto> getCouponRecord(PayCouponDto payCouponDto, HttpHeaders headers);

    ResponseJson couponCheck(Integer recordId);

    ResponseJson vipCheck(Integer recordId);

    /**
     * 开通认证通会员-微信线上支付
     * @param payVipDto {
     *               vipRecordId   会员购买记录Id
     *               returnUrl     页面回调地址
     *               code          微信小程序code
     *               state         微信公众号state参数
     * }
     */
    ResponseJson<JSONObject> payAuthVipByWeChat(PayVipDto payVipDto, HttpHeaders headers);
    /**
     * 开通认证通会员-支付宝线上支付
     * @param payVipDto {
     *               vipRecordId   会员购买记录Id
     *               returnUrl     页面回调地址
     * }
     */
    ResponseJson<JSONObject> payAuthVipByAlipay(PayVipDto payVipDto, HttpHeaders headers);
    /**
     * 开通认证通会员-银联线上支付
     * @param payVipDto {
     *               vipRecordId   会员购买记录Id
     *               returnUrl     页面回调地址
     *               bankCode      银行编码(银联支付使用)
     *               userType      用户类型(银联支付使用)企业:ENTERPRISE，个人:USER
     * }
     */
    ResponseJson<JSONObject> payAuthVipByUnionPay(PayVipDto payVipDto, HttpHeaders headers);
    /**
     * 开通认证通会员-支付回调
     */
    String paymentAuthVipCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException;
    /**
     * 开通认证通会员-线上退款
     * @param payVipDto {
     *               vipRecordId   会员购买记录Id
     *               code          退款口令
     * }
     */
    ResponseJson<JSONObject> authVipOnlineRefund(PayVipDto payVipDto) throws NoSuchAlgorithmException, InvalidKeySpecException;

    /**
     * 购买认证通会员-查询是否购买成功
     * @return
     */
    ResponseJson authVipCheck(Integer vipRecordId);

    ResponseJson<JSONObject> paySuperVipByTransfer(PayVipDto payVipDto, HttpHeaders headers);

    ResponseJson<JSONObject> payCouponByTransfer(PayCouponDto payCouponDto, HttpHeaders headers);
}
