package com.caimei365.order.service;

import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.PostMapping;

/**
 * 订单推送服务
 *
 * @author : Charles
 * @date : 2021/7/15
 */
public interface RemoteCallService {
    /**
     * 生成短链接
     *
     * @param length 链接长度
     * @param markId 跳转类型
     * @param url    跳转地址
     * @return
     */
    String getShortLink(int length, int markId, String url);

    /**
     * 请求发短信接口
     *
     * @param markId  短信类型
     * @param mobile  手机号
     * @param content 内容
     * @param type    短信类型：1通知短信，2验证码短信，3营销短信
     * @return
     */
    boolean getSendSms(Integer type,int markId, String mobile, String content);

    /**
     * 微信小程序授权登录（调用user服务，发送服务间调用POST请求）
     * @param code              微信code
     * @param encryptedData     微信加密数据
     * @param iv                微信加密算法的初始向量
     * @param headers           HttpHeaders
     */
    String appletsAuthorization(String code, String encryptedData, String iv, HttpHeaders headers);

    /**
     * 微信小程序授权登录（调用user服务，发送服务间调用POST请求） - 组织
     * @param code              微信code
     * @param encryptedData     微信加密数据
     * @param iv                微信加密算法的初始向量
     * @param headers           HttpHeaders
     * @param userId            userId
     * @return
     */
    String appletsOrganizeAuthorization(String code, String encryptedData, String iv, HttpHeaders headers, Integer userId);

    /**
     * 智能获取物流信息
     *
     * @param number      快递单号
     * @param companyCode 快递公司的编码
     * @param mobile      收/寄件人的电话号码(顺丰必填)
     */
    String getLogisticsByNumber(String number, String companyCode, String mobile);

    /**
     * 更新商品销量
     * @param productInfo [ // 商品id，数量
     *                   {"productId": 2789, "productCount": 1},
     *                   {"productId": 2790, "productCount": 2}
     *                 ]
     */
    String productSaleUpdate(String productInfo);

    void getOldUser();

    void getOldUserTow();
}
