package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.components.ProductService;
import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.mapper.CartClubMapper;
import com.caimei365.order.mapper.OrderClubMapper;
import com.caimei365.order.mapper.OrderCommonMapper;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.CartDto;
import com.caimei365.order.model.dto.InvoiceDto;
import com.caimei365.order.model.dto.InvoiceOrganizeDto;
import com.caimei365.order.model.po.CartPo;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.CartClubService;
import com.caimei365.order.service.RemoteCallService;
import com.caimei365.order.utils.ImageUtil;
import com.caimei365.order.utils.MathUtil;
import com.google.common.collect.Lists;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.aspectj.weaver.ast.Var;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import redis.clients.jedis.Jedis;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static com.alibaba.fastjson.JSON.parseArray;

import static com.alibaba.fastjson.JSON.parseArray;
import static com.alibaba.fastjson.JSON.parseObject;


/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/25
 */
@Slf4j
@Service
public class CartClubServiceImpl implements CartClubService {
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private CartClubMapper cartClubMapper;
    @Resource
    private OrderCommonMapper orderCommonMapper;
    @Resource
    private OrderClubMapper orderClubMapper;
    @Resource
    private ProductService productService;
    @Resource
    private RemoteCallService remoteCallService;
    @Value("${spring.cloud.config.profile}")
    private String profile;

    /**
     * 购物车列表详细数据
     *
     * @param userId 用户Id
     * @param source 来源 : 1 网站 ; 2 小程序
     */
    @Override
    public ResponseJson<Map<String, Object>> getShoppingCartList(Integer userId, Integer source) {
        // 用户组织Id
        Integer organizeId = baseMapper.getOrganizeId(userId);
        /**
         * 初始化返回数据
         */
        // 购物车供应商列表
        List<CartShopVo> shopInfoList = cartClubMapper.getCartShops(userId);
        // 失效商品列表
        List<CartItemVo> invalidList = new ArrayList<>();
        // 商品总数量
        AtomicInteger totalCount = new AtomicInteger(0);
        // 商品种类
        AtomicInteger kindCount = new AtomicInteger(0);
        // 统计商品总金额
        AtomicDouble totalPrice = new AtomicDouble(0);
        // 统计总促销满减
        AtomicDouble reducedPrice = new AtomicDouble(0);
        // 统计总超级会员优惠
        AtomicDouble svipReducedPrice = new AtomicDouble(0);
        // 统计总划线价
        AtomicDouble totalOriginalPrice = new AtomicDouble(0);
        /**
         * 处理数据
         */
        // 促销活动（总）
        List<PromotionsVo> totalPromotions = new ArrayList<>();
        List<Integer> promotionsIds = new ArrayList<>();
        // 用户注册时间
        Date registerTime = orderCommonMapper.getUserRegisterTime(userId);
        // 用户可用优惠券（总）
        List<CouponVo> couponList = orderCommonMapper.getClubCouponList(userId);
        // 用户可领取优惠券（总）
        List<CouponVo> preCouponList = orderCommonMapper.getPrevCouponList(userId, registerTime);
        List<CouponVo> totalCouponList = new ArrayList<>();
        totalCouponList.addAll(couponList);
        totalCouponList.addAll(preCouponList);
        List<CartItemVo> cartAllProducts = new ArrayList<>();
        // 用户身份：  0个人，1协销，2会员机构，3供应商，4普通机构
        Integer userIdentity = baseMapper.getIdentityByUserId(userId);
        userIdentity = null == userIdentity ? 0 : userIdentity;
        // 会员机构类型：1医美，2生美
        Integer userClubType = 0;
        if (userIdentity == 2) {
            userClubType = cartClubMapper.getClubTypeById(userId);
            userClubType = null == userClubType ? 0 : userClubType;
        }
        // 超级会员标识
        Integer svipUserId = baseMapper.getSvipUserIdByUserId(userId);
        boolean svipUserFlag = null != svipUserId;
        if (null != shopInfoList && shopInfoList.size() > 0) {
            // 删除空数据
            shopInfoList.removeIf(Objects::isNull);
            Integer finalUserIdentity = userIdentity;
            Integer finalUserClubType = userClubType;
            // 遍历供应商列表
            shopInfoList.forEach(shop -> {
                // 默认未选中状态（前端要求）
                shop.setIsChecked(false);
                // 该供应商下商品种类
                AtomicInteger shopKindCount = new AtomicInteger(0);
                // 该供应商总价
                AtomicDouble shopPrice = new AtomicDouble(0);
                // 该供应商下可参与店铺促销的商品总价
                AtomicDouble shopPromotionFee = new AtomicDouble(0);
                // 该供应商满减金额（供应商满减，单品满减）
                AtomicDouble shopReducedPrice = new AtomicDouble(0);
                // 该供应商svip优惠金额
                AtomicDouble shopSvipReducedPrice = new AtomicDouble(0);
                // 该供应商划线价
                AtomicDouble shopOriginalPrice = new AtomicDouble(0);
                // 供应商促销优惠活动，以活动分类排序，店铺>凑单>单品
                PromotionsVo shopPromotion = baseMapper.getPromotionByShopId(shop.getShopId());
                if (shopPromotion != null && shopPromotion.getSeen() != null && shopPromotion.getSeen() == 2 && finalUserIdentity == 1) {
                    shopPromotion = null;
                }
                // 供应商下商品列表 ，购物车里的该供应商商品
                List<CartItemVo> productList = null;
                if (0 == organizeId) {
                    productList = cartClubMapper.getCartProductsByShopId(shop.getShopId(), userId);
                } else {
                    productList = cartClubMapper.getCartOrganizeProductsByShopId(organizeId, shop.getShopId(), userId);
                }
                // 迭代器设置商品信息
                Iterator<CartItemVo> productIterator = productList.iterator();
                while (productIterator.hasNext()) {
                    CartItemVo cartItemVo = productIterator.next();
                    if (0 == organizeId) {
                        cartItemVo.setSkus(orderClubMapper.findSkus(cartItemVo.getProductId()));
                    } else {
                        cartItemVo.setSkus(orderClubMapper.getOrganizeSku(cartItemVo.getProductId()));
                    }
                    if (0 == organizeId) {
                        // 设置商品图片及税费
                        boolean taxFlag = productService.setCartItemImgAndTax(cartItemVo);
                        // 商品可见度： 3:所有人可见，2:普通机构可见，1:会员机构可见，4:仅医美机构可见
                        Integer visibility = cartItemVo.getVisibility();
                        boolean visibleFlag = visibility == 3 || visibility == 2 || (visibility == 1 && (finalUserIdentity == 2 || svipUserFlag)) || (visibility == 4 && finalUserClubType == 1);
                        if (!visibleFlag) {
                            //不可见商品 不加入失效商品，直接去除
                            productIterator.remove();
                        }
                        // 已上架商品
                        else if (cartItemVo.getValidFlag() == 2) {
                            // 设置商品有效
                            cartItemVo.setStatus(0);
                            // 默认所有商品未选中状态（前端要求）
                            cartItemVo.setIsChecked(false);
                            // 价格可见度： 0公开价格，1不公开价格，2仅对会员机构公开，3仅对医美机构公开
                            boolean priceVisible = (cartItemVo.getPriceFlag() == 0 || (cartItemVo.getPriceFlag() == 2 && (finalUserIdentity == 2 || svipUserFlag)) || (cartItemVo.getPriceFlag() == 3 && finalUserClubType == 1));
                            // 是否库存充足  只看所有sku的stock是否为0
    //                        boolean isStocked = (cartItemVo.getStock() != null && cartItemVo.getStock() > 0 && cartItemVo.getStock() >= cartItemVo.getMin() && cartItemVo.getStock() >= cartItemVo.getNumber());
                            boolean isStocked = orderCommonMapper.findMaxStock(cartItemVo.getProductId()) > 0;
                            if (priceVisible && isStocked) {
                                    if (1 == cartItemVo.getSvipProductFlag()) {
                                        // 超级会员优惠商品,不参与促销活动（普通机构购买该商品可参与店铺促销）/阶梯价/复购价
                                        // 超级会员设置商品优惠价
                                        productService.setSvipProductPrice(cartItemVo, taxFlag, svipUserFlag);
                                        if (svipUserFlag) {
                                            // 超级会员设置超级会员优惠总额 + 商品优惠金额
                                            shopSvipReducedPrice.set(MathUtil.add(shopSvipReducedPrice, cartItemVo.getSvipTotalReducedPrice()).doubleValue());
                                        }
                                    } else {
                                        // 获取商品促销信息
                                        PromotionsVo promotions = null;
                                        // 没有店铺促销时，商品促销才有效
                                        if (null == shopPromotion) {
                                            // 获取商品促销信息
                                            promotions = baseMapper.getPromotionByProductId(cartItemVo.getProductId());
                                            // 促销活动如果协销不可见直接置空
                                            if (promotions != null && promotions.getSeen() != null && promotions.getSeen() == 2 && finalUserIdentity == 1) {
                                                promotions = null;
                                            }
                                            /*
                                             * 设置商品促销优惠
                                             */
                                            if (null != promotions) {
                                                if (1 == promotions.getMode()) {
                                                    promotions.setTouchPrice(baseMapper.getTouchPriceBySku(cartItemVo.getSkuId(), promotions.getId()));
                                                }
                                                // 当前促销活动的价格计算列表
                                                List<PromotionPriceVo> promotionPriceList = productService.getPromotionProducts(promotions, cartItemVo, taxFlag);
                                                BigDecimal totalAmount = MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice());
                                                Double floor = Math.floor(MathUtil.div(totalAmount, promotions.getTouchPrice()).doubleValue());
                                                // 更新到总促销列表
                                                productService.updateTotalPromotions(totalPromotions, promotionsIds, promotions, promotionPriceList, floor);
                                                //单品满减-计算供应商总价/满减金额
                                                // 叠加优惠计算
                                                if (promotions.getType() == 1 && promotions.getMode() == 2) {
                                                    if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                                                        //叠加优惠计算
                                                        //叠加倍数
                                                        shopPrice.set(MathUtil.sub(shopPrice.get(), MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                                        shopReducedPrice.set(MathUtil.add(shopReducedPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                                    } else {
                                                        // 如果满足促销条件，设置供应商价格-满减金额，满减总额 + 当前促销满减金额
                                                        shopPrice.set(MathUtil.sub(shopPrice.get(), promotions.getReducedPrice()).doubleValue());
                                                        shopReducedPrice.set(MathUtil.add(shopReducedPrice, promotions.getReducedPrice()).doubleValue());
                                                    }
                                                }
                                                cartItemVo.setPromotions(promotions);
                                            }
                                        }
                                        if (null != promotions || null != shopPromotion) {
                                            // 商品处于活动状态
                                            cartItemVo.setActStatus(1);
                                            // 关闭阶梯价格，活动优先
                                            cartItemVo.setLadderFlag(0);
                                        } else {
                                            if (1 == cartItemVo.getLadderFlag()) {
                                                // 设置阶梯价
                                                productService.setCartLadderPrices(cartItemVo, taxFlag);
                                            }
                                            /*else {
                                                // 复购价
                                                Double repurchase = baseMapper.getRepurchasePrice(cartItemVo.getSkuId(), userId);
                                                if (null != repurchase && repurchase > 0) {
                                                    if (taxFlag) {
                                                        BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(repurchase, cartItemVo.getTaxRate()), 100, 2);
                                                        cartItemVo.setPrice(MathUtil.add(repurchase, cartItemTax).doubleValue());
                                                    } else {
                                                        cartItemVo.setPrice(repurchase);
                                                    }
                                                }
                                            }*/
                                        }
                                    }
                                // 该供应商下价格累加
                                shopPrice.set(MathUtil.add(shopPrice, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                                shopOriginalPrice.set(MathUtil.add(shopOriginalPrice.get(), MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getOriginalPrice())).doubleValue());
                                // 该供应商下可参与店铺促销的商品价格累加
                                if (0 == organizeId) {
                                    if (!(1 == cartItemVo.getSvipProductFlag() && svipUserFlag)) {
                                        // 超级会员购买svip商品不享受店铺促销优惠
                                        shopPromotionFee.set(MathUtil.add(shopPromotionFee, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                                    }
                                }
                                // 该供应商下商品种类 +1
                                shopKindCount.incrementAndGet();
                                // 购物车总数量 + 当前商品购买数量
                                totalCount.updateAndGet(v -> v + cartItemVo.getNumber());
                            } else {
                                //失效无库存不统计
                                totalCount.updateAndGet(v -> v - 1);
                                kindCount.updateAndGet(v -> v - 1);
                                // 失效商品
                                if (cartItemVo.getPriceFlag() == 1) {
                                    // 未公开价格
                                    cartItemVo.setStatus(6);
                                } else if (cartItemVo.getPriceFlag() == 2 && finalUserIdentity == 4) {
                                    // 价格仅会员可见
                                    cartItemVo.setStatus(5);
                                } else if (cartItemVo.getPriceFlag() == 3 && finalUserClubType != 1) {
                                    // 价格仅医美机构可见
                                    cartItemVo.setStatus(8);
                                } else if (cartItemVo.getStock() == null || cartItemVo.getStock() == 0) {
                                    // 售罄
                                    cartItemVo.setStatus(4);
                                } else if (cartItemVo.getStock() != null && (cartItemVo.getStock() < cartItemVo.getMin() || cartItemVo.getStock() < cartItemVo.getNumber())) {
                                    // 库存不足
                                    cartItemVo.setStatus(7);
                                }
                                invalidList.add(cartItemVo);
                                productIterator.remove();
                            }
                        } else {
                            // 失效商品
                            if (cartItemVo.getValidFlag() == 0) {
                                // 后台逻辑删除,已停售
                                cartItemVo.setStatus(1);
                                invalidList.add(cartItemVo);
                            } else if (cartItemVo.getValidFlag() == 10) {
                                // 已冻结，已丢失
                                cartItemVo.setStatus(2);
                                invalidList.add(cartItemVo);
                            } else if (cartItemVo.getValidFlag() == 3) {
                                // 已下架
                                cartItemVo.setStatus(3);
                                invalidList.add(cartItemVo);
                            }
                            //隐身商品validFlag = 9 不加入失效商品，直接去除
                            productIterator.remove();
                        }
                    } else {
                        // 设置商品图片及税费
                        productService.setCartItemImgAndTax(cartItemVo);
                        if (cartItemVo.getValidFlag() == 2) {
                            // 设置商品有效
                            cartItemVo.setStatus(0);
                            // 该供应商下价格累加
                            shopPrice.set(MathUtil.add(shopPrice, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                            shopOriginalPrice.set(MathUtil.add(shopOriginalPrice.get(), MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getOriginalPrice())).doubleValue());

                            // 该供应商下商品种类 +1
                            shopKindCount.incrementAndGet();
                            // 购物车总数量 + 当前商品购买数量
                            totalCount.updateAndGet(v -> v + cartItemVo.getNumber());
                        } else {
                            // 设置商品失效
                            cartItemVo.setStatus(3);
                            invalidList.add(cartItemVo);
                        }
                    }

                }
                cartAllProducts.addAll(productList);

                // 供应商下商品种类
                shop.setCount(shopKindCount.get());
                if (shopKindCount.get() > 0) {
                    // 店铺促销
                    if (null != shopPromotion) {
                        shop.setPromotions(shopPromotion);
                        if (!promotionsIds.contains(shopPromotion.getId())) {
                            promotionsIds.add(shopPromotion.getId());
                            // 店铺满赠
                            Double floor = Math.floor(MathUtil.div(shopPromotionFee, shopPromotion.getTouchPrice()).doubleValue());
                            if (shopPromotion.getMode() == 3) {
                                // 获取赠品
                                List<CartItemVo> giftList = baseMapper.getPromotionGifts(shopPromotion.getId());
                                // 叠加赠品
                                if (shopPromotion.getDiscount() != null && shopPromotion.getDiscount() == 1 && floor > 1) {
                                    Integer in = floor.intValue();
                                    giftList.forEach(g -> g.setNumber(MathUtil.mul(g.getNumber(), in).intValue()));
                                }
                                shopPromotion.setGiftList(giftList);
                            }
                            // 设置该优惠下的商品列表
                            List<PromotionPriceVo> promotionPriceList = new ArrayList<>();
                            productList.forEach(item -> {
                                if (!(1 == item.getSvipProductFlag() && svipUserFlag)) {
                                    // 超级会员购买svip商品不享受店铺促销优惠
                                    PromotionPriceVo promotionPrice = new PromotionPriceVo();
                                    promotionPrice.setProductId(item.getProductId());
                                    promotionPrice.setNumber(item.getNumber());
                                    promotionPrice.setPrice(item.getPrice());
                                    promotionPriceList.add(promotionPrice);
                                }
                            });
                            shopPromotion.setProductList(promotionPriceList);
                            // 添加到总促销
                            totalPromotions.add(shopPromotion);
                            // 店铺满减-计算供应商总价/满减金额
                            if (shopPromotion.getMode() == 2 && MathUtil.compare(shopPromotionFee, shopPromotion.getTouchPrice()) > -1) {
                                if (shopPromotion.getDiscount() != null && shopPromotion.getDiscount() == 1 && floor > 1) {
                                    // 该供应商总价 - 满减金额
                                    shopPrice.set(MathUtil.sub(shopPrice.get(), MathUtil.mul(shopPromotion.getReducedPrice(), floor)).doubleValue());
                                    // 该供应商优惠总额 + 满减金额
                                    shopReducedPrice.set(MathUtil.add(shopReducedPrice.get(), MathUtil.mul(shopPromotion.getReducedPrice(), floor)).doubleValue());
                                } else {
                                    // 该供应商总价 - 满减金额
                                    shopPrice.set(MathUtil.sub(shopPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                                    // 该供应商优惠总额 + 满减金额
                                    shopReducedPrice.set(MathUtil.add(shopReducedPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                                }
                            }
                        }
                    }
                    // 是否可领取优惠券
                    Boolean couponsLogo = setCouponsLogo(shop.getShopId(), source, productList, totalCouponList);
                    shop.setCouponsLogo(couponsLogo);

                    // 供应商商品
                    shop.setCartList(productList);
                    // 供应商总价
                    shop.setTotalPrice(shopPrice.get());
                    // 供应商总优惠
                    shop.setReducedPrice(shopReducedPrice.get());
                    // 供应商总超级会员优惠
                    shop.setSvipReducedPrice(shopSvipReducedPrice.get());
                    // 供应商划线价
                    shop.setOriginalPrice(shopOriginalPrice.get());
                    // 计算总价
                    totalPrice.set(MathUtil.add(totalPrice, shop.getTotalPrice()).doubleValue());
                    // 优惠总额
                    reducedPrice.set(MathUtil.add(reducedPrice, shop.getReducedPrice()).doubleValue());
                    // 超级会员优惠总额
                    svipReducedPrice.set(MathUtil.add(svipReducedPrice, shop.getSvipReducedPrice()).doubleValue());
                    // 总划线价
                    totalOriginalPrice.set(MathUtil.add(totalOriginalPrice, shop.getOriginalPrice()).doubleValue());
                    // 商品种类
                    kindCount.updateAndGet(v -> v + shopKindCount.get());
                }
            });
            // 删除空数据
            shopInfoList.removeIf(shop -> (shop.getCount() == 0));
            // 总促销计算
            totalPromotions.forEach(promotions -> {
                // 凑单满减
                if (promotions.getType() == 2 && promotions.getMode() == 2) {
                    // 该促销内商品总价
                    double touchPrice = promotions.getProductList().stream().mapToDouble(product -> product.getNumber() * product.getPrice()).sum();
                    //叠加倍数
                    Double floor = Math.floor(MathUtil.div(touchPrice, promotions.getTouchPrice()).doubleValue());
                    if (MathUtil.compare(touchPrice, promotions.getTouchPrice()) > -1) {
                        if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                            //叠加优惠计算
                            // 总价 - 满减金额
                            totalPrice.set(MathUtil.sub(totalPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                            // 优惠总额 + 满减金额
                            reducedPrice.set(MathUtil.add(reducedPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                        } else {
                            // 总价 - 满减金额
                            totalPrice.set(MathUtil.sub(totalPrice, promotions.getReducedPrice()).doubleValue());
                            // 优惠总额 + 满减金额
                            reducedPrice.set(MathUtil.add(reducedPrice, promotions.getReducedPrice()).doubleValue());
                        }
                    }
                }
            });
        }
        // 过滤与当前购物车商品无关的优惠券
        filterCoupon(source, cartAllProducts, couponList);
        //超级会员优惠券剔除
        List<Integer> vipCoupon = cartClubMapper.findVipCoupon();
        if (couponList != null && couponList.size() > 0) {
            couponList.removeIf(couponVo -> vipCoupon.contains(couponVo.getCouponId()));
        }
        /*
         * 返回结果
         */
        Map<String, Object> resultMap = new HashMap<>(8);
        resultMap.put("list", shopInfoList);
        resultMap.put("invalid", invalidList);
        resultMap.put("kindCount", kindCount);
        resultMap.put("totalCount", totalCount);
        resultMap.put("totalPrice", totalPrice);
        resultMap.put("reducedPrice", reducedPrice);
        resultMap.put("svipReducedPrice", svipReducedPrice);
        resultMap.put("totalOriginalPrice", totalOriginalPrice);
        resultMap.put("promotions", totalPromotions);
        resultMap.put("couponList", couponList);
        // 返回数据
        return ResponseJson.success(resultMap);
    }

    /**
     * 过滤与当前商品列表无关的优惠券
     */
    private void filterCoupon(Integer source, List<CartItemVo> cartItems, List<CouponVo> couponList) {
        if (couponList != null && couponList.size() > 0) {
            Iterator<CouponVo> iterator = couponList.iterator();
            A:
            while (iterator.hasNext()) {
                CouponVo coupon = iterator.next();
                if (coupon.getCouponType() == 0 && coupon.getProductType() == 2) {
                    // 活动券商品Ids
                    List<Integer> productIds = orderCommonMapper.getCouponProductIds(coupon.getCouponId(), source);
                    coupon.setProductIds(productIds);
                    for (CartItemVo item : cartItems) {
                        if (!productIds.contains(item.getProductId())) {
                            iterator.remove();
                            continue A;
                        }
                    }
                }
                if (coupon.getCouponType() == 1) {
                    for (CartItemVo item : cartItems) {
                        if (!coupon.getCategoryType().equals(item.getCommodityType())) {
                            iterator.remove();
                            continue A;
                        }
                    }
                }
                if (coupon.getCouponType() == 3) {
                    for (CartItemVo item : cartItems) {
                        if (!coupon.getShopId().equals(item.getShopId())) {
                            iterator.remove();
                            continue A;
                        }
                    }
                }
                if (coupon.getCouponType() == 3 && coupon.getShopId() != null) {
                    //店铺券
                    String shopName = baseMapper.getShopNameById(coupon.getShopId());
                    if (StringUtils.isNotBlank(shopName)) {
                        if (shopName.length() > 7) {
                            shopName = shopName.substring(0, 6) + "...";
                        }
                        coupon.setShopName(shopName);
                    }
                }
            }
        }
    }

    /**
     * 优惠券标识是否显示
     */
    public Boolean setCouponsLogo(Integer shopId, Integer source, List<CartItemVo> cartItems, List<CouponVo> couponList) {
        boolean couponsLogo = false;
        if (couponList != null && couponList.size() > 0) {
            A:
            for (CouponVo coupon : couponList) {
                if (coupon.getCouponType() == 4 || coupon.getCouponType() == 2) {
                    //用户券与用户专享券
                    couponsLogo = true;
                    break;
                }
                if (coupon.getCouponType() == 0) {
                    if (coupon.getProductType() == 1) {
                        //活动券-全商城商品
                        couponsLogo = true;
                        break;
                    } else {
                        // 活动券商品Ids
                        List<Integer> productIds = orderCommonMapper.getCouponProductIds(coupon.getCouponId(), source);
                        for (CartItemVo item : cartItems) {
                            if (productIds.contains(item.getProductId())) {
                                couponsLogo = true;
                                break A;
                            }
                        }
                    }
                }
                if (coupon.getCouponType() == 3 && shopId.equals(coupon.getShopId())) {
                    //店铺券
                    couponsLogo = true;
                    break;
                }
                if (coupon.getCouponType() == 1) {
                    //品类券
                    for (CartItemVo item : cartItems) {
                        if (coupon.getCategoryType().equals(item.getCommodityType())) {
                            couponsLogo = true;
                            break A;
                        }
                    }
                }
            }
        }
        return couponsLogo;
    }

    /**
     * 网站顶部购物车数据
     *
     * @param userId 用户Id
     */
    @Override
    public ResponseJson<Map<String, Object>> getShoppingCartHead(Integer userId) {
        /*
         * 初始化返回数据
         */
        // 商品总数量
        AtomicInteger totalCount = new AtomicInteger(0);
        // 商品种类
        AtomicInteger kindCount = new AtomicInteger(0);
        // 统计商品总金额
        AtomicDouble totalPrice = new AtomicDouble(0);
        // 购物车商品列表
        List<CartItemVo> cartList = null;
        /*
         * 处理数据
         */
        // 促销活动（总）
        List<PromotionsVo> totalPromotions = new ArrayList<>();
        List<Integer> promotionsIds = new ArrayList<>();
        // 用户身份
        Integer userIdentity = baseMapper.getIdentityByUserId(userId);
        // 会员机构类型：1医美，2生美
        Integer userClubType = 0;
        if (null != userIdentity && userIdentity == 2) {
            userClubType = cartClubMapper.getClubTypeById(userId);
            userClubType = null == userClubType ? 0 : userClubType;
        }
        // 超级会员标识
        Integer svipUserId = baseMapper.getSvipUserIdByUserId(userId);
        boolean svipUserFlag = null != svipUserId;
        // 获取购物车商品列表（不区分供应商）
        cartList = cartClubMapper.getCartProductList(userId);
        if (null != cartList && cartList.size() > 0) {
            // 移除不可见商品,价格不可见商品,无库存
            Integer finalUserClubType = userClubType;
            Integer finalUserIdentity = userIdentity;
            cartList.removeIf(cartItemVo -> !(
                    // visibility商品可见度： 3:所有人可见，2:普通机构可见，1:会员机构可见，4:仅医美机构可见
                    (cartItemVo.getVisibility() == 3 || cartItemVo.getVisibility() == 2 || (cartItemVo.getVisibility() == 1 && (finalUserIdentity == 2 || svipUserFlag)) || (cartItemVo.getVisibility() == 4 && finalUserClubType == 1))
                            // 价格可见度： 0公开价格，1不公开价格，2仅对会员机构公开，3仅对医美机构公开
                            || (cartItemVo.getPriceFlag() == 0 || (cartItemVo.getPriceFlag() == 2 && (finalUserIdentity == 2 || svipUserFlag)) || (cartItemVo.getPriceFlag() == 3 && finalUserClubType == 1))
                            // 是否库存充足
                            || (cartItemVo.getStock() != null && cartItemVo.getStock() > 0 && cartItemVo.getStock() >= cartItemVo.getMin() && cartItemVo.getStock() >= cartItemVo.getNumber())
            ));
            cartList.forEach(cartItemVo -> {
                // 设置商品图片及税费
                boolean taxFlag = productService.setCartItemImgAndTax(cartItemVo);
                // 获取商品促销信息
                PromotionsVo promotions = baseMapper.getPromotionByProductId(cartItemVo.getProductId());
                // 如果促销活动协销不可见，移除促销
                if (promotions != null && promotions.getSeen() != null && promotions.getSeen() == 2 && finalUserIdentity == 1) {
                    promotions = null;
                }
                /*
                 * 设置商品促销优惠
                 */
                if (null != promotions) {
                    if (1 == promotions.getMode()) {
                        promotions.setTouchPrice(baseMapper.getTouchPriceBySku(cartItemVo.getSkuId(), promotions.getId()));
                    }
                    BigDecimal totalAmount = MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice());
                    Double floor = Math.floor(MathUtil.div(totalAmount, promotions.getTouchPrice()).doubleValue());
                    // 关闭阶梯价格，活动优先
                    cartItemVo.setLadderFlag(0);
                    // 商品处于活动状态
                    cartItemVo.setActStatus(1);
                    // 当前促销活动的价格计算列表
                    List<PromotionPriceVo> promotionPriceList = productService.getPromotionProducts(promotions, cartItemVo, taxFlag);
                    // 更新到总促销列表
                    productService.updateTotalPromotions(totalPromotions, promotionsIds, promotions, promotionPriceList, floor);
                } else {
                    if (cartItemVo.getLadderFlag() == 1) {
                        // 设置阶梯价
                        productService.setCartLadderPrices(cartItemVo, taxFlag);
                        // 头部只展示简略信息，不显示详细阶梯价格
                        cartItemVo.setLadderPrices(null);
                    } else {
                        // 复购价
                        Double repurchase = baseMapper.getRepurchasePrice(cartItemVo.getSkuId(), userId);
                        if (null != repurchase && repurchase > 0) {
                            if (taxFlag) {
                                BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(repurchase, cartItemVo.getTaxRate()), 100, 2);
                                cartItemVo.setPrice(MathUtil.add(repurchase, cartItemTax).doubleValue());
                            } else {
                                cartItemVo.setPrice(repurchase);
                            }
                        }
                    }
                }
                // 商品总金额累加
                totalPrice.set(MathUtil.add(totalPrice, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                // 商品种类 +1
                kindCount.incrementAndGet();
                // 购物车总数量 + 当前商品购买数量
                totalCount.updateAndGet(v -> v + cartItemVo.getNumber());
            });
            // 总促销-满减计算
            totalPromotions.forEach(promotions -> {
                // 满减
                // 满减叠加
                if (promotions.getMode() == 2) {
                    // 该促销内商品总价
                    double touchPrice = promotions.getProductList().stream().mapToDouble(product -> product.getNumber() * product.getPrice()).sum();
                    //叠加倍数
                    Double floor = Math.floor(MathUtil.div(touchPrice, promotions.getTouchPrice()).doubleValue());
                    if (MathUtil.compare(touchPrice, promotions.getTouchPrice()) > -1) {
                        if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                            //叠加优惠计算
                            // 总价 - 满减金额
                            totalPrice.set(MathUtil.sub(totalPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                        } else {
                            // 总价 - 满减金额
                            totalPrice.set(MathUtil.sub(totalPrice, promotions.getReducedPrice()).doubleValue());
                        }
                    }
                }
            });
        }
        /*
         * 返回结果
         */
        Map<String, Object> resultMap = new HashMap<>(4);
        resultMap.put("list", cartList);
        resultMap.put("kindCount", kindCount);
        resultMap.put("totalCount", totalCount);
        resultMap.put("totalPrice", totalPrice);
        // 返回数据
        return ResponseJson.success(resultMap);
    }

    /**
     * 添加购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                productCount 商品数量
     *                }
     */
    @Override
    public ResponseJson<Integer> addShoppingCart(CartDto cartDto) {
        saveShoppingCart(cartDto);
        // 获取购物车数量（商品种类数）
        int cartCount = getCartCount(cartDto.getUserId());
        return ResponseJson.success("添加成功！返回购物车数量", cartCount);
    }

    @Transactional(rollbackFor = Exception.class)
    public void saveShoppingCart(CartDto cartDto) {
        CartPo cart = cartClubMapper.getCartPo(cartDto);
        if (cart != null) {
            // 购物车已存在该商品，更新数量
            cart.setProductCount(cart.getProductCount() + cartDto.getProductCount());
            cart.setAddTime(new Date());
            cartClubMapper.updateCart(cart);
        } else {
            // 添加新购物车
            cart = new CartPo();
            cart.setUserId(cartDto.getUserId());
            cart.setProductCount(cartDto.getProductCount());
            cart.setSkuId(cartDto.getSkuId());
            //判断是否是再次购买  由商品id改为skuid
            Double repurchase = baseMapper.getRepurchasePrice(cartDto.getSkuId(), cartDto.getUserId());
            if (null != repurchase && repurchase > 0) {
                // 再次购买
                cart.setReBuyFlag(1);
            } else {
                cart.setReBuyFlag(0);
            }
            cart.setAddTime(new Date());
            cartClubMapper.insertCart(cart);
        }
    }

    /**
     * 批量添加购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productInfo  商品及数量信息:[   // 商品id,skuId,数量
     *                {"productId": 2789,  "skuId":123, "productCount": 1},
     *                {"productId": 2790,  "skuId":123, "productCount": 1}
     *                ]
     *                }
     */
    @Override
    public ResponseJson<Integer> addShoppingCartBulk(CartDto cartDto) {
        JSONArray productInfo = null;
        try {
            productInfo = parseArray(cartDto.getProductInfo());
        } catch (Exception e) {
            log.error("批量添加购物车参数解析异常try-catch:", e);
            return ResponseJson.error("商品及数量信息参数解析异常！", null);
        }
        if (null == productInfo || productInfo.isEmpty()) {
            return ResponseJson.error("商品及数量信息参数异常！", null);
        }
        for (Object infoObject : productInfo) {
            JSONObject tempObj = (JSONObject) infoObject;
            Integer productId = (Integer) tempObj.get("productId");
            Integer productCount = (Integer) tempObj.get("productCount");
            Integer skuId = (Integer) tempObj.get("skuId");
            CartDto cartBo = new CartDto();
            cartBo.setUserId(cartDto.getUserId());
            cartBo.setProductId(productId);
            cartBo.setProductCount(productCount);
            cartBo.setSkuId(skuId);
            saveShoppingCart(cartBo);
        }
        // 获取购物车数量（商品种类数）
        int cartCount = getCartCount(cartDto.getUserId());
        return ResponseJson.success("添加成功！返回购物车数量", cartCount);
    }

    /**
     * 更新购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                skuId        skuId
     *                productCount 商品数量
     *                }
     */
    @Override
    public ResponseJson<Integer> updateShoppingCart(CartDto cartDto) {
        CartPo cart = new CartPo();
        cart.setUserId(cartDto.getUserId());
        cart.setSkuId(cartDto.getSkuId());
        cart.setProductCount(cartDto.getProductCount());
        cart.setAddTime(new Date());
        cartClubMapper.updateCart(cart);
        // 获取购物车数量（商品种类数）
        int cartCount = getCartCount(cartDto.getUserId());
        return ResponseJson.success("更新成功！返回购物车数量", cartCount);
    }

    /**
     * 删除购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productIds   商品ids,逗号隔开
     *                }
     */
    @Override
    public ResponseJson<Integer> deleteShoppingCart(CartDto cartDto) {
        List<String> skuIdList = Lists.newArrayList();
        if (cartDto.getSkuIds().contains(",")) {
            String[] productArr = cartDto.getSkuIds().split(",");
            Collections.addAll(skuIdList, productArr);
        } else {
            skuIdList.add(cartDto.getSkuIds());
        }
        cartClubMapper.deleteCartByProductIds(cartDto.getUserId(), skuIdList);
        // 获取购物车数量（商品种类数）
        int cartCount = getCartCount(cartDto.getUserId());
        return ResponseJson.success("删除成功！返回购物车数量", cartCount);
    }

    /**
     * 获取机构购物车数量（商品种类数）
     *
     * @param userId 用户ID
     */
    @Override
    public ResponseJson<Integer> getUserCartCount(Integer userId) {
        // 获取购物车数量（商品种类数）
        int cartCount = getCartCount(userId);
        return ResponseJson.success(cartCount);
    }

    /**
     * 获取机构购物车数量（商品种类数） -- 组织
     *
     * @param userId
     * @return
     */
    @Override
    public ResponseJson<Integer> getShoppingCartOrganizeCount(Integer userId) {
        Integer count = 0;
        if (null == userId) {
            return ResponseJson.success(count);
        }
        // 获取购物车商品列表（不区分供应商）
        List<CartItemVo> cartList = cartClubMapper.getCartOrganizeProductList(userId);
        if (null != cartList && cartList.size() > 0) {

            cartList.removeIf(cartItemVo ->
                    cartItemVo.getValidFlag() == 3
            );
            return ResponseJson.success(cartList.size());
        } else {
            return ResponseJson.success(count);
        }
    }

    /**
     * 获取机构购物车数量（商品种类数）
     *
     * @param userId 用户ID
     * @return int
     */
    private int getCartCount(Integer userId) {
        if (null == userId) {
            return 0;
        }
        // 获取购物车商品列表（不区分供应商）
        List<CartItemVo> cartList = cartClubMapper.getCartProductList(userId);
        if (null != cartList && cartList.size() > 0) {
            // 用户身份
            Integer userIdentity = baseMapper.getIdentityByUserId(userId);
            userIdentity = null == userIdentity ? 0 : userIdentity;
            // 超级会员标识
            Integer svipUserId = baseMapper.getSvipUserIdByUserId(userId);
            boolean svipUserFlag = null != svipUserId;
            // 会员机构类型：1医美，2生美
            Integer userClubType = 0;
            if (null != userIdentity && userIdentity == 2) {
                userClubType = cartClubMapper.getClubTypeById(userId);
                userClubType = null == userClubType ? 0 : userClubType;
            }
            int finalUserIdentity = userIdentity;
            int finalUserClubType = userClubType;
            cartList.removeIf(cartItemVo -> !(
                    // visibility商品可见度： 3:所有人可见，2:普通机构可见，1:会员机构可见，4:仅医美机构可见
                    (cartItemVo.getVisibility() == 3 || cartItemVo.getVisibility() == 2 || (cartItemVo.getVisibility() == 1 && (finalUserIdentity == 2 || svipUserFlag)) || (cartItemVo.getVisibility() == 4 && finalUserClubType == 1))
                            // 价格可见度： 0公开价格，1不公开价格，2仅对会员机构公开，3仅对医美机构公开
                            || (cartItemVo.getPriceFlag() == 0 || (cartItemVo.getPriceFlag() == 2 && (finalUserIdentity == 2 || svipUserFlag)) || (cartItemVo.getPriceFlag() == 3 && finalUserClubType == 1))
                            // 是否库存充足
                            || (cartItemVo.getStock() != null && cartItemVo.getStock() > 0 && cartItemVo.getStock() >= cartItemVo.getMin() && cartItemVo.getStock() >= cartItemVo.getNumber())
            ));
            return cartList.size();
        } else {
            return 0;
        }
    }


    /**
     * 购物车结算
     *
     * @param userId 用户ID
     * @param skuIds skuIds,逗号隔开
     * @param source 来源 : 1 网站 ; 2 小程序
     */
    @Override
    public ResponseJson<Map<String, Object>> settlementShoppingCart(Integer userId, Integer cityId, String skuIds, Integer source) {
        // 用户组织Id
        Integer organizeId = baseMapper.getOrganizeId(userId);
        log.info("￥￥￥￥￥￥￥￥￥￥ > 商品结算");
        // 商品种类
        AtomicInteger kindCount = new AtomicInteger(0);
        // 商品总数量
        AtomicInteger totalCount = new AtomicInteger(0);
        // 统计商品总金额
        AtomicDouble totalPrice = new AtomicDouble(0);
        // 统计总促销满减
        AtomicDouble reducedPrice = new AtomicDouble(0);
        // 统计总超级会员优惠
        AtomicDouble svipReducedPrice = new AtomicDouble(0);
        // 统计总划线价
        AtomicDouble totalOriginalPrice = new AtomicDouble(0);
        // 促销活动（总）
        List<PromotionsVo> totalPromotions = new ArrayList<>();
        // 是否包含充值商品，默认false
        AtomicBoolean includeRecharge = new AtomicBoolean(false);
        //所有的商品数据
        List<CartItemVo> totalProductList = new ArrayList<>();
        // 促销活动Id集合，用于合并促销活动
        List<Integer> promotionsIds = new ArrayList<>();
        // 供应商Id集合，用户判断赠品供应商是否在当前供应商中
        List<Integer> shopIds = new ArrayList<>();
        // 运费模板是否异常
        AtomicInteger templateFlag = new AtomicInteger(0);
        // 用户身份
        Integer userIdentity = baseMapper.getIdentityByUserId(userId);
        if (null == userIdentity) {
            userIdentity = 0;
        }
        // 会员机构类型：1医美，2生美
        Integer userClubType = 0;
        if (userIdentity == 2) {
            userClubType = cartClubMapper.getClubTypeById(userId);
            if (null == userClubType) {
                userClubType = 0;
            }
        }
        // 超级会员标识
        Integer svipUserId = baseMapper.getSvipUserIdByUserId(userId);
        boolean svipUserFlag = null != svipUserId;

        // 前端接收商品Id信息
        List<String> skuIdList = new ArrayList<>();
        if (skuIds.contains(",")) {
            skuIdList = Stream.of(skuIds.split(",")).collect(Collectors.toList());
        } else {
            skuIdList.add(skuIds);
        }

        // 购物车商品的分帐号分组的供应商列表
        // 1带分账号，2不带
        List<CartShopVo> shopList = cartClubMapper.getCartShopsByProductIds(userId, skuIdList, 1);
        int firstSize = shopList.size();
        shopList.removeIf(s -> StringUtils.isBlank(s.getSplitCode()));
        if (firstSize != shopList.size()) {
            shopList = cartClubMapper.getCartShopsByProductIds(userId, skuIdList, 2);
        }
        List<CartShopVo> goodsStutas = cartClubMapper.getProductReturnGoodsStutas(skuIdList);
        String helpContent = null;
        boolean flag = false;
        if (null != shopList && shopList.size() > 0) {
            // 遍历供应商列表
            List<String> finalIdList = skuIdList;//购物车商品skuId列表
            Integer finalUserIdentity = userIdentity;
            Integer finalUserClubType = userClubType;
            Integer finalUserIdentity1 = userIdentity;
            for (int i = 0; i < goodsStutas.size(); i++) {
                //订单中存在不能退货的商品则给出不能退货提示
                if (null != goodsStutas.get(i).getReturnGoodsStutas() && goodsStutas.get(i).getReturnGoodsStutas() == 2) {
                    flag = true;
                    helpContent = cartClubMapper.getHelpContent(1040);
                    break;
                }
            }
            shopList.forEach(shop -> {
                if (null != shop) {
                    // 该供应商下商品种类
                    AtomicInteger shopKindCount = new AtomicInteger(0);
                    // 该供应商总价
                    AtomicDouble shopPrice = new AtomicDouble(0);
                    // 该供应商下参与店铺促销的商品总价
                    AtomicDouble shopPromotionFee = new AtomicDouble(0);
                    // 该供应商满减金额（供应商满减，单品满减）
                    AtomicDouble shopReducedPrice = new AtomicDouble(0);
                    // 该供应商svip优惠金额
                    AtomicDouble shopSvipReducedPrice = new AtomicDouble(0);
                    // 该供应商划线价
                    AtomicDouble shopOriginalPrice = new AtomicDouble(0);
                    // 运费方式 0 包邮 1 到付 2 运费模板
                    List<Integer> shopPostageFlag = new ArrayList<>();
                    // 运费
                    AtomicDouble shopPostage = new AtomicDouble(0);
                    // 供应商促销优惠活动
                    PromotionsVo shopPromotion = baseMapper.getPromotionByShopId(shop.getShopId());
                    if (shopPromotion != null && shopPromotion.getSeen() != null && shopPromotion.getSeen() == 2 && finalUserIdentity1 == 1) {
                        shopPromotion = null;
                    }
                    // 供应商下商品列表
                    // 过滤保存已上架商品
                    List<CartItemVo> productList = null;
                    if (0 == organizeId) {
                        productList = cartClubMapper.getCartProductsByShopIdAndProductIds(userId, shop.getShopId(), finalIdList, shop.getSplitCode());
                    } else {
                        productList = cartClubMapper.getCartProductsOrganize(userId, shop.getShopId(), organizeId, finalIdList, shop.getSplitCode());
                    }
                    if (null != productList && productList.size() > 0) {
                        if (0 == organizeId) {
                            // 去除不可见商品，visibility商品可见度： 3:所有人可见，2:普通机构可见，1:会员机构可见，4:仅医美机构可见
                            productList.removeIf(cartItemVo -> !(cartItemVo.getVisibility() == 3 || cartItemVo.getVisibility() == 2 || (cartItemVo.getVisibility() == 1 && (finalUserIdentity == 2 || svipUserFlag)) || (cartItemVo.getVisibility() == 4 && finalUserClubType == 1)));
                            // 去除价格不可见商品，价格可见度： 0公开价格，1不公开价格，2仅对会员机构公开，3仅对医美机构公开
                            productList.removeIf(cartItemVo -> !(cartItemVo.getPriceFlag() == 0 || (cartItemVo.getPriceFlag() == 2 && (finalUserIdentity == 2 || svipUserFlag)) || (cartItemVo.getPriceFlag() == 3 && finalUserClubType == 1)));
                            // 去除库存不足商品
                            productList.removeIf(cartItemVo -> !(cartItemVo.getStock() != null && cartItemVo.getStock() > 0 && cartItemVo.getStock() >= cartItemVo.getMin() && cartItemVo.getStock() >= cartItemVo.getNumber()));
                        }
                        // 是否包含需要冷链运输商品
                        List<Integer> collect = productList.stream().map(CartItemVo::getProductId).collect(Collectors.toList());
                        if (collect.stream().allMatch(c -> c == 7881)) {
                            shop.setDesignatedFlag(1);
                        } else if (!collect.contains(7881)) {
                            shop.setDesignatedFlag(2);
                        } else {
                            shop.setDesignatedFlag(3);
                        }
                        // 冷链费
                        Double coldChain = 0d;
                        // 迭代器设置商品信息
                        for (CartItemVo cartItemVo : productList) {
                            // 设置商品运费
                            if (7881 == cartItemVo.getProductId()) {
                                if (cartItemVo.getNumber() >= 100) {
                                    coldChain = 0d;
                                } else {
                                    coldChain = baseMapper.getFreightAmount(cartItemVo.getProductId());
                                    // shop.setPostageFlag(1);
                                }
                            }
                            // 设置商品图片及税费
                            boolean taxFlag = productService.setCartItemImgAndTax(cartItemVo);
                            // 是否充值商品
                            boolean recharge = productService.isRechargeProduct(cartItemVo.getProductId());
                            if (recharge) {
                                includeRecharge.set(true);
                            }
                            if (0 == organizeId) {
                                if (1 == cartItemVo.getSvipProductFlag()) {
                                    // 超级会员优惠商品,不参与促销活动（普通机构购买该商品可参与店铺促销）/阶梯价/复购价
                                    // 超级会员设置商品优惠价
                                    productService.setSvipProductPrice(cartItemVo, taxFlag, svipUserFlag);
                                    if (svipUserFlag) {
                                        // 超级会员设置超级会员优惠总额 + 商品优惠金额
                                        shopSvipReducedPrice.set(MathUtil.add(shopSvipReducedPrice, cartItemVo.getSvipTotalReducedPrice()).doubleValue());
                                    }
                                } else {
                                    // 非超级会员优惠商品参与促销活动
                                    // 获取商品促销信息
                                    PromotionsVo promotions = null;
                                    // 没有店铺促销时，商品促销才有效 单品/凑单活动
                                    if (null == shopPromotion) {
                                        // 获取商品促销信息
                                        promotions = baseMapper.getPromotionByProductId(cartItemVo.getProductId());
                                        // 手动获取skuId优惠价
                                        if (promotions != null && (promotions.getTouchPrice() == null || promotions.getTouchPrice() == 0d)) {
                                            Double touchPrice = baseMapper.getTouchPriceBySku(cartItemVo.getSkuId(), promotions.getId());
                                            promotions.setTouchPrice(touchPrice);
                                        }
                                        // 如果促销活动协销不可见，移除促销
                                        if (promotions != null && promotions.getSeen() != null && promotions.getSeen() == 2 && finalUserIdentity1 == 1) {
                                            promotions = null;
                                        }
                                        /*
                                         * 设置商品促销优惠
                                         */
                                        if (null != promotions) {
                                            BigDecimal totalAmount = MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice());
                                            //叠加倍数
                                            Double floor = Math.floor(MathUtil.div(totalAmount, promotions.getTouchPrice()).doubleValue());
                                            // 当前促销活动的价格计算列表
                                            List<PromotionPriceVo> promotionPriceList = productService.getPromotionProducts(promotions, cartItemVo, taxFlag);
                                            // 更新到总促销列表
                                            productService.updateTotalPromotions(totalPromotions, promotionsIds, promotions, promotionPriceList, floor);
                                            //单品满减-计算供应商总价/满减金额
                                            if (promotions.getType() == 1 && promotions.getMode() == 2) {
                                                if (MathUtil.compare(totalAmount, promotions.getTouchPrice()) > -1) {
                                                    if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                                                        //叠加优惠计算
                                                        shopPrice.set(MathUtil.sub(shopPrice.get(), MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                                        shopReducedPrice.set(MathUtil.add(shopReducedPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                                    } else {
                                                        // 如果满足促销条件，设置供应商价格-满减金额，满减总额 + 当前促销满减金额
                                                        shopPrice.set(MathUtil.sub(shopPrice.get(), promotions.getReducedPrice()).doubleValue());
                                                        shopReducedPrice.set(MathUtil.add(shopReducedPrice, promotions.getReducedPrice()).doubleValue());
                                                    }
                                                }
                                            }
                                            cartItemVo.setPromotions(promotions);
                                        }
                                    }
                                    if (null != promotions || null != shopPromotion) {
                                        // 商品处于活动状态
                                        cartItemVo.setActStatus(1);
                                        // 关闭阶梯价格，活动优先
                                        cartItemVo.setLadderFlag(0);
                                    } else {
                                        if (cartItemVo.getLadderFlag() == 1) {
                                            // 设置阶梯价
                                            productService.setCartLadderPrices(cartItemVo, taxFlag);
                                        }
                                        /*else {
                                            // 复购价
                                            Double repurchase = baseMapper.getRepurchasePrice(cartItemVo.getSkuId(), userId);
                                            if (null != repurchase && repurchase > 0) {
                                                if (taxFlag) {
                                                    BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(repurchase, cartItemVo.getTaxRate()), 100, 2);
                                                    cartItemVo.setPrice(MathUtil.add(repurchase, cartItemTax).doubleValue());
                                                } else {
                                                    cartItemVo.setPrice(repurchase);
                                                }
                                            }
                                        }*/
                                    }
                                }
                            }
                            // 该供应商下价格累加
                            shopPrice.set(MathUtil.add(shopPrice, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                            shopOriginalPrice.set(MathUtil.add(shopOriginalPrice.get(), MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getOriginalPrice())).doubleValue());
                            if (0 == organizeId) {
                                // 该供应商下可参与店铺促销的商品价格累加
                                if (!(1 == cartItemVo.getSvipProductFlag() && svipUserFlag)) {
                                    // 超级会员购买svip商品不享受店铺促销优惠
                                    shopPromotionFee.set(MathUtil.add(shopPromotionFee, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                                }
                            }
                            // 设置商品运费
                            if (null != cityId) {
                                List<String> cartSkuIds = new ArrayList<>();
                                cartSkuIds.add(cartItemVo.getSkuId().toString());
                                Map<String, Object> postageMap = productService.computePostage(userId, cityId, cartItemVo.getNumber(), MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice()).doubleValue(), cartSkuIds);
                                Integer postageFlag = (Integer) postageMap.get("postageFlag");
                                shopPostageFlag.add(postageFlag);
                                if (999 == postageFlag) {
                                    templateFlag.set(-1);
                                } else {
                                    Double postage = (Double) postageMap.get("postage");
                                    shopPostage.set(MathUtil.add(shopPostage.get(), postage).doubleValue());
                                    log.info("shopPostage.get()======"+shopPostage.get());
                                }
                                shop.setUserBeans((Integer) postageMap.get("userBeans"));
                            }
                            // 该供应商下商品种类 +1
                            shopKindCount.incrementAndGet();
                            // 总数量 + 当前商品购买数量
                            totalCount.updateAndGet(v -> v + cartItemVo.getNumber());
                        }
                        // 运费方式
                        if (shopPostageFlag.contains(1)) {
                            shop.setPostageFlag(1);
                        } else if (!shopPostageFlag.contains(1) && shopPostageFlag.contains(2)) {
                            shop.setPostageFlag(2);
                        } else {
                            shop.setPostageFlag(0);
                        }
                        shop.setPostage(shopPostage.get());
                        // 运费
                        // 包邮或者到付 冷链费为 0
                        coldChain = 0 == shop.getPostage() ? 0 : coldChain;
                        shop.setPostage(MathUtil.sub(shop.getPostage(), coldChain).doubleValue());
                        shop.setColdChain(coldChain);
                        if (0 != coldChain) {
                            shop.setIsColdChina(true);
                        } else {
                            shop.setIsColdChina(false);
                        }
                        log.info("shop.getPostage()******"+shop.getPostage());
                        // 供应商总金额
                        shopPrice.set(MathUtil.add(MathUtil.add(shopPrice.get(), shop.getPostage()),coldChain).doubleValue());
                        if (shopKindCount.get() > 0) {
                            // 店铺促销
                            if (null != shopPromotion) {
                                Double floor = Math.floor(MathUtil.div(shopPromotionFee, shopPromotion.getTouchPrice()).doubleValue());
                                shop.setPromotions(shopPromotion);
                                if (!promotionsIds.contains(shopPromotion.getId())) {
                                    promotionsIds.add(shopPromotion.getId());
                                    // 店铺满赠
                                    if (shopPromotion.getMode() == 3 && MathUtil.compare(shopPromotionFee, shopPromotion.getTouchPrice()) > -1) {
                                        // 获取赠品
                                        List<CartItemVo> giftList = baseMapper.getPromotionGifts(shopPromotion.getId());
                                        //叠加满赠
                                        if (shopPromotion.getDiscount() != null && shopPromotion.getDiscount() == 1 && floor > 1) {
                                            Integer in = floor.intValue();
                                            giftList.forEach(g -> g.setNumber(MathUtil.mul(g.getNumber(), in).intValue()));
                                        }
                                        shopPromotion.setGiftList(giftList);
                                    }
                                    // 设置该优惠下的商品列表
                                    List<PromotionPriceVo> promotionPriceList = new ArrayList<>();
                                    productList.forEach(item -> {
                                        if (!(1 == item.getSvipProductFlag() && svipUserFlag)) {
                                            // 超级会员购买svip商品不享受店铺促销优惠
                                            PromotionPriceVo promotionPrice = new PromotionPriceVo();
                                            promotionPrice.setProductId(item.getProductId());
                                            promotionPrice.setNumber(item.getNumber());
                                            promotionPrice.setPrice(item.getPrice());
                                            promotionPriceList.add(promotionPrice);
                                        }
                                    });
                                    shopPromotion.setProductList(promotionPriceList);
                                    // 添加到总促销
                                    totalPromotions.add(shopPromotion);
                                    // 店铺满减-计算供应商总价/满减金额
                                    // 满减叠加计算
                                    if (shopPromotion.getMode() == 2 && MathUtil.compare(shopPromotionFee, shopPromotion.getTouchPrice()) > -1) {
                                        if (shopPromotion.getDiscount() != null && shopPromotion.getDiscount() == 1 && floor > 1) {
                                            shopPrice.set(MathUtil.sub(shopPrice.get(), MathUtil.mul(shopPromotion.getReducedPrice(), floor)).doubleValue());
                                            shopReducedPrice.set(MathUtil.add(shopReducedPrice, MathUtil.mul(shopPromotion.getReducedPrice(), floor)).doubleValue());
                                        } else {
                                            // 该供应商总价 - 满减金额
                                            shopPrice.set(MathUtil.sub(shopPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                                            // 该供应商优惠总额 + 满减金额
                                            shopReducedPrice.set(MathUtil.add(shopReducedPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                                        }
                                    }
                                }
                            }
                            // 供应商商品
                            shop.setCartList(productList);
                            // 供应商总价
                            shop.setTotalPrice(shopPrice.get());
                            // 供应商总优惠
                            shop.setReducedPrice(shopReducedPrice.get());
                            // 供应商总超级会员优惠
                            shop.setSvipReducedPrice(shopSvipReducedPrice.get());
                            // 供应商划线价
                            shop.setOriginalPrice(shopOriginalPrice.get());
                            // 添加供应商Id集合
                            shopIds.add(shop.getShopId());
                            // 计算总价
                            totalPrice.set(MathUtil.add(totalPrice, shop.getTotalPrice()).doubleValue());
                            // 优惠总额
                            reducedPrice.set(MathUtil.add(reducedPrice, shop.getReducedPrice()).doubleValue());
                            // 超级会员优惠总额
                            svipReducedPrice.set(MathUtil.add(svipReducedPrice, shop.getSvipReducedPrice()).doubleValue());
                            // 总划线价
                            totalOriginalPrice.set(MathUtil.add(totalOriginalPrice, shop.getOriginalPrice()).doubleValue());
                            // 商品种类
                            kindCount.updateAndGet(v -> v + shopKindCount.get());
                        }
                        totalProductList.addAll(productList);
                    }
                    // 供应商下商品种类
                    shop.setCount(shopKindCount.get());
                }
            });
            // 删除空数据
            shopList.removeIf(shop -> (null == shop || shop.getCount() == 0));

        }
        // 总促销计算
        // 满减满赠叠加计算
        List<CartShopVo> finalShopList = shopList;
        totalPromotions.forEach(promotions -> {
            // 该促销内商品总价
            double touchPrice = promotions.getProductList().stream().mapToDouble(product -> product.getNumber() * product.getPrice()).sum();
            // 满足促销条件
            if (MathUtil.compare(touchPrice, promotions.getTouchPrice()) > -1) {
                //叠加倍数
                Double floor = Math.floor(MathUtil.div(touchPrice, promotions.getTouchPrice()).doubleValue());
                // 凑单满减
                if (promotions.getType() == 2 && promotions.getMode() == 2) {
                    if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                        //叠加优惠计算
                        // 总价 - 满减金额
                        totalPrice.set(MathUtil.sub(totalPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                        // 优惠总额 + 满减金额
                        reducedPrice.set(MathUtil.add(reducedPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                    } else {
                        // 总价 - 满减金额
                        totalPrice.set(MathUtil.sub(totalPrice, promotions.getReducedPrice()).doubleValue());
                        // 优惠总额 + 满减金额
                        reducedPrice.set(MathUtil.add(reducedPrice, promotions.getReducedPrice()).doubleValue());
                    }
                } else if (promotions.getMode() == 3) {
                    //满赠叠加
                    List<CartItemVo> giftList = baseMapper.getPromotionGifts(promotions.getId());
                    //满足叠加
                    if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                        Integer in = floor.intValue();
                        giftList.forEach(g -> g.setNumber(MathUtil.mul(g.getNumber(), in).intValue()));
                    }
                    promotions.setGiftList(giftList);
                    // 全部满赠
                    promotions.getGiftList().forEach(gift -> {
                        if (shopIds.contains(gift.getShopId())) {
                            // 赠品在当前订单内的供应商下
                            finalShopList.forEach(shop -> {
                                if (shop.getShopId().equals(gift.getShopId())) {
                                    shop.getCartList().add(gift);
                                }
                            });
                        } else {
                            // 获取赠品供应商
                            CartShopVo giftShop = baseMapper.getShopByProductId(gift.getProductId());
                            shopIds.add(giftShop.getShopId());
                            giftShop.setCartList(new ArrayList<>());
                            giftShop.getCartList().add(gift);
                            finalShopList.add(giftShop);
                        }
                    });
                }
            }
        });

        // 发票信息
        InvoiceVo invoice = baseMapper.getUserInvoice(userId);
        // 可用余额
        Double availableMoney = baseMapper.getAbleUserMoney(userId);
        // 可用采美豆
        Integer userBeans = baseMapper.getUserBeans(userId);
        // 用户可用优惠券（总）
        List<CouponVo> couponList = getUserCartCoupons(totalProductList, source, userId);

        // 包装返回数据
        Map<String, Object> resultData = new HashMap<>();
        resultData.put("list", finalShopList);
        resultData.put("kindCount", kindCount);
        resultData.put("totalCount", totalCount);
        resultData.put("totalPrice", totalPrice);
        resultData.put("reducedPrice", reducedPrice);
        resultData.put("templateFlag", templateFlag.get());
        resultData.put("svipReducedPrice", svipReducedPrice);
        resultData.put("totalOriginalPrice", totalOriginalPrice);
        resultData.put("promotions", totalPromotions);
        resultData.put("invoice", invoice);
        resultData.put("userMoney", availableMoney);
        resultData.put("userBeans", userBeans);
        resultData.put("includeRecharge", includeRecharge);
        resultData.put("couponList", couponList);
        // 联合丽格冷链费
        if (0 != organizeId) {
            double coldChain = 0.00d;
            if (null != totalProductList) {
                for (CartItemVo cart :totalProductList) {
                    if (7881 == cart.getProductId()) {
                        if (cart.getNumber() >= 100) {
                            coldChain = 0.00d;
                        } else {
                            coldChain = baseMapper.getFreightAmount(cart.getProductId());
                        }
                        break;
                    } else {
                        coldChain = 0.00d;
                    }
                }
            }
            resultData.put("coldChain", coldChain);
        }
        if (flag) {
            resultData.put("returnGoodsStutas", 2);
            resultData.put("helpContent", helpContent);
        } else {
            resultData.put("returnGoodsStutas", 1);
            resultData.put("helpContent", null);
        }
        return ResponseJson.success(resultData);
    }

    /**
     * 组合商品立即购买
     *
     * @param userId      用户ID
     * @param productInfo 组合商品信息：Json字符串格式[{"id":4351,"count":1,"skuId":123},{}]
     * @param source      来源 : 1 网站 ; 2 小程序
     * @return
     */
    @Override
    public ResponseJson<Map<String, Object>> MultipleBuyNow(Integer userId, Integer cityId, String productInfo, Integer source) {
        log.info("￥￥￥￥￥￥￥￥￥￥ > 组合商品立即购买");
        // 商品种类
        AtomicInteger kindCount = new AtomicInteger(0);
        // 商品总数量
        AtomicInteger totalCount = new AtomicInteger(0);
        // 统计商品总金额
        AtomicDouble totalPrice = new AtomicDouble(0);
        // 统计总促销满减
        AtomicDouble reducedPrice = new AtomicDouble(0);
        // 统计总超级会员优惠
        AtomicDouble svipReducedPrice = new AtomicDouble(0);
        // 统计总划线价
        AtomicDouble totalOriginalPrice = new AtomicDouble(0);
        // 促销活动（总）
        List<PromotionsVo> totalPromotions = new ArrayList<>();
        // 是否包含充值商品，默认false
        AtomicBoolean includeRecharge = new AtomicBoolean(false);
        //所有的商品数据
        List<CartItemVo> totalProductList = new ArrayList<>();
        // 促销活动Id集合，用于合并促销活动
        List<Integer> promotionsIds = new ArrayList<>();
        // 供应商Id集合，用户判断赠品供应商是否在当前供应商中
        List<Integer> shopIds = new ArrayList<>();
        // 商品信息map
        Map<String, Integer> productInfos = new HashMap<>();
        // 前端接收商品Id信息
        List<String> productIdList = new ArrayList<>();
        //skuIdList
        List<Integer> skuIdList = new ArrayList<>();
        // 运费模板是否异常
        AtomicInteger templateFlag = new AtomicInteger(0);
        // 用户身份
        Integer userIdentity = baseMapper.getIdentityByUserId(userId);
        if (null == userIdentity) {
            userIdentity = 0;
        }
        // 会员机构类型：1医美，2生美
        Integer userClubType = 0;
        if (userIdentity == 2) {
            userClubType = cartClubMapper.getClubTypeById(userId);
            if (null == userClubType) {
                userClubType = 0;
            }
        }
        // 超级会员标识
        Integer svipUserId = baseMapper.getSvipUserIdByUserId(userId);
        boolean svipUserFlag = null != svipUserId;
        // productInfo拆解
        try {
            JSONArray productArr = parseArray(productInfo);
            for (Object o : productArr) {
                JSONObject product = (JSONObject) o;
                String productId = product.getString("id");
                Integer productCount = product.getInteger("count");
                Integer skuId = product.getInteger("skuId");
                productInfos.put(skuId.toString(), productCount);
                productIdList.add(productId);
                skuIdList.add(skuId);
            }
        } catch (Exception e) {
            log.error("组合商品信息格式错误", e);
            return ResponseJson.error("组合商品信息格式错误！Json字符串格式[{\"id\":4351,\"count\":1},{}]", null);
        }

        // 商品的供应商列表
        List<CartShopVo> shopList = cartClubMapper.getShopsByProductIds(productIdList);
        List<CartShopVo> goodsStutas = cartClubMapper.getProductReturnGoodsStutas(skuIdList.stream().map(String::valueOf).collect(Collectors.toList()));
        //按供应商分类统计商品
        String helpContent = null;
        boolean flag = false;
        if (null != shopList && shopList.size() > 0) {
            // 遍历供应商列表
            List<Integer> finalIdList = skuIdList;//商品id列表
            Integer finalUserIdentity = userIdentity;
            Integer finalUserClubType = userClubType;
            Integer finalUserIdentity1 = userIdentity;
            for (int i = 0; i < goodsStutas.size(); i++) {
                //订单中存在不能退货的商品则给出不能退货提示
                if (null != goodsStutas.get(i).getReturnGoodsStutas() && goodsStutas.get(i).getReturnGoodsStutas() == 2) {
                    flag = true;
                    helpContent = cartClubMapper.getHelpContent(1040);
                    break;
                }
            }
            shopList.forEach(shop -> {
                if (null != shop) {
                    // 该供应商下商品种类
                    AtomicInteger shopKindCount = new AtomicInteger(0);
                    // 该供应商总价
                    AtomicDouble shopPrice = new AtomicDouble(0);
                    // 该供应商下参与店铺促销的商品总价
                    AtomicDouble shopPromotionFee = new AtomicDouble(0);
                    // 该供应商满减金额（供应商满减，单品满减）
                    AtomicDouble shopReducedPrice = new AtomicDouble(0);
                    // 该供应商svip优惠金额
                    AtomicDouble shopSvipReducedPrice = new AtomicDouble(0);
                    // 该供应商划线价
                    AtomicDouble shopOriginalPrice = new AtomicDouble(0);
                    // 运费方式 0 包邮 1 到付 2 运费模板
                    List<Integer> shopPostageFlag = new ArrayList<>();
                    // 运费
                    AtomicDouble shopPostage = new AtomicDouble(0);
                    // 供应商促销优惠活动
                    PromotionsVo shopPromotion = baseMapper.getPromotionByShopId(shop.getShopId());
                    if (shopPromotion != null && shopPromotion.getSeen() != null && shopPromotion.getSeen() == 2 && finalUserIdentity1 == 1) {
                        shopPromotion = null;
                    }
                    // 供应商下商品列表
                    // 过滤保存已上架商品
                    List<CartItemVo> productList = cartClubMapper.getProductsByShopIdAndProductIds(shop.getShopId(), finalIdList);
                    productList.forEach(p -> p.setNumber(productInfos.get(p.getSkuId().toString())));
                    if (null != productList && productList.size() > 0) {
                        // 是否包含需要冷链运输商品
                        List<Integer> collect = productList.stream().map(CartItemVo::getProductId).collect(Collectors.toList());
                        if (collect.stream().allMatch(c -> c == 7881)) {
                            shop.setDesignatedFlag(1);
                        } else if (!collect.contains(7881)) {
                            shop.setDesignatedFlag(2);
                        } else {
                            shop.setDesignatedFlag(3);
                        }
                        // 去除不可见商品，visibility商品可见度： 3:所有人可见，2:普通机构可见，1:会员机构可见，4:仅医美机构可见
                        productList.removeIf(cartItemVo -> !(cartItemVo.getVisibility() == 3 || cartItemVo.getVisibility() == 2 || (cartItemVo.getVisibility() == 1 && (finalUserIdentity == 2 || svipUserFlag)) || (cartItemVo.getVisibility() == 4 && finalUserClubType == 1)));
                        // 去除价格不可见商品，价格可见度： 0公开价格，1不公开价格，2仅对会员机构公开，3仅对医美机构公开
                        productList.removeIf(cartItemVo -> !(cartItemVo.getPriceFlag() == 0 || (cartItemVo.getPriceFlag() == 2 && (finalUserIdentity == 2 || svipUserFlag)) || (cartItemVo.getPriceFlag() == 3 && finalUserClubType == 1)));
                        // 去除库存不足商品
                        productList.removeIf(cartItemVo -> !(cartItemVo.getStock() != null && cartItemVo.getStock() > 0 && cartItemVo.getStock() >= cartItemVo.getMin() && cartItemVo.getStock() >= cartItemVo.getNumber()));


                        Double coldChain = 0d;
                        // 迭代器设置商品信息
                        for (CartItemVo cartItemVo : productList) {
                            // 设置商品运费
                            if (null != cityId) {
                                if (7881 == cartItemVo.getProductId()) {
                                    if (cartItemVo.getNumber() >= 100) {
                                        coldChain = 0d;
                                    } else {
                                        coldChain = baseMapper.getFreightAmount(cartItemVo.getProductId());
                                        // shop.setPostageFlag(1);
                                    }
                                }
                            }
                            // 设置商品图片及税费
                            boolean taxFlag = productService.setCartItemImgAndTax(cartItemVo);
                            // 是否充值商品
                            boolean recharge = productService.isRechargeProduct(cartItemVo.getProductId());
                            if (recharge) {
                                includeRecharge.set(true);
                            }

                            if (1 == cartItemVo.getSvipProductFlag()) {
                                // 超级会员优惠商品,不参与促销活动（普通机构购买该商品可参与店铺促销）/阶梯价/复购价
                                // 超级会员设置商品优惠价
                                productService.setSvipProductPrice(cartItemVo, taxFlag, svipUserFlag);
                                if (svipUserFlag) {
                                    // 超级会员设置超级会员优惠总额 + 商品优惠金额
                                    shopSvipReducedPrice.set(MathUtil.add(shopSvipReducedPrice, cartItemVo.getSvipTotalReducedPrice()).doubleValue());
                                }
                            } else {
                                // 非超级会员优惠商品参与促销活动
                                // 获取商品促销信息
                                PromotionsVo promotions = null;
                                // 没有店铺促销时，商品促销才有效 单品/凑单活动
                                if (null == shopPromotion) {
                                    // 获取商品促销信息
                                    promotions = baseMapper.getPromotionByProductId(cartItemVo.getProductId());
                                    // 如果促销活动协销不可见，移除促销
                                    if (promotions != null && promotions.getSeen() != null && promotions.getSeen() == 2 && finalUserIdentity1 == 1) {
                                        promotions = null;
                                    }
                                    /*
                                     * 设置商品促销优惠
                                     */
                                    if (null != promotions) {
                                        BigDecimal totalAmount = MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice());
                                        //叠加倍数
                                        Double floor = Math.floor(MathUtil.div(totalAmount, promotions.getTouchPrice()).doubleValue());
                                        // 当前促销活动的价格计算列表
                                        List<PromotionPriceVo> promotionPriceList = productService.getPromotionProducts(promotions, cartItemVo, taxFlag);
                                        // 更新到总促销列表
                                        productService.updateTotalPromotions(totalPromotions, promotionsIds, promotions, promotionPriceList, floor);
                                        //单品满减-计算供应商总价/满减金额
                                        if (promotions.getType() == 1 && promotions.getMode() == 2) {
                                            if (MathUtil.compare(totalAmount, promotions.getTouchPrice()) > -1) {
                                                if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                                                    //叠加优惠计算
                                                    shopPrice.set(MathUtil.sub(shopPrice.get(), MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                                    shopReducedPrice.set(MathUtil.add(shopReducedPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                                } else {
                                                    // 如果满足促销条件，设置供应商价格-满减金额，满减总额 + 当前促销满减金额
                                                    shopPrice.set(MathUtil.sub(shopPrice.get(), promotions.getReducedPrice()).doubleValue());
                                                    shopReducedPrice.set(MathUtil.add(shopReducedPrice, promotions.getReducedPrice()).doubleValue());
                                                }
                                            }
                                        }
                                        cartItemVo.setPromotions(promotions);
                                    }
                                }
                                if (null != promotions || null != shopPromotion) {
                                    // 商品处于活动状态
                                    cartItemVo.setActStatus(1);
                                    // 关闭阶梯价格，活动优先
                                    cartItemVo.setLadderFlag(0);
                                } else {
                                    if (cartItemVo.getLadderFlag() == 1) {
                                        // 设置阶梯价
                                        productService.setCartLadderPrices(cartItemVo, taxFlag);
                                    }
                                    /*else {
                                        // 复购价
                                        Double repurchase = baseMapper.getRepurchasePrice(cartItemVo.getSkuId(), userId);
                                        if (null != repurchase && repurchase > 0) {
                                            if (taxFlag) {
                                                BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(repurchase, cartItemVo.getTaxRate()), 100, 2);
                                                cartItemVo.setPrice(MathUtil.add(repurchase, cartItemTax).doubleValue());
                                            } else {
                                                cartItemVo.setPrice(repurchase);
                                            }
                                        }
                                    }*/
                                }
                            }
                            // 该供应商下价格累加
                            shopPrice.set(MathUtil.add(shopPrice, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                            shopOriginalPrice.set(MathUtil.add(shopOriginalPrice.get(), MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getOriginalPrice())).doubleValue());
                            // 该供应商下可参与店铺促销的商品价格累加
                            if (!(1 == cartItemVo.getSvipProductFlag() && svipUserFlag)) {
                                // 超级会员购买svip商品不享受店铺促销优惠
                                shopPromotionFee.set(MathUtil.add(shopPromotionFee, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                            }
                            if (null != cityId) {
                                List<String> cartSkuIds = new ArrayList<>();
                                cartSkuIds.add(cartItemVo.getSkuId().toString());
                                Map<String, Object> postageMap = productService.computePostage(userId, cityId, cartItemVo.getNumber(), MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice()).doubleValue(), cartSkuIds);
                                Integer postageFlag = (Integer) postageMap.get("postageFlag");
                                shopPostageFlag.add(postageFlag);
                                if (999 == postageFlag) {
                                    templateFlag.set(-1);
                                } else {
                                    Double postage = (Double) postageMap.get("postage");
                                    shopPostage.set(MathUtil.add(shopPostage.get(), postage).doubleValue());
                                }
                                shop.setUserBeans((Integer) postageMap.get("userBeans"));
                            }
                            // 该供应商下商品种类 +1
                            shopKindCount.incrementAndGet();
                            // 总数量 + 当前商品购买数量
                            totalCount.updateAndGet(v -> v + cartItemVo.getNumber());
                        }
                        // 运费方式
                        if (shopPostageFlag.contains(1)) {
                            shop.setPostageFlag(1);
                        } else if (!shopPostageFlag.contains(1) && shopPostageFlag.contains(2)) {
                            shop.setPostageFlag(2);
                        } else {
                            shop.setPostageFlag(0);
                        }
                        shop.setPostage(shopPostage.get());

                        // 总费用 + 冷链费
                        // shopPrice.set(MathUtil.add(shopPrice.get(), coldChain).doubleValue());
                        // 只有冷链费商品 运费为0
                        /*if (null != townId) {
                            if (1 == shop.getDesignatedFlag()) {
                                shop.setPostage(0d);
                            }
                        }*/
                        // 总运费
                        // 包邮或者到付 冷链费为 0
                        coldChain = 0 == shop.getPostage() ? 0 : coldChain;
                        shop.setPostage(MathUtil.sub(shop.getPostage(), coldChain).doubleValue());
                        shop.setColdChain(coldChain);
                        if (0 != coldChain) {
                            shop.setIsColdChina(true);
                        } else {
                            shop.setIsColdChina(false);
                        }
                        // 供应商总金额
                        shopPrice.set(MathUtil.add(MathUtil.add(shopPrice.get(), shop.getPostage()),coldChain).doubleValue());
                        if (shopKindCount.get() > 0) {
                            // 店铺促销
                            if (null != shopPromotion) {
                                Double floor = Math.floor(MathUtil.div(shopPromotionFee, shopPromotion.getTouchPrice()).doubleValue());
                                shop.setPromotions(shopPromotion);
                                if (!promotionsIds.contains(shopPromotion.getId())) {
                                    promotionsIds.add(shopPromotion.getId());
                                    // 店铺满赠
                                    if (shopPromotion.getMode() == 3 && MathUtil.compare(shopPromotionFee, shopPromotion.getTouchPrice()) > -1) {
                                        // 获取赠品
                                        List<CartItemVo> giftList = baseMapper.getPromotionGifts(shopPromotion.getId());
                                        //叠加满赠
                                        if (shopPromotion.getDiscount() != null && shopPromotion.getDiscount() == 1 && floor > 1) {
                                            Integer in = floor.intValue();
                                            giftList.forEach(g -> g.setNumber(MathUtil.mul(g.getNumber(), in).intValue()));
                                        }
                                        shopPromotion.setGiftList(giftList);
                                    }
                                    // 设置该优惠下的商品列表
                                    List<PromotionPriceVo> promotionPriceList = new ArrayList<>();
                                    productList.forEach(item -> {
                                        if (!(1 == item.getSvipProductFlag() && svipUserFlag)) {
                                            // 超级会员购买svip商品不享受店铺促销优惠
                                            PromotionPriceVo promotionPrice = new PromotionPriceVo();
                                            promotionPrice.setProductId(item.getProductId());
                                            promotionPrice.setNumber(item.getNumber());
                                            promotionPrice.setPrice(item.getPrice());
                                            promotionPriceList.add(promotionPrice);
                                        }
                                    });
                                    shopPromotion.setProductList(promotionPriceList);
                                    // 添加到总促销
                                    totalPromotions.add(shopPromotion);
                                    // 店铺满减-计算供应商总价/满减金额
                                    // 满减叠加计算
                                    if (shopPromotion.getMode() == 2 && MathUtil.compare(shopPromotionFee, shopPromotion.getTouchPrice()) > -1) {
                                        if (shopPromotion.getDiscount() != null && shopPromotion.getDiscount() == 1 && floor > 1) {
                                            shopPrice.set(MathUtil.sub(shopPrice.get(), MathUtil.mul(shopPromotion.getReducedPrice(), floor)).doubleValue());
                                            shopReducedPrice.set(MathUtil.add(shopReducedPrice, MathUtil.mul(shopPromotion.getReducedPrice(), floor)).doubleValue());
                                        } else {
                                            // 该供应商总价 - 满减金额
                                            shopPrice.set(MathUtil.sub(shopPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                                            // 该供应商优惠总额 + 满减金额
                                            shopReducedPrice.set(MathUtil.add(shopReducedPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                                        }
                                    }
                                }
                            }
                            // 供应商商品
                            shop.setCartList(productList);
                            // 供应商总价
                            shop.setTotalPrice(shopPrice.get());
                            // 供应商总优惠
                            shop.setReducedPrice(shopReducedPrice.get());
                            // 供应商总超级会员优惠
                            shop.setSvipReducedPrice(shopSvipReducedPrice.get());
                            // 供应商划线价
                            shop.setOriginalPrice(shopOriginalPrice.get());
                            // 添加供应商Id集合
                            shopIds.add(shop.getShopId());
                            // 计算总价
                            totalPrice.set(MathUtil.add(totalPrice, shop.getTotalPrice()).doubleValue());
                            // 优惠总额
                            reducedPrice.set(MathUtil.add(reducedPrice, shop.getReducedPrice()).doubleValue());
                            // 超级会员优惠总额
                            svipReducedPrice.set(MathUtil.add(svipReducedPrice, shop.getSvipReducedPrice()).doubleValue());
                            // 总划线价
                            totalOriginalPrice.set(MathUtil.add(totalOriginalPrice, shop.getOriginalPrice()).doubleValue());
                            // 商品种类
                            kindCount.updateAndGet(v -> v + shopKindCount.get());
                        }
                        totalProductList.addAll(productList);
                    }
                    // 供应商下商品种类
                    shop.setCount(shopKindCount.get());
                }
            });
            // 删除空数据
            shopList.removeIf(shop -> (null == shop || shop.getCount() == 0));

        }
        // 总促销计算
        // 满减满赠叠加计算
        totalPromotions.forEach(promotions -> {
            // 该促销内商品总价
            double touchPrice = promotions.getProductList().stream().mapToDouble(product -> product.getNumber() * product.getPrice()).sum();
            // 满足促销条件
            if (MathUtil.compare(touchPrice, promotions.getTouchPrice()) > -1) {
                //叠加倍数
                Double floor = Math.floor(MathUtil.div(touchPrice, promotions.getTouchPrice()).doubleValue());
                // 凑单满减
                if (promotions.getType() == 2 && promotions.getMode() == 2) {
                    if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                        //叠加优惠计算
                        // 总价 - 满减金额
                        totalPrice.set(MathUtil.sub(totalPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                        // 优惠总额 + 满减金额
                        reducedPrice.set(MathUtil.add(reducedPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                    } else {
                        // 总价 - 满减金额
                        totalPrice.set(MathUtil.sub(totalPrice, promotions.getReducedPrice()).doubleValue());
                        // 优惠总额 + 满减金额
                        reducedPrice.set(MathUtil.add(reducedPrice, promotions.getReducedPrice()).doubleValue());
                    }
                } else if (promotions.getMode() == 3) {
                    //满赠叠加
                    List<CartItemVo> giftList = baseMapper.getPromotionGifts(promotions.getId());
                    //满足叠加
                    if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                        Integer in = floor.intValue();
                        giftList.forEach(g -> g.setNumber(MathUtil.mul(g.getNumber(), in).intValue()));
                    }
                    promotions.setGiftList(giftList);
                    // 全部满赠
                    promotions.getGiftList().forEach(gift -> {
                        if (shopIds.contains(gift.getShopId())) {
                            // 赠品在当前订单内的供应商下
                            shopList.forEach(shop -> {
                                if (shop.getShopId().equals(gift.getShopId())) {
                                    shop.getCartList().add(gift);
                                }
                            });
                        } else {
                            // 获取赠品供应商
                            CartShopVo giftShop = baseMapper.getShopByProductId(gift.getProductId());
                            shopIds.add(giftShop.getShopId());
                            giftShop.setCartList(new ArrayList<>());
                            giftShop.getCartList().add(gift);
                            shopList.add(giftShop);
                        }
                    });
                }
            }
        });

        // 发票信息
        InvoiceVo invoice = baseMapper.getUserInvoice(userId);
        // 可用余额
        Double availableMoney = baseMapper.getAbleUserMoney(userId);
        // 可用采美豆
        Integer userBeans = baseMapper.getUserBeans(userId);
        // 用户可用优惠券（总）
        List<CouponVo> couponList = getUserCartCoupons(totalProductList, source, userId);

        // 包装返回数据
        Map<String, Object> resultData = new HashMap<>();
        resultData.put("list", shopList);
        resultData.put("kindCount", kindCount);
        resultData.put("totalCount", totalCount);
        resultData.put("totalPrice", totalPrice);
        resultData.put("reducedPrice", reducedPrice);
        resultData.put("templateFlag", templateFlag.get());
        resultData.put("svipReducedPrice", svipReducedPrice);
        resultData.put("totalOriginalPrice", totalOriginalPrice);
        resultData.put("promotions", totalPromotions);
        resultData.put("invoice", invoice);
        resultData.put("userMoney", availableMoney);
        resultData.put("userBeans", userBeans);
        resultData.put("includeRecharge", includeRecharge);
        resultData.put("couponList", couponList);
        if (flag) {
            resultData.put("returnGoodsStutas", 2);
            resultData.put("helpContent", helpContent);
        } else {
            resultData.put("returnGoodsStutas", 1);
            resultData.put("helpContent", null);
        }
        return ResponseJson.success(resultData);
    }

    @Override
    public ResponseJson checkSkuId(Integer userId, Integer oldSkuId, Integer newSkuId, Integer count) {
        cartClubMapper.deleteCartSku(userId, oldSkuId);
        CartDto cartDto = new CartDto();
        cartDto.setUserId(userId);
        cartDto.setSkuId(newSkuId);
        cartDto.setProductCount(count);
        saveShoppingCart(cartDto);
        return ResponseJson.success();
    }

    /**
     * 立即购买
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                productCount 商品数量
     *                source       来源 : 1 网站 ; 2 小程序
     *                }
     */
    @Override
    public ResponseJson<Map<String, Object>> buyNowProduct(CartDto cartDto) {
        log.info("￥￥￥￥￥￥￥￥￥￥ > 立即购买");
        // 用户组织Id
        Integer organizeId = baseMapper.getOrganizeId(cartDto.getUserId());
        // 用户身份
        Integer userIdentity = baseMapper.getIdentityByUserId(cartDto.getUserId());
        if (null == userIdentity) {
            userIdentity = 0;
        }
        // 供应商信息
        CartShopVo shop = baseMapper.getShopByProductId(cartDto.getProductId());
        if (null == shop) {
            return ResponseJson.error("商品供应商异常！", null);
        }
        CartItemVo cartItemVo = null;
        if (0 == organizeId) {
            if(userIdentity==7){
                //分销人员sku
                cartItemVo = cartClubMapper.getDistributionProductId(cartDto.getSkuId());
            }else {
                cartItemVo = cartClubMapper.getCartItemByProductId(cartDto.getSkuId());
            }
        } else {
            cartItemVo = cartClubMapper.getCartOrganizeProductId(cartDto.getSkuId());
        }
        if (null == cartItemVo) {
            return ResponseJson.error("商品数据异常！", null);
        } else {
            //是否能退货
            if (null != cartItemVo.getReturnGoodsStutas() && 2 == cartItemVo.getReturnGoodsStutas()) {
                String helpContent = orderCommonMapper.getHelpContent(1040);
                cartItemVo.setHelpContent(helpContent);
            }
        }

        // 会员机构类型：1医美，2生美
        Integer userClubType = 0;
        if (userIdentity == 2) {
            userClubType = cartClubMapper.getClubTypeById(cartDto.getUserId());
            if (null == userClubType) {
                userClubType = 0;
            }
        }
        // 超级会员标识
        Integer svipUserId = baseMapper.getSvipUserIdByUserId(cartDto.getUserId());
        boolean svipUserFlag = null != svipUserId;
        if (0 == organizeId&&userIdentity!=7) {
            // visibility商品可见度： 3:所有人可见，2:普通机构可见，1:会员机构可见，4:仅医美机构可见
            if (!(cartItemVo.getVisibility() == 3 || cartItemVo.getVisibility() == 2 || (cartItemVo.getVisibility() == 1 && (userIdentity == 2 || svipUserFlag)) || (cartItemVo.getVisibility() == 4 && userClubType == 1))) {
                return ResponseJson.error("商品不可见！", null);
            }
            // 价格可见度： 0公开价格，1不公开价格，2仅对会员机构公开，3仅对医美机构公开

            if (!(cartItemVo.getPriceFlag() == 0 || (cartItemVo.getPriceFlag() == 2 && (userIdentity == 2 || svipUserFlag)) || (cartItemVo.getPriceFlag() == 3 && userClubType == 1))) {
                return ResponseJson.error("商品价格不可见！", null);
            }
        }
        cartItemVo.setNumber(cartDto.getProductCount());
        // 库存不足商品
//        if (!(cartItemVo.getStock() != null && cartItemVo.getStock() > 0 && cartItemVo.getStock() >= cartItemVo.getMin() && cartItemVo.getStock() >= cartItemVo.getNumber())) {
//            return ResponseJson.error("商品库存不足！", null);
//        }
        // 供应商Id集合，用户判断赠品供应商是否在当前供应商中
        List<Integer> shopIds = new ArrayList<>();
        shopIds.add(shop.getShopId());
        List<CartShopVo> shopList = new ArrayList<>();
        // 设置商品图片及税费
        boolean taxFlag = productService.setCartItemImgAndTax(cartItemVo);

        // 促销活动（总）
        List<PromotionsVo> totalPromotions = new ArrayList<>();
        // 统计商品总金额
        AtomicDouble originalPrice = new AtomicDouble(0);
        AtomicDouble totalPrice = new AtomicDouble(0);
        // 统计总促销满减
        AtomicDouble reducedPrice = new AtomicDouble(0);
        // svip优惠金额
        AtomicDouble svipReducedPrice = new AtomicDouble(0);
        // 供应商促销优惠活动
        PromotionsVo promotions = baseMapper.getPromotionByShopId(shop.getShopId());
        if (promotions != null && promotions.getSeen() != null && promotions.getSeen() == 2 && userIdentity == 1) {
            promotions = null;
        }
        boolean hasGift = false;
        if (0 == organizeId&&userIdentity!=7) {
            if (1 == cartItemVo.getSvipProductFlag()) {
                // 超级会员优惠商品,不参与促销活动（普通机构购买该商品可参与店铺促销）/阶梯价/复购价
                // 超级会员设置商品优惠价
                productService.setSvipProductPrice(cartItemVo, taxFlag, svipUserFlag);
                if (svipUserFlag) {
                    // 超级会员设置超级会员优惠总额 + 商品优惠金额
                    svipReducedPrice.set(MathUtil.add(svipReducedPrice, cartItemVo.getSvipTotalReducedPrice()).doubleValue());
                }
            } else {
                // 没有店铺促销时，商品促销才有效
                if (null == promotions) {
                    // 获取商品促销信息
                    promotions = baseMapper.getPromotionByProductId(cartItemVo.getProductId());
                    //协销不可见
                    if (promotions != null && promotions.getSeen() != null && promotions.getSeen() == 2 && userIdentity == 1) {
                        promotions = null;
                    }
                } else {
                    shop.setPromotions(promotions);
                }
                /*
                 * 设置商品促销优惠
                 */
                if (null != promotions) {
                    // 当前促销活动的价格计算列表
                    List<PromotionPriceVo> promotionPriceList = productService.getPromotionProducts(promotions, cartItemVo, taxFlag);
                    promotions.setProductList(promotionPriceList);
                    Double totalAmount = MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice()).doubleValue();
                    Double floor = Math.floor(MathUtil.div(totalAmount, promotions.getTouchPrice()).doubleValue());
                    // 满足促销条件
                    if (MathUtil.compare(totalAmount, promotions.getTouchPrice()) >= 0) {
                        // 满减
                        if (promotions.getMode() == 2) {
                            // 总价 - 满减金额
                            if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                                totalPrice.set(MathUtil.sub(totalPrice.get(), MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                // 优惠总额 + 满减金额
                                reducedPrice.set(MathUtil.add(reducedPrice.get(), MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                            } else {
                                totalPrice.set(MathUtil.sub(totalPrice.get(), promotions.getReducedPrice()).doubleValue());
                                // 优惠总额 + 满减金额
                                reducedPrice.set(MathUtil.add(reducedPrice.get(), promotions.getReducedPrice()).doubleValue());
                            }
                        } else if (promotions.getMode() == 3) {
                            // 满足满赠
                            hasGift = true;
                            List<CartItemVo> giftList = baseMapper.getPromotionGifts(promotions.getId());
                            //满足叠加
                            if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                                Integer in = floor.intValue();
                                giftList.forEach(g -> g.setNumber(MathUtil.mul(g.getNumber(), in).intValue()));
                            }
                            promotions.setGiftList(giftList);
                        }
                    }
                    // 添加到总促销
                    totalPromotions.add(promotions);
                    // 商品处于活动状态
                    cartItemVo.setActStatus(1);
                    // 关闭阶梯价格，活动优先
                    cartItemVo.setLadderFlag(0);
                    cartItemVo.setPromotions(promotions);
                } else {
                    cartItemVo.setActStatus(0);
                    if (cartItemVo.getLadderFlag() == 1) {
                        // 设置阶梯价
                        productService.setCartLadderPrices(cartItemVo, taxFlag);
                    }
                    /*else {
                        // 复购价
                        Double repurchase = baseMapper.getRepurchasePrice(cartItemVo.getSkuId(), cartDto.getUserId());
                        if (null != repurchase && repurchase > 0) {
                            if (taxFlag) {
                                BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(repurchase, cartItemVo.getTaxRate()), 100, 2);
                                cartItemVo.setPrice(MathUtil.add(repurchase, cartItemTax).doubleValue());
                            } else {
                                cartItemVo.setPrice(repurchase);
                            }
                        }
                    }*/
                }
            }
        }
        // 计算运费
        Double coldChain = 0d;
        if (null != cartDto.getCityId()) {
            // 是否包含需要冷链运输商品 1 冷链费商品 2 不含冷链费商品 3 包含冷链费商品
            if (cartDto.getProductId() == 7881) {
                shop.setDesignatedFlag(1);
            } else {
                shop.setDesignatedFlag(2);
            }
            List<String> skuIds = new ArrayList<>();
            skuIds.add(cartDto.getSkuId().toString());
            Map<String, Object> postageMap = productService.computePostage(cartDto.getUserId(), cartDto.getCityId(), cartItemVo.getNumber(), MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice()).doubleValue(), skuIds);
            Integer postageFlag = (Integer) postageMap.get("postageFlag");
            if (999 == postageFlag) {
                return ResponseJson.error(-1,"商品运费规则异常",null);
            } else {
                shop.setPostageFlag(postageFlag);
                shop.setPostage((Double) postageMap.get("postage"));
                shop.setUserBeans((Integer) postageMap.get("userBeans"));
            }

            if (7881 == cartItemVo.getProductId()) {
                if (cartItemVo.getNumber() >= 100) {
                    coldChain = 0d;
                } else {
                    coldChain = baseMapper.getFreightAmount(cartItemVo.getProductId());
                    // shop.setPostageFlag(1);
                    // totalPrice.set(MathUtil.add(totalPrice.get(), coldChain).doubleValue());
                    if (1 == postageFlag) {
                        shop.setIsColdChina(true);
                    }
                }
            } else {
                shop.setIsColdChina(false);
            }
            if (1 == postageFlag) {
                shop.setColdChain(coldChain);
            } else {
                shop.setColdChain(0d);
            }
        }
        // 只有冷链费商品 运费为0
        /*if (null != cartDto.getTownId()) {
            if (1 == shop.getDesignatedFlag()) {
                shop.setPostage(0d);
            }
        }*/
        // 总运费
        // 包邮或者到付 冷链费为 0
        coldChain = 0 == shop.getPostage() ? 0 : coldChain;
        shop.setPostage(MathUtil.sub(shop.getPostage(), coldChain).doubleValue());
        // 供应商总金额
        totalPrice.set(MathUtil.add(MathUtil.add(totalPrice.get(), shop.getPostage()), coldChain).doubleValue());
        // 商品总金额累加
        originalPrice.set(MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getOriginalPrice()).doubleValue());
        totalPrice.set(MathUtil.add(totalPrice.get(), MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
        // 供应商商品
        List<CartItemVo> cartList = new ArrayList<>(1);
        cartList.add(cartItemVo);
        shop.setCartList(cartList);
        shop.setCount(1);
        // 供应商总价
        shop.setTotalPrice(totalPrice.get());
        // 供应商总优惠
        shop.setReducedPrice(reducedPrice.get());
        // 供应商划线价
        shop.setOriginalPrice(originalPrice.get());
        // svip优惠金额
        shop.setSvipReducedPrice(svipReducedPrice.get());
        // 当前供应商添加到首位
        shopList.add(0, shop);
        // 满足促销满赠
        if (hasGift) {
            List<CartItemVo> giftList = baseMapper.getPromotionGifts(promotions.getId());
            Double totalAmount = MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice()).doubleValue();
            Double floor = Math.floor(MathUtil.div(totalAmount, promotions.getTouchPrice()).doubleValue());
            //满足叠加
            if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                Integer in = floor.intValue();
                giftList.forEach(g -> g.setNumber(MathUtil.mul(g.getNumber(), in).intValue()));
            }
            promotions.setGiftList(giftList);

            promotions.getGiftList().forEach(gift -> {
                if (shopIds.contains(gift.getShopId())) {
                    // 赠品在当前订单内的供应商下
                    shopList.forEach(s -> {
                        if (s.getShopId().equals(gift.getShopId())) {
                            s.getCartList().add(gift);
                        }
                    });
                } else {
                    // 获取赠品供应商
                    CartShopVo giftShop = baseMapper.getShopByProductId(gift.getProductId());
                    shopIds.add(giftShop.getShopId());
                    giftShop.setCartList(new ArrayList<>());
                    giftShop.getCartList().add(gift);
                    shopList.add(giftShop);
                }
            });
        }
        // 是否充值商品
        boolean includeRecharge = false;
        boolean recharge = productService.isRechargeProduct(cartDto.getProductId());
        if (recharge) {
            includeRecharge = true;
        }
        // 发票信息
        InvoiceVo invoice = baseMapper.getUserInvoice(cartDto.getUserId());
        // 可用余额
        Double availableMoney = baseMapper.getAbleUserMoney(cartDto.getUserId());
        // 可用采美豆
        Integer userBeans = baseMapper.getUserBeans(cartDto.getUserId());
        // 获取用户可用优惠券
        List<CouponVo> couponList = getUserCartCoupons(cartList, cartDto.getSource(), cartDto.getUserId());
        //订单中存在不能退货的商品则给出不能退货提示
        boolean flag = false;
        String helpContent = null;
        if (null != shopList && shopList.size() > 0) {
            for (CartShopVo so : shopList) {
                List<CartItemVo> carList = so.getCartList();
                if (null != carList && carList.size() > 0) {
                    for (CartItemVo op : carList) {
                        if (null != op.getReturnGoodsStutas() && op.getReturnGoodsStutas() == 2) {
                            flag = true;
                            helpContent = op.getHelpContent();
                        }
                    }
                }
            }
        }

        // 包装返回数据
        Map<String, Object> resultData = new HashMap<>();
        resultData.put("list", shopList);
        resultData.put("kindCount", 1);
        resultData.put("totalCount", cartDto.getProductCount());
        resultData.put("totalPrice", totalPrice.get());
        resultData.put("reducedPrice", reducedPrice.get());
        resultData.put("svipReducedPrice", svipReducedPrice.get());
        resultData.put("totalOriginalPrice", originalPrice.get());
        resultData.put("invoice", invoice);
        resultData.put("userMoney", availableMoney);
        resultData.put("userBeans", userBeans);
        resultData.put("includeRecharge", includeRecharge);
        resultData.put("promotions", totalPromotions);
        resultData.put("couponList", couponList);
        // 联合丽格冷链费
        if (0 != organizeId) {
            resultData.put("coldChain", coldChain);
        }
        if (flag) {
            resultData.put("returnGoodsStutas", 2);
            resultData.put("helpContent", helpContent);
        } else {
            resultData.put("returnGoodsStutas", 1);
            resultData.put("helpContent", null);
        }
        return ResponseJson.success(resultData);
    }

    private List<CouponVo> getUserCartCoupons(List<CartItemVo> cartList, Integer source, Integer userId) {
        // 用户可用优惠券（总）
        List<CouponVo> couponList = orderCommonMapper.getClubCouponList(userId);
        Iterator<CouponVo> iterator = couponList.iterator();
        while (iterator.hasNext()) {
            CouponVo coupon = iterator.next();
            BigDecimal totalAmount = BigDecimal.ZERO;
            Integer couponType = coupon.getCouponType();
            for (CartItemVo item : cartList) {
                if (couponType == 0) {
                    if (coupon.getProductType() == 1) {
                        //活动券-全商城商品
                        BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                        totalAmount = MathUtil.add(price, totalAmount);
                    } else if (coupon.getProductType() == 2) {
                        //活动券-指定商品
                        List<Integer> couponProductIds = orderCommonMapper.getCouponProductIds(coupon.getCouponId(), source);
                        if (couponProductIds.contains(item.getProductId())) {
                            BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                            totalAmount = MathUtil.add(price, totalAmount);
                        }
                    }
                } else if (couponType == 1) {
                    //品类券
                    if (coupon.getCategoryType().equals(item.getCommodityType())) {
                        BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                        totalAmount = MathUtil.add(price, totalAmount);
                    }
                } else if (couponType == 2 || couponType == 4) {
                    //用户专享券或新用户券
                    BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                    totalAmount = MathUtil.add(price, totalAmount);
                } else if (coupon.getCouponType() == 3 && coupon.getShopId() != null) {
                    //店铺券
                    String shopName = baseMapper.getShopNameById(coupon.getShopId());
                    if (StringUtils.isNotBlank(shopName)) {
                        if (shopName.length() > 7) {
                            shopName = shopName.substring(0, 6) + "...";
                        }
                        coupon.setShopName(shopName);
                    }
                    if (coupon.getShopId().equals(item.getShopId())) {
                        BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                        totalAmount = MathUtil.add(price, totalAmount);
                    }
                }
            }
            if (MathUtil.compare(totalAmount, coupon.getTouchPrice()) < 0) {
                iterator.remove();
            }
        }
        couponList.sort((o1, o2) -> o2.getCouponAmount().compareTo(o1.getCouponAmount()));
        return couponList;
    }

    /**
     * 获取结算商品运费
     *
     * @param userId 用户ID
     * @param skuIds skuIds,逗号隔开
     * @param townId 地区Id
     */
    @Override
    public ResponseJson<Map<String, Object>> getProductsPostage(Integer userId, String skuIds, Integer cityId) {
        // 商品Id信息
        List<String> skuIdList = new ArrayList<>();
        if (skuIds.contains(",")) {
            skuIdList = Stream.of(skuIds.split(",")).collect(Collectors.toList());
        } else {
            skuIdList.add(skuIds);
        }
        // 计算运费
        Map<String, Object> resultMap = productService.computePostage(userId, cityId, 0, 0d, skuIdList);
        return ResponseJson.success(resultMap);
    }

    /**
     * 获取用户发票信息
     *
     * @param userId 用户Id
     */
    @Override
    public ResponseJson<InvoiceVo> getUserInvoice(Integer userId) {
        InvoiceVo invoice = cartClubMapper.getUserInvoice(userId);
        return ResponseJson.success(invoice);
    }

    /**
     * 更新用户发票
     *
     * @param invoiceDto {
     *                   userId:用户Id
     *                   id:id
     *                   invoiceTitle:单位名，发票抬头
     *                   corporationTaxNum: 企业税号、纳税人识别号
     *                   registeredAddress:注册地址
     *                   registeredPhone:注册电话
     *                   bankAccountNo:开户银行账户
     *                   openBank:开户银行
     *                   }
     */
    @Override
    public ResponseJson<Integer> updateUserInvoice(InvoiceDto invoiceDto) {
        if (null == invoiceDto.getId()) {
            // 只保留一份用户发票信息
            cartClubMapper.deleteUserInvoice(invoiceDto.getUserId());
            cartClubMapper.insertUserInvoice(invoiceDto);
        } else {
            cartClubMapper.updateUserInvoice(invoiceDto);
        }
        return ResponseJson.success(1);
    }

    /**
     * 更新用户发票
     *
     * @param invoiceDto {
     *                   userId:用户Id
     *                   id:id
     *                   invoiceTitle:单位名，发票抬头
     *                   corporationTaxNum: 企业税号、纳税人识别号
     *                   registeredAddress:注册地址
     *                   registeredPhone:注册电话
     *                   bankAccountNo:开户银行账户
     *                   openBank:开户银行
     *                   }
     */
    @Override
    public ResponseJson<Integer> updateOrganizeUserInvoice(InvoiceOrganizeDto invoiceDto) {
        if (null == invoiceDto.getId()) {
            // 只保留一份用户发票信息
            cartClubMapper.deleteUserInvoice(invoiceDto.getUserId());
            cartClubMapper.insertOrganizeUserInvoice(invoiceDto);
        } else {
            cartClubMapper.updateOrganizeUserInvoice(invoiceDto);
        }
        return ResponseJson.success(1);
    }

    /**
     * 供应商优惠券
     *
     * @param userId 用户id
     * @param shopId 供应商id
     * @param source 来源 : 1 网站 ; 2 小程序
     * @param status 状态: 1未领取 2已领取
     */
    @Override
    public ResponseJson<Map<String, Object>> getShopCoupons(Integer userId, Integer shopId, Integer source, Integer status) {
        // 供应商下购物车商品列表
        List<CartItemVo> productList = cartClubMapper.getCartProductsByShopId(shopId, userId);
        // 用户注册时间
        Date registerTime = orderCommonMapper.getUserRegisterTime(userId);
        // 未领取的优惠券
        List<CouponVo> preCouponList = orderCommonMapper.getPrevCouponList(userId, registerTime);
        preCouponList.forEach(p -> p.setCouponBtnType(0));
        //剔除超级会员优惠券
        List<Integer> vipCoupon = cartClubMapper.findVipCoupon();
        if (preCouponList != null && preCouponList.size() > 0) {
            preCouponList.removeIf(couponVo -> vipCoupon.contains(couponVo.getCouponId()));
        }
        // 过滤与当前购物车商品无关的优惠券
        filterCoupon(source, productList, preCouponList);
        // 用户可用优惠券（已领取）
        List<CouponVo> couponList = orderCommonMapper.getClubCouponList(userId);
        couponList.forEach(c -> c.setCouponBtnType(1));
        //剔除超级会员优惠券
        if (couponList != null && couponList.size() > 0) {
            couponList.removeIf(couponVo -> vipCoupon.contains(couponVo.getCouponId()));
        }
        // 过滤与当前购物车商品无关的优惠券
        filterCoupon(source, productList, couponList);
        Map<String, Object> map = new HashMap<>(3);
        map.put("notCouponNum", preCouponList == null ? 0 : preCouponList.size());
        map.put("couponNum", couponList == null ? 0 : couponList.size());
        if (status == 1) {
            //未领取
            map.put("couponList", preCouponList);
        } else {
            //已领取
            map.put("couponList", couponList);
        }
        return ResponseJson.success(map);
    }

}
