package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.components.ProductService;
import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.mapper.CartSellerMapper;
import com.caimei365.order.mapper.OrderClubMapper;
import com.caimei365.order.mapper.OrderCommonMapper;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.AgainBuyDto;
import com.caimei365.order.model.dto.SellerCartDto;
import com.caimei365.order.model.po.SellerCartPo;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.CartSellerService;
import com.caimei365.order.utils.ImageUtil;
import com.caimei365.order.utils.MathUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.base.Joiner;
import com.google.common.collect.Lists;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static com.alibaba.fastjson.JSON.parseArray;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/29
 */
@Slf4j
@Service
public class CartSellerServiceImpl implements CartSellerService {
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private CartSellerMapper cartSellerMapper;
    @Resource
    private OrderCommonMapper orderCommonMapper;
    @Resource
    private ProductService productService;
    @Resource
    private OrderClubMapper orderClubMapper;

    /**
     * 协销购物车列表数据
     *
     * @param serviceProviderId  协销Id
     * @param clubId             机构Id
     * @param againBuyProductIds 再来一单商品skuIds
     * @param pageNum            页码
     * @param pageSize           每页数量
     */
    @Override
    public ResponseJson<Map<String, Object>> getSellerCartList(Integer serviceProviderId, Integer clubId, String againBuyProductIds, int pageNum, int pageSize) {
        List<Integer> againBuyIdList = Lists.newArrayList();
        if (StringUtils.isNotBlank(againBuyProductIds)) {
            if (againBuyProductIds.contains(",")) {
                String[] againBuyArr = againBuyProductIds.split(",");
                for (String id : againBuyArr) {
                    againBuyIdList.add(Integer.valueOf(id));
                }
            } else {
                againBuyIdList.add(Integer.valueOf(againBuyProductIds));
            }
        }
        // 失效商品列表（总）
        List<CartItemVo> invalidList = new ArrayList<>();
        // 促销活动（总）
        List<PromotionsVo> totalPromotions = new ArrayList<>();
        List<Integer> promotionsIds = new ArrayList<>();
        // 获取机构用户Id
        Integer clubUserId = baseMapper.getUserIdByClubId(clubId);
        // 超级会员标识
        Integer svipUserId = baseMapper.getSvipUserIdByClubId(clubId);
        boolean svipUserFlag = null != svipUserId;
        // 开始分页请求供应商数据
        PageHelper.startPage(pageNum, pageSize);
        // 协销购物车供应商列表
        List<CartShopVo> shopInfoList = cartSellerMapper.getSellerCartShops(serviceProviderId, clubId);
        if (null != shopInfoList && shopInfoList.size() > 0) {
            shopInfoList.removeIf(Objects::isNull);
            // 遍历供应商列表
            shopInfoList.forEach(shop -> {
                // 默认未选中状态（前端要求）
                shop.setIsChecked(false);
                // 该供应商下商品种类
                AtomicInteger shopKindCount = new AtomicInteger(0);
                // 该供应商总价
                AtomicDouble shopPrice = new AtomicDouble(0);
                // 该供应商下参与店铺促销的商品总价
                AtomicDouble shopPromotionFee = new AtomicDouble(0);
                // 该供应商满减金额（供应商满减，单品满减）
                AtomicDouble shopReducedPrice = new AtomicDouble(0);
                // 该供应商svip优惠金额
                AtomicDouble shopSvipReducedPrice = new AtomicDouble(0);
                // 供应商促销优惠活动
                PromotionsVo shopPromotion = baseMapper.getPromotionByShopId(shop.getShopId());
                if (shopPromotion != null && shopPromotion.getSeen() != null && shopPromotion.getSeen() == 2) {
                    shopPromotion = null;
                }
                // 供应商下商品列表
                List<CartItemVo> productList = cartSellerMapper.getSellerCartProducts(serviceProviderId, clubId, shop.getShopId());
                // 迭代器设置商品信息
                Iterator<CartItemVo> productIterator = productList.iterator();
                while (productIterator.hasNext()) {
                    CartItemVo cartItemVo = productIterator.next();
                    // 默认所有商品未选中状态（前端要求）
                    cartItemVo.setIsChecked(false);
                    // 设置skus
                    cartItemVo.setSkus(orderClubMapper.findSkus(cartItemVo.getProductId()));
                    if (cartItemVo.getValidFlag() == 0) {
                        // 后台逻辑删除,已停售
                        cartItemVo.setStatus(1);
                        invalidList.add(cartItemVo);
                    } else if (cartItemVo.getValidFlag() == 10) {
                        // 已冻结，已丢失
                        cartItemVo.setStatus(2);
                        invalidList.add(cartItemVo);
                    } else {
                        // 设置商品有效
                        cartItemVo.setStatus(0);
                        // 设置商品图片及税费
                        boolean taxFlag = productService.setCartItemImgAndTax(cartItemVo);
                        if (1 == cartItemVo.getSvipProductFlag()) {
                            // 超级会员优惠商品,不参与促销活动（普通机构购买该商品可参与店铺促销）/阶梯价/复购价
                            // 超级会员设置商品优惠价
                            productService.setSvipProductPrice(cartItemVo, taxFlag, svipUserFlag);
                            if (svipUserFlag) {
                                // 超级会员设置超级会员优惠总额 + 商品优惠金额
                                shopReducedPrice.set(MathUtil.add(shopReducedPrice, cartItemVo.getSvipTotalReducedPrice()).doubleValue());
                            }
                        } else {
                            // 获取商品促销信息
                            PromotionsVo promotions = null;
                            // 没有店铺促销时，商品促销才有效
                            if (null == shopPromotion) {
                                // 获取商品促销信息
                                promotions = baseMapper.getPromotionByProductId(cartItemVo.getProductId());
                                // 促销活动如果协销不可见直接置空
                                if (promotions != null && promotions.getSeen() != null && promotions.getSeen() == 2) {
                                    promotions = null;
                                }
                                /*
                                 * 设置商品促销优惠
                                 */
                                if (null != promotions) {
                                    if (1 == promotions.getMode()) {
                                        promotions.setTouchPrice(baseMapper.getTouchPriceBySku(cartItemVo.getSkuId(), promotions.getId()));
                                    }
                                    BigDecimal totalAmount = MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice());
                                    //叠加倍数
                                    Double floor = Math.floor(MathUtil.div(totalAmount, promotions.getTouchPrice()).doubleValue());
                                    // 当前促销活动的价格计算列表
                                    List<PromotionPriceVo> promotionPriceList = productService.getPromotionProducts(promotions, cartItemVo, taxFlag);
                                    // 更新到总促销列表
                                    productService.updateTotalPromotions(totalPromotions, promotionsIds, promotions, promotionPriceList, floor);
                                    //单品满减-计算供应商总价/满减金额
                                    if (promotions.getType() == 1 && promotions.getMode() == 2) {
                                        if (MathUtil.compare(totalAmount, promotions.getTouchPrice()) > -1) {
                                            if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                                                //叠加优惠计算
                                                shopPrice.set(MathUtil.sub(shopPrice.get(), MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                                shopReducedPrice.set(MathUtil.add(shopReducedPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                            } else {
                                                // 如果满足促销条件，设置供应商价格-满减金额，满减总额 + 当前促销满减金额
                                                shopPrice.set(MathUtil.sub(shopPrice.get(), promotions.getReducedPrice()).doubleValue());
                                                shopReducedPrice.set(MathUtil.add(shopReducedPrice, promotions.getReducedPrice()).doubleValue());
                                            }
                                        }
                                    }
                                    cartItemVo.setPromotions(promotions);
                                }
                            }
                            if (null != promotions || null != shopPromotion) {
                                // 商品处于活动状态
                                cartItemVo.setActStatus(1);
                                // 关闭阶梯价格，活动优先
                                cartItemVo.setLadderFlag(0);
                            } else {
                                if (cartItemVo.getLadderFlag() == 1) {
                                    // 设置阶梯价
                                    productService.setCartLadderPrices(cartItemVo, taxFlag);
                                }
                                /*else {
                                    // 复购价
                                    Double repurchase = baseMapper.getRepurchasePrice(cartItemVo.getSkuId(), clubUserId);
                                    if (null != repurchase && repurchase > 0) {
                                        if (taxFlag) {
                                            BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(repurchase, cartItemVo.getTaxRate()), 100, 2);
                                            cartItemVo.setPrice(MathUtil.add(repurchase, cartItemTax).doubleValue());
                                        } else {
                                            cartItemVo.setPrice(repurchase);
                                        }
                                    }
                                }*/
                            }
                        }
                        // 再来一单的商品前端默认勾选
                        if (againBuyIdList.contains(cartItemVo.getProductId())) {
                            cartItemVo.setIsChecked(true);
                        }
                        // 该供应商下价格累加
                        shopPrice.set(MathUtil.add(shopPrice, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                        // 该供应商下可参与店铺促销的商品价格累加
                        if (!(1 == cartItemVo.getSvipProductFlag() && svipUserFlag)) {
                            // 超级会员购买svip商品不享受店铺促销优惠
                            shopPromotionFee.set(MathUtil.add(shopPromotionFee, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                        }
                        // 该供应商下商品种类 +1
                        shopKindCount.incrementAndGet();
                    }
                }
                // 店铺促销
                if (null != shopPromotion) {
                    shop.setPromotions(shopPromotion);
                    if (!promotionsIds.contains(shopPromotion.getId())) {
                        promotionsIds.add(shopPromotion.getId());
                        Double floor = Math.floor(MathUtil.div(shopPromotionFee, shopPromotion.getTouchPrice()).doubleValue());
                        // 店铺满赠
                        if (shopPromotion.getMode() == 3) {
                            // 获取赠品
                            List<CartItemVo> giftList = baseMapper.getPromotionGifts(shopPromotion.getId());
                            // 叠加赠品
                            if (shopPromotion.getDiscount() != null && shopPromotion.getDiscount() == 1 && floor > 1) {
                                Integer in = floor.intValue();
                                giftList.forEach(g -> g.setNumber(MathUtil.mul(g.getNumber(), in).intValue()));
                            }
                            shopPromotion.setGiftList(giftList);
                        }
                        // 设置该优惠下的商品列表
                        List<PromotionPriceVo> promotionPriceList = new ArrayList<>();
                        productList.forEach(item -> {
                            PromotionPriceVo promotionPrice = new PromotionPriceVo();
                            promotionPrice.setProductId(item.getProductId());
                            promotionPrice.setNumber(item.getNumber());
                            promotionPrice.setPrice(item.getPrice());
                            promotionPriceList.add(promotionPrice);
                        });
                        shopPromotion.setProductList(promotionPriceList);
                        // 添加到总促销
                        totalPromotions.add(shopPromotion);
                        // 店铺满减-计算供应商总价/满减金额
                        if (shopPromotion.getMode() == 2 && MathUtil.compare(shopPrice, shopPromotion.getTouchPrice()) > -1) {
                            if (shopPromotion.getDiscount() != null && shopPromotion.getDiscount() == 1 && floor > 1) {
                                // 该供应商总价 - 满减金额
                                shopPrice.set(MathUtil.sub(shopPrice.get(), MathUtil.mul(shopPromotion.getReducedPrice(), floor)).doubleValue());
                                // 该供应商优惠总额 + 满减金额
                                shopReducedPrice.set(MathUtil.add(shopReducedPrice.get(), MathUtil.mul(shopPromotion.getReducedPrice(), floor)).doubleValue());
                            } else {
                                // 该供应商总价 - 满减金额
                                shopPrice.set(MathUtil.sub(shopPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                                // 该供应商优惠总额 + 满减金额
                                shopReducedPrice.set(MathUtil.add(shopReducedPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                            }
                        }
                    }
                }
                // 供应商商品
                shop.setCartList(productList);
                // 供应商总价
                shop.setTotalPrice(shopPrice.get());
                // 当有超级会员优惠时，取超级会员优惠
                Double shopReduction = MathUtil.compare(shopSvipReducedPrice, 0) > 0 ? shopSvipReducedPrice.get() : shopReducedPrice.get();
                // 供应商总优惠
                shop.setReducedPrice(shopReducedPrice.get());
                // 供应商总超级会员优惠
                shop.setSvipReducedPrice(shopSvipReducedPrice.get());
                // 供应商划线价
                shop.setOriginalPrice(MathUtil.add(shopPrice.get(), shopReduction).doubleValue());
                // 供应商下商品种类
                shop.setCount(shopKindCount.get());
            });
            // 删除空数据
            shopInfoList.removeIf(shop -> (null == shop || shop.getCount() == 0));
        }
        if (null == shopInfoList) {
            shopInfoList = new ArrayList<>();
        }
        PageInfo<CartShopVo> pageInfo = new PageInfo(shopInfoList);
        /*
         * 返回结果
         */
        Map<String, Object> resultMap = new HashMap<>(3);
        resultMap.put("pageDate", pageInfo);
        resultMap.put("invalidProductList", invalidList);
        resultMap.put("promotionsList", totalPromotions);
        // 返回数据
        return ResponseJson.success(resultMap);
    }

    /**
     * 协销购物车列表数据 -- 组织
     *
     * @param serviceProviderId 协销Id
     * @param clubId            机构Id
     * @param pageNum           页码
     * @param pageSize          每页数量
     */
    @Override
    public ResponseJson<Map<String, Object>> getSellerCartOrganizeList(Integer serviceProviderId, Integer clubId, int pageNum, int pageSize) {
        List<Integer> againBuyIdList = Lists.newArrayList();
        // 失效商品列表（总）
        List<CartItemVo> invalidList = new ArrayList<>();
        // 开始分页请求供应商数据
        PageHelper.startPage(pageNum, pageSize);
        log.info("serviceProviderId, clubId"+serviceProviderId+"---"+clubId);
        // 协销购物车供应商列表
        List<CartShopVo> shopInfoList = cartSellerMapper.getSellerCartShops(serviceProviderId, clubId);
        if (null != shopInfoList && shopInfoList.size() > 0) {
            shopInfoList.removeIf(Objects::isNull);
            // 遍历供应商列表
            shopInfoList.forEach(shop -> {
                // 默认未选中状态（前端要求）
                shop.setIsChecked(false);
                // 该供应商下商品种类
                AtomicInteger shopKindCount = new AtomicInteger(0);
                // 该供应商总价
                AtomicDouble shopPrice = new AtomicDouble(0);

                // 供应商下商品列表
                List<CartItemVo> productList = cartSellerMapper.getSellerCartOrganizeProducts(serviceProviderId, clubId, shop.getShopId());
                // 迭代器设置商品信息
                Iterator<CartItemVo> productIterator = productList.iterator();
                while (productIterator.hasNext()) {
                    CartItemVo cartItemVo = productIterator.next();
                    // 设置商品图片及税费
                    productService.setCartItemImgAndTax(cartItemVo);
                    // 默认所有商品未选中状态（前端要求）
                    cartItemVo.setIsChecked(false);
                    // 设置skus
                    cartItemVo.setSkus(orderClubMapper.getOrganizeSku(cartItemVo.getProductId()));
                    if (cartItemVo.getValidFlag() != 2) {
                        // 后台逻辑删除,已停售
                        cartItemVo.setStatus(1);
                        invalidList.add(cartItemVo);
                    } else {
                        // 设置商品有效
                        cartItemVo.setStatus(0);

                        // 再来一单的商品前端默认勾选
                        if (againBuyIdList.contains(cartItemVo.getProductId())) {
                            cartItemVo.setIsChecked(true);
                        }
                        // 该供应商下商品种类 +1
                        shopKindCount.incrementAndGet();
                    }
                }
                // 供应商商品
                shop.setCartList(productList);
                // 供应商总价
                shop.setTotalPrice(shopPrice.get());
                // 供应商下商品种类
                shop.setCount(shopKindCount.get());
            });
            // 删除空数据
            shopInfoList.removeIf(shop -> (null == shop || shop.getCount() == 0));
        }
        if (null == shopInfoList) {
            shopInfoList = new ArrayList<>();
        }
        PageInfo<CartShopVo> pageInfo = new PageInfo(shopInfoList);
        /*
         * 返回结果
         */
        Map<String, Object> resultMap = new HashMap<>(2);
        resultMap.put("pageDate", pageInfo);
        resultMap.put("invalidProductList", invalidList);
        // 返回数据
        return ResponseJson.success(resultMap);
    }

    /**
     * 获取协销购物车商品种类
     *
     * @param serviceProviderId 协销Id
     * @param clubId            机构Id
     */
    @Override
    public ResponseJson<Integer> getSellerCartCount(Integer serviceProviderId, Integer clubId) {
        Integer count = cartSellerMapper.getSellerCartCount(serviceProviderId, clubId);
        if (null == count) {
            count = 0;
        }
        return ResponseJson.success(count);
    }

    /**
     * 协销添加购物车
     *
     * @param sellerCartDto {
     *                      serviceProviderId 协销Id
     *                      clubId       机构ID
     *                      type         商品类型：1单一商品，2组合商品
     *                      skuId        单一商品skuId
     *                      productCount 单一商品数量
     *                      productInfo  组合商品信息：Json字符串格式[{"skuId":4351,"count":1},{}]
     *                      }
     */
    @Override
    public ResponseJson<Integer> addShoppingCart(SellerCartDto sellerCartDto) {
        if (1 == sellerCartDto.getType()) {
            // 单一商品
            addSellerCart(sellerCartDto);
        } else if (2 == sellerCartDto.getType()) {
            try {
                JSONArray productArr = parseArray(sellerCartDto.getProductInfo());
                for (Object o : productArr) {
                    JSONObject product = (JSONObject) o;
                    Integer skuId = product.getInteger("skuId");
                    Integer productCount = product.getInteger("count");
                    sellerCartDto.setSkuId(skuId);
                    sellerCartDto.setProductCount(productCount);
                    addSellerCart(sellerCartDto);
                }
            } catch (Exception e) {
                log.error("组合商品信息格式错误", e);
                return ResponseJson.error("组合商品信息格式错误！Json字符串格式[{\"id\":4351,\"count\":1},{}]", null);
            }
        }
        // 返回购物车种类数量
        return getSellerCartCount(sellerCartDto.getServiceProviderId(), sellerCartDto.getClubId());
    }

    /**
     * 添加购物车，插入数据库
     *
     * @param sellerCartDto SellerCartDto
     */
    private void addSellerCart(SellerCartDto sellerCartDto) {
        SellerCartPo cart = cartSellerMapper.getSellerCart(sellerCartDto);

        if (cart != null) {
            // 购物车已存在该商品，更新数量
            cart.setNum(cart.getNum() + sellerCartDto.getProductCount());
            cart.setAddTime(new Date());
            cartSellerMapper.updateSellerCart(cart);
        } else {
            // 添加新购物车
            Integer shopId = baseMapper.getShopIdByproductId(sellerCartDto.getSkuId());
            cart = new SellerCartPo();
            cart.setServiceProviderId(sellerCartDto.getServiceProviderId());
            cart.setShopId(shopId);
            cart.setClubId(sellerCartDto.getClubId());
            cart.setProductId(sellerCartDto.getProductId());
            cart.setNum(sellerCartDto.getProductCount());
            cart.setAddTime(new Date());
            cart.setSkuId(sellerCartDto.getSkuId());
            cartSellerMapper.insertSellerCart(cart);
        }
    }

    /**
     * 协销更新购物车
     *
     * @param sellerCartDto {
     *                      serviceProviderId 协销Id
     *                      id           购物车Id
     *                      productCount 商品数量
     *                      }
     */
    @Override
    public ResponseJson<Integer> updateSellerCart(SellerCartDto sellerCartDto) {
        SellerCartPo cart = cartSellerMapper.getSellerCartById(sellerCartDto.getId());
        if (!sellerCartDto.getServiceProviderId().equals(cart.getServiceProviderId())) {
            return ResponseJson.error("协销Id不正确！", null);
        }
        cart.setNum(sellerCartDto.getProductCount());
        cart.setAddTime(new Date());
        cartSellerMapper.updateSellerCart(cart);
        return ResponseJson.success(0);
    }

    /**
     * 协销删除购物车
     *
     * @param sellerCartDto {
     *                      serviceProviderId 协销Id
     *                      cartIds           购物车IDs
     *                      }
     */
    @Override
    public ResponseJson<Integer> deleteSellerCart(SellerCartDto sellerCartDto) {
        List<Integer> cartIdList = Lists.newArrayList();
        if (sellerCartDto.getCartIds().contains(",")) {
            String[] productArr = sellerCartDto.getCartIds().split(",");
            for (String id : productArr) {
                cartIdList.add(Integer.parseInt(id));
            }
        } else {
            cartIdList.add(Integer.parseInt(sellerCartDto.getCartIds()));
        }
        cartSellerMapper.deleteSellerCartByIds(sellerCartDto.getServiceProviderId(), cartIdList);
        return ResponseJson.success(0);
    }

    /**
     * 协销购物车结算
     *
     * @param serviceProviderId 协销Id
     * @param clubId            机构Id
     * @param skuIds            skuIds
     */
    @Override
    public ResponseJson<Map<String, Object>> settlementSellerCart(Integer serviceProviderId, Integer cityId, Integer clubId, String skuIds) {
        log.info("￥￥￥￥￥￥￥￥￥￥ > 商品结算(协销帮机构下单)");
        // 商品种类
        AtomicInteger kindCount = new AtomicInteger(0);
        // 商品总数量
        AtomicInteger totalCount = new AtomicInteger(0);
        // 统计商品总金额
        AtomicDouble totalPrice = new AtomicDouble(0);
        // 统计总促销满减
        AtomicDouble reducedPrice = new AtomicDouble(0);
        // 统计总划线价
        AtomicDouble totalOriginalPrice = new AtomicDouble(0);
        // 是否包含充值商品，默认false
        AtomicBoolean includeRecharge = new AtomicBoolean(false);
        //所有的商品数据
        List<CartItemVo> totalProductList = new ArrayList<>();
        // 促销活动（总）
        List<PromotionsVo> totalPromotions = new ArrayList<>();
        // 促销活动Id集合，用于合并促销活动
        List<Integer> promotionsIds = new ArrayList<>();
        // 供应商Id集合，用户判断赠品供应商是否在当前供应商中
        List<Integer> shopIds = new ArrayList<>();
        // 运费模板是否异常
        AtomicInteger templateFlag = new AtomicInteger(0);
        // 前端接收商品Id信息
        List<String> skuIdList = new ArrayList<>();
        if (skuIds.contains(",")) {
            skuIdList = Stream.of(skuIds.split(",")).collect(Collectors.toList());
        } else {
            skuIdList.add(skuIds);
        }
        // 获取机构用户Id
        Integer clubUserId = baseMapper.getUserIdByClubId(clubId);
        // 超级会员标识
        Integer svipUserId = baseMapper.getSvipUserIdByClubId(clubId);
        boolean svipUserFlag = null != svipUserId;
        // 供应商列表
        List<CartShopVo> shopList = cartSellerMapper.getCartShopsByProductIds(serviceProviderId, clubId, skuIdList);
        if (null != shopList && shopList.size() > 0) {
            // 遍历供应商列表
            List<String> finalIdList = skuIdList;
            shopList.forEach(shop -> {
                // 该供应商下商品种类
                AtomicInteger shopKindCount = new AtomicInteger(0);
                // 该供应商总价
                AtomicDouble shopPrice = new AtomicDouble(0);
                // 该供应商下参与店铺促销的商品总价
                AtomicDouble shopPromotionFee = new AtomicDouble(0);
                // 该供应商满减金额（供应商满减，单品满减）
                AtomicDouble shopReducedPrice = new AtomicDouble(0);
                // 该供应商svip优惠金额
                AtomicDouble shopSvipReducedPrice = new AtomicDouble(0);
                // 运费方式 0 包邮 1 到付 2 运费模板
                List<Integer> shopPostageFlag = new ArrayList<>();
                // 运费
                AtomicDouble shopPostage = new AtomicDouble(0);
                // 供应商促销优惠活动
                PromotionsVo shopPromotion = baseMapper.getPromotionByShopId(shop.getShopId());
                if (shopPromotion != null && shopPromotion.getSeen() != null && shopPromotion.getSeen() == 2) {
                    shopPromotion = null;
                }
                // 供应商下商品列表
                List<CartItemVo> productList = cartSellerMapper.getCartProductsByProductIds(serviceProviderId, clubId, shop.getShopId(), finalIdList);

                // 冷链费
                AtomicDouble coldChain = new AtomicDouble(0);
                if (null != productList && productList.size() > 0) {
                    // 是否包含需要冷链运输商品
                    List<Integer> collect = productList.stream().map(CartItemVo::getProductId).collect(Collectors.toList());
                    if (collect.stream().allMatch(c -> c == 7881)) {
                        shop.setDesignatedFlag(1);
                    } else if (!collect.contains(7881)) {
                        shop.setDesignatedFlag(2);
                    } else {
                        shop.setDesignatedFlag(3);
                    }
                    for (CartItemVo cartItemVo : productList) {
                        if (7881 == cartItemVo.getProductId()) {
                            if (cartItemVo.getNumber() >= 100) {
                                coldChain.set(0);
                            } else {
                                coldChain.set(baseMapper.getFreightAmount(cartItemVo.getProductId()));
                                // shop.setPostageFlag(1);
                            }
                        }
                        // 设置商品图片及税费
                        boolean taxFlag = productService.setCartItemImgAndTax(cartItemVo);
                        // 是否充值商品
                        boolean recharge = productService.isRechargeProduct(cartItemVo.getProductId());
                        if (recharge) {
                            includeRecharge.set(true);
                        }
                        if (1 == cartItemVo.getSvipProductFlag()) {
                            // 超级会员优惠商品,不参与促销活动（普通机构购买该商品可参与店铺促销）/阶梯价/复购价
                            // 超级会员设置商品优惠价
                            productService.setSvipProductPrice(cartItemVo, taxFlag, svipUserFlag);
                            if (svipUserFlag) {
                                // 超级会员设置超级会员优惠总额 + 商品优惠金额
                                shopReducedPrice.set(MathUtil.add(shopReducedPrice, cartItemVo.getSvipTotalReducedPrice()).doubleValue());
                            }
                        } else {
                            // 获取商品促销信息
                            PromotionsVo promotions = null;
                            // 没有店铺促销时，商品促销才有效
                            if (null == shopPromotion) {
                                // 获取商品促销信息
                                promotions = baseMapper.getPromotionByProductId(cartItemVo.getProductId());
                                // 手动获取skuId优惠价
                                if (promotions != null && (promotions.getTouchPrice() == null || promotions.getTouchPrice() == 0d)) {
                                    Double touchPrice = baseMapper.getTouchPriceBySku(cartItemVo.getSkuId(),promotions.getId());
                                    promotions.setTouchPrice(touchPrice);
                                }
                                // 促销活动如果协销不可见直接置空
                                if (promotions != null && promotions.getSeen() != null && promotions.getSeen() == 2) {
                                    promotions = null;
                                }
                                /*
                                 * 设置商品促销优惠
                                 */
                                if (null != promotions) {
                                    BigDecimal totalAmount = MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice());
                                    //叠加倍数
                                    Double floor = Math.floor(MathUtil.div(totalAmount, promotions.getTouchPrice()).doubleValue());
                                    // 当前促销活动的价格计算列表
                                    List<PromotionPriceVo> promotionPriceList = productService.getPromotionProducts(promotions, cartItemVo, taxFlag);
                                    // 更新到总促销列表
                                    if (MathUtil.compare(totalAmount, promotions.getTouchPrice()) > -1) {
                                        productService.updateTotalPromotions(totalPromotions, promotionsIds, promotions, promotionPriceList, floor);
                                    }
                                    //单品满减-计算供应商总价/满减金额
                                    if (promotions.getType() == 1 && promotions.getMode() == 2) {
                                        if (MathUtil.compare(totalAmount, promotions.getTouchPrice()) > -1) {
                                            if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                                                //叠加优惠计算
                                                shopPrice.set(MathUtil.sub(shopPrice.get(), MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                                shopReducedPrice.set(MathUtil.add(shopReducedPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                                            } else {
                                                // 如果满足促销条件，设置供应商价格-满减金额，满减总额 + 当前促销满减金额
                                                shopPrice.set(MathUtil.sub(shopPrice.get(), promotions.getReducedPrice()).doubleValue());
                                                shopReducedPrice.set(MathUtil.add(shopReducedPrice, promotions.getReducedPrice()).doubleValue());
                                            }
                                        }
                                    }
                                    cartItemVo.setPromotions(promotions);
                                }
                            }
                            if (null != promotions || null != shopPromotion) {
                                // 商品处于活动状态
                                cartItemVo.setActStatus(1);
                                // 关闭阶梯价格，活动优先
                                cartItemVo.setLadderFlag(0);
                            } else {
                                if (cartItemVo.getLadderFlag() == 1) {
                                    // 设置阶梯价
                                    productService.setCartLadderPrices(cartItemVo, taxFlag);
                                }
                                /*else {
                                    // 复购价
                                    Double repurchase = baseMapper.getRepurchasePrice(cartItemVo.getSkuId(), clubUserId);
                                    if (null != repurchase && repurchase > 0) {
                                        if (taxFlag) {
                                            BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(repurchase, cartItemVo.getTaxRate()), 100, 2);
                                            cartItemVo.setPrice(MathUtil.add(repurchase, cartItemTax).doubleValue());
                                        } else {
                                            cartItemVo.setPrice(repurchase);
                                        }
                                    }
                                }*/
                            }
                        }
                        // 该供应商下价格累加
                        shopPrice.set(MathUtil.add(shopPrice, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                        // 该供应商下可参与店铺促销的商品价格累加
                        if (!(1 == cartItemVo.getSvipProductFlag() && svipUserFlag)) {
                            // 超级会员购买svip商品不享受店铺促销优惠
                            shopPromotionFee.set(MathUtil.add(shopPromotionFee, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                        }
                        // 设置商品运费
                        if (null != cityId) {
                            List<String> cartSkuIds = new ArrayList<>();
                            cartSkuIds.add(cartItemVo.getSkuId().toString());
                            Map<String, Object> postageMap = productService.computePostage(clubUserId, cityId, cartItemVo.getNumber(), MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice()).doubleValue(), cartSkuIds);
                            Integer postageFlag = (Integer) postageMap.get("postageFlag");
                            shopPostageFlag.add(postageFlag);
                            if (999 == postageFlag) {
                                templateFlag.set(-1);
                            } else {
                                Double postage = (Double) postageMap.get("postage");
                                shopPostage.set(MathUtil.add(shopPostage.get(), postage).doubleValue());
                            }
                            shop.setUserBeans((Integer) postageMap.get("userBeans"));
                        }
                        // 该供应商下商品种类 +1
                        shopKindCount.incrementAndGet();
                        // 总数量 + 当前商品购买数量
                        totalCount.updateAndGet(v -> v + cartItemVo.getNumber());
                    }
                    // 运费方式
                    if (shopPostageFlag.contains(1)) {
                        shop.setPostageFlag(1);
                    } else if (!shopPostageFlag.contains(1) && shopPostageFlag.contains(2)) {
                        shop.setPostageFlag(2);
                    } else {
                        shop.setPostageFlag(0);
                    }
                    shop.setPostage(shopPostage.get());

                    // 只有冷链费商品 运费为0
                    /*if (null != townId) {
                        if (1 == shop.getDesignatedFlag()) {
                            shop.setPostage(0d);
                        }
                    }*/
                    // 总运费
                    // 包邮或者到付 冷链费为 0
                    coldChain.set(0 == shop.getPostage() ? 0 : coldChain.get());
                    shop.setColdChain(coldChain.get());
                    shop.setPostage(MathUtil.sub(shop.getPostage(),coldChain.get()).doubleValue());
                    if (0 != coldChain.get()) {
                        shop.setIsColdChina(true);
                    } else {
                        shop.setIsColdChina(false);
                    }
                    // 供应商总金额
                    shopPrice.set(MathUtil.add(MathUtil.add(shopPrice.get(), shop.getPostage()), coldChain.get()).doubleValue());
                    if (shopKindCount.get() > 0) {
                        // 店铺促销
                        if (null != shopPromotion) {
                            shop.setPromotions(shopPromotion);
                            if (!promotionsIds.contains(shopPromotion.getId())) {
                                promotionsIds.add(shopPromotion.getId());
                                // 店铺满赠
                                Double floor = Math.floor(MathUtil.div(shopPromotionFee, shopPromotion.getTouchPrice()).doubleValue());
                                if (shopPromotion.getMode() == 3) {
                                    // 获取赠品
                                    List<CartItemVo> giftList = baseMapper.getPromotionGifts(shopPromotion.getId());
                                    // 叠加赠品
                                    if (shopPromotion.getDiscount() != null && shopPromotion.getDiscount() == 1 && floor > 1) {
                                        Integer in = floor.intValue();
                                        giftList.forEach(g -> g.setNumber(MathUtil.mul(g.getNumber(), in).intValue()));
                                    }
                                    shopPromotion.setGiftList(giftList);
                                }
                                // 设置该优惠下的商品列表
                                List<PromotionPriceVo> promotionPriceList = new ArrayList<>();
                                productList.forEach(item -> {
                                    if (!(1 == item.getSvipProductFlag() && svipUserFlag)) {
                                        // 超级会员购买svip商品不享受店铺促销优惠
                                        PromotionPriceVo promotionPrice = new PromotionPriceVo();
                                        promotionPrice.setProductId(item.getProductId());
                                        promotionPrice.setNumber(item.getNumber());
                                        promotionPrice.setPrice(item.getPrice());
                                        promotionPriceList.add(promotionPrice);
                                    }
                                });
                                shopPromotion.setProductList(promotionPriceList);
                                // 添加到总促销
                                totalPromotions.add(shopPromotion);
                                // 店铺满减-计算供应商总价/满减金额
                                if (shopPromotion.getMode() == 2 && MathUtil.compare(shopPromotionFee, shopPromotion.getTouchPrice()) > -1) {
                                    if (shopPromotion.getDiscount() != null && shopPromotion.getDiscount() == 1 && floor > 1) {
                                        // 该供应商总价 - 满减金额
                                        shopPrice.set(MathUtil.sub(shopPrice.get(), MathUtil.mul(shopPromotion.getReducedPrice(), floor)).doubleValue());
                                        // 该供应商优惠总额 + 满减金额
                                        shopReducedPrice.set(MathUtil.add(shopReducedPrice.get(), MathUtil.mul(shopPromotion.getReducedPrice(), floor)).doubleValue());
                                    } else {
                                        // 该供应商总价 - 满减金额
                                        shopPrice.set(MathUtil.sub(shopPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                                        // 该供应商优惠总额 + 满减金额
                                        shopReducedPrice.set(MathUtil.add(shopReducedPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                                    }
                                }
                            }
                        }
                        // 供应商商品
                        shop.setCartList(productList);
                        // 供应商总价
                        shop.setTotalPrice(shopPrice.get());
                        // 当有超级会员优惠时，取超级会员优惠
                        Double shopReduction = MathUtil.compare(shopSvipReducedPrice, 0) > 0 ? shopSvipReducedPrice.get() : shopReducedPrice.get();
                        // 供应商总优惠
                        shop.setReducedPrice(shopReducedPrice.get());
                        // 供应商总超级会员优惠
                        shop.setSvipReducedPrice(shopSvipReducedPrice.get());
                        // 供应商划线价
                        shop.setOriginalPrice(MathUtil.sub(MathUtil.add(shopPrice.get(), shopReduction), shop.getPostage()).doubleValue());

                        // 添加供应商Id集合
                        shopIds.add(shop.getShopId());
                        // 计算总价
                        totalPrice.set(MathUtil.add(totalPrice, shop.getTotalPrice()).doubleValue());
                        // 优惠总额
                        reducedPrice.set(MathUtil.add(reducedPrice, shop.getReducedPrice()).doubleValue());
                        // 总划线价
                        totalOriginalPrice.set(MathUtil.add(totalOriginalPrice, shop.getOriginalPrice()).doubleValue());
                        // 商品种类
                        kindCount.updateAndGet(v -> v + shopKindCount.get());
                    }
                    totalProductList.addAll(productList);
                }

                // 供应商下商品种类
                shop.setCount(shopKindCount.get());
            });
            // 删除空数据
            shopList.removeIf(shop -> (null == shop || shop.getCount() == 0));

        }
        // 总促销计算
        totalPromotions.forEach(promotions -> {
            // 该促销内商品总价
            double touchPrice = promotions.getProductList().stream().mapToDouble(product -> product.getNumber() * product.getPrice()).sum();
            // 凑单满减
            Double floor = Math.floor(MathUtil.div(touchPrice, promotions.getTouchPrice()).doubleValue());
            if (promotions.getType() == 2 && promotions.getMode() == 2) {
                if (MathUtil.compare(touchPrice, promotions.getTouchPrice()) > -1) {
                    if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                        //叠加优惠计算
                        // 总价 - 满减金额
                        totalPrice.set(MathUtil.sub(totalPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                        // 优惠总额 + 满减金额
                        reducedPrice.set(MathUtil.add(reducedPrice, MathUtil.mul(promotions.getReducedPrice(), floor)).doubleValue());
                    } else {
                        // 总价 - 满减金额
                        totalPrice.set(MathUtil.sub(totalPrice, promotions.getReducedPrice()).doubleValue());
                        // 优惠总额 + 满减金额
                        reducedPrice.set(MathUtil.add(reducedPrice, promotions.getReducedPrice()).doubleValue());
                    }
                }
            } else if (promotions.getMode() == 3) {
                //满赠叠加
                List<CartItemVo> giftList = baseMapper.getPromotionGifts(promotions.getId());
                //满足叠加
                if (promotions.getDiscount() != null && promotions.getDiscount() == 1 && floor > 1) {
                    Integer in = floor.intValue();
                    giftList.forEach(g -> g.setNumber(MathUtil.mul(g.getNumber(), in).intValue()));
                }
                promotions.setGiftList(giftList);
                // 全部满赠
                promotions.getGiftList().forEach(gift -> {
                    if (shopIds.contains(gift.getShopId())) {
                        // 赠品在当前订单内的供应商下
                        shopList.forEach(shop -> {
                            if (shop.getShopId().equals(gift.getShopId())) {
                                shop.getCartList().add(gift);
                            }
                        });
                    } else {
                        // 获取赠品供应商
                        CartShopVo giftShop = baseMapper.getShopByProductId(gift.getProductId());
                        shopIds.add(giftShop.getShopId());
                        giftShop.setCartList(new ArrayList<>());
                        giftShop.getCartList().add(gift);
                        shopList.add(giftShop);
                    }
                });
            }
        });
        // 发票信息
        InvoiceVo invoice = baseMapper.getUserInvoice(clubUserId);
        // 可用余额
        Double availableMoney = baseMapper.getAbleUserMoney(clubUserId);
        // 可用采美豆
        Integer userBeans = baseMapper.getUserBeans(clubUserId);

        //优惠券相关信息
        List<CouponVo> couponList = orderCommonMapper.getClubCouponList(clubUserId);
        Iterator<CouponVo> iterator = couponList.iterator();
        while (iterator.hasNext()) {
            CouponVo coupon = iterator.next();
            BigDecimal totalAmount = BigDecimal.ZERO;
            Integer couponType = coupon.getCouponType();
            for (CartItemVo item : totalProductList) {
                if (couponType == 0) {
                    if (coupon.getProductType() == 1) {
                        //活动券-全商城商品
                        BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                        totalAmount = MathUtil.add(price, totalAmount);
                    } else if (coupon.getProductType() == 2) {
                        //活动券-指定商品
                        List<Integer> couponProductIds = orderCommonMapper.getCouponProductIds(coupon.getCouponId(), 2);
                        if (couponProductIds.contains(item.getProductId())) {
                            BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                            totalAmount = MathUtil.add(price, totalAmount);
                        }
                    }
                } else if (couponType == 1) {
                    //品类券
                    if (coupon.getCategoryType().equals(item.getCommodityType())) {
                        BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                        totalAmount = MathUtil.add(price, totalAmount);
                    }
                } else if (couponType == 2 || couponType == 4) {
                    //用户专享券或新用户券
                    BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                    totalAmount = MathUtil.add(price, totalAmount);
                } else if (coupon.getCouponType() == 3 && coupon.getShopId() != null) {
                    //店铺券
                    String shopName = baseMapper.getShopNameById(coupon.getShopId());
                    if (StringUtils.isNotBlank(shopName)) {
                        if (shopName.length() > 7) {
                            shopName = shopName.substring(0, 6) + "...";
                        }
                        coupon.setShopName(shopName);
                    }
                    if (coupon.getShopId().equals(item.getShopId())) {
                        BigDecimal price = MathUtil.mul(item.getPrice(), item.getNumber());
                        totalAmount = MathUtil.add(price, totalAmount);
                    }
                }
            }
            if (MathUtil.compare(totalAmount, coupon.getTouchPrice()) < 0) {
                iterator.remove();
            }
        }
        couponList.sort((o1, o2) -> o2.getCouponAmount().compareTo(o1.getCouponAmount()));
        // 获取条款列表
        List<ClauseVo> clauseList = baseMapper.getClauseList();

        // 包装返回数据
        Map<String, Object> resultData = new HashMap<>();
        resultData.put("list", shopList);
        resultData.put("kindCount", kindCount);
        resultData.put("totalCount", totalCount);
        resultData.put("totalPrice", totalPrice);
        resultData.put("reducedPrice", reducedPrice);
        resultData.put("templateFlag", templateFlag.get());
        resultData.put("totalOriginalPrice", totalOriginalPrice);
        resultData.put("promotions", totalPromotions);
        resultData.put("invoice", invoice);
        resultData.put("userMoney", availableMoney);
        resultData.put("userBeans", userBeans);
        resultData.put("includeRecharge", includeRecharge);
        resultData.put("couponList", couponList);
        resultData.put("clauseList", clauseList);
        return ResponseJson.success(resultData);
    }

    /**
     * 协销二手商品立即购买
     *
     * @param serviceProviderId 协销Id
     * @param clubId            机构Id
     * @param productId         商品Id
     * @param productCount      商品数量
     */
    @Override
    public ResponseJson<Map<String, Object>> settlementSecond(Integer serviceProviderId, Integer clubId, Integer productId, Integer productCount) {
        // 统计商品总金额
        Double totalPrice = 0d;
        // 供应商信息
        CartShopVo shop = baseMapper.getShopByProductId(productId);
        if (null != shop) {
            CartItemVo second = cartSellerMapper.getSecondByProductId(productId);
            second.setActStatus(0);
            second.setNumber(productCount);
            // 计算总价
            totalPrice = MathUtil.mul(second.getNumber(), second.getPrice()).doubleValue();
            shop.setTotalPrice(totalPrice);
            List<CartItemVo> cartList = new ArrayList<>(1);
            cartList.add(second);
            shop.setCartList(cartList);
        }
        // 获取机构用户Id
        Integer clubUserId = baseMapper.getUserIdByClubId(clubId);
        // 发票信息
        InvoiceVo invoice = baseMapper.getUserInvoice(clubUserId);
        // 可用余额
        Double availableMoney = baseMapper.getAbleUserMoney(clubUserId);
        // 可用采美豆
        Integer userBeans = baseMapper.getUserBeans(clubUserId);
        // 包装返回数据
        Map<String, Object> resultData = new HashMap<>();
        List<CartShopVo> shopList = new ArrayList<>();
        shopList.add(shop);
        resultData.put("list", shopList);
        resultData.put("kindCount", 1);
        resultData.put("totalCount", productCount);
        resultData.put("totalPrice", totalPrice);
        resultData.put("invoice", invoice);
        resultData.put("userMoney", availableMoney);
        resultData.put("userBeans", userBeans);
        return ResponseJson.success(resultData);
    }

    /**
     * 协销再来一单
     *
     * @param againBuyDto {
     *                    serviceProviderId      协销Id
     *                    orderId                订单Id
     *                    confirmFlag            确认标识 0未确认 1已确认-将失效商品以外的正常商品加入购物车
     *                    }
     */
    @Override
    public ResponseJson<Map<String, Object>> addCartBuyAgain(AgainBuyDto againBuyDto) {
        // 返回数据
        Map<String, Object> resultData = new HashMap<>();
        // 获取订单里用户Id
        Integer orderUserId = cartSellerMapper.getOrderUserIdByShopOrderId(againBuyDto.getShopOrderId());
        Integer orderClubId = cartSellerMapper.getOrderClubIdByShopOrderId(againBuyDto.getShopOrderId());
        if (null == orderUserId || orderUserId == 0 || null == orderClubId || orderClubId == 0) {
            return ResponseJson.error("订单异常或不存在！", null);
        }
        List<CartItemVo> productList = cartSellerMapper.getShopOrderProductNum(againBuyDto.getShopOrderId());
        // 移除运费商品
        productList.removeIf(cartItemVo -> (null == cartItemVo || null == cartItemVo.getValidFlag() || cartItemVo.getProductId() == 999));
        // 总商品种类数量
        int total = productList.size();
        // 移除失效商品
        productList.removeIf(cartItemVo -> (cartItemVo.getValidFlag() != 2 && cartItemVo.getValidFlag() != 3 && cartItemVo.getValidFlag() != 9));
        // 失效商品数量
        int invalid = total - productList.size();
        if (productList.size() == 0) {
            return ResponseJson.error(-2, "订单内的商品已全部失效，不能下单！", null);
        }
        if (invalid > 0 && (null == againBuyDto.getConfirmFlag() || 1 != againBuyDto.getConfirmFlag())) {
            resultData.put("productList", productList);
            return ResponseJson.error(-3, "部分商品已失效，不能进行购买。是否先将以下商品加入购物车？", resultData);
        }
        // 订单商品加入协销购物车
        SellerCartDto cart = new SellerCartDto();
        cart.setServiceProviderId(againBuyDto.getServiceProviderId());
        cart.setClubId(orderClubId);
        List<Integer> productIds = new ArrayList<>();
        productList.forEach(cartItemVo -> {
            cart.setSkuId(cartItemVo.getSkuId());
            cart.setProductId(cartItemVo.getProductId());
            cart.setProductCount(cartItemVo.getNumber());
            addSellerCart(cart);
            productIds.add(cartItemVo.getProductId());
        });
        String againBuyProductIds = Joiner.on(",").join(productIds);
        // 包装返回数据
        resultData.put("userId", orderUserId);
        resultData.put("clubId", orderClubId);
        resultData.put("productIds", againBuyProductIds);
        return ResponseJson.success(resultData);
    }

    @Override
    public ResponseJson checkClubSkuId(SellerCartDto sellerCartDto) {
        cartSellerMapper.deleteClubCart(sellerCartDto);
        sellerCartDto.setSkuId(sellerCartDto.getNewSkuId());
        SellerCartPo cart = cartSellerMapper.getSellerCart(sellerCartDto);
        if (cart != null) {
            // 购物车已存在该商品，更新数量
            cart.setNum(cart.getNum() + sellerCartDto.getProductCount());
            cart.setAddTime(new Date());
            cartSellerMapper.updateSellerCart(cart);
        } else {
            // 添加新购物车
            cart = new SellerCartPo();
            Integer shopId = baseMapper.getShopIdByproductId(sellerCartDto.getSkuId());
            cart.setSkuId(sellerCartDto.getSkuId());
            cart.setServiceProviderId(sellerCartDto.getServiceProviderId());
            cart.setClubId(sellerCartDto.getClubId());
            cart.setShopId(shopId);
            cart.setProductId(sellerCartDto.getProductId());
            cart.setNum(sellerCartDto.getProductCount());
            cart.setAddTime(new Date());
            cartSellerMapper.insertSellerCart(cart);
        }
        return ResponseJson.success();
    }
}
