package com.caimei365.order.service.impl;

import java.util.Date;
import java.util.List;

import com.caimei365.order.mapper.CmReportingClubMapper;
import com.caimei365.order.model.po.CmReportingClub;
import com.caimei365.order.service.CmReportingClubService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * 机构报备Service业务层处理
 *
 * @author Kaick
 * @date 2023-10-11
 */
@Service
public class CmReportingClubServiceImpl implements CmReportingClubService {
    @Resource
    private CmReportingClubMapper cmReportingClubMapper;

    /**
     * 通过对象查询机构报备列表
     *
     * @param cmReportingClub 机构报备
     * @return 机构报备
     */
    @Override
    public List<CmReportingClub> getCmReportingClubList(CmReportingClub cmReportingClub) {
        return cmReportingClubMapper.getCmReportingClubList(cmReportingClub);
    }

    /**
     * 通过Id查询机构报备
     *
     * @param id 机构报备主键
     * @return 机构报备
     */
    @Override
    public CmReportingClub getCmReportingClubById(String id) {
        return cmReportingClubMapper.getCmReportingClubById(id);
    }

    /**
     * 通过对象查询机构报备
     *
     * @param cmReportingClub 机构报备
     * @return 机构报备
     */
    @Override
    public CmReportingClub getByCmReportingClub(CmReportingClub cmReportingClub) {
        return cmReportingClubMapper.getByCmReportingClub(cmReportingClub);
    }

    /**
     * 通过对象查询机构报备Id
     *
     * @param cmReportingClub 机构报备
     * @return String
     */
    @Override
    public String getById(CmReportingClub cmReportingClub) {
        return cmReportingClubMapper.getById(cmReportingClub);
    }


    /**
     * 通过对象查询机构报备记录总数
     *
     * @param cmReportingClub 机构报备
     * @return int
     */
    @Override
    public int getCmReportingClubCount(CmReportingClub cmReportingClub) {
        return cmReportingClubMapper.getCmReportingClubCount(cmReportingClub);
    }

    /**
     * 新增机构报备
     *
     * @param cmReportingClub 机构报备
     * @return 结果
     */
    @Override
    public int addCmReportingClub(CmReportingClub cmReportingClub) {
        cmReportingClub.setCreateTime(new Date());
        return cmReportingClubMapper.addCmReportingClub(cmReportingClub);
    }

    /**
     * 修改机构报备
     *
     * @param cmReportingClub 机构报备
     * @return 结果
     */
    @Override
    public int updateCmReportingClub(CmReportingClub cmReportingClub) {
        cmReportingClub.setUpdateTime(new Date());
        return cmReportingClubMapper.updateCmReportingClub(cmReportingClub);
    }


    /**
     * 删除机构报备信息
     *
     * @param id 机构报备主键
     * @return 结果
     */
    @Override
    public int delCmReportingClubById(String id) {
        return cmReportingClubMapper.updateCmReportingClub(new CmReportingClub().id(id).delFlag(1));
        //return cmReportingClubMapper.delCmReportingClubById(id);
    }
}

