package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.components.HeliPayUtil;
import com.caimei365.order.components.OnlinePostFormUtil;
import com.caimei365.order.components.OrderCommonService;
import com.caimei365.order.components.WeChatService;
import com.caimei365.order.constant.Constant;
import com.caimei365.order.mapper.*;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.bo.PayParamBo;
import com.caimei365.order.model.dto.BankListDto;
import com.caimei365.order.model.dto.HeliDto;
import com.caimei365.order.model.enums.AppPayType;
import com.caimei365.order.model.enums.OrderStatus;
import com.caimei365.order.model.enums.PayType;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.HeliPayService;
import com.caimei365.order.service.RemoteCallService;
import com.caimei365.order.utils.ImageUtil;
import com.caimei365.order.utils.MathUtil;
import com.caimei365.order.utils.helipay.Disguiser;
import com.caimei365.order.utils.helipay.HttpClientService;
import com.caimei365.order.utils.helipay.MessageHandle;
import com.caimei365.order.utils.helipay.MyBeanUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpStatus;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.beans.IntrospectionException;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicReference;

import okhttp3.*;

import static com.caimei365.order.constant.Constant.*;

/**
 * Description
 *
 * @author : zzj
 * @date : 2021/11/11
 */
@Slf4j
@Service
public class HeliPayServiceImpl implements HeliPayService {

    public static OkHttpClient client = new OkHttpClient.Builder()
            .connectTimeout(3, TimeUnit.SECONDS)
            .readTimeout(20, TimeUnit.SECONDS)
            .build();
    @Resource
    private OrderCommonMapper orderCommonMapper;
    @Resource
    private HeliPayUtil heliPayUtil;
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private PayOrderMapper payOrderMapper;
    @Resource
    private RemoteCallService remoteCallService;
    @Resource
    private OrderCommonService orderCommonService;
    @Resource
    private WeChatService weChatService;
    @Resource
    private OrderRefundMapper refundMapper;
    @Resource
    private CmReportingClubMapper cmReportingClubMapper;
    @Resource
    private CmReportingMemberMapper cmReportingMemberMapper;
    @Value("${caimei.wwwDomain}")
    private String domain;

    /**
     * 微信/支付宝扫码接口
     * 交易类型	P1_bizType	是	String(20)	AppPay	 交易类型
     * 商户订单号 P2_orderId	是	String(50)	p_20170302185347	商户系统内部订单号，要求50字符以内，同一商户号下订单号唯一
     * 商户编号	P3_customerNumber	是	String(15)	C1800000002	合利宝分配的商户号
     * 支付类型	P4_payType	是	String(15)		SCAN:扫码(主扫)
     * 交易金额	P5_orderAmount	是	Number(10,2)	0.01	订单金额，以元为单位，最小金额为0.01
     * 币种类型	P6_currency	是	String(30)	CNY	CNY:人民币
     * 授权码	P7_authcode	是	String(50)	1234567890	payType为刷卡(被扫)时传入一组字符串(付款码),主扫支付类型传入1即可.
     * 客户端类型	P8_appType	是	String(20)  ALIPAY：支付宝  WXPAY:微信
     * 通知回调地址	P9_notifyUrl	否	String(300)	http://wap.helipay.com/notify.php
     * 异步接收合利宝支付结果通知的回调地址，通知url必须为外网可访问的url，不能携带参数。
     * 成功跳转URL	P10_successToUrl	否	String(300)	http://wap.helipay.com/success.php 支付完成后，展示支付结果的页面地址(暂不用)
     * 商户IP	P11_orderIp	是	String(20)	192.168.10.1	用户下单IP
     * 商品名称	P12_goodsName	是	String(128)	Iphone7	商品名称,账单上显示
     * 商品详情	P13_goodsDetail	否	String(8000)商品的优惠活动: 单品优惠活动该字段必传，且必须按照规范上传，JSON字符串格式，详见附录【6.3单品优惠活动字段说明】
     * 备注	P14_desc	否	String(100)	备注	订单备注信息，原样返回
     * 公众号appId	P16_appId	否	String(30)	wxdeaaaa2311	刷卡时可传.(不参与签名)
     * 签名	sign	是	String(40)	d3382b9a9b08cefc1a79d276ec03d83a	MD5 签名结果，详见“第 5 章 数字签名”
     */
    @Override
    public ResponseJson<JSONObject> payByWeChat(HeliDto heliDto, HttpHeaders headers) throws IntrospectionException, InvocationTargetException, IllegalAccessException {
        log.info("--------进入支付二维码创建接口----------");
        AppCreateOrderVo pay = new AppCreateOrderVo();
        String splitCode = payOrderMapper.findShopOrderSplitCode(heliDto.getShopOrderId());
        if (StringUtils.isBlank(splitCode)) {
            return ResponseJson.error("子订单无商户号！", null);
        }
        pay.setP3_customerNumber(splitCode);
        //合利宝主扫接口参数赋值
        heliPayUtil.setValue(pay, heliDto, "order", headers);
        if (!(StringUtils.equals(AppPayType.ALIPAY.name(), pay.getP8_appType())
                && StringUtils.equals(PayType.SWIPE.name(), pay.getP4_payType()))) {
            pay.setOpenId(null);
            pay.setAuthConfirmMode(null);
        }
        Map<String, String> map = MyBeanUtils.convertBean(pay, new LinkedHashMap());
        String oriMessage = MyBeanUtils.getSignedByPresetParameter(map, AppCreateOrderVo.NEED_SIGN_PARAMS);
        //密钥拼接
        oriMessage += SPLIT + Constant.SAOMA;
        log.info("签名原文串：" + oriMessage);
        String sign = Disguiser.disguiseMD5(oriMessage.trim());
        log.info("签名串：" + sign);
        map.put("sign", sign);
        log.info("发送参数：" + map);
        Map<String, Object> resultMap = HttpClientService.getHttpResp(map, Constant.REQUEST_URL);
        log.info("响应结果：" + resultMap);
        if ((Integer) resultMap.get("statusCode") == HttpStatus.SC_OK) {
            String resultMsg = (String) resultMap.get("response");
            AppCreateOrderResponseVo orderResponseVo = JSONObject.parseObject(resultMsg, AppCreateOrderResponseVo.class);
            String assemblyRespOriSign = MyBeanUtils.getSignedByPresetParameter(orderResponseVo, AppCreateOrderResponseVo.NEED_SIGN_PARAMS);
            assemblyRespOriSign += Constant.SPLIT + Constant.SAOMA;
            log.info("组装返回结果签名串：" + assemblyRespOriSign);
            String responseSign = orderResponseVo.getSign();
            log.info("响应签名：" + responseSign);
            String checkSign = Disguiser.disguiseMD5(assemblyRespOriSign.trim());
            JSONObject jsonObject = JSONObject.parseObject(resultMsg);
            if (checkSign.equals(responseSign) && "0000".equals(orderResponseVo.getRt2_retCode())) {
                return ResponseJson.success("二维码创建成功", jsonObject);
            } else {
                return ResponseJson.error("二维码创建失败", jsonObject);
            }
        } else {
            return ResponseJson.error("二维码创建失败", null);
        }
    }

    @Override
    public ResponseJson<JSONObject> payOnline(HeliDto heliDto, HttpHeaders headers) throws IntrospectionException, InvocationTargetException, IllegalAccessException {
        log.info("--------进入公众号/小程序预创建订单接口----------");
        // 组织Id
        Integer organizeId = baseMapper.getOrganizeIdShop(heliDto.getShopOrderId());
        heliDto.setOrganizeId(organizeId);
        AppPayPublicCreateOrderVo pay = new AppPayPublicCreateOrderVo();
        String splitCode = payOrderMapper.findShopOrderSplitCode(heliDto.getShopOrderId());
        if (StringUtils.isBlank(splitCode)) {
            return ResponseJson.error("子订单无商户号！", null);
        }
        pay.setP3_customerNumber(splitCode);
        //合利宝主扫接口参数赋值
        heliPayUtil.setOnlineValue(pay, heliDto, "order", headers);
        if ("GZH".equals(heliDto.getPayType()) || "XCX".equals(heliDto.getPayType())) {
            //公众号/小程序
            String openId = getOpenId(heliDto, headers);
            if (StringUtils.isEmpty(openId)) {
                return ResponseJson.error("微信openId获取失败", null);
            }
            pay.setP8_openid(openId);
        }
        Map<String, String> map = MyBeanUtils.convertBean(pay, new LinkedHashMap());
        String oriMessage = MyBeanUtils.getSignedByPresetParameter(map, AppPayPublicCreateOrderVo.NEED_SIGN_PARAMS);
        oriMessage += SPLIT + Constant.SAOMA;
        log.info("签名原文串：" + oriMessage);
        String sign = Disguiser.disguiseMD5(oriMessage.trim());
        log.info("签名串：" + sign);
        map.put("sign", sign);
        log.info("发送参数：" + map);
        Map<String, Object> resultMap = HttpClientService.getHttpResp(map, Constant.REQUEST_URL);
        log.info("响应结果：" + resultMap);
        if ((Integer) resultMap.get("statusCode") == HttpStatus.SC_OK) {
            String resultMsg = (String) resultMap.get("response");
            AppPayPublicOrderResponseVo orderResponseVo = JSONObject.parseObject(resultMsg, AppPayPublicOrderResponseVo.class);
            String assemblyRespOriSign = MyBeanUtils.getSignedByPresetParameter(orderResponseVo, AppPayPublicOrderResponseVo.NEED_SIGN_PARAMS);
            assemblyRespOriSign += Constant.SPLIT + Constant.SAOMA;
            log.info("组装返回结果签名串：" + assemblyRespOriSign);
            String responseSign = orderResponseVo.getSign();
            log.info("响应签名：" + responseSign);
            String checkSign = Disguiser.disguiseMD5(assemblyRespOriSign.trim());
            if (checkSign.equals(responseSign) && "0000".equals(orderResponseVo.getRt2_retCode())) {
                JSONObject jsonObject = JSONObject.parseObject(resultMsg);
                return ResponseJson.success("请求成功", jsonObject);
            } else {
                return ResponseJson.error("请求参数有误", JSONObject.parseObject(resultMsg));
            }
        } else {
            return ResponseJson.error("请求失败", null);
        }
    }

    @Override
    public String paymentCallback(NotifyResponseVo res) throws IntrospectionException, InvocationTargetException, IllegalAccessException {
        log.info("******************** 支付异步回调 start *******************");
        // 签名验证
        String sign = res.getSign();
        log.info("回调签名" + sign);
        String oriMessage = MyBeanUtils.getSigned(res, null);
        String oriMessage1 = oriMessage + SPLIT + Constant.SAOMA;
        String checkSign1 = Disguiser.disguiseMD5(oriMessage1.trim());
        boolean b = sign.equals(checkSign1);
        if (!b) {
            return "验签名失败！";
        }
        // 订单状态 INIT:已接收 DOING:处理中 SUCCESS:成功 FAIL:失败 CLOSE:关闭 CANCEL:撤销
        String orderStatus = res.getRt4_status();
        log.info("【支付异步回调】>>>>>>>>>>>>>>支付订单状态:" + orderStatus);
        if (!"SUCCESS".equals(orderStatus)) {
            return "支付失败";
        }
        String s = checkOrderStatus(res, 1);
        if ("SUCCESS".equals(s)) {
            return "SUCCESS";
        }
        return null;
    }

    /**
     * 1 其他支付，2快捷支付
     *
     * @param o
     * @param status
     * @return
     */
    public String checkOrderStatus(Object o, Integer status) {
        // 平台唯一流水号
        String mbOrderId = "";
        String orderRequestNo = "";
        Double amount = null;
        Integer orderId = null;
        String payType = "";
        Integer shopOrderId = null;
        Integer infoId = null;
        if (1 == status) {
            NotifyResponseVo res = (NotifyResponseVo) o;
            mbOrderId = res.getRt3_systemSerial();
            // 商户唯一订单号
            orderRequestNo = res.getRt2_orderId();
            // 本次支付金额，以元为单位
            amount = Double.valueOf(res.getRt5_orderAmount());
            String[] split = res.getRt8_desc().split(",");
            //0位置订单id
            orderId = Integer.valueOf(split[0]);
            //1位置支付类型
            payType = split[1];
            //2位置子订单id
            shopOrderId = Integer.valueOf(split[2]);
        } else {
            QuickPayConfirmPayResponseVo res = (QuickPayConfirmPayResponseVo) o;
            orderRequestNo = res.getRt5_orderId();
            mbOrderId = res.getRt6_serialNumber();
            amount = Double.valueOf(res.getRt8_orderAmount());
            String[] s = orderRequestNo.split("_");
            //  orderId = order.getOrderNo() + "_" + order.getOrderId() + "_" + heliDto.getShopOrderId() + "_" + infoId;
            orderId = Integer.valueOf(s[1]);
            shopOrderId = Integer.valueOf(s[2]);
            //infoId
            infoId = Integer.valueOf(s[3]);
            String bankCode = res.getRt11_bankId();
            Integer bankType = "DEBIT".equals(res.getRt12_onlineCardType()) ? 1 : 2;
            payOrderMapper.updateQuickInfo(bankCode, bankType, infoId);
            String userId = res.getRt14_userId().substring(1);
            // 快捷支付借记卡支付29，快捷支付贷记卡支付30
            payType = 1 == bankType ? "29" : "30";
            payOrderMapper.insertBindId(res.getRt10_bindId(), res.getRt4_customerNumber(), userId, infoId);
        }
        // 订单信息
        OrderVo order = orderCommonMapper.getOrderByOrderId(orderId);
        ShopOrderVo shopOrder = orderCommonMapper.getShopOrderByOrderId(shopOrderId);
        if (null == order) {
            return "订单不存在";
        }
        if (null == shopOrder) {
            return "子订单不存在";
        }
        // 支付记录
        List<DiscernReceiptVo> discernReceiptList = orderCommonMapper.getDiscernReceipt(order.getOrderId(), order.getShopOrderIds());
        double receiptAmount = 0d;
        if (null != discernReceiptList && discernReceiptList.size() > 0) {
            for (DiscernReceiptVo discernReceipt : discernReceiptList) {
                receiptAmount = MathUtil.add(receiptAmount, discernReceipt.getAssociateAmount()).doubleValue();
            }
        }
        order.setReceiptAmount(MathUtil.add(receiptAmount, amount).doubleValue());
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        if (MathUtil.compare(order.getPayableAmount(), order.getReceiptAmount()) == 0) {
            /*
             * 订单全部支付
             * 0待确认,11待收待发,12待收部发,13待收全发,21部收待发,22部收部发,23部收全发,
             * 31已收待发,32已收部发,33已收全发,4交易完成,5订单完成,6已关闭,7交易全退
             */
            if (OrderStatus.UNRECEIVED_AND_UNSHIPPED.getCode() == order.getStatus() || OrderStatus.PART_RECEIVED_AND_UNSHIPPED.getCode() == order.getStatus()) {
                order.setStatus(31);
            } else if (OrderStatus.UNRECEIVED_AND_PART_SHIPPED.getCode() == order.getStatus() || OrderStatus.PART_RECEIVED_AND_PART_SHIPPED.getCode() == order.getStatus()) {
                order.setStatus(32);
            } else {
                order.setStatus(33);
            }
            order.setPayFlag(1);
            order.setOnlinePayFlag(0);
            //（收款买家）收款状态：1待收款、2部分收款、3已收款
            order.setReceiptStatus(3);
            log.info("【支付异步回调】>>>>>>>>>>>>>>订单(全部支付),修改订单状态：" + order.getStatus() + "，orderId：" + orderId);
        } else {
            // 部分支付
            if (OrderStatus.UNRECEIVED_AND_UNSHIPPED.getCode() == order.getStatus() || OrderStatus.PART_RECEIVED_AND_UNSHIPPED.getCode() == order.getStatus()) {
                order.setStatus(21);
            } else if (OrderStatus.UNRECEIVED_AND_PART_SHIPPED.getCode() == order.getStatus() || OrderStatus.PART_RECEIVED_AND_PART_SHIPPED.getCode() == order.getStatus()) {
                order.setStatus(22);
            } else {
                order.setStatus(23);
            }
            order.setOnlinePayFlag(0);
            //（收款买家）收款状态：1待收款、2部分收款、3已收款
            order.setReceiptStatus(2);
            log.info("【支付异步回调】>>>>>>>>>>>>>>订单(部分支付),修改订单状态：" + order.getStatus() + "，orderId：" + orderId);
        }
        // 更新付款次数
        Integer paySuccessCounter = (null == order.getPaySuccessCounter()) ? 1 : order.getPaySuccessCounter() + 1;
        Integer paySuccess = (null == shopOrder.getPaySuccessCounter()) ? 1 : shopOrder.getPaySuccessCounter() + 1;
        shopOrder.setPaySuccessCounter(paySuccess);
        order.setPaySuccessCounter(paySuccessCounter);
        order.setUpdateDate(curDateStr);
        // 更新订单支付状态
        payOrderMapper.updateOrderStatus(order);
        // 支付记录
        List<DiscernReceiptVo> shopOrderDiscernReceiptList = orderCommonMapper.getShopOrderDiscernReceipt(shopOrderId);
        double shopOrderReceiptAmount = 0d;
        if (null != shopOrderDiscernReceiptList && shopOrderDiscernReceiptList.size() > 0) {
            for (DiscernReceiptVo discernReceipt : shopOrderDiscernReceiptList) {
                shopOrderReceiptAmount = MathUtil.add(shopOrderReceiptAmount, discernReceipt.getAssociateAmount()).doubleValue();
            }
        }
        shopOrder.setReceiptAmount(MathUtil.add(shopOrderReceiptAmount, amount).doubleValue());
        //均摊后needpay:已支付
        if (MathUtil.compare(shopOrder.getRealPay(), shopOrder.getReceiptAmount()) == 0) {
            shopOrder.setReceiptStatus(3);
        } else {
            shopOrder.setReceiptStatus(2);
        }
        payOrderMapper.updateShopOrder(shopOrder);
        // 保存收款记录
        DiscernReceiptPo discernReceipt = new DiscernReceiptPo();
        discernReceipt.setPayWay(1);
        discernReceipt.setPayType(Integer.valueOf(payType));
        discernReceipt.setReceiptType(1);
        discernReceipt.setReceiptStatus(3);
        discernReceipt.setReceiptAmount(amount);
        discernReceipt.setConfirmType(4);
        discernReceipt.setRePayFlag(1);
        discernReceipt.setFormData(JSONObject.toJSONString(o));
        discernReceipt.setReceiptDate(curDateStr);
        discernReceipt.setConfirmDate(curDateStr);
        discernReceipt.setReviewDate(curDateStr);
        discernReceipt.setUpdateDate(curDateStr);
        discernReceipt.setDelFlag(0);
        if (2 == status) {
            //快捷支付回调，存入支付银行卡信息infoId
            discernReceipt.setInfoId(infoId);
        }
        // 保存 收款记录
        baseMapper.insertDiscernReceipt(discernReceipt);
        log.info("【支付异步回调】>>>>>>>>>>>保存识别款项(insert[cm_discern_receipt])id：" + discernReceipt.getId() + "，orderId：" + orderId);
        // 收款项和订单关系表
        OrderReceiptRelationPo relation = new OrderReceiptRelationPo();
        relation.setReceiptId(discernReceipt.getId());
        relation.setOrderId(orderId);
        relation.setAssociateAmount(amount);
        relation.setMbOrderId(mbOrderId);
        relation.setOrderRequestNo(orderRequestNo);
        relation.setSplitStatus(0);
        relation.setRelationType(2);
        relation.setDelFlag(0);
        relation.setShopOrderId(shopOrderId);
        // 保存 收款项和订单关系
        baseMapper.insertOrderReceiptRelation(relation);
        log.info("【支付异步回调】>>>>>>>>>>>收款项和订单关系(insert[cm_receipt_order_relation])id：" + relation.getId() + "，orderId：" + orderId);
        Integer identity = baseMapper.getIdentityByUserId(order.getUserId());
        //分销人员下单修改报备记录
        if (identity == 8) {
            CmReportingClub byCmReportingClub = cmReportingClubMapper.getByCmReportingClub(new CmReportingClub()
                    .orderId(order.getOrderId())
                    .orderStatus(0)
            );
            //商品报备佣金
            BigDecimal commission = cmReportingClubMapper.getDistributionProductCommission(byCmReportingClub.productId());
            byCmReportingClub
                    .distributionId(shopOrder.getSpId())
                    .lockStatus(1)
                    .lockTime(new Date())
                    .orderStatus(1)
                    .commission(commission)
                    .splitSettleStatus(1)
                    .settleStatus(0);
            cmReportingClubMapper.updateCmReportingClub(byCmReportingClub);
            List<CmReportingMember> cmReportingMemberList = cmReportingMemberMapper.getCmReportingMemberList(new CmReportingMember()
                    .clubReportingId(Integer.valueOf(byCmReportingClub.id()))
                    .auditStatus(2)
            );
            //商品报备成员佣金
            BigDecimal memberCommission = MathUtil.div(commission, cmReportingMemberList.size(), 2);
            cmReportingMemberList.forEach(s -> {
                s.commission(memberCommission).settleStatus(2);
                cmReportingMemberMapper.updateCmReportingMember(s);
            });
        }
        //当前版本无法线上走充值/订金订单
        // 商品数据
//        List<OrderProductVo> orderProductList = orderCommonMapper.getOrderProductByOrderId(orderId);
        // 判断是否是充值商品
//        int rechargeFlag = 0;
//        // 缴纳订金订单
//        int[] productId1 = {6060, 6061, 6062, 6063, 6064};
//        // 充值余额订单
//        int[] productId2 = {6065, 6066, 6067, 6068, 6069};
//        for (OrderProductVo product : orderProductList) {
//            if (ArrayUtils.contains(productId1, product.getProductId())) {
//                rechargeFlag = 1;
//                break;
//            }
//            if (ArrayUtils.contains(productId2, product.getProductId())) {
//                rechargeFlag = 2;
//                break;
//            }
//        }
//        if (rechargeFlag > 0) {
//            // 账户余额
//            double oldUserMoney = baseMapper.getUserMoney(order.getUserId());
//            double userMoney = MathUtil.add(oldUserMoney, amount).doubleValue();
//            // 可用余额
//            Double oldAvailableMoney = baseMapper.getAbleUserMoney(order.getUserId());
//            double availableMoney = MathUtil.add(oldAvailableMoney, amount).doubleValue();
//            payOrderMapper.updateMoneyByUserId(userMoney, availableMoney, order.getUserId());
//            log.info("【支付异步回调】>>>>>>>>>>>更新用户余额(update[user])userId：" + order.getUserId() + "，orderId：" + orderId);
//            //保存余额到余额收支记录
//            BalanceRecordPo balanceRecord = new BalanceRecordPo();
//            balanceRecord.setUserId(order.getUserId());
//            balanceRecord.setType(1);
//            if (rechargeFlag == 1) {
//                balanceRecord.setBalanceType(6);
//            } else {
//                balanceRecord.setBalanceType(7);
//            }
//            balanceRecord.setAddDate(new Date());
//            balanceRecord.setAmount(amount);
//            balanceRecord.setOrderId(orderId);
//            balanceRecord.setRemark("订单商品充值余额");
//            balanceRecord.setDelFlag(0);
//            // 保存 余额收支记录
//            baseMapper.insertBalanceRecord(balanceRecord);
//            log.info("【支付异步回调】>>>>>>>>>>>>>>>>>>>>>>>>>>订单商品充值余额(insert[cm_user_balance_record])orderId：" + orderId);
//        }

        // 线上支付与自主下单送豆（已全部收款），最后一笔线上支付成功后，赠送200采美豆
        if (3 == order.getReceiptStatus() && 0 == order.getSecondHandOrderFlag()&&identity != 8) {
            UserBeansHistoryPo beansHistory = new UserBeansHistoryPo();
            beansHistory.setUserId(order.getUserId());
            beansHistory.setOrderId(orderId);
            beansHistory.setBeansType(6);
            beansHistory.setType(1);
            beansHistory.setNum(200);
            beansHistory.setPushStatus(0);
            beansHistory.setAddTime(date);
            beansHistory.setDelFlag(0);
            // 保存 采美豆使用记录
            baseMapper.insertBeansHistory(beansHistory);
            // 用户采美豆
            Integer userBeans = baseMapper.getUserBeans(order.getUserId());
            // 更新用户剩余采美豆数量
            userBeans = userBeans + beansHistory.getNum();

            // 自主下单且没有促销活动送豆
            Integer orderPromotionsId = payOrderMapper.getOrderPromotionsId(order.getOrderId());
            if (1 == order.getOrderType() && null == orderPromotionsId) {
                // 机构自主下单，根据订单总额的价格区间赠送采美豆数量规则
                Double payTotalFee = order.getPayTotalFee();
                int num;
                if (MathUtil.compare(1000, payTotalFee) > -1) {
                    num = 0;
                } else if (MathUtil.compare(5000, payTotalFee) > -1) {
                    num = 1000;
                } else if (MathUtil.compare(10000, payTotalFee) > -1) {
                    num = 2500;
                } else if (MathUtil.compare(25000, payTotalFee) > -1) {
                    num = 5000;
                } else if (MathUtil.compare(100000, payTotalFee) > -1) {
                    num = 7500;
                } else if (MathUtil.compare(250000, payTotalFee) > -1) {
                    num = 10000;
                } else if (MathUtil.compare(500000, payTotalFee) > -1) {
                    num = 12500;
                } else {
                    num = 20000;
                }
                // 超级会员用户标识
                Integer svipUserId = baseMapper.getSvipUserIdByUserId(order.getUserId());
                if (null != svipUserId && svipUserId.equals(order.getUserId())) {
                    // 超级会员用户采美豆翻倍
                    num = num * 2;
                }
                beansHistory.setBeansType(5);
                beansHistory.setNum(num);
                if (num > 0) {
                    // 保存 采美豆使用记录
                    baseMapper.insertBeansHistory(beansHistory);
                    // 更新用户剩余采美豆数量
                    userBeans = userBeans + beansHistory.getNum();
                }
            }
            baseMapper.updateUserBeans(beansHistory.getUserId(), userBeans);
            log.info("【支付异步回调】>>>>>更新用户采美豆(update[user(userBeans:" + userBeans + ")])，userId:" + beansHistory.getUserId());
        }
        // 已支付短信推送 不是本地或者测试
//        boolean smsPushFlag = !orderRequestNo.contains("BETA") && !orderRequestNo.contains("DEV");
//        String bindMobile = baseMapper.getBindMobileByUserId(order.getUserId());
//        if (smsPushFlag && StringUtils.isNotBlank(bindMobile)) {
//            String shortLink = remoteCallService.getShortLink(8, 6, "https://www.caimei365.com/user/order/detail.html?orderId=" + orderId);
//            String content = "您已成功支付订单（订单编号：" + order.getOrderNo() + "）全部款项，支付总金额￥" + order.getPayTotalFee() + "，采美平台将立即安排发货。您可关注采美公众号或者访问采美微信小程序和网站查看订单。" +
//                    "平台公众号：微信搜索“采美365网”; 微信小程序：微信搜索“采美采购商城”；网址：www.caimei365.com/t/" + shortLink;
//            boolean sendSms = remoteCallService.getSendSms(6, bindMobile, content);
//            if (!sendSms) {
//                log.info("取消订单推送失败，orderId>>>>" + orderId);
//            }
//        }
        return "SUCCESS";
    }

    @Override
    public ResponseJson<String> link(HeliDto heliDto, HttpHeaders headers) {
        log.info("--------进入线上支付接口----------");
        // 订单信息
        OrderVo order = orderCommonMapper.getOrderByShopOrderId(heliDto.getShopOrderId());
        if (null == order) {
            log.error("订单不存在");
            return ResponseJson.error("订单不存在");
        }
        if (OrderStatus.UNCONFIRMED.getCode() == order.getStatus() ||
                OrderStatus.RECEIVED_AND_FULL_SHIPPED.getCode() == order.getStatus() ||
                OrderStatus.TRANSACTION_COMPLETED.getCode() == order.getStatus() ||
                OrderStatus.ORDER_COMPLETED.getCode() == order.getStatus() ||
                OrderStatus.CLOSED.getCode() == order.getStatus() ||
                OrderStatus.FULL_RETURNED.getCode() == order.getStatus()) {
            log.error("订单状态错误");
            return ResponseJson.error("订单状态错误", null);
        }
        // 支付记录
        List<DiscernReceiptVo> discernReceiptList = orderCommonMapper.getDiscernReceipt(order.getOrderId(), order.getShopOrderIds());
        if (!discernReceiptList.isEmpty()) {
            AtomicReference<Boolean> offlineFlag = new AtomicReference<>(false);
            discernReceiptList.forEach(discernReceipt -> {
                if (2 == discernReceipt.getPayWay()) {
                    offlineFlag.set(true);
                }
            });
            if (offlineFlag.get()) {
                log.error("已经线下支付过，只能线下支付！");
                return ResponseJson.error("已经线下支付过，只能线下支付！", null);
            }
        }
        log.info(">>>>>>>>>>>>>>本次付款金额:" + heliDto.getPayAmount());
        if (MathUtil.compare(MathUtil.mul(order.getPayableAmount(), 100), heliDto.getPayAmount()) < 0) {
            log.error("付款金额错误！");
            return ResponseJson.error("付款金额错误！", null);
        }
        try {
            HeliOnlineVo heliOnlineVo = new HeliOnlineVo();
            String splitCode = payOrderMapper.findShopOrderSplitCode(heliDto.getShopOrderId());
            if (StringUtils.isBlank(splitCode)) {
                return ResponseJson.error("子订单无商户号！", null);
            }
            heliOnlineVo.setP3_customerNumber(splitCode);
            heliPayUtil.setLinkValue(heliOnlineVo, heliDto, "order", headers);
            Map<String, String> map = MyBeanUtils.convertBean(heliOnlineVo, new LinkedHashMap());
            String oriMessage = MyBeanUtils.getSignedByPresetParameter(map, HeliOnlineVo.NEED_SIGN_PARAMS);
            oriMessage += SPLIT + Constant.WANGYIN;
            String sign = Disguiser.disguiseMD5(oriMessage.trim());
            Map<String, String> sPara = new HashMap<String, String>();
            sPara.put("P1_bizType", heliOnlineVo.getP1_bizType());
            sPara.put("P2_orderId", heliOnlineVo.getP2_orderId());
            sPara.put("P3_customerNumber", splitCode);
            sPara.put("P4_orderAmount", heliOnlineVo.getP4_orderAmount());
            sPara.put("P5_bankId", heliOnlineVo.getP5_bankId());
            sPara.put("P6_business", heliOnlineVo.getP6_business());
            sPara.put("P7_timestamp", heliOnlineVo.getP7_timestamp());
            sPara.put("P8_goodsName", heliOnlineVo.getP8_goodsName());
            sPara.put("P9_period", heliOnlineVo.getP9_period());
            sPara.put("P10_periodUnit", heliOnlineVo.getP10_periodUnit());
            sPara.put("P11_callbackUrl", heliOnlineVo.getP11_callbackUrl());
            sPara.put("P12_serverCallbackUrl", heliOnlineVo.getP12_serverCallback());
            sPara.put("P13_orderIp", heliOnlineVo.getP13_orderIp());
            sPara.put("P14_onlineCardType", heliOnlineVo.getP14_onlineCardType());
            sPara.put("P15_desc", heliOnlineVo.getP15_desc());
            sPara.put("sign", sign);
            StringBuffer sbHtml = new StringBuffer();
            List<String> keys = new ArrayList<String>(sPara.keySet());
            //		post方式传递
            sbHtml.append("<form id=\"onlineForm\" name=\"onlineForm\" action=\"").append(YL).append("\" method=\"post\">");
            String name = "";
            String value = "";
            for (int i = 0; i < keys.size(); i++) {
                name = (String) keys.get(i);
                value = (String) sPara.get(name);
                if (value != null && !"".equals(value)) {
                    sbHtml.append("<input type=\"hidden\" name=\"").append(name).append("\" value=\"" + value + "\"/>");
                }
            }
            //submit按钮控件请不要含有name属性
            sbHtml.append("<input type=\"submit\" value=\"确认付款\"></form>");
            return ResponseJson.success(sbHtml.toString());
        } catch (Exception e) {
            log.error("错误信息", e);
            return ResponseJson.error("支付失败！");
        }
    }

    private String getOpenId(HeliDto heliDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(heliDto, payParam);
        // 微信线上支付
        String openId = null;
        if (null == payParam.getState()) {
            // 小程序微信授权获取登录信息
            ResponseJson<Map<String, Object>> appletsInfo = null;
            if (0 == heliDto.getOrganizeId()) {
                appletsInfo = weChatService.getInfoMapByApplets(payParam.getCode(), headers, 1);
            } else {
                appletsInfo = weChatService.getInfoMapByApplets(payParam.getCode(), headers, 3);
            }
            if (appletsInfo.getCode() == -1) {
                return null;
            }
            Map<String, Object> infoData = appletsInfo.getData();
            openId = (String) infoData.get(WeChatService.Keys.OPEN_ID);
            if (StringUtils.isEmpty(openId)) {
                return null;
            }
        } else {
            try {
                // 微信公众号，通过code获取微信用户信息
                Map<String, Object> map = weChatService.getInfoMapByWeb(payParam.getCode(), "crm");
                openId = (String) map.get(WeChatService.Keys.OPEN_ID);
            } catch (Exception e) {
                log.error("openId获取失败", e);
            }
            if (StringUtils.isEmpty(openId)) {
                return null;
            }
        }
        return openId;
    }

//    @Override
//    @Transactional(rollbackFor = Exception.class)
//    public void delayedSplitting(String callUrl) {
//        log.info("【延时分账】>>>>>>>>>>延时分账,每一小时执行一次");
//        Calendar calendar = Calendar.getInstance();
//        calendar.setTime(new Date());
//        //todo 测试 不减时间，现在有就分账
//        //测试查5分钟前的单
//        calendar.add(Calendar.SECOND, -5);
////        calendar.add(Calendar.DAY_OF_MONTH, -1);
//        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//        String currentTime = format.format(calendar.getTime());
//        // 查询未分账已支付收款      排除线上订金/充值订单
//        List<OrderReceiptRelationPo> orderRelations = payOrderMapper.getUndividedPaidReceipt(currentTime);
//        if (null != orderRelations && orderRelations.size() > 0) {
//            for (OrderReceiptRelationPo orderRelation : orderRelations) {
//                log.info("【延时分账】>>>>>>>>>>订单id:" + orderRelation.getOrderId() + "进入延时分账");
//                // 收款对应的订单信息
//                OrderVo order = orderCommonMapper.getOrderByOrderId(orderRelation.getOrderId());
//                PayParamBo payParam = new PayParamBo();
//                //支付金额
//                payParam.setPayAmount(MathUtil.mul(orderRelation.getAssociateAmount(), 100).intValue());
//                if (12 == orderRelation.getPayType()) {
//                    // 网银支付
//                    payParam.setPayWay("UNIONPAY");
//                }
//                if (17 == orderRelation.getPayType()) {
//                    //b2c网银
//                    payParam.setPayWay("B2C");
//                }
//                //微信0.65%手续费
//                if (8 == orderRelation.getPayType() || 13 == orderRelation.getPayType() || 15 == orderRelation.getPayType()) {
//                    payParam.setPayWay("WX");
//                }
//                List<SplitAccountPo> splitBillDetail = setSplitAccountDetail(order, payParam);
//                List<Map<String, String>> maps = new ArrayList<>();
//                List<ShopOrderVo> shopOrderList = orderCommonMapper.getShopOrderListByOrderId(order.getOrderId());
//                for (ShopOrderVo shopOrder : shopOrderList) {
//                    double shopTotalAmount = 0.00;
//                    String subUserNo = "";
//                    for (SplitAccountPo account : splitBillDetail) {
//                        //供应商有支付平台账户type=4
//                        if (null != account.getType() && 4 == account.getType() && shopOrder.getShopId().equals(account.getShopId())) {
//                            shopTotalAmount = MathUtil.add(shopTotalAmount, account.getSplitAccount()).doubleValue();
//                            subUserNo = account.getSubUserNo();
//                        }
//                    }
////                    addMaps(maps, shopTotalAmount, subUserNo);
//                }
//                if (null == splitBillDetail || splitBillDetail.size() == 0) {
//                    log.info("无满足条件分账单号");
//                    return;
//                }
//                //公账-专票总金额,私账-无票总金额,公账-普票总金额
//                //Type2，3奥泰，1，5信息
//                //todo 正式改
//                String sp1 = "";
//                BigDecimal totalAmount1 = BigDecimal.ZERO;
//                String sp2 = "";
//                BigDecimal totalAmount2 = BigDecimal.ZERO;
//                for (SplitAccountPo account : splitBillDetail) {
//                    if (1 == account.getType() || 5 == account.getType()) {
//                        totalAmount1 = MathUtil.add(totalAmount1, account.getSplitAccount());
//                        sp1 = account.getSubUserNo();
//                    }
//                    if (2 == account.getType() || 3 == account.getType()) {
//                        totalAmount2 = MathUtil.add(totalAmount2, account.getSplitAccount());
//                        sp2 = account.getSubUserNo();
//                    }
//                }
//                ArrayList<AccountPayOrder.AccountPayOrderExt.SplitBillRule> splitBillRules = new ArrayList<>();
//                if (MathUtil.compare(totalAmount1, 0.01) > 0) {
//                    AccountPayOrder.AccountPayOrderExt.SplitBillRule splitBillRule = new AccountPayOrder.AccountPayOrderExt.SplitBillRule();
//                    splitBillRule.setSplitBillAmount(totalAmount1);
//                    splitBillRule.setSplitBillMerchantNo(sp1);
//                    splitBillRules.add(splitBillRule);
//                }
//                if (MathUtil.compare(totalAmount2, 0.01) > 0) {
//                    AccountPayOrder.AccountPayOrderExt.SplitBillRule splitBillRule = new AccountPayOrder.AccountPayOrderExt.SplitBillRule();
//                    splitBillRule.setSplitBillAmount(totalAmount2);
//                    splitBillRule.setSplitBillMerchantNo(sp2);
//                    splitBillRules.add(splitBillRule);
//                }
//                //第三方分账接口
//                try {
//                    AccountPayOrder accountPayOrder = new AccountPayOrder();
//                    accountPayOrder.setP1_bizType("AccountPaySub");
//                    accountPayOrder.setP2_signType("MD5");
//                    String format1 = new SimpleDateFormat("yyyy-MM-dd_HH:mm:ss.SSS").format(new Date());
//                    accountPayOrder.setP3_timestamp(format1);
//                    String substring = format1.substring(20);
//                    // fz+当前微秒时间+原唯一订单号
//                    accountPayOrder.setP4_orderId("FZ" + substring + orderRelation.getOrderRequestNo());
//                    //todo 付款账户商编暂用网络
//                    accountPayOrder.setP5_customerNumber(Constant.CUSTOMERNUM2);
//                    AccountPayOrder.AccountPayOrderExt accountPayOrderExt = new AccountPayOrder.AccountPayOrderExt();
//                    //收款账户商编  填写splitBillRules时候不填写MerchantNo，Amount并且即使填写这两个参数不生效！！
////                    accountPayOrderExt.setInMerchantNo(splitMoneyVo.getName());
//                    accountPayOrderExt.setOrderType(AccountPayOrderType.TRANSFER);
////                    accountPayOrderExt.setAmount(splitMoneyVo.getSplitMoney());
//                    accountPayOrderExt.setServerCallbackUrl(callUrl);
//                    accountPayOrderExt.setGoodsName("分账");
//
//                    if (null != splitBillRules && splitBillRules.size() > 0) {
//                        accountPayOrderExt.setSplitBillRules(splitBillRules);
//                    }
//                    String ext = JSON.toJSONString(accountPayOrderExt);
//                    log.info("分账规则串json串：" + ext);
//                    accountPayOrder.setP6_ext(ext);
//                    // 生成签名
//                    StringBuilder builder = new StringBuilder();
//                    builder.append(SPLIT)
//                            .append(accountPayOrder.getP1_bizType()).append(SPLIT)
//                            .append(accountPayOrder.getP2_signType()).append(SPLIT)
//                            .append(accountPayOrder.getP3_timestamp()).append(SPLIT)
//                            .append(accountPayOrder.getP4_orderId()).append(SPLIT)
//                            .append(accountPayOrder.getP5_customerNumber()).append(SPLIT)
//                            .append(accountPayOrder.getP6_ext()).append(SPLIT)
//                            .append(XUNI);
//                    String sign = Disguiser.disguiseMD5(builder.toString().trim());
//                    Map<String, String> bean = convertBean(accountPayOrder);
//                    log.info("--------------------> 发送分账参数：  " + bean);
//                    Map<String, String> map = postForm(bean, Constant.FZ, sign, Map.class);
//                    log.info("----------------分账返回数据: " + map.toString());
//                    if (map != null) {
//                        String code = map.get("rt5_retCode");
//                        if (!"0000".equals(code)) {
//                            String msg = map.get("rt6_retMsg");
//                            log.info("【延时分账】>>>>>>>>>>第三方延迟分账失败>>>>>>>msg:" + msg);
//                        } else {
//                            for (SplitAccountPo splitAccount : splitBillDetail) {
//                                splitAccount.setMbOrderId(orderRelation.getMbOrderId());
//                                splitAccount.setOrderRequestNo(substring + orderRelation.getOrderRequestNo());
//                                splitAccount.setPayStatus(1);
//                                // 保存分账详情
//                                payOrderMapper.insertSplitAccount(splitAccount);
//                            }
//                            log.info("【延时分账】>>>>>>>>>>此订单分账结束");
//                        }
//                    }
//                } catch (Exception e) {
//                    log.error("【延时分账】>>>>>>>>>>错误信息", e);
//                }
//
//            }
//        }
//    }

    @Override
    public String delayedSplittingCallback(AccountResVo data) {
        log.info("延时分账异步回调参数-------------------》 " + data.toString());
        try {
            String oriMessage = MyBeanUtils.getSigned(data, null);
            oriMessage = oriMessage + SPLIT + XUNI;
            String checkSign = Disguiser.disguiseMD5(oriMessage.trim());
            log.info("回调签名 ：" + data.getSign());
            log.info("checkSign ： " + checkSign);
            if (!checkSign.equals(data.getSign())) {
                log.info("延时分账异步回调验签失败------------------");
                return "验签失败";
            }
            // 子订单id
            String shopOrderId = data.getRt7_orderId().substring(5);
            String status = data.getRt10_orderStatus();
            log.info("【延时分账回调】>>>>>>>>>>分账状态:" + status);
            if (!"SUCCESS".equals(status)) {
                return "分账失败";
            }
            // 修改收款分账状态为1
            payOrderMapper.updateBySplitStatusByNo(shopOrderId);
        } catch (Exception e) {
            log.error("【延时分账回调】>>>>>>>>>>分账异步通知异常", e);
            return "分账失败";
        }
        return "SUCCESS";
    }

    //银行通道码获取
    @Override
    public ResponseJson<Map<String, Object>> bankCode() {
        List<BankCodeVo> B2B = payOrderMapper.findB2BBankCode();
        List<BankCodeVo> B2C = payOrderMapper.findB2CBankCode();
        Map<String, Object> banks = new HashMap<>(2);
        banks.put("B2B", B2B);
        banks.put("B2C", B2C);
        return ResponseJson.success("获取成功", banks);
    }

    @Override
    public ResponseJson<JSONObject> getPayOrderResult(String mbOrderId, Integer orderFlag, Integer shopOrderId) throws IntrospectionException, InvocationTargetException, IllegalAccessException {
        log.info("--------进入交易订单查询接口----------");
        //区分网银和其他产品
        QueryOrderVo queryOrderVo = new QueryOrderVo();
        queryOrderVo.setP1_bizType("AppPayQuery");
        queryOrderVo.setP2_orderId(mbOrderId);
        if (0 == orderFlag) {
            queryOrderVo.setP3_customerNumber(Constant.CUSTOMERNUM);
        } else {
            if (null == shopOrderId) {
                return ResponseJson.error("子订单Id不能为空！", null);
            }
            String splitCode = payOrderMapper.getSplitCodeByShopOrderId(shopOrderId);
            if (null == splitCode) {
                return ResponseJson.error("订单信息不正确！", null);
            }
            queryOrderVo.setP3_customerNumber(splitCode);
        }

        Map<String, String> map = MyBeanUtils.convertBean(queryOrderVo, new LinkedHashMap());
        String oriMessage = MyBeanUtils.getSignedByPresetParameter(map, QueryOrderVo.NEED_SIGN_PARAMS);
        oriMessage += Constant.SPLIT + Constant.SAOMA;
        log.info("签名原文串：" + oriMessage);
        String sign = Disguiser.disguiseMD5(oriMessage.trim());
        log.info("md5加密签名串：" + sign);
        map.put("sign", sign);
        log.info("发送参数：" + map);
        Map<String, Object> resultMap = HttpClientService.getHttpResp(map, Constant.REQUEST_URL);
        log.info("响应结果：" + resultMap);
        if ((Integer) resultMap.get("statusCode") == HttpStatus.SC_OK) {
            String resultMsg = (String) resultMap.get("response");
            QueryOrderResponseVo queryOrderResponseVo = JSONObject.parseObject(resultMsg, QueryOrderResponseVo.class);
            String assemblyRespOriSign = MyBeanUtils.getSignedByPresetParameter(queryOrderResponseVo, QueryOrderResponseVo.NEED_SIGN_PARAMS);
            log.info("组装返回结果签名串：" + assemblyRespOriSign);
            assemblyRespOriSign += Constant.SPLIT + Constant.SAOMA;
            String responseSign = queryOrderResponseVo.getSign();
            log.info("响应签名：" + responseSign);
            String checkSign = Disguiser.disguiseMD5(assemblyRespOriSign.trim());
            if (checkSign.equals(responseSign)) {
                if ("0000".equals(queryOrderResponseVo.getRt2_retCode())) {
                    JSONObject jsonObject = JSONObject.parseObject(resultMsg, JSONObject.class);
                    return ResponseJson.success("查询成功", jsonObject);
                } else {
                    return ResponseJson.error("验签失败", null);
                }
            } else {
                return ResponseJson.error("验签失败", null);
            }
        } else {
            return ResponseJson.error("查询失败", null);
        }

    }


    @Override
    public String unionCallback(UnionResVo res) throws IntrospectionException, InvocationTargetException, IllegalAccessException {
        log.info("******************** 网银支付异步回调 start *******************");
        // 签名验证
        String sign = res.getSign();
        log.info("回调签名" + sign);
        String oriMessage = MyBeanUtils.getSignedByPresetParameter(res, UnionResVo.NEED_SIGN_PARAMS);
        oriMessage += SPLIT + WANGYIN;
        String checkSign2 = Disguiser.disguiseMD5(oriMessage.trim());
        if (!sign.equals(checkSign2)) {
            log.error("网银支付异步回调验签失败------------------------------------");
            return "验签名失败！";
        }
        // INIT：未支付
        //SUCCESS:已支付
        //CANCELLED：已取消
        String orderStatus = res.getRt11_orderStatus();
        // 平台唯一流水号
        String mbOrderId = res.getRt12_serialNumber();
        // 商户唯一订单号
        String orderRequestNo = res.getRt5_orderId();
        // 订单金额，以元为单位
        Double amount = Double.valueOf(res.getRt6_orderAmount());
        log.info("【支付异步回调】>>>>>>>>>>>>>>支付订单状态:" + orderStatus);
        if (!"SUCCESS".equals(orderStatus)) {
            return "支付失败";
        }
        String[] split = res.getRt13_desc().split(",");
        //0位置订单id
        Integer orderId = Integer.valueOf(split[0]);
        //1位置支付类型
        String payType = split[1];
        //2位置子订单id
        Integer shopOrderId = Integer.valueOf(split[2]);
        // 订单信息
        OrderVo order = orderCommonMapper.getOrderByOrderId(orderId);
        ShopOrderVo shopOrder = orderCommonMapper.getShopOrderByOrderId(shopOrderId);
        if (null == order) {
            return "订单不存在";
        }
        if (null == shopOrder) {
            return "子订单不存在";
        }
        // 支付记录
        List<DiscernReceiptVo> discernReceiptList = orderCommonMapper.getDiscernReceipt(order.getOrderId(), order.getShopOrderIds());
        double receiptAmount = 0d;
        if (null != discernReceiptList && discernReceiptList.size() > 0) {
            for (DiscernReceiptVo discernReceipt : discernReceiptList) {
                receiptAmount = MathUtil.add(receiptAmount, discernReceipt.getAssociateAmount()).doubleValue();
            }
        }
        order.setReceiptAmount(MathUtil.add(receiptAmount, amount).doubleValue());
        log.info("【支付异步回调】>>>>>>>>>>>>>>已付金额+本次支付金额：" + order.getReceiptAmount());
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        if (MathUtil.compare(order.getPayableAmount(), order.getReceiptAmount()) == 0) {
            /*
             * 订单全部支付
             * 0待确认,11待收待发,12待收部发,13待收全发,21部收待发,22部收部发,23部收全发,
             * 31已收待发,32已收部发,33已收全发,4交易完成,5订单完成,6已关闭,7交易全退
             */
            if (OrderStatus.UNRECEIVED_AND_UNSHIPPED.getCode() == order.getStatus() || OrderStatus.PART_RECEIVED_AND_UNSHIPPED.getCode() == order.getStatus()) {
                order.setStatus(31);
            } else if (OrderStatus.UNRECEIVED_AND_PART_SHIPPED.getCode() == order.getStatus() || OrderStatus.PART_RECEIVED_AND_PART_SHIPPED.getCode() == order.getStatus()) {
                order.setStatus(32);
            } else {
                order.setStatus(33);
            }
            order.setPayFlag(1);
            order.setOnlinePayFlag(0);
            //（收款买家）收款状态：1待收款、2部分收款、3已收款
            order.setReceiptStatus(3);
            log.info("【支付异步回调】>>>>>>>>>>>>>>订单(全部支付),修改订单状态：" + order.getStatus() + "，orderId：" + orderId);
        } else {
            // 部分支付
            if (OrderStatus.UNRECEIVED_AND_UNSHIPPED.getCode() == order.getStatus() || OrderStatus.PART_RECEIVED_AND_UNSHIPPED.getCode() == order.getStatus()) {
                order.setStatus(21);
            } else if (OrderStatus.UNRECEIVED_AND_PART_SHIPPED.getCode() == order.getStatus() || OrderStatus.PART_RECEIVED_AND_PART_SHIPPED.getCode() == order.getStatus()) {
                order.setStatus(22);
            } else {
                order.setStatus(23);
            }
            order.setOnlinePayFlag(0);
            //（收款买家）收款状态：1待收款、2部分收款、3已收款
            order.setReceiptStatus(2);
            log.info("【支付异步回调】>>>>>>>>>>>>>>订单(部分支付),修改订单状态：" + order.getStatus() + "，orderId：" + orderId);
        }
        // 更新付款次数
        Integer paySuccessCounter = (null == order.getPaySuccessCounter()) ? 1 : order.getPaySuccessCounter() + 1;
        order.setPaySuccessCounter(paySuccessCounter);
        order.setUpdateDate(curDateStr);
        // 更新订单支付状态
        payOrderMapper.updateOrderStatus(order);
        // 修改支付链接状态
        OrderPayLinkVo orderPayLink = payOrderMapper.getOrderPayLink(orderId, amount);
        if (null != orderPayLink && ("12".equals(payType) || "17".equals(payType))) {
            orderPayLink.setPayStatus(1);
            payOrderMapper.updateOrderPayLinkStatus(orderPayLink);
        }
        // 支付记录
        //todo 子订单返佣？？？？
        List<DiscernReceiptVo> shopOrderDiscernReceiptList = orderCommonMapper.getShopOrderDiscernReceipt(shopOrderId);
        double shopOrderReceiptAmount = 0d;
        if (null != shopOrderDiscernReceiptList && shopOrderDiscernReceiptList.size() > 0) {
            for (DiscernReceiptVo discernReceipt : shopOrderDiscernReceiptList) {
                shopOrderReceiptAmount = MathUtil.add(shopOrderReceiptAmount, discernReceipt.getAssociateAmount()).doubleValue();
            }
        }
        shopOrder.setReceiptAmount(MathUtil.add(shopOrderReceiptAmount, amount).doubleValue());
        //均摊后needpay:已支付
        if (MathUtil.compare(shopOrder.getRealPay(), shopOrder.getReceiptAmount()) == 0) {
            shopOrder.setReceiptStatus(3);
        } else {
            shopOrder.setReceiptStatus(2);
        }
        payOrderMapper.updateShopOrder(shopOrder);
        // 保存收款记录
        DiscernReceiptPo discernReceipt = new DiscernReceiptPo();
        discernReceipt.setPayWay(1);
        discernReceipt.setPayType(Integer.valueOf(payType));
        discernReceipt.setReceiptType(1);
        discernReceipt.setReceiptStatus(3);
        discernReceipt.setReceiptAmount(amount);
        discernReceipt.setConfirmType(4);
        discernReceipt.setRePayFlag(1);
        discernReceipt.setFormData(JSONObject.toJSONString(res));
        discernReceipt.setReceiptDate(curDateStr);
        discernReceipt.setConfirmDate(curDateStr);
        discernReceipt.setReviewDate(curDateStr);
        discernReceipt.setUpdateDate(curDateStr);
        discernReceipt.setDelFlag(0);
        // 保存 收款记录
        baseMapper.insertDiscernReceipt(discernReceipt);
        log.info("【支付异步回调】>>>>>>>>>>>保存识别款项(insert[cm_discern_receipt])id：" + discernReceipt.getId() + "，orderId：" + orderId);
        // 收款项和订单关系表
        OrderReceiptRelationPo relation = new OrderReceiptRelationPo();
        relation.setReceiptId(discernReceipt.getId());
        relation.setOrderId(orderId);
        relation.setAssociateAmount(amount);
        relation.setMbOrderId(mbOrderId);
        relation.setOrderRequestNo(orderRequestNo);
        relation.setSplitStatus(0);
        relation.setRelationType(2);
        relation.setDelFlag(0);
        relation.setShopOrderId(shopOrderId);
        // 保存 收款项和订单关系
        baseMapper.insertOrderReceiptRelation(relation);
        log.info("【支付异步回调】>>>>>>>>>>>收款项和订单关系(insert[cm_receipt_order_relation])id：" + relation.getId() + "，orderId：" + orderId);
        Integer identity = baseMapper.getIdentityByUserId(order.getUserId());
        //分销人员下单修改报备记录
        if (identity == 8) {
            CmReportingClub byCmReportingClub = cmReportingClubMapper.getByCmReportingClub(new CmReportingClub()
                    .orderId(order.getOrderId())
                    .orderStatus(0)
            );
            //商品报备佣金
            BigDecimal commission = cmReportingClubMapper.getDistributionProductCommission(byCmReportingClub.productId());
            byCmReportingClub
                    .distributionId(shopOrder.getSpId())
                    .lockStatus(1)
                    .lockTime(new Date())
                    .orderStatus(1)
                    .commission(commission)
                    .splitSettleStatus(1)
                    .settleStatus(0);
            cmReportingClubMapper.updateCmReportingClub(byCmReportingClub);
            List<CmReportingMember> cmReportingMemberList = cmReportingMemberMapper.getCmReportingMemberList(new CmReportingMember()
                    .clubReportingId(Integer.valueOf(byCmReportingClub.id()))
                    .auditStatus(2)
            );
            //商品报备成员佣金
            BigDecimal memberCommission = MathUtil.div(commission, cmReportingMemberList.size(), 2);
            cmReportingMemberList.forEach(s -> {
                s.commission(memberCommission).settleStatus(2);
                cmReportingMemberMapper.updateCmReportingMember(s);
            });
        }
        // 线上支付与自主下单送豆（已全部收款），最后一笔线上支付成功后，赠送200采美豆
        if (3 == order.getReceiptStatus() && 0 == order.getSecondHandOrderFlag()&&identity != 8) {
            UserBeansHistoryPo beansHistory = new UserBeansHistoryPo();
            beansHistory.setUserId(order.getUserId());
            beansHistory.setOrderId(orderId);
            beansHistory.setBeansType(6);
            beansHistory.setType(1);
            beansHistory.setNum(200);
            beansHistory.setPushStatus(0);
            beansHistory.setAddTime(date);
            beansHistory.setDelFlag(0);
            // 保存 采美豆使用记录
            baseMapper.insertBeansHistory(beansHistory);
            // 用户采美豆
            Integer userBeans = baseMapper.getUserBeans(order.getUserId());
            // 更新用户剩余采美豆数量
            userBeans = userBeans + beansHistory.getNum();

            // 自主下单且没有促销活动送豆
            Integer orderPromotionsId = payOrderMapper.getOrderPromotionsId(order.getOrderId());
            if (1 == order.getOrderType() && null == orderPromotionsId) {
                // 机构自主下单，根据订单总额的价格区间赠送采美豆数量规则
                Double payTotalFee = order.getPayTotalFee();
                int num;
                if (MathUtil.compare(1000, payTotalFee) > -1) {
                    num = 0;
                } else if (MathUtil.compare(5000, payTotalFee) > -1) {
                    num = 1000;
                } else if (MathUtil.compare(10000, payTotalFee) > -1) {
                    num = 2500;
                } else if (MathUtil.compare(25000, payTotalFee) > -1) {
                    num = 5000;
                } else if (MathUtil.compare(100000, payTotalFee) > -1) {
                    num = 7500;
                } else if (MathUtil.compare(250000, payTotalFee) > -1) {
                    num = 10000;
                } else if (MathUtil.compare(500000, payTotalFee) > -1) {
                    num = 12500;
                } else {
                    num = 20000;
                }
                // 超级会员用户标识
                Integer svipUserId = baseMapper.getSvipUserIdByUserId(order.getUserId());
                if (null != svipUserId && svipUserId.equals(order.getUserId())) {
                    // 超级会员用户采美豆翻倍
                    num = num * 2;
                }
                beansHistory.setBeansType(5);
                beansHistory.setNum(num);
                if (num > 0) {
                    // 保存 采美豆使用记录
                    baseMapper.insertBeansHistory(beansHistory);
                    // 更新用户剩余采美豆数量
                    userBeans = userBeans + beansHistory.getNum();
                }
            }
            baseMapper.updateUserBeans(beansHistory.getUserId(), userBeans);
            log.info("【支付异步回调】>>>>>更新用户采美豆(update[user(userBeans:" + userBeans + ")])，userId:" + beansHistory.getUserId());
        }
        // 已支付短信推送 不是本地或者测试
        boolean smsPushFlag = !orderRequestNo.contains("BETA") && !orderRequestNo.contains("DEV");
        String bindMobile = baseMapper.getBindMobileByUserId(order.getUserId());
        if (smsPushFlag && StringUtils.isNotBlank(bindMobile)) {
            String shortLink = remoteCallService.getShortLink(8, 6, "https://www.caimei365.com/user/order/detail.html?orderId=" + orderId);
            String content = "您已成功支付订单（订单编号：" + order.getOrderNo() + "）全部款项，支付总金额￥" + order.getPayTotalFee() + "，采美平台将立即安排发货。您可关注采美公众号或者访问采美微信小程序和网站查看订单。" +
                    "平台公众号：微信搜索“采美365网”; 微信小程序：微信搜索“采美采购商城”；网址：www.caimei365.com/t/" + shortLink;
            boolean sendSms = remoteCallService.getSendSms(1, 6, bindMobile, content);
            if (!sendSms) {
                log.info("取消订单推送失败，orderId>>>>" + orderId);
            }
        }
        return "SUCCESS";
    }

    @Override
    public ResponseJson<List<ShopOrderVo>> getShopOrders(Integer orderId, Integer shopOrderId) {
        String[] shopOrderIdArr;
        List<String> shopOrderIds = new ArrayList<>();
        if (null != orderId) {
            String getShopOrderId = baseMapper.findShopOrderIds(orderId);
            // 子订单
            if (StringUtils.isBlank(getShopOrderId)) {
                return ResponseJson.error("子订单为空！", null);
            }
             shopOrderIdArr = getShopOrderId.split(",");
             shopOrderIds = Arrays.asList(shopOrderIdArr);
        } else {
            if (null == shopOrderId) {
                return ResponseJson.error("子订单为空！", null);
            }
            shopOrderIds.add(shopOrderId.toString());
        }
        List<ShopOrderVo> shopOrderList = orderCommonMapper.getPayShopOrderList(shopOrderIds);
        shopOrderList.forEach(shopOrder -> {
            // 店铺促销活动
            PromotionsVo shopPromotion = null;
            if (null != shopOrder.getOrderPromotionsId() && shopOrder.getOrderPromotionsId() > 0) {
                shopPromotion = orderCommonMapper.getOrderPromotionsById(shopOrder.getOrderPromotionsId());
                shopOrder.setShopPromotion(shopPromotion);
            }
            AtomicReference<Boolean> reference = new AtomicReference<>(true);
            List<OrderProductVo> orderProductList = orderCommonMapper.getShopOrderProduct(shopOrder.getShopOrderId());
            orderProductList.removeIf(Objects::isNull);
            orderProductList.forEach(orderProduct -> {
                // 不含税可开票商品,单价/折后单价在原基础上加上税费
                boolean taxFlag = (Integer.valueOf(0).equals(orderProduct.getIncludedTax()) && (Integer.valueOf(1).equals(orderProduct.getInvoiceType()) || Integer.valueOf(2).equals(orderProduct.getInvoiceType())));
                if (taxFlag) {
                    Double valueTax = MathUtil.div(MathUtil.mul(orderProduct.getPrice(), orderProduct.getTaxRate()), 100).doubleValue();
                    // orderProduct.setPrice(MathUtil.add(orderProduct.getPrice(), valueTax).doubleValue());
                    // orderProduct.setDiscountPrice(MathUtil.add(orderProduct.getDiscountPrice(), orderProduct.getAddedValueTax()).doubleValue());
                }
                orderProduct.setImage(ImageUtil.getImageUrl("product", orderProduct.getImage(), domain));
                // 查询订单下商品的促销活动
                if (null != orderProduct.getOrderPromotionsId() && orderProduct.getOrderPromotionsId() > 0) {
                    PromotionsVo promotions = orderCommonMapper.getOrderPromotionsById(orderProduct.getOrderPromotionsId());
                    if (null != promotions) {
                        if (1 == promotions.getType() && 1 == promotions.getMode()) {
                            //单品优惠价的时候取当前sku的touchprice
                            promotions.setTouchPrice(baseMapper.getTouchPriceBySku(orderProduct.getSkuId(), promotions.getId()));
                        }
                        if (taxFlag && Integer.valueOf(1).equals(promotions.getType()) && Integer.valueOf(1).equals(promotions.getMode())) {
                            promotions.setTouchPrice(MathUtil.add(promotions.getTouchPrice(), MathUtil.div(MathUtil.mul(promotions.getTouchPrice(), orderProduct.getTaxRate()), 100)).doubleValue());
                        }
                        orderProduct.setProductPromotion(promotions);
                    }
                }
                // 超级会员优惠商品设置优惠标签
                if (null != orderProduct.getSvipPriceFlag() && 1 == orderProduct.getSvipPriceFlag()) {
                    if (1 == orderProduct.getSvipPriceType()) {
                        orderProduct.setSvipPriceTag(MathUtil.div(orderProduct.getSvipDiscount(), 10, 1) + "折");
                    } else if (2 == orderProduct.getSvipPriceType()) {
                        orderProduct.setSvipPriceTag("￥" + orderProduct.getDiscountPrice());
                    }
                }
                //前端要求把price置为折后单价，totalAmount置为折后总价
                if (null != orderProduct.getDiscountPrice()) {
                    orderProduct.setPrice(orderProduct.getDiscountPrice());
                }
                if (null != orderProduct.getTotalFee()) {
                    orderProduct.setTotalAmount(orderProduct.getTotalFee());
                }
                // 商品分帐号
                if (StringUtils.isEmpty(orderProduct.getSplitCode()) || StringUtils.isBlank(orderProduct.getSplitCode())) {
                    reference.set(false);
                }
            });
            if (null == shopOrder.getRealPay()) {
                shopOrder.setRealPay(shopOrder.getNeedPayAmount());
            }
            shopOrder.setOrderProductList(orderProductList);
            shopOrder.setShopLogo(ImageUtil.getImageUrl("shopLogo", shopOrder.getShopLogo(), domain));
            // 子订单支付方式
            AtomicReference<Integer> onlinePay = new AtomicReference<>(0);

            // 供应商是否支持线上下线
            if (null != shopOrder.getPayway()) {
                onlinePay.set(shopOrder.getPayway());
            }
            // 线上分帐号
            String shopCode = payOrderMapper.getShopCode(shopOrder.getShopId());
            if (null == shopCode) {
                onlinePay.set(2);
            }
            // 支付记录
            List<DiscernReceiptVo> shopOrderReceipt = orderCommonMapper.getShopOrderReceipt(shopOrder.getShopOrderId());
            AtomicDouble finalReceiptAmount = new AtomicDouble(0d);
            // 1线上，2线下，3余额抵扣
            if (null != shopOrderReceipt) {
                shopOrderReceipt.forEach(receipt -> {
                    finalReceiptAmount.set(MathUtil.add(finalReceiptAmount.get(), receipt.getAssociateAmount()).doubleValue());
                    if (1 == receipt.getPayWay()) {
                        onlinePay.set(1);
                    } else {
                        onlinePay.set(2);
                    }
                });
            }
            // 商品无分帐号线下
            if (!reference.get()) {
                onlinePay.set(2);
            }
            // 余额抵扣只能下线
            if (1 == shopOrder.getUseBalanceFlag()) {
                onlinePay.set(2);
            }
            shopOrder.setReceiptAmount(MathUtil.add(shopOrder.getAccountAmount(), finalReceiptAmount.get()).doubleValue());
            // 待付金额
            shopOrder.setObligation(MathUtil.sub(shopOrder.getRealPay(), shopOrder.getReceiptAmount()).doubleValue());
            if (998 == shopOrder.getShopId()) {
                shopOrder.setObligation(shopOrder.getTotalAmount());
            }
            shopOrder.setOnlinePay(onlinePay.get());
            orderCommonService.setPostageInfo(shopOrder);
        });
        return ResponseJson.success(shopOrderList);
    }

    @Override
    public ResponseJson<JSONObject> getUnionResult(String mbOrderId, Integer orderFlag, Integer shopOrderId) {
        String customerNumber = "";
        if (0 == orderFlag) {
            customerNumber = Constant.CUSTOMERNUM;
        } else {
            if (null == shopOrderId) {
                return ResponseJson.error("子订单Id不能为空！", null);
            }
            String splitCode = payOrderMapper.getSplitCodeByShopOrderId(shopOrderId);
            if (null == splitCode) {
                return ResponseJson.error("订单信息不正确！", null);
            }
            customerNumber = splitCode;
        }
        Map<String, String> sPara = new HashMap<String, String>();
        sPara.put("P1_bizType", "OnlineQuery");
        sPara.put("P2_orderId", mbOrderId);
        sPara.put("P3_customerNumber", customerNumber);
        StringBuffer sb = new StringBuffer();
        sb.append("&").append("OnlineQuery").append("&").append(mbOrderId).append("&")
                .append(customerNumber).append("&").append(WANGYIN);
        String sign = Disguiser.disguiseMD5(sb.toString());
        sPara.put("sign", sign);
        try {
            OnlineCheckVo on = OnlinePostFormUtil.postForm(sPara, YL, sign, OnlineCheckVo.class);
            JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(on), JSONObject.class);
            if (!"0000".equals(on.getRt2_retCode())) {
                log.info("查询失败:" + on.toString());
                return ResponseJson.error("查询失败", jsonObject);
            } else {
                String assemblyRespOriSign = MyBeanUtils.getSignedByPresetParameter(on, OnlineCheckVo.NEED_SIGN_PARAMS);
                log.info("组装返回结果签名串：" + assemblyRespOriSign);
                assemblyRespOriSign += Constant.SPLIT + WANGYIN;
                String checkSign = Disguiser.disguiseMD5(assemblyRespOriSign.trim());
                if (!checkSign.equals(on.getSign())) {
                    return ResponseJson.error("验签失败", jsonObject);
                } else {
                    return ResponseJson.success("查询成功", jsonObject);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public ResponseJson<Map<String, String>> quickPay(HeliDto heliDto, HttpHeaders headers) {
        FirstQuickPayVo firstQuickPayVo = new FirstQuickPayVo();
        log.info("helidto------------" + heliDto);
        heliPayUtil.setQuickValue(heliDto, firstQuickPayVo, headers);
        if (Double.parseDouble(firstQuickPayVo.getP15_orderAmount()) <= 0.1) {
            return ResponseJson.error("订单支付金额不能小于最低手续费！");
        }
        Map reqestMap = null;
        try {
            reqestMap = MessageHandle.getReqestMap(firstQuickPayVo);
            log.info("首次支付下单请求参数：" + reqestMap);
            Map<String, Object> resultMap = HttpClientService.getHttpResp(reqestMap, Constant.REQUEST_URL_QUICKPAY);
            log.info("首次支付下单响应结果：" + resultMap);
            String resultMsg = (String) resultMap.get("response");
            if ((Integer) resultMap.get("statusCode") != org.apache.commons.httpclient.HttpStatus.SC_OK) {
                //状态码不对请求失败
                log.info("-----------------请求失败------------------");
                return ResponseJson.error("请求失败");
            }
            if (!MessageHandle.isJSON(resultMsg)) {
                //状态码不对请求失败
                log.info("-----------------交易失败------------------");
                return ResponseJson.error("交易失败");
            }
            QuickPayCreateOrderResponseVo responseVo = JSONObject.parseObject(resultMsg, QuickPayCreateOrderResponseVo.class);
            if (!MessageHandle.checkSign(responseVo)) {
                log.info("-----------------验签失败------------------");
                return ResponseJson.error("验签失败");
            }
            if (!"0000".equals(responseVo.getRt2_retCode()) || "FAIL".equals(responseVo.getSmsStatus())) {
                log.info("交易失败-------------->" + responseVo.getRt3_retMsg());
                return ResponseJson.error(responseVo.getSmsMsg());
            }

//            /**
//             * 手动发短信
//             */
//            FirstPaySendValidateCodeVo sendVo = new FirstPaySendValidateCodeVo();
//            sendVo.setP1_bizType("FirstPaySendValidateCode");
//            sendVo.setP2_customerNumber(firstQuickPayVo.getP2_customerNumber());
//            sendVo.setP3_orderId(firstQuickPayVo.getP4_orderId());
//            sendVo.setP4_timestamp(firstQuickPayVo.getP5_timestamp());
//            sendVo.setSignatureType("SM3WITHSM2");
//            Map req = MessageHandle.getReqestMap(sendVo);
//            Map<String, Object> reqMap = HttpClientService.getHttpResp(req, Constant.REQUEST_URL_QUICKPAY);
//            log.info("发送短信响应结果：" + reqMap);
//            if ((Integer) resultMap.get("statusCode") != org.apache.commons.httpclient.HttpStatus.SC_OK) {
//                log.info("发送短信请求失败！");
//                return ResponseJson.error("发送短信请求失败!");
//            }
//            String reqMsg = (String) reqMap.get("response");
//            QuickPaySendValidateCodeResponseVo respVo = JSONObject.parseObject(reqMsg, QuickPaySendValidateCodeResponseVo.class);
//            if (!MessageHandle.checkSign(respVo)) {
//                log.info("发送短信验签失败！");
//                return ResponseJson.error("发送短信验签失败!");
//            }
//            if (!"0000".equals(respVo.getRt2_retCode())) {
//                log.info("短信发送失败！");
//                return ResponseJson.error("短信发送失败!");
//            }


            HashMap<String, String> map = new HashMap<>();
            map.put("customer", responseVo.getRt4_customerNumber());
            map.put("orderId", responseVo.getRt5_orderId());
            return ResponseJson.success(map);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public String quickPayCallback(QuickPayConfirmPayResponseVo res) {
        try {
            if (MessageHandle.checkSign(res)) {
                if ("SUCCESS".equals(res.getRt9_orderStatus())) {
                    log.info("快捷支付付款成功============================> res:" + res);
                    /**
                     * 验签通过 查看交易状态码是否成功，是则继续业务逻辑
                     * 保存身份证，银行卡，姓名，银行编码, 合利宝绑定ip 若已经存在，则不保存
                     */
                    String s = checkOrderStatus(res, 2);
                    if ("SUCCESS".equals(s)) {
                        return "SUCCESS";
                    }
                } else {
                    // 支付验证码校验失败删除银行卡信息
                    log.info("快捷支付付款状态异常============================> res:" + res);
                    delQuickInfo(res);
                }
            } else {
                // 支付验证码校验失败删除银行卡信息
                log.info("快捷支付回调验签失败=========================> res:" + res);
                delQuickInfo(res);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public void delQuickInfo(QuickPayConfirmPayResponseVo res) {
        // 未成功删除cm_quickpay_info中预存的userInfo信息
        String[] s = res.getRt5_orderId().split("_");
        String infoId = s[3];
        QuickInfoVo infoNotDel = payOrderMapper.findInfoNotDel(infoId);
        if (null == infoNotDel) {
            payOrderMapper.delQuickInfo(infoId);
        }
    }

    @Override
    public ResponseJson<Map<String, String>> bindBankCard(HeliDto heliDto) {
        // 校验是否已绑
        QuickInfoVo infoByBankNumber = payOrderMapper.findInfoByBankNumber(heliDto.getQuickPayBankNumber());
        if (null != infoByBankNumber && null != infoByBankNumber.getId()) {
            return ResponseJson.error("卡已绑定！");
        }
        log.info("helidto-------------" + heliDto);
        QuickPayBindCardPreOrderVo confirmBindCardVo = new QuickPayBindCardPreOrderVo();
        String time = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
        String orderId = "bd_" + heliDto.getUserId() + "_" + time;
        confirmBindCardVo.setP1_bizType("QuickPayBindCardPreOrder");
        //一次请求只能绑定一个e商户，暂定绑定网络，有支付请求走预下单直接绑定
        confirmBindCardVo.setP2_customerNumber(CUSTOMERNUM2);
        confirmBindCardVo.setP3_userId("U" + heliDto.getUserId());
        confirmBindCardVo.setP4_orderId(orderId);
        confirmBindCardVo.setP5_timestamp(time);
        confirmBindCardVo.setP6_payerName(heliDto.getQuickPayUserName());
        confirmBindCardVo.setP7_idCardType("IDCARD");
        confirmBindCardVo.setP8_idCardNo(heliDto.getIdCard());
        confirmBindCardVo.setP9_cardNo(heliDto.getQuickPayBankNumber());
        if (StringUtils.isNotBlank(heliDto.getQuickPayBankExpireTime())) {
            String[] split = heliDto.getQuickPayBankExpireTime().split("-");
            confirmBindCardVo.setP10_year(split[0]);
            confirmBindCardVo.setP11_month(split[1]);
            confirmBindCardVo.setP12_cvv2(heliDto.getCvvCode());
        }
        confirmBindCardVo.setP13_phone(heliDto.getQuickPayMobile());
        confirmBindCardVo.setProtocolType("protocol");
        confirmBindCardVo.setSendValidateCode("true");
        confirmBindCardVo.setSignatureType("SM3WITHSM2");
        QuickInfoVo quick = new QuickInfoVo();
        Map reqestMap = null;
        Map<String, String> map = new HashMap<>();
        try {
            reqestMap = MessageHandle.getReqestMap(confirmBindCardVo);
            Map<String, Object> resultMap = HttpClientService.getHttpResp(reqestMap, Constant.REQUEST_URL_QUICKPAY);
            log.info("响应结果：" + resultMap);
            if ((Integer) resultMap.get("statusCode") != org.apache.commons.httpclient.HttpStatus.SC_OK) {
                log.info("请求失败--------------------");
                return ResponseJson.error("请求失败！");
            }
            String resultMsg = (String) resultMap.get("response");
            BindCardPreOrderResponseVo responseVo = JSONObject.parseObject(resultMsg, BindCardPreOrderResponseVo.class);
            if (!MessageHandle.checkSign(responseVo)) {
                log.info("验签失败");
                return ResponseJson.error("验签失败！");
            }
            if (!"0000".equals(responseVo.getRt2_retCode())) {
                log.info("短信发送失败------------------------");
                return ResponseJson.error("短信发送失败！");
            }
            //绑定成功，存卡
            quick.setBankNumber(heliDto.getQuickPayBankNumber());
            quick.setBindMobile(heliDto.getQuickPayMobile());
            quick.setUserId(heliDto.getUserId());
            quick.setQuickPayUserId("U" + heliDto.getUserId());
            quick.setIdCard(heliDto.getIdCard());
            quick.setBankUserName(heliDto.getQuickPayUserName());
            if (null != heliDto.getQuickPayBankExpireTime()) {
                quick.setBankCvvCode(heliDto.getCvvCode());
                quick.setBankExpireTime(heliDto.getQuickPayBankExpireTime());
            }
            payOrderMapper.insertQuickPayInfo(quick);
        } catch (Exception e) {
            e.printStackTrace();
        }
        map.put("orderId", orderId);
        map.put("infoId", quick.getId().toString());
        return ResponseJson.success(map);
    }

    @Override
    public ResponseJson bindByCode(Integer flag, String bindCode, String orderId, String infoId, String customer) {
        ConfirmBindCardVo confirmBindCardVo = new ConfirmBindCardVo();
        Object response = new Object();
        //绑卡ConfirmBindCard  首次支付FirstPayConfirmPay  flag 1绑卡2付款
        if (1 == flag) {
            confirmBindCardVo.setP1_bizType("ConfirmBindCard");
            confirmBindCardVo.setP2_customerNumber(CUSTOMERNUM2);
        } else {
            confirmBindCardVo.setP1_bizType("FirstPayConfirmPay");
            confirmBindCardVo.setP2_customerNumber(CUSTOMERNUM);
        }
        if (StringUtils.isNotBlank(customer)) {
            //下单商户号随动
            confirmBindCardVo.setP2_customerNumber(customer);
        }
        confirmBindCardVo.setP3_orderId(orderId);
        confirmBindCardVo.setP4_timestamp(new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()));
        confirmBindCardVo.setP5_validateCode(bindCode);
        confirmBindCardVo.setSignatureType("SM3WITHSM2");
        Map reqestMap = null;
        try {
            log.info("验证码发送参数---------------------" + confirmBindCardVo);
            reqestMap = MessageHandle.getReqestMap(confirmBindCardVo);
            Map<String, Object> resultMap = HttpClientService.getHttpResp(reqestMap, Constant.REQUEST_URL_QUICKPAY);
            log.info("响应结果：" + resultMap);
            if ((Integer) resultMap.get("statusCode") != org.apache.commons.httpclient.HttpStatus.SC_OK) {
                log.info("--------------请求失败----------------");
                //失败删除info
                payOrderMapper.delQuickInfo(infoId);
                return ResponseJson.error("请求失败");
            }
            String resultMsg = (String) resultMap.get("response");
            QuickInfoVo infoById = payOrderMapper.findInfoNotDel(infoId);
            if (!MessageHandle.isJSON(resultMsg)) {
                //失败删除info
                if (null == infoById) {
                    payOrderMapper.delQuickInfo(infoId);
                }
                log.info("-------------------验证码校验失败！------------------");
                return ResponseJson.error("验证码校验失败！");
            }
            if (1 == flag) {
                response = JSONObject.parseObject(resultMsg, ConfirmBindCardResponseVo.class);
            } else {
                response = JSONObject.parseObject(resultMsg, QuickPayConfirmPayResponseVo.class);
            }
            if (!MessageHandle.checkSign(response)) {
                //失败删除info
                if (null == infoById) {
                    payOrderMapper.delQuickInfo(infoId);
                }
                log.info("------------验签失败---------------");
                return ResponseJson.error("验签失败！");
            }
            if (1 == flag) {
                ConfirmBindCardResponseVo res = (ConfirmBindCardResponseVo) response;
                // 8015短信验证码错误或已过期
                if ("8015".equals(res.getRt2_retCode())) {
                    return ResponseJson.error(-2, res.getRt3_retMsg());
                } else if (!"0000".equals(res.getRt2_retCode())) {
                    //失败删除info
                    if (null == infoById) {
                        payOrderMapper.delQuickInfo(infoId);
                    }
                    log.info("------------绑定失败---------------");
                    return ResponseJson.error("绑定失败！");
                }
                /**
                 *  绑卡接口返回值无银行卡类型 需要查询绑卡信息
                 */
                BindCardQueryVo bindCardQueryVo = new BindCardQueryVo();
                bindCardQueryVo.setP1_bizType("BankCardbindList");
                bindCardQueryVo.setP2_customerNumber(confirmBindCardVo.getP2_customerNumber());
                bindCardQueryVo.setP3_userId(res.getRt5_userId());
                bindCardQueryVo.setP4_bindId(res.getRt10_bindId());
                bindCardQueryVo.setP5_timestamp(new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()));
                bindCardQueryVo.setSignatureType("SM3WITHSM2");
                BindCardQueryResponseVo bindCardInfo = getBindCardInfo(bindCardQueryVo);
                ObjectMapper objectMapper = new ObjectMapper();
                List<Map<String, String>> objectList = objectMapper.readValue(bindCardInfo.getRt5_bindCardList(), List.class);
                AtomicReference<String> cartType = new AtomicReference<>("");
                objectList.forEach(o -> cartType.set(o.get("cardType")));
                Integer bankType = "DEBIT".equals(cartType.get()) ? 1 : 2;
                log.info("cartType---------" + cartType.get().toString());
                /**
                 *  绑卡修改剩余信息
                 */
                payOrderMapper.updateQuickInfo(res.getRt8_bankId(), bankType, Integer.valueOf(infoId));
                payOrderMapper.insertBindId(res.getRt10_bindId(), res.getRt4_customerNumber(), res.getRt5_userId().substring(1), Integer.valueOf(infoId));
            } else {
                QuickPayConfirmPayResponseVo res = (QuickPayConfirmPayResponseVo) response;
                String[] s = res.getRt5_orderId().split("_");
                boolean vp = orderId.startsWith("VP");
                boolean avp = orderId.startsWith("AVP");
                boolean cp = orderId.startsWith("CP");
                if (vp || avp) {
                    infoId = s[4];
                } else if (cp) {
                    infoId = s[5];
                } else {
                    infoId = s[3];
                }
                QuickInfoVo info = payOrderMapper.findInfoNotDel(infoId);
                // 8015短信验证码错误或已过期
                if ("8015".equals(res.getRt2_retCode())) {
                    if (null == info) {
                        payOrderMapper.delQuickInfo(infoId);
                    }
                    log.info("------------下单支付验证码错误或过期---------------");
                    return ResponseJson.error(-2, res.getRt3_retMsg());
                } else if (!"0000".equals(res.getRt2_retCode())) {
                    if (null == info) {
                        payOrderMapper.delQuickInfo(infoId);
                    }
                    // 下单支付在回调中修改绑卡信息
                    log.info("------------下单支付验证码校验失败---------------");
                    return ResponseJson.error(res.getRt3_retMsg());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResponseJson.success();
    }

    public BindCardQueryResponseVo getBindCardInfo(BindCardQueryVo bindCardQueryVo) {
        try {
            Map reqestMap = MessageHandle.getReqestMap(bindCardQueryVo);
            Map<String, Object> resultMap = HttpClientService.getHttpResp(reqestMap, Constant.REQUEST_URL_QUICKPAY);
            log.info("响应结果：" + resultMap);
            if ((Integer) resultMap.get("statusCode") != org.apache.commons.httpclient.HttpStatus.SC_OK) {
                log.info("请求失败");
            }
            String resultMsg = (String) resultMap.get("response");
            BindCardQueryResponseVo responseVo = JSONObject.parseObject(resultMsg, BindCardQueryResponseVo.class);
            if (!MessageHandle.checkSign(responseVo)) {
                log.info("验签失败");
            }
            return responseVo;
        } catch (Exception e) {
            e.printStackTrace();
            log.info("交易异常：" + e.getMessage());
        }
        return null;
    }

    @Override
    public ResponseJson<List<BankListDto>> bankInfos(Integer cardType) {
        List<String> ins = payOrderMapper.findQuickBankInitial(cardType);
        List<BankListDto> bankList = new ArrayList<>();
        ins.forEach(i -> {
            BankListDto bankListDto = new BankListDto();
            bankListDto.setInitial(i);
            bankListDto.setBanks(payOrderMapper.findQuickBank(cardType, i));
            bankList.add(bankListDto);
        });
        return ResponseJson.success(bankList);
    }

    @Override
    public ResponseJson unBoundCard(Integer cardId) {
//        BankCardUnbindVo unbind = new BankCardUnbindVo();
        QuickInfoVo info = payOrderMapper.getQuickInfoById(cardId);
        if (null == info) {
            return ResponseJson.error("当前卡不存在！");
        }
//        String time = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
//        unbind.setP1_bizType("BankCardUnbind");
//        unbind.setP2_customerNumber(info.getCustomer());
//        unbind.setP3_userId(info.getQuickPayUserId());
//        unbind.setP4_bindId(info.getQuickPaybindId());
//        unbind.setP5_orderId("ub_" + time + cardId);
//        unbind.setP6_timestamp(time);
//        unbind.setSignatureType("SM3WITHSM2");
//        try {
//            Map reqestMap = MessageHandle.getReqestMap(unbind);
//            Map<String, Object> resultMap = HttpClientService.getHttpResp(reqestMap, REQUEST_URL_QUICKPAY);
//            log.info("响应结果：" + resultMap);
//            if ((Integer) resultMap.get("statusCode") != org.apache.commons.httpclient.HttpStatus.SC_OK) {
//                log.info("----------------解绑银行卡请求失败-----------------");
//                return ResponseJson.error("请求失败！");
//            }
//            String resultMsg = (String) resultMap.get("response");
//            BankCardUnbindResponseVo responseVo = JSONObject.parseObject(resultMsg, BankCardUnbindResponseVo.class);
//            if (!MessageHandle.checkSign(responseVo)) {
//                log.info("------------------------解绑银行卡验签失败-------------------");
//                return ResponseJson.error("解绑银行卡验签失败！");
//            }
//            if (!"0000".equals(responseVo.getRt2_retCode())) {
//                log.info("-----------------银行卡解绑失败---------------------");
//                return ResponseJson.error("银行卡解绑失败！");
//            }
        /** 解绑成功 cm_quickpay_info置为删除
         *  当前版本一次只能绑定/解绑一个商户号，请求解绑暂无意义
         */
        payOrderMapper.delQuickInfo(cardId.toString());
//            payOrderMapper.delQuickBindId(info.getQuickPaybindId());
//        } catch (Exception e) {
//            e.printStackTrace();
//            log.info("交易异常：" + e.getMessage());
//        }
        return ResponseJson.success("解绑成功");
    }

    @Override
    public ResponseJson<String> bindCardNoCode(HeliDto heliDto) {
        OnceBindCardVo once = new OnceBindCardVo();
        once.setP1_bizType("QuickBindCard");
        once.setP2_customerNumber("E1807062884");
        once.setP3_userId("U" + heliDto.getUserId());
        String time = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
        once.setP4_orderId("bind_" + time + "_" + heliDto.getUserId());
        once.setP5_timestamp(time);
        once.setP6_payerName(heliDto.getQuickPayUserName());
        once.setP7_idCardType("IDCARD");
        once.setP8_idCardNo(heliDto.getIdCard());
        once.setP9_phone(heliDto.getQuickPayMobile());
        once.setP10_bankId(heliDto.getBankCode());
        String cardType = 1 == heliDto.getCardType() ? "DEBIT" : "CREDIT";
        once.setP11_onlineCardType(cardType);
        once.setP12_serverCallbackUrl("http://localhost:8080/manager/text");
        once.setP13_bankRdrctToMchUrl("www.caimei365.com");
        once.setSignatureType("MD5WITHRSA");
        Map reqestMap = null;
        String html = "";
        try {
            reqestMap = MessageHandle.getReqestMap(once);
            log.info("请求参数：" + reqestMap);
            Map<String, Object> resultMap = HttpClientService.getHttpResp(reqestMap, REQUEST_URL_QUICKPAY);
            log.info("响应结果：" + resultMap);
            if ((Integer) resultMap.get("statusCode") != org.apache.commons.httpclient.HttpStatus.SC_OK) {
                log.info("请求失败");
            }
            String resultMsg = (String) resultMap.get("response");
            OnceBindResponseVo responseVo = JSONObject.parseObject(resultMsg, OnceBindResponseVo.class);
            if (!MessageHandle.checkSign(responseVo)) {
                log.info("验签失败");
            }
            if (!"0000".equals(responseVo.getRt2_retCode())) {
                log.info(responseVo.getRt3_retMsg());
            }
            html = responseVo.getRt5_formData();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResponseJson.success(html);
    }

    @Override
    public ResponseJson<List<QuickBankVo>> unCodeInfos() {
        /**
         * 快捷支付免密绑卡支持银行
         */
        List<QuickBankVo> banks = payOrderMapper.findUnCodeBanks();
        return ResponseJson.success(banks);
    }

    @Override
    public String refundSplittingCallback(AccountResVo data) {
        log.info("网银退款分账异步回调参数-------------------》 " + data.toString());
        try {
            String oriMessage = MyBeanUtils.getSigned(data, null);
            oriMessage = oriMessage + SPLIT + XUNI;
            String checkSign = Disguiser.disguiseMD5(oriMessage.trim());
            log.info("网银退款回调签名 ：" + data.getSign());
            log.info("checkSign ： " + checkSign);
            if (!checkSign.equals(data.getSign())) {
                log.info("网银退款分账异步回调验签失败------------------");
                return "验签失败";
            }
            // wytk+sss + orderRequestNo
            String orderRequestNo = data.getRt7_orderId().substring(7);
            String status = data.getRt10_orderStatus();
            log.info("【网银退款分账回调】>>>>>>>>>>分账状态:" + status);
            if (!"SUCCESS".equals(status)) {
                return "分账失败";
            }
            OrderReceiptRelationPo po = refundMapper.findOrderRelationByNo(orderRequestNo);
            // 网银退款
            Map<String, String> sPara = new HashMap<String, String>();
            sPara.put("P1_bizType", "OnlineRefund");
            sPara.put("P2_orderId", orderRequestNo);
            sPara.put("P3_customerNumber", po.getSplitCode());
            sPara.put("P4_refundAmount", po.getAssociateAmount().toString());
            sPara.put("P5_refundOrderId", "TK" + orderRequestNo);
            sPara.put("P6_desc", "无");
            sPara.put("P7_serverCallbackUrl", "");
            StringBuffer sb = new StringBuffer();
            sb.append("&").append("OnlineRefund").append("&").append(orderRequestNo).append("&").append(po.getSplitCode()).append("&").
                    append(po.getAssociateAmount().toString()).append("&").append("TK" + orderRequestNo).append("&").append("无").append("&").append("&").append(Constant.WANGYIN);
            String sign = Disguiser.disguiseMD5(sb.toString());
            sPara.put("sign", sign);
            OnlineRefundData on = OnlinePostFormUtil.refundPostForm(sPara, Constant.YL, sign, OnlineRefundData.class);
            if (!"0000".equals(on.getRt2_retCode())) {
                log.info("网银退款失败:" + on.toString());
                return "分账失败";
            } else {
                String assemblyRespOriSign = MyBeanUtils.getSignedByPresetParameter(on, OnlineRefundData.NEED_SIGN_PARAMS);
                log.info("组装返回结果签名串：" + assemblyRespOriSign);
                assemblyRespOriSign += Constant.SPLIT + Constant.WANGYIN;
                String backSign = Disguiser.disguiseMD5(assemblyRespOriSign.trim());
                if (!backSign.equals(on.getSign())) {
                    log.error("网银退款验签失败========================" + on.getRt3_retMsg());
                    return "分账失败";
                } else {
                    CmRefundRecord cmRefundRecord = new CmRefundRecord();
                    cmRefundRecord.setRefundAmount(Double.parseDouble(po.getAssociateAmount().toString()));
                    cmRefundRecord.setOrderRequestNo(orderRequestNo);
                    cmRefundRecord.setShopOrderId(po.getShopOrderId());
                    refundMapper.insertCmRefundRecord(cmRefundRecord);
                    log.info("网银退款成功");
                }
            }
        } catch (Exception e) {
            log.error("【网银退款手续费转账失败】>>>>>>>>>>网银退款分账异常", e);
            return "分账失败";
        }
        return "SUCCESS";
    }

    public static <T> T postForm(Map<String, String> params, String url, String sign, Class<T> clazz) {
        FormBody.Builder builder = new FormBody.Builder();
        for (Map.Entry<String, String> entry : params.entrySet()) {
            builder.add(entry.getKey(), entry.getValue());
        }
        builder.add("sign", sign);

        Request request = new Request.Builder() // okHttp post
                .url(url)
                .post(builder.build())
                .build();

        Response response = null;
        try {
            response = client.newCall(request).execute();
        } catch (IOException e) {
            throw new IllegalStateException("请求出错", e);
        }
        if (!response.isSuccessful()) {
            try {
                log.info(response.body().string());
            } catch (IOException e) {
                e.printStackTrace();
            }
            throw new RuntimeException("请求失败了: http response code: " + response.code());
        }

        ResponseBody body = response.body();
        String content = null;
        try {
            content = body.string();
        } catch (IOException e) {
            throw new IllegalStateException("IO异常", e);
        }
        JSONObject res = JSON.parseObject(content);
        if (!res.getBooleanValue("rt4_success")) {
            log.error("error: " + res.getString("rt6_retMsg"));
        }
        /** rt4_success 为 true，需验签  **/
        return res.toJavaObject(clazz);
    }

    public static Map<String, String> convertBean(Object bean) {
        Class clazz = bean.getClass();
        Field[] fields = clazz.getDeclaredFields();
        for (Field f : fields) {
            f.setAccessible(true);
        }
        try {
            Map<String, String> retMap = new LinkedHashMap<>();
            for (Field f : fields) {
                String key = f.toString().substring(f.toString().lastIndexOf(".") + 1);
                Object value = f.get(bean);
                if (value == null) {
                    value = "";
                }
                retMap.put(key, (String) value);
            }
            return retMap;
        } catch (Exception e) {
            log.info("分账", e);
            throw new IllegalStateException("分账异常", e);
        }
    }


    /**
     * 分账详情
     */
    private List<SplitAccountPo> setSplitAccountDetail(OrderVo order, PayParamBo payParam) {
        List<SplitAccountPo> list = new ArrayList<>();
        // 本次支付金额,单位/元
        double payAmount = MathUtil.div(payParam.getPayAmount(), 100).doubleValue();
        // 待分账总金额
        double splitAmount = payAmount;
        // 总手续费
        double procedureFee;
        if ("UNIONPAY".equals(payParam.getPayWay())) {
            procedureFee = 10.00;
        } else if ("B2C".equals(payParam.getPayWay())) {
            //b2c 0.2%
            procedureFee = MathUtil.mul(payAmount, 0.002, 2).doubleValue();
            //b2c最低手续费0.1
            if (procedureFee < 0.1) {
                procedureFee = 0.1;
            }
        } else if ("WX".equals(payParam.getPayWay())) {
            //微信0.65%
            procedureFee = MathUtil.mul(payAmount, 0.0065, 2).doubleValue();
        } else {
            //手续费 其他0.25%
            procedureFee = MathUtil.mul(payAmount, 0.0025, 2).doubleValue();
        }
        if (MathUtil.compare(procedureFee, 0.01) <= 0) {
            procedureFee = 0.01;
        }
        //减去收款手续费
        splitAmount = MathUtil.sub(splitAmount, procedureFee).doubleValue();
        //分账手续费0.1%
        Double amount = Math.max(MathUtil.mul(splitAmount, 0.001, 2).doubleValue(), 0.01);
        splitAmount = MathUtil.sub(splitAmount, amount).doubleValue();
        if (splitAmount <= 0) {
            return null;
        }
        // 商品数据
        List<OrderProductVo> orderProductList = orderCommonMapper.getOrderProductByOrderId(order.getOrderId());
        for (OrderProductVo orderProduct : orderProductList) {
            //成本价*数量
            double costPrice = MathUtil.mul(orderProduct.getCostPrice(), orderProduct.getNum()).doubleValue();
            // 不含税能开发票
            if (Integer.valueOf(0).equals(orderProduct.getIncludedTax()) && !Integer.valueOf(3).equals(orderProduct.getInvoiceType())) {
                //应付总税费
                Double payableTax = MathUtil.mul(orderProduct.getSingleShouldPayTotalTax(), orderProduct.getNum()).doubleValue();
                //成本+税费
                costPrice = MathUtil.add(costPrice, payableTax).doubleValue();
            }
            // 判断是否支付过
            Double paidAmount = payOrderMapper.getOrderProductPaidAmount(orderProduct.getOrderProductId());
            // 支付过金额大于0
            if (paidAmount != null && MathUtil.compare(paidAmount, 0) > 0) {
                //成本-支付过的金额
                costPrice = MathUtil.sub(costPrice, paidAmount).doubleValue();
            }
            // 没支付过或者支付过的金额-成本<0(成本还没分够)
            if (paidAmount == null || MathUtil.compare(paidAmount, costPrice) < 0) {
                // 待分账金额>=本次待分账金额(还要分账的成本)
                if (MathUtil.compare(splitAmount, costPrice) > 0) {
                    // 待分账金额=待分账金额-成本
                    splitAmount = MathUtil.sub(splitAmount, costPrice).doubleValue();
                    log.info("待分账金额大于成本  splitAmount = splitAmount-costPrice --------->splitAmount= " + splitAmount);
                } else {
                    // 待分账金额小于成本
                    log.info("待分账金额小于成本  costPrice = splitAmount --------->splitAmount= " + splitAmount);
                    costPrice = splitAmount;
                    splitAmount = 0.00;
                }
            }
            //todo 子商户有商编暂不考虑
            //分账对象
            SplitAccountPo splitAccount = new SplitAccountPo();
            splitAccount.setOrderId(order.getOrderId());
            splitAccount.setOrderProductId(orderProduct.getOrderProductId());
            splitAccount.setShopId(orderProduct.getShopId());
            //成本设置给分账金额
            splitAccount.setSplitAccount(costPrice);
            splitAccount.setProductType(1);
            if (1 == orderProduct.getInvoiceType()) {
                // 开增值税发票，则分账到公账-专票
                splitAccount.setType(1);
                //todo 专票测试暂入信息
                splitAccount.setSubUserNo(CUSTOMERNUM);
            } else if (2 == orderProduct.getInvoiceType()) {
                // 开普通发票，则分账到公账-普票
                splitAccount.setType(3);
                //todo 公账-普票 暂入奥泰
                splitAccount.setSubUserNo(CUSTOMERNUM3);
            } else {
                // 未知或不能开票，则分账到私账
                splitAccount.setType(2);
                //todo 私账测试暂入奥泰
                splitAccount.setSubUserNo(CUSTOMERNUM3);
            }
            if (splitAccount.getSplitAccount() > 0) {
                list.add(splitAccount);
                log.info("============设置成本分账详情对象： " + splitAccount.toString());
            }
            //没钱了就break
            if (MathUtil.compare(splitAmount, 0) == 0) {
                break;
            }
        }

        // 还有钱就付供应商运费
        // 付供应商运费,是以供应商为单位的
        if (MathUtil.compare(splitAmount, 0) > 0) {
            List<ShopOrderVo> shopOrderList = orderCommonMapper.getShopOrderListByOrderId(order.getOrderId());
            for (ShopOrderVo shopOrder : shopOrderList) {
                // 运费
                Double shopPostFee = shopOrder.getShopPostFee();
                if (MathUtil.compare(shopPostFee, 0) > 0) {
                    // 查询已支付运费
                    Double shipping = payOrderMapper.getPaidShipping(order.getOrderId(), shopOrder.getShopId());
                    shopPostFee = MathUtil.sub(shopPostFee, shipping).doubleValue();
                    if (MathUtil.compare(splitAmount, shopPostFee) > -1) {
                        //减去运费
                        splitAmount = MathUtil.sub(splitAmount, shopPostFee).doubleValue();
                    } else {
                        //钱不够运费就分剩余
                        shopPostFee = splitAmount;
                        splitAmount = 0.00;
                    }
                    //todo 暂不考虑供应商有商户号，有分给供应商，没有进私账
                    SplitAccountPo splitAccount = new SplitAccountPo();
                    splitAccount.setOrderId(order.getOrderId());
                    splitAccount.setShopId(shopOrder.getShopId());
                    splitAccount.setSplitAccount(shopPostFee);
                    splitAccount.setProductType(2);
                    splitAccount.setType(2);
                    //todo 私账-无票,子商户商编 测试暂入奥泰
                    splitAccount.setSubUserNo(CUSTOMERNUM3);
                    if (splitAccount.getSplitAccount() > 0) {
                        log.info("============设置付供应商运费分账详情对象： " + splitAccount.toString());
                        list.add(splitAccount);
                    }
                }
            }
        }
        // 如果还有钱则为佣金,分到采美网络
        if (MathUtil.compare(splitAmount, 0) > 0) {
            SplitAccountPo splitAccount = new SplitAccountPo();
            splitAccount.setOrderId(order.getOrderId());
            splitAccount.setSplitAccount(splitAmount);
            splitAccount.setProductType(3);
            splitAccount.setType(5);
            //todo 佣金应入采美网络，测试暂入信息
            splitAccount.setSubUserNo(Constant.CUSTOMERNUM);
            if (splitAccount.getSplitAccount() > 0) {
                log.info("============设置佣金分账详情对象： " + splitAccount.toString());
                list.add(splitAccount);
            }
        }
        return list;
    }

}
