package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.components.OrderCommonService;
import com.caimei365.order.components.ProductService;
import com.caimei365.order.components.WeChatService;
import com.caimei365.order.mapper.*;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.VoucherDto;
import com.caimei365.order.model.dto.VoucherImageDto;
import com.caimei365.order.model.enums.OrderStatus;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.RemoteCallService;
import com.caimei365.order.service.OrderClubService;
import com.caimei365.order.utils.ImageUtil;
import com.caimei365.order.utils.MathUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/19
 */
@Slf4j
@Service
public class OrderClubServiceImpl implements OrderClubService {
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private AddressMapper addressMapper;
    @Resource
    private OrderClubMapper orderClubMapper;
    @Resource
    private OrderCommonMapper orderCommonMapper;
    @Resource
    private ProductService productService;
    @Resource
    private RemoteCallService remoteCallService;
    @Resource
    private OrderCommonService orderCommonService;
    @Resource
    private WeChatService weChatService;
    @Resource
    private PayOrderMapper payOrderMapper;
    @Resource
    private CmReportingClubMapper cmReportingClubMapper;

    /**
     * 获取各状态订单数量
     *
     * @param userId 用户id
     */
    @Override
    public ResponseJson<Map<String, Object>> getOrderCount(Integer userId) {
        Map<String, Object> map = new HashMap<>();
        Integer identity = baseMapper.getIdentityByUserId(userId);
        //全部数量
        Integer totalCount = orderClubMapper.getOrderCount(userId, 0);
        //非协销剔除不可见订单
        if (identity != 1) {
            Integer orderSeenCount = orderClubMapper.getOrderSeenCount(userId, 0);
            totalCount -= orderSeenCount;
        }
        map.put("totalCount", totalCount);
        //待确认数量
        Integer confirmedCount = orderClubMapper.getOrderCount(userId, 1);
        if (identity != 1) {
            Integer orderSeenCount = orderClubMapper.getOrderSeenCount(userId, 1);
            confirmedCount -= orderSeenCount;
        }
        map.put("confirmedCount", confirmedCount);
        //待付款数量
        Integer paymentCount = orderClubMapper.getOrderCount(userId, 2);
        paymentCount = paymentCount == null ? 0 : paymentCount;
        if (identity != 1) {
            Integer orderSeenCount = orderClubMapper.getOrderSeenCount(userId, 2);
            paymentCount -= orderSeenCount;
        }
        map.put("paymentCount", paymentCount);
        //待发货数量
        Integer waitShipmentsCount = orderClubMapper.getOrderCount(userId, 3);
        waitShipmentsCount = waitShipmentsCount == null ? 0 : waitShipmentsCount;
        if (identity != 1) {
            Integer orderSeenCount = orderClubMapper.getOrderSeenCount(userId, 3);
            waitShipmentsCount -= orderSeenCount;
        }
        map.put("waitShipmentsCount", waitShipmentsCount);
        //已发货数量
        Integer shipmentsCount = orderClubMapper.getOrderCount(userId, 4);
        shipmentsCount = shipmentsCount == null ? 0 : shipmentsCount;
        if (identity != 1) {
            Integer orderSeenCount = orderClubMapper.getOrderSeenCount(userId, 4);
            shipmentsCount -= orderSeenCount;
        }
        map.put("shipmentsCount", shipmentsCount);
        //退货款数量
        Integer salesReturnCount = orderClubMapper.getOrderCount(userId, 5);
        salesReturnCount = salesReturnCount == null ? 0 : salesReturnCount;
        if (identity != 1) {
            Integer orderSeenCount = orderClubMapper.getOrderSeenCount(userId, 5);
            salesReturnCount -= orderSeenCount;
        }
        map.put("salesReturnCount", salesReturnCount);
        return ResponseJson.success(map);
    }

    /**
     * 机构订单列表
     *
     * @param userId     用户Id
     * @param orderState 1:待确认,2:待付款,3:待发货,4:已发货,5:退货款
     * @param orderNo    订单编号
     * @param beginTime  开始时间
     * @param endTime    截止时间
     * @param pageNum    页码
     * @param pageSize   每页数量
     */
    @Override
    public ResponseJson<PageInfo<ShopOrderVo>> getOrderList(Integer userId, Integer orderState, String orderNo, String beginTime, String endTime, int pageNum, int pageSize) {
        // 分页请求数据
        PageHelper.startPage(pageNum, pageSize);
        List<ShopOrderVo> shopOrderList = orderClubMapper.getShopOrderList(userId, orderState, orderNo, beginTime, endTime);
        shopOrderList.forEach(shopOrder -> {
            // 设置订单状态
            orderCommonService.setShopOrderStatus(shopOrder);
            // 设置数据
            orderCommonService.setShopOrderInfo(shopOrder);
            // 设置子订单付款金额
            orderCommonService.getDiscernReceiptAndSetShoporder(shopOrder);
            // 联合丽格 [待付款]或者线下支付[部分付款] 的子订单 状态
            if ((4 != shopOrder.getStatus() && 5 != shopOrder.getStatus()) && ((1 == shopOrder.getReceiptStatus() && (2 == shopOrder.getOnlinePay() || 0 == shopOrder.getOnlinePay())) || (2 == shopOrder.getReceiptStatus() && 2 == shopOrder.getOnlinePay()))) {
                shopOrder.setCheckFlag(2);
            } else {
                shopOrder.setCheckFlag(1);
            }
        });
        /*List<OrderVo> orderList = orderClubMapper.getOrderList(userId, orderState, orderNo, beginTime, endTime);
        orderList.forEach(order -> {
            // 设置订单状态
            orderCommonService.setOrderStatus(order);
            // 设置子订单数据
            orderCommonService.getShopOrderData(order);
            // 设置付款金额
            orderCommonService.getDiscernReceiptAndSetOrder(order);
            // 联合丽格 [待付款]或者线下支付[部分付款] 的主订单 状态
            if (0 != order.getOrganizeId()) {
                if ((6 != order.getStatus() && 7 != order.getStatus()) && (1 == order.getReceiptStatus() || (2 == order.getReceiptStatus() && 1 == order.getOnlinePayFlag()))) {
                    order.setCheckFlag(1);
                } else {
                    order.setCheckFlag(0);
                }
            }
        });*/
        PageInfo<ShopOrderVo> pageInfo = new PageInfo(shopOrderList);
        return ResponseJson.success(pageInfo);
    }

    /**
     * 根据关键词搜索订单
     *
     * @param userId     用户Id
     * @param searchWord 搜索关键词
     * @param pageNum    页码
     * @param pageSize   每页数量
     */
    @Override
    public ResponseJson<PageInfo<ShopOrderVo>> getOrderListByKeyword(Integer userId, String searchWord, int pageNum, int pageSize) {
        Integer identity = baseMapper.getIdentityByUserId(userId);
        // 设置搜索关键词历史记录
        orderCommonService.setHistoryRecord(userId, searchWord);
        PageHelper.startPage(pageNum, pageSize);
        // List<OrderVo> orderList = orderClubMapper.getOrderListByKeyword(searchWord, userId);
        List<ShopOrderVo> shopOrderList = orderClubMapper.getShopOrderListByKeyword(searchWord, userId);
        //获取主订单数据
        shopOrderList.forEach(shopOrder -> {
            // 设置订单状态
            orderCommonService.setShopOrderStatus(shopOrder);
            // 设置子订单数据
            orderCommonService.setShopOrderInfo(shopOrder);
            // 设置付款金额
            orderCommonService.getDiscernReceiptAndSetShoporder(shopOrder);
            if ((4 != shopOrder.getStatus() && 5 != shopOrder.getStatus()) && ((1 == shopOrder.getReceiptStatus() && (2 == shopOrder.getOnlinePay() || 0 == shopOrder.getOnlinePay())) || (2 == shopOrder.getReceiptStatus() && 2 == shopOrder.getOnlinePay()))) {
                shopOrder.setCheckFlag(2);
            } else {
                shopOrder.setCheckFlag(1);
            }
        });
        //不是协销，订单不可见屏蔽
        if (identity != 1) {
            shopOrderList.removeIf(o -> o.getOrderSeen().equals(2));
        }
        PageInfo<ShopOrderVo> pageInfo = new PageInfo(shopOrderList);
        return ResponseJson.success(pageInfo);
    }

    /**
     * 机构订单详情
     *
     * @param userId      用户Id
     * @param shopOrderId 子订单Id
     */
    @Override
    public ResponseJson<Map<String, Object>> getOrderDetail(Integer userId, Integer shopOrderId) {
//        OrderPo orderPo = orderClubMapper.getMainOrderByOrderId(orderId);
        ShopOrderVo shopOrder = orderClubMapper.getShopOrderById(shopOrderId);
        if (null == shopOrder) {
            return ResponseJson.error("该订单不存在！", null);
        }
        if (!userId.equals(shopOrder.getUserId())) {
            return ResponseJson.error("无权限查看此订单！", null);
        }
//        OrderVo order = new OrderVo();
        // OrderPo -> OrderVo
//        BeanUtils.copyProperties(orderPo, order);

        shopOrder.setOrderMark("#" + shopOrder.getShopOrderId() + "#");
        // 设置订单状态
        orderCommonService.setShopOrderStatus(shopOrder);
        // 设置子订单信息
        orderCommonService.setShopOrderInfo(shopOrder);
        // 子订单列表
//        List<ShopOrderVo> shopOrderList = order.getShopOrderList();
        AtomicDouble expensesOfTaxation = new AtomicDouble(0d);
//        shopOrderList.forEach(shopOrder -> {
        List<OrderProductVo> orderProductList = shopOrder.getOrderProductList();
        orderProductList.forEach(orderProduct -> {
            // 是否充值商品
            boolean recharge = productService.isRechargeProduct(orderProduct.getProductId());
            if (recharge) {
                shopOrder.setRechargeGoods(true);
            }
            //是否能退货
            if (null != orderProduct.getReturnGoodsStutas() && 2 == orderProduct.getReturnGoodsStutas()) {
                String helpContent = orderCommonMapper.getHelpContent(1040);
                orderProduct.setHelpContent(helpContent);
            }
            // 总税费
            expensesOfTaxation.set(MathUtil.add(expensesOfTaxation.get(), orderProduct.getTotalAddedValueTax()).doubleValue());
            //已发货数量
            orderProduct.setShipmentsNum(orderProduct.getNum() + orderProduct.getPresentNum() - orderProduct.getNotOutStore());
            //已退货数量
            Integer returnedNum = orderCommonMapper.countReturnedNum(orderProduct.getShopOrderId(), orderProduct.getProductId());
            returnedNum = null != returnedNum ? returnedNum : 0;
            orderProduct.setReturnedNum(returnedNum);
            //已取消发货数量
            Integer actualCancelNum = orderCommonMapper.countActualCancelNum(orderProduct.getShopOrderId(), orderProduct.getProductId());
            actualCancelNum = null != actualCancelNum ? actualCancelNum : 0;
            orderProduct.setActualCancelNum(actualCancelNum);
            //判断商品价格是否含税
            boolean taxFlag = (Integer.valueOf(1).equals(orderProduct.getIncludedTax()) || (Integer.valueOf(0).equals(orderProduct.getIncludedTax()) && (Integer.valueOf(1).equals(orderProduct.getInvoiceType()) || Integer.valueOf(2).equals(orderProduct.getInvoiceType()))));
            if (taxFlag) {
                orderProduct.setIncludedTaxFlag(2);
            } else if (Integer.valueOf(0).equals(orderProduct.getIncludedTax()) && Integer.valueOf(3).equals(orderProduct.getInvoiceType())) {
                orderProduct.setIncludedTaxFlag(1);
            } else {
                orderProduct.setIncludedTaxFlag(3);
            }
        });
        // 支付凭证
        List<VoucherPo> voucher = orderClubMapper.getVoucher(shopOrder.getShopOrderId());
        // 支付凭证图片
        if (null != voucher) {
            List<Integer> ids = voucher.stream().map(VoucherPo::getId).collect(Collectors.toList());
            List<VoucherImageDto> voucherImage = orderClubMapper.getVoucherImage(ids);
            voucher.forEach(v -> {
                List<VoucherImageDto> imageList = voucherImage.stream().filter(m -> m.getVoucherId().equals(v.getId())).collect(Collectors.toList());
                v.setImgs(imageList);
            });
        }
        shopOrder.setVoucher(voucher);
//        });
        shopOrder.setExpensesOfTaxation(expensesOfTaxation.get());
        // 发票信息
        InvoiceVo invoice = null;
        if (null != shopOrder.getInvoiceFlag() && shopOrder.getInvoiceFlag() > 0) {
            invoice = orderCommonMapper.getOrderInvoice(shopOrder.getOrderId());
            invoice.setUserId(userId);
        }
        // 可用余额
        Double availableMoney = baseMapper.getAbleUserMoney(userId);
        // 付款金额
        List<DiscernReceiptVo> discernReceiptList = orderCommonService.getDiscernReceiptAndSetShoporder(shopOrder);
        if (null != discernReceiptList && discernReceiptList.size() > 0) {
            discernReceiptList.forEach(d -> {
                if (29 == d.getPayType() || 30 == d.getPayType()) {
                    //快捷支付借记卡支付29，快捷支付贷记卡支付30
                    Integer infoId = d.getInfoId();
                    QuickInfoVo v = payOrderMapper.findInfoById(infoId);
                    String cardType = "1".equals(v.getBankType()) ? "借记卡" : "信用卡";
                    String substring = v.getBankNumber().substring(v.getBankNumber().length() - 4);
                    String cardStr = v.getBankName() + cardType + "(" + substring + ")";
                    d.setQuickPayStr(cardStr);
                }
            });
        }
        // 退款记录
        List<ReturnedPurchaseVo> returnedPurchaseList = orderClubMapper.getReturnedPurchaseList(shopOrder.getOrderId());
        if (!returnedPurchaseList.isEmpty()) {
            AtomicDouble returnedPurchaseFee = new AtomicDouble(0d);
            returnedPurchaseList.forEach(returnedPurchase -> returnedPurchaseFee.set(MathUtil.add(returnedPurchaseFee.get(), returnedPurchase.getRefundFee()).doubleValue()));
            //退款总金额
            shopOrder.setReturnedPurchaseFee(returnedPurchaseFee.get());
        }
        if ((4 != shopOrder.getStatus() && 5 != shopOrder.getStatus()) && ((1 == shopOrder.getReceiptStatus() && (2 == shopOrder.getOnlinePay() || 0 == shopOrder.getOnlinePay())) || (2 == shopOrder.getReceiptStatus() && 2 == shopOrder.getOnlinePay()))) {
            shopOrder.setCheckFlag(2);
        } else {
            shopOrder.setCheckFlag(1);
        }
        //售后条款
        ClauseVo clause = null;
        // 非二手
        if (0 == shopOrder.getSecondHandOrderFlag()) {
            Integer clauseId = shopOrder.getClauseId() == null ? 1 : shopOrder.getClauseId();
            clause = orderClubMapper.getClauseById(clauseId);
        }
        // 收货地址
        OrderUserinfoVo userInfo = addressMapper.getOrderUserinfo(shopOrder.getOrderId());
        //订单中存在不能退货的商品则给出不能退货提示
        boolean flag = false;
        String helpContent = null;
        if (null != orderProductList && orderProductList.size() > 0) {
            for (OrderProductVo op : orderProductList) {
                if (null != op.getReturnGoodsStutas() && op.getReturnGoodsStutas() == 2) {
                    flag = true;
                    helpContent = op.getHelpContent();
                }
            }
        }

        Map<String, Object> map = new HashMap<>();
        map.put("shopOrder", shopOrder);
        map.put("voucher", voucher);
        map.put("orderInvoice", invoice);
        map.put("ableUserMoney", availableMoney);
        map.put("userInfo", userInfo);
        map.put("discernReceiptList", discernReceiptList);
        map.put("returnedPurchaseList", returnedPurchaseList);
        map.put("clause", clause);
        if (flag) {
            map.put("returnGoodsStutas", 2);
            map.put("helpContent", helpContent);
        } else {
            map.put("returnGoodsStutas", 1);
            map.put("helpContent", null);
        }
        return ResponseJson.success(map);
    }


    /**
     * 上传支付凭证
     *
     * @param voucherDto
     * @return
     */
    @Override
    public ResponseJson voucher(VoucherDto voucherDto) {
        if (null == voucherDto.getShopOrderId()) {
            return ResponseJson.error(-1, "订单Id不能为空", null);
        }
        // 上传支付凭证
        orderClubMapper.insertVoucher(voucherDto);
        // 修改支付凭证图片
        if (!"".equals(voucherDto.getVoucherImgs())) {
            if (voucherDto.getVoucherImgs().contains("##")) {
                String[] split = voucherDto.getVoucherImgs().split("##");
                for (String str : split) {
                    VoucherImageDto voucherImageDto = new VoucherImageDto();
                    voucherImageDto.setVoucherId(voucherDto.getId());
                    voucherImageDto.setImg(str);
                    orderClubMapper.insertVoucherImage(voucherImageDto);
                }
            } else {
                VoucherImageDto voucherImageDto = new VoucherImageDto();
                voucherImageDto.setVoucherId(voucherDto.getId());
                voucherImageDto.setImg(voucherDto.getVoucherImgs());
                orderClubMapper.insertVoucherImage(voucherImageDto);
            }
        } else {
            return ResponseJson.error(-1, "请上传支付凭证", null);
        }

        return ResponseJson.success();
    }

    /**
     * 机构最新订单
     *
     * @param userId 用户Id
     */
    @Override
    public ResponseJson<OrderVo> getOrderLatest(Integer userId) {
        // 机构最新订单Id
        Integer orderId = orderClubMapper.getLatestOrderId(userId);
        OrderPo orderPo = orderClubMapper.getMainOrderByOrderId(orderId);
        //机构不可见置为空
        if (orderPo.getOrderSeen() == 2) {
            orderPo = null;
        }
        if (null == orderPo) {
            return ResponseJson.error("该订单不存在！", null);
        }
        if (!userId.equals(orderPo.getUserId())) {
            return ResponseJson.error("无权限查看此订单！", null);
        }
        OrderVo order = new OrderVo();
        // OrderPo -> OrderVo
        BeanUtils.copyProperties(orderPo, order);
        order.setOrderMark("#" + order.getOrderId() + "#");
        // 设置订单状态
        orderCommonService.setOrderStatus(order);
        // 设置子订单数据
        orderCommonService.getShopOrderData(order);
        // 设置付款金额
        orderCommonService.getDiscernReceiptAndSetOrder(order);
        return ResponseJson.success(order);
    }

    /**
     * 机构确认订单
     *
     * @param shopOrderId 订单Id
     */
    @Override
    public ResponseJson<Void> confirmMainOrder(Integer shopOrderId) {
        ShopOrderVo shopOrder = orderClubMapper.getShopOrderById(shopOrderId);
        if (null == shopOrder || 0 != shopOrder.getStatus()) {
            // 非待确认订单
            return ResponseJson.error("订单异常！", null);
        }
        /*if (MathUtil.compare(order.getCouponAmount(), 0) > 0) {
            // 确认订单时才真正使用优惠券
            CouponOrderRecordPo couponOrderRecord = orderClubMapper.getOrderCouponRecord(orderId);
            CouponVo coupon = orderClubMapper.getClubCouponById(couponOrderRecord.getClubCouponId(), orderClubMapper.findUserId(orderId));
            if (coupon == null) {
                return ResponseJson.error("订单内优惠券已被使用", null);
            } else {
                orderClubMapper.updateCouponClubStatus(couponOrderRecord.getClubCouponId(), orderId);
            }
        }*/
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        String note = "【确认订单】";
        shopOrder.setConfirmTime(curDateStr);
        List<OrderProductPo> orderProducts = orderClubMapper.getShopOrderProductList(shopOrderId);
        orderProducts.forEach(product -> {
            // 是否二手商品
            boolean secondFlag = false;
            if (null != product.getProductCategory() && 2 == product.getProductCategory()) {
                secondFlag = true;
            }
            // 更新复购价格库
            // productService.savePurchasePrice(order, product, secondFlag, note);
            // 二手商品确认商品如果是二手仪器则需要把该二手商品置为已售
            if (secondFlag) {
                orderClubMapper.updateSecondStatus(product.getProductId(), 1);
            }
        });
        if (null != shopOrder.getAccountAmount() && MathUtil.compare(shopOrder.getAccountAmount(), 0) > 0) {
            double userMoney = baseMapper.getUserMoney(shopOrder.getUserId());
            double updateMoney = MathUtil.sub(userMoney, shopOrder.getAccountAmount()).doubleValue();
            // 扣除用户余额
            orderClubMapper.updateUserMoneyByUserId(updateMoney, shopOrder.getUserId());
            log.info(note + ">>>>>>>>>>>更新用户余额(update[user(userMoney)])userId：" + shopOrder.getUserId() + "，shopOrderId：" + shopOrderId);
            // 保存余额到收款记录
            productService.saveShopOrderDiscernReceipt(shopOrder.getAccountAmount(), shopOrder, note);
        }
        if (null == shopOrder.getAccountAmount() || shopOrder.getAccountAmount() == 0d) {
            shopOrder.setReceiptStatus(1);
        } else if (null != shopOrder.getAccountAmount() && MathUtil.compare(shopOrder.getAccountAmount(), shopOrder.getTotalAmount()) < 0) {
            shopOrder.setReceiptStatus(2);
        } else {
            shopOrder.setReceiptStatus(3);
        }
        Integer count = 0;
        log.info("********************下单微信公众号消息推送***************************");
        /*if (0 == order.getSecondHandOrderFlag() && 0 == order.getRebateFlag() && !orderClubMapper.getProductOrder(order.getOrderId()).contains(6060) && order.getOrderType() != 0) {
            log.info("*******" + order.getSecondHandOrderFlag() + "**********" + order.getRebateFlag());
            log.info("*******获取公众名" + orderClubMapper.getOrderIds(orderId).toString());
            String name = "0";
            try {
                String accessToken = weChatService.getAccessToken();
                List<String> openidList = orderClubMapper.getOpenidListByPermission(orderClubMapper.FromUnionId(order.getOrderId()));
                openidList.removeIf(Objects::isNull);
                String i = orderClubMapper.getOrderIds(orderId).toString();
                if (i.length() < 10) {
                    name = StringUtils.strip(i, "[]");
                } else {
                    name = StringUtils.strip(i.substring(0, 11), "[]");
                    name += "...";
                }
                String associateTitle = "订单已成功提交，等待付款！";
                Double money = order.getPayTotalFee();
                String orderno = orderClubMapper.getOrderNo(orderId);
                String remarkText = "下单时间：" + order.getOrderTime() + "点击详情进行付款";
                // 跳转到【小程序付款-选择支付方式页面】
                String pagePath = "pages/user/order/order-details?type=share&orderId=" + orderId;
                for (String openid : openidList) {
                    // sendTemplateMsg(openid, 标题, 金额, 收款日期, 备注, 跳转链接)
                    weChatService.sendTemplateMsgs(accessToken, openid, associateTitle, name, orderno, money, remarkText, pagePath);
                    count++;
                }
            } catch (Exception e) {
                log.error("【订单发货通知】获取微信公众号access_token异常！", e);
            }
            //推送信息中心-交易物流
            MessageCenter messageCenter = new MessageCenter();
            messageCenter.setShopID(null);
            messageCenter.setClubID(order.getClubId());
            messageCenter.setUserType(1);
            messageCenter.setMessageType(1);
            messageCenter.setOrderMessageType(1);
            messageCenter.setOrderID(orderId);
            messageCenter.setContent(null);
            messageCenter.setTime(curDateStr);
            orderClubMapper.addMessageCenter(messageCenter);
            log.info(">>>>>>>>>>>>>>>>>>>>>>" + orderId);
            log.info("********************确认订单短信消息推送***************************");
            if (count <= 0) {
                ScheduledExecutorService executorService = new ScheduledThreadPoolExecutor(2);
                executorService.schedule(new Runnable() {
                    @Override
                    public void run() {
                        OrderVo orderVo = orderClubMapper.getOrderMess(orderId);
                        String bindMobile = baseMapper.getBindMobileByUserId(order.getBuyUserId());
                        log.info(">>>>>>>>>>>>>>>>>>>>>>" + orderId);
                        log.info(">>>>>>>>>>>>>>>>>>>>>>" + orderVo.getProCount());
                        String name = orderVo.getProName();
                        if (name.length() > 10) {
                            name = StringUtils.strip(name.substring(0, 11));
                            name += "...";
                        }
                        if (orderVo.getPayFlag() == 1 && orderVo.getOnlinePayFlag() == 0) {
                            String shortLink = remoteCallService.getShortLink(8, 2, "https://www.caimei365.com");
                            String content = "您已成功下单“" + name + "”等" + orderVo.getProCount() + "件商品，订单编号：" + orderVo.getOrderNo() + "订单等待支付，支付完成后采美将尽快安排发货。您可点击采美网址www.caimei365.com或使用微信小程序“采美采购商城”查看并支付订单。";
                            boolean sendSms = remoteCallService.getSendSms(1, 2, bindMobile, content);
                            if (!sendSms) {
                                log.info("》》》》》》》》》》》》》》》》协销下单短信推送失败");
                            } else {
                                log.info("》》》》》》》》》》》》》》》》协销下单短信推送成功");
                            }
                        }
                    }
                }, 300000, TimeUnit.MILLISECONDS);
            } else {
                log.info("》》》》》推送了协销下单短信");
            }
        }*/
        /*order.setUpdateDate(curDateStr);
        order.setConfirmTime(curDateStr);*/
        //确认订单状态
        shopOrder.setStatus(1);
        // 更新订单状态
        orderClubMapper.updateShopOrderStatus(shopOrder);
        // 修改订单确认时间
        orderClubMapper.updateOrder(shopOrder.getOrderId());
        return ResponseJson.success("确认订单成功！", null);
    }

    /**
     * 机构取消订单
     *
     * @param shopOrderId  订单Id
     * @param userIdentity 用户身份 1协销 其他用户
     * @param remark       备注（订单取消原因）
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseJson<Void> cancelMainOrder(Integer shopOrderId, Integer userIdentity, String remark) {
        ShopOrderVo shopOrder = orderClubMapper.getShopOrderById(shopOrderId);
        boolean cancelFlag = (null != shopOrder && (0 == shopOrder.getStatus() || (1 == shopOrder.getStatus() && 1 == shopOrder.getReceiptStatus())));
        if (!cancelFlag) {
            // 非待确认(或待收待发)订单
            return ResponseJson.error("订单异常！", null);
        }
        Integer identity = baseMapper.getIdentityByUserId(shopOrder.getUserId());
        //分销人员取消订单修改报备记录
        if (identity == 8) {
            cmReportingClubMapper.updateCmReportingClub(new CmReportingClub()
                    .orderId(shopOrder.getOrderId())
                    .orderStatus(0)
                    .lockStatus(0)
            );
        }
        Date date = new Date();
        String note = "【取消订单】";
        // 余额退回原账户
        if (null != shopOrder.getAccountAmount() && MathUtil.compare(shopOrder.getAccountAmount(), 0) > 0) {
            double ableUserMoney = baseMapper.getAbleUserMoney(shopOrder.getUserId());
            double updateMoney = MathUtil.add(ableUserMoney, shopOrder.getAccountAmount()).doubleValue();
            // 退回用户可用余额
            orderClubMapper.updateAbleUserMoneyByUserId(updateMoney, shopOrder.getUserId());
            log.info(note + ">>>>>>>>>>>可用余额退回原账户(update[user(ableUserMoney)])userId：" + shopOrder.getUserId() + "，shopOrderId：" + shopOrderId);
            // 取消收支记录
            BalanceRecordPo balanceRecord = new BalanceRecordPo();
            balanceRecord.setUserId(shopOrder.getUserId());
            balanceRecord.setShopOrderId(shopOrderId);
            balanceRecord.setRemark("取消订单，余额记录无效");
            balanceRecord.setDelFlag(1);
            // 取消 余额收支记录
            orderClubMapper.cancelBalanceRecord(balanceRecord);
            log.info(note + ">>>>>>>>>>>>取消余额收支记录(update[cm_user_balance_record])shopOrderId：" + shopOrderId);
        }
        shopOrder.setStatus(4);
        // 取消 订单
        orderClubMapper.cancelMainShopOrder(shopOrder);
        //二手订单取消订单, 二手商品标记未售
        if (1 == shopOrder.getSecondHandOrderFlag()) {
            List<OrderProductPo> orderProducts = orderClubMapper.getShopOrderProductList(shopOrderId);
            orderProducts.forEach(product -> {
                // 是否二手商品
                boolean secondFlag = false;
                if (null != product.getProductCategory() && 2 == product.getProductCategory()) {
                    secondFlag = true;
                }
                // 二手仪器标记未售
                if (secondFlag) {
                    orderClubMapper.updateSecondStatus(product.getProductId(), 0);
                }
            });
        }
        // 子订单状态
        List<Integer> shopOrderStatus = orderClubMapper.getShopOrderStatus(shopOrder.getOrderId());
        boolean flag = false;
        if (shopOrderStatus.stream().allMatch(s -> s == 4)) {
            flag = true;
        }
        // 退回 采美豆
        if (null != shopOrder.getUserBeans() && shopOrder.getUserBeans() > 0 && flag) {
            UserBeansHistoryPo beansHistory = new UserBeansHistoryPo();
            beansHistory.setUserId(shopOrder.getUserId());
            beansHistory.setOrderId(shopOrder.getOrderId());
            beansHistory.setBeansType(9);
            beansHistory.setType(1);
            beansHistory.setNum(shopOrder.getUserBeans());
            beansHistory.setPushStatus(0);
            beansHistory.setAddTime(date);
            beansHistory.setDelFlag(0);
            // 用户采美豆
            Integer userBeans = baseMapper.getUserBeans(shopOrder.getUserId());
            // 更新用户剩余采美豆数量
            int beans = userBeans + beansHistory.getNum();
            productService.updateUserBeans(beansHistory, beans, note);
        }
        // 客服确认注册机构，一天内取消有效
        Integer clubStatus = baseMapper.getClubStatus(shopOrder.getUserId());
        if (null != clubStatus && 94 == clubStatus) {
            // 已冻结(客服确认机构)
            // 查询最后的确认时间
            Date confirmTime = baseMapper.getClubConfirmTime(shopOrder.getUserId());
            if (confirmTime != null) {
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(confirmTime);
                calendar.add(Calendar.DATE, 1);
                if (calendar.getTime().compareTo(new Date()) > 0) {
                    // 机构状态修改为：已确认(客服确认机构)
                    baseMapper.updateUserClubStatus(93, shopOrder.getUserId());
                    baseMapper.updateClubStatus(93, shopOrder.getUserId());
                }
            }
        }
        //优惠券退回
        if (MathUtil.compare(shopOrder.getCouponAmount(), 0) > 0) {
            CouponOrderRecordPo couponRecord = orderClubMapper.getOrderCouponRecord(shopOrder.getOrderId(), shopOrder.getCouponClubId());
            if (couponRecord != null) {
                orderClubMapper.deleteCouponClubById(couponRecord.getClubCouponId());
                CouponClubPo couponClub = orderClubMapper.getCouponClubById(couponRecord.getClubCouponId());
                couponClub.setOrderId(null);
                couponClub.setReturnedId(0);
                couponClub.setSource(3);
                couponClub.setStatus("1");
                couponClub.setCreateDate(new Date());
                couponClub.setUseDate(null);
                couponClub.setDelFlag("0");
                orderClubMapper.insertCouponClub(couponClub);
            }
        }
        return ResponseJson.success("取消订单成功！", null);
    }

    /**
     * 机构删除订单
     *
     * @param shopOrderId 订单Id
     */
    @Override
    public ResponseJson<Void> deleteMainOrder(Integer shopOrderId) {
//        OrderPo order = orderClubMapper.getMainOrderByOrderId(orderId);
        ShopOrderVo shopOrder = orderClubMapper.getShopOrderById(shopOrderId);
        if (null == shopOrder || 4 != shopOrder.getStatus()) {
            // 非已关闭订单
            return ResponseJson.error("订单异常！", null);
        }
        // 逻辑删除子订单
        // String[] shopOrderIdArr = order.getShopOrderIds().split(",");
        List<String> shopOrderIds = new ArrayList<>();
        shopOrderIds.add(shopOrder.getShopOrderId().toString());
        orderClubMapper.deleteShopOrder(shopOrderIds);
        log.info("【删除订单】>>>>>>>>>逻辑删除子订单(update[cm_shop_order])shopOrderId：" + shopOrder.getShopOrderId());
        // 逻辑删除主订单
        List<Integer> shopOrderStatus = orderClubMapper.getShopOrderStatus(shopOrder.getOrderId());
        boolean flag = false;
        if (shopOrderStatus.stream().allMatch(s -> s == 4)) {
            flag = true;
        }
        if (flag) {
            orderClubMapper.deleteMainOrder(shopOrder.getOrderId(), "用户发起删除订单");
            log.info("【删除订单】>>>>>>>>>逻辑删除主订单(update[cm_order])orderId：" + shopOrder.getOrderId());
        }

        // 逻辑删除收款记录
        List<String> receiptIds = orderClubMapper.getShopOrderReceiptIds(shopOrderId);
        if (!receiptIds.isEmpty()) {
            orderClubMapper.deleteShopOrderReceiptRelation(shopOrderId);
            orderClubMapper.deleteDiscernReceipt(receiptIds);
            log.info("【删除订单】>>>>>>>>>逻辑删除收款记录(update[cm_discern_receipt])orderId：" + shopOrderId);
        }
        // 逻辑删除付款记录
        List<String> payIds = orderClubMapper.getPayShopIds(shopOrderIds);
        if (!payIds.isEmpty()) {
            orderClubMapper.deletePayShopRecord(shopOrderIds);
            orderClubMapper.deletePayShop(payIds);
            log.info("【删除订单】>>>>>>>>>逻辑删除付款记录(update[cm_pay_shop])shopOrderIds：" + shopOrderIds);
        }

        return ResponseJson.success("删除订单成功！", null);
    }

    /**
     * 机构确认收货
     *
     * @param shopOrderId 子订单Id
     */
    @Override
    public ResponseJson<Void> receiveGoods(Integer shopOrderId) {
        ShopOrderVo shopOrder = orderClubMapper.getShopOrderById(shopOrderId);
        if (null == shopOrder) {
            return ResponseJson.error("订单不存在！", null);
        }
        orderCommonService.setShopOrderStatus(shopOrder);
        if (33 == shopOrder.getStatus() && 3 == shopOrder.getPayStatus()) {
            // 已付款供应商,订单完成
            shopOrder.setStatus(3);
        } else if (33 == shopOrder.getStatus()) {
            // 交易完成
            shopOrder.setStatus(2);
        } else {
            return ResponseJson.error("订单异常！", null);
        }
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        String note = "【确认收货】";
        shopOrder.setUpdateTime(date);
        // 更新订单状态
        orderClubMapper.updateShopOrderStatus(shopOrder);
        // 更新物流状态
        orderClubMapper.updateLogisticsBatch(shopOrder.getOrderId(), shopOrderId);

        // 主动确认收货，赠送100采美豆
        UserBeansHistoryPo beansHistory = new UserBeansHistoryPo();
        beansHistory.setUserId(shopOrder.getUserId());
        beansHistory.setOrderId(shopOrder.getOrderId());
        beansHistory.setBeansType(7);
        beansHistory.setType(1);
        beansHistory.setNum(100);
        beansHistory.setPushStatus(0);
        beansHistory.setAddTime(date);
        beansHistory.setDelFlag(0);
        // 用户采美豆
        Integer userBeans = baseMapper.getUserBeans(shopOrder.getUserId());
        // 更新用户剩余采美豆数量
        int beans = userBeans + beansHistory.getNum();
        productService.updateUserBeans(beansHistory, beans, note);

        return ResponseJson.success("确认收货成功！", null);
    }

    /**
     * 根据用户Id查找订单搜索历史记录
     *
     * @param userId 用户Id
     */
    @Override
    public ResponseJson<List<SearchHistoryVo>> getOrderSearchHistory(Integer userId) {
        List<SearchHistoryVo> historyList = orderClubMapper.getOrderSearchHistory(userId);
        return ResponseJson.success(historyList);
    }

    /**
     * 根据用户Id删除订单搜索历史记录
     *
     * @param userId 用户Id
     */
    @Override
    public ResponseJson<Void> deleteOrderSearchHistory(Integer userId) {
        orderClubMapper.deleteOrderSearchHistory(userId);
        return ResponseJson.success(null);
    }

    /**
     * 物流详情
     *
     * @param shopOrderId 子订单Id
     */
    @Override
    public ResponseJson<List<LogisticsBatchVo>> getOrderLogistics(Integer shopOrderId) {
        int orderCount = orderClubMapper.countMainShopOrderByOrderId(shopOrderId);
        if (0 == orderCount) {
            return ResponseJson.error("订单不存在！", null);
        }
        // 获取物流批次列表
        List<LogisticsBatchVo> batchList = orderCommonMapper.getShopLogisticsBatchList(shopOrderId);
        batchList.forEach(batch -> {
            // 根据物流批次获取子订单
            // List<String> shopOrderIds = orderClubMapper.getBatchShopOrderIds(orderId, batch.getId());
            List<String> shopOrderIds = new ArrayList<>();
            shopOrderIds.add(shopOrderId.toString());
            List<ShopOrderVo> shopOrderList = orderCommonMapper.getShopOrderList(shopOrderIds);
            shopOrderList.forEach(shopOrder -> {
                shopOrder.setShopLogo(ImageUtil.getImageUrl("shopLogo", shopOrder.getShopLogo(), domain));
                // 获取发货物流记录
                List<LogisticsRecordVo> logisticsRecordList = orderCommonMapper.getLogisticsRecord(shopOrder.getShopOrderId(), batch.getId());
                shopOrder.setLogisticsRecordList(logisticsRecordList);
            });
            // 获取物流信息
            List<LogisticsInformationVo> logisticsInfoList = orderCommonMapper.getLogisticsInfoList(batch.getId());
            logisticsInfoList.forEach(logisticsInfo -> {
                List<LogisticsRouterVo> routers = JSONArray.parseArray(logisticsInfo.getInfo(), LogisticsRouterVo.class);
                if (!CollectionUtils.isEmpty(routers)) {
                    routers.forEach(router -> {
                        if (StringUtils.isEmpty(router.getDesc())) {
                            router.setDesc(router.getContext());
                        }
                    });
                    logisticsInfo.setRouterList(routers);
                }
            });
            batch.setLogisticsInformationList(logisticsInfoList);
            batch.setShopOrderList(shopOrderList);
        });

        return ResponseJson.success(batchList);
    }

    /**
     * 分享订单，获取分享码
     *
     * @param shopOrderId 订单Id
     */
    @Override
    public ResponseJson<String> getOrderShareCode(Integer shopOrderId) {
        // int orderCount = orderClubMapper.countMainOrderByOrderId(orderId);
        int shopOrderCount = orderClubMapper.countMainShopOrderByOrderId(shopOrderId);
        if (0 == shopOrderCount) {
            return ResponseJson.error("订单不存在！", null);
        }
        ShopOrderVo shopOrder = orderClubMapper.getShopOrderById(shopOrderId);
        // 生成分享码
        String shareCode = RandomStringUtils.random(4, false, true);
        // 过期时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DATE, 1);
        // 获取数据库已有分享码
        OrderShareCodeVo orderCodeVo = orderClubMapper.getShopOrderCodeById(shopOrderId);
        if (orderCodeVo != null && orderCodeVo.getExpiredTime().compareTo(new Date()) >= 0) {
            // 旧分享码还未过期，直接返回
            return ResponseJson.success(orderCodeVo.getShareCode());
        } else if (orderCodeVo != null) {
            // 如果旧分享码已经过期，删除分享码使用记录
            orderClubMapper.deleteShopOrderShareCodeRecord(shopOrderId);
            // 更新数据库的分享码
            orderCodeVo.setShareCode(shareCode);
            orderCodeVo.setExpiredTime(calendar.getTime());
            orderCodeVo.setDelFlag(0);
            orderClubMapper.updateOrderShareCode(orderCodeVo);
        } else {
            // 如果没有分享记录,新增数据库的分享码
            OrderShareCodeVo newShareCodeVo = new OrderShareCodeVo();
            newShareCodeVo.setOrderId(shopOrder.getOrderId());
            newShareCodeVo.setShopOrderId(shopOrderId);
            newShareCodeVo.setShareCode(shareCode);
            newShareCodeVo.setExpiredTime(calendar.getTime());
            newShareCodeVo.setAddTime(new Date());
            newShareCodeVo.setDelFlag(0);
            orderClubMapper.insertOrderShareCode(newShareCodeVo);
        }
        // 返回新生成的分享码
        return ResponseJson.success(shareCode);
    }

    /**
     * 分享订单，用户身份
     *
     * @param shopOrderId       订单Id
     * @param userId            用户Id
     * @param serviceProviderId 协销Id
     * @param code              微信code
     * @param encryptedData     微信加密数据
     * @param iv                微信加密算法的初始向量
     * @param headers           HttpHeaders
     */
    @Override
    public ResponseJson<String> identityOrderShareCode(Integer shopOrderId, Integer userId, Integer serviceProviderId, String code, String encryptedData, String iv, HttpHeaders headers) {
        // 用户组织Id
        Integer organizeId = baseMapper.getOrganizeId(userId);
        // int orderCount = orderClubMapper.countMainOrderByOrderId(orderId);
        int shopOrderCount = orderClubMapper.countMainShopOrderByOrderId(shopOrderId);
        if (0 == shopOrderCount) {
            return ResponseJson.error("订单不存在！", null);
        }
        // 微信小程序授权登录（调用user服务，发送服务间调用POST请求）
        String loginData = null;
        if (0 == organizeId) {
            loginData = remoteCallService.appletsAuthorization(code, encryptedData, iv, headers);
        } else {
            loginData = remoteCallService.appletsOrganizeAuthorization(code, encryptedData, iv, headers, userId);
        }
        if (StringUtils.isEmpty(loginData)) {
            return ResponseJson.error("微信授权登录异常！", null);
        }
        // 解析登录后得到 userIdentity,  openId等
        JSONObject userInfo = JSONObject.parseObject(loginData);
        String openId = userInfo.getString("openId");
        if (StringUtils.isEmpty(openId)) {
            return ResponseJson.error("微信授权登录异常！", null);
        }
        // 用户身份: 1协销 2会员机构 3供应商 4普通机构
        Integer userIdentity = userInfo.getInteger("userIdentity");
        Integer loginUserId = userInfo.getInteger("userId");
        Integer loginSpId = userInfo.getInteger("serviceProviderId");
        // 协销身份
        boolean spFlag = (null != userIdentity && 1 == userIdentity);
        // 机构身份
        boolean clubFlag = (null != userIdentity && (2 == userIdentity || 4 == userIdentity));
        if (spFlag) {
            if (null != serviceProviderId && serviceProviderId.equals(loginSpId)) {
                // 该机构的协销
                return ResponseJson.success(2, "该机构的协销查看分享订单!", loginData);
            } else {
                // 非该机构协销（游客）
                return ResponseJson.success(3, "非该机构协销查看分享订单！", null);
            }
        } else if (clubFlag && userId.equals(loginUserId)) {
            // 该机构的运营人员, 旧接口返回  0,true
            return ResponseJson.success(1, "该机构的运营人员查看分享订单!", loginData);
        }
        // 根据openId统计数据库内是否有有效分享码
        int codeCount = orderClubMapper.countShareCodeRecord(shopOrderId, openId);
        if (codeCount > 0) {
            // 游客权限
            return ResponseJson.success(3, "游客权限查看分享订单！", null);
        }
        /*
         * 默认数据按游客权限处理
         */
        return ResponseJson.success(0, "默认游客分享订单商品数据！", null);
    }

    /**
     * 分享订单，校验分享码
     *
     * @param shopOrderId   订单Id
     * @param shareCode     分享码
     * @param code          微信code
     * @param encryptedData 微信加密数据
     * @param iv            微信加密算法的初始向量
     * @param headers       HttpHeaders
     */
    @Override
    public ResponseJson<String> checkOrderShareCode(Integer userId, Integer shopOrderId, String shareCode, String code, String encryptedData, String iv, HttpHeaders headers) {
        // 用户组织Id
        Integer organizeId = baseMapper.getOrganizeId(userId);
        // int orderCount = orderClubMapper.countMainOrderByOrderId(orderId);
        int shopOrderCount = orderClubMapper.countMainShopOrderByOrderId(shopOrderId);
        if (0 == shopOrderCount) {
            return ResponseJson.error("订单不存在！", null);
        }
        // 微信小程序授权登录（调用user服务，发送服务间调用POST请求）
        String loginData = null;
        if (0 == organizeId) {
            loginData = remoteCallService.appletsAuthorization(code, encryptedData, iv, headers);
        } else {
            loginData = remoteCallService.appletsOrganizeAuthorization(code, encryptedData, iv, headers, userId);
        }
        log.info("loginData=====" + loginData);
        if (StringUtils.isEmpty(loginData)) {
            return ResponseJson.error("微信授权登录异常！", null);
        }
        // 解析登录后得到 userIdentity,  openId等
        JSONObject userInfo = JSONObject.parseObject(loginData);
        String openId = userInfo.getString("openId");
        if (StringUtils.isEmpty(openId)) {
            return ResponseJson.error("微信授权登录异常！", null);
        }
        // 根据订单Id获取分享码
        OrderShareCodeVo orderCodeVo = orderClubMapper.getShopOrderCodeById(shopOrderId);
        if (null == orderCodeVo) {
            return ResponseJson.error("该订单没有生成分享码！", null);
        }
        // 校验分享码
        if (orderCodeVo.getShareCode().equals(shareCode)) {
            if (orderCodeVo.getExpiredTime().compareTo(new Date()) > 0) {
                // 分享码有效, 保存分享码使用记录
                OrderShareCodeRecordVo codeRecord = new OrderShareCodeRecordVo();
                codeRecord.setOpenId(openId);
                codeRecord.setAddTime(new Date());
                codeRecord.setOrderId(orderCodeVo.getOrderId());
                codeRecord.setShopOrderId(shopOrderId);
                codeRecord.setDelFlag(0);
                codeRecord.setShareCodeId(orderCodeVo.getId());
                orderClubMapper.insertOrderShareCodeRecord(codeRecord);
                // 返回
                return ResponseJson.success(0, "游客权限查看分享订单！", null);
            } else {
                // 分享码过期
                return ResponseJson.error("分享码已失效！", null);
            }
        } else {
            return ResponseJson.error("分享码不正确！", null);
        }
    }
    /**
     * 分享订单，校验分享码
     *
     * @param shopOrderId   订单Id
     * @param shareCode     分享码
     */
    @Override
    public ResponseJson<String> checkOrderShareCode(Integer userId, Integer shopOrderId, String shareCode) {
        // 用户组织Id
        Integer identity = baseMapper.getIdentityByUserId(userId);
        int shopOrderCount = orderClubMapper.countMainShopOrderByOrderId(shopOrderId);
        if (8!= identity) {
            return ResponseJson.error("userId不存在！", null);
        }
        if (0 == shopOrderCount) {
            return ResponseJson.error("订单不存在！", null);
        }
        // 根据订单Id获取分享码
        OrderShareCodeVo orderCodeVo = orderClubMapper.getShopOrderCodeById(shopOrderId);
        if (null == orderCodeVo) {
            return ResponseJson.error("该订单没有生成分享码！", null);
        }
        // 校验分享码
        if (orderCodeVo.getShareCode().equals(shareCode)) {
            if (orderCodeVo.getExpiredTime().compareTo(new Date()) > 0) {
                // 分享码有效, 保存分享码使用记录
                OrderShareCodeRecordVo codeRecord = new OrderShareCodeRecordVo();
                codeRecord.setUserId(userId);
                codeRecord.setAddTime(new Date());
                codeRecord.setOrderId(orderCodeVo.getOrderId());
                codeRecord.setShopOrderId(shopOrderId);
                codeRecord.setDelFlag(0);
                codeRecord.setShareCodeId(orderCodeVo.getId());
                orderClubMapper.insertOrderShareCodeRecord(codeRecord);
                // 返回
                return ResponseJson.success(0, "查看分享订单！", null);
            } else {
                // 分享码过期
                return ResponseJson.error("分享码已失效！", null);
            }
        } else {
            return ResponseJson.error("分享码不正确！", null);
        }
    }

    /**
     * 分享订单，初始商品数据
     *
     * @param shopOrderId 订单Id
     */
    @Override
    public ResponseJson<Map<String, Object>> getOrderShareInitProduct(Integer shopOrderId) {
        List<OrderProductVo> orderProductList = orderCommonMapper.getshopOrderProductByOrderId(shopOrderId);
        orderProductList.forEach(orderProduct -> orderProduct.setImage(ImageUtil.getImageUrl("product", orderProduct.getImage(), domain)));
        ShopOrderVo shopOrderVo = orderCommonMapper.getShopOrderById(shopOrderId);
        orderCommonService.setShopOrderStatus(shopOrderVo);
        HashMap<String, Object> result = new HashMap<>(2);
        orderCommonService.getDiscernReceiptAndSetShoporder(shopOrderVo);
        result.put("productList", orderProductList);
        result.put("orderStatus", shopOrderVo.getStatus());
        //2021.12.14 去掉paybutton判断
//        result.put("payButton", order.isPayButton());
        return ResponseJson.success(result);
    }

    /**
     * 二手订单 确认打款供应商
     *
     * @param orderId 订单Id
     * @param userId  用户Id
     */
    @Override
    public ResponseJson<Void> secondPaymentConfirm(Integer orderId, Integer userId) {
        OrderPo orderPo = orderClubMapper.getMainOrderByOrderId(orderId);
        if (null == orderPo) {
            return ResponseJson.error("该订单不存在！", null);
        }
        if (!userId.equals(orderPo.getUserId())) {
            return ResponseJson.error("需机构自身确认打款供应商！", null);
        }
        boolean flag = 1 == orderPo.getSecondHandOrderFlag() && 0 == orderPo.getRefundType() && 0 == orderPo.getAffirmPaymentFlag();
        if (flag) {
            if (orderPo.getStatus().toString().startsWith("3") && 3 == orderPo.getPayStatus()) {
                orderPo.setAffirmPaymentFlag(1);
                orderPo.setStatus(5);
            } else if (orderPo.getStatus().toString().startsWith("3")) {
                orderPo.setAffirmPaymentFlag(1);
                orderPo.setStatus(4);
            } else if (orderPo.getStatus().toString().startsWith("2")) {
                orderPo.setAffirmPaymentFlag(1);
            } else {
                return ResponseJson.error("订单状态异常！", null);
            }
            // 更新订单状态
            orderClubMapper.updateOrderStatus(orderPo);
        } else {
            return ResponseJson.error("订单状态异常！", null);
        }
        return ResponseJson.success(null);
    }

    /**
     * 根据售后条款ID查看售后条款内容
     *
     * @param clauseId 售后条款Id
     */
    @Override
    public ResponseJson<ClauseVo> getClauseById(Integer clauseId) {
        ClauseVo clause = orderClubMapper.getClauseById(clauseId);
        return ResponseJson.success(clause);
    }

    /**
     * 查询供应商银行卡信息
     *
     * @param shopId
     * @return
     */
    @Override
    public ResponseJson<ShopPo> getShopBankAccount(Integer shopId) {
        // 账号归属 0采美 1 供应商
        Integer accountOwnership = orderClubMapper.getShopAccountOwnership(shopId);
        ShopPo shop = null;
        if (1 == accountOwnership) {
            shop = orderClubMapper.getShopBankAccount(shopId);
        } else {
            shop = orderClubMapper.getShopCmBankAccount(shopId);
        }
        return ResponseJson.success(shop);
    }

}
