package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.components.OnlinePostFormUtil;
import com.caimei365.order.constant.Constant;
import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.mapper.OrderRefundMapper;
import com.caimei365.order.mapper.PayOrderMapper;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.enums.AccountPayOrderType;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.OrderRefundService;
import com.caimei365.order.utils.MathUtil;
import com.caimei365.order.utils.helipay.Disguiser;
import com.caimei365.order.utils.helipay.HttpClientService;
import com.caimei365.order.utils.helipay.MessageHandle;
import com.caimei365.order.utils.helipay.MyBeanUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.httpclient.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;

/**
 * @author zzj
 */
@Slf4j
@Service
public class OrderRefundServiceImpl implements OrderRefundService {

    @Resource
    private OrderRefundMapper refundMapper;
    @Resource
    private PayOrderMapper payOrderMapper;
    @Resource
    private BaseMapper baseMapper;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseJson refundShopOrder(Integer shopOrderId) throws Exception {
        /** todo
         *  当前版本退款为原路退回,无自定义退款和部分退,仅可全退。 线上/线下/余额 三种支付方式,
         *  如果线上退回,退款并改变主订单/子订单状态
         *  如果线下退回,仅改变状态,不做其他处理
         *  如果余额支付,退回余额,改变状态
         *  所有支付方式,有采美豆使用的,一律退回采美豆,当前版本无采美豆抵扣运费,优惠券退回当前版本暂不考虑
         */
        OrderVo order = refundMapper.findOrder(shopOrderId);
        UserVo user = refundMapper.findUser(order.getUserId());

        List<OrderReceiptRelationPo> relationPos = refundMapper.findPayWays(shopOrderId);
        CmReturnedPurchase cmReturnedPurchase = new CmReturnedPurchase();
        String returnNo = "T" + System.currentTimeMillis() + new Random().nextInt(1000);
        String format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date());
        cmReturnedPurchase.setReturnedWay(1);
        cmReturnedPurchase.setReturnedNo(returnNo);
        cmReturnedPurchase.setShopOrderId(shopOrderId);
        cmReturnedPurchase.setUserId(order.getUserId());
        cmReturnedPurchase.setOrderId(order.getOrderId());
        cmReturnedPurchase.setStatus("2");
        cmReturnedPurchase.setOperatingOrderStatus(order.getStatus().toString());
        cmReturnedPurchase.setReturnTime(format);
        cmReturnedPurchase.setConfirmReturnTime(format);
        cmReturnedPurchase.setDelFlag(1);
        refundMapper.insertReturns(cmReturnedPurchase);


        AtomicReference<Double> refundBalanceFee = new AtomicReference<>(0d);
        AtomicReference<Double> refundOnlineFee = new AtomicReference<>(0d);
        AtomicReference<Double> refundOfflineFee = new AtomicReference<>(0d);

        for (OrderReceiptRelationPo relationPo : relationPos) {
            switch (relationPo.getPayWay()) {
                case 1:
                    returnOnline(relationPo);
                    refundOnlineFee.updateAndGet(v -> v + relationPo.getAssociateAmount());
                    break;
                case 2:
                    returnOffline(relationPo);
                    refundOfflineFee.updateAndGet(v -> v + relationPo.getAssociateAmount());
                    break;
                case 3:
                    returnBalance(relationPo, cmReturnedPurchase, order);
                    refundBalanceFee.updateAndGet(v -> v + relationPo.getAssociateAmount());
                    break;
                default:
                    throw new IllegalStateException("payWay Unexpected ------>" + relationPo.getPayWay());
            }
        }
        user.setUserMoney(user.getUserMoney() + refundBalanceFee.get());
        user.setAbleUserMoney(user.getAbleUserMoney() + refundBalanceFee.get());
        cmReturnedPurchase.setRefundBalanceFee(refundBalanceFee.get());
        cmReturnedPurchase.setRefundOnlineFee(refundOnlineFee.get());
        cmReturnedPurchase.setRefundOfflineFee(refundOfflineFee.get());
        cmReturnedPurchase.setRefundFee(refundBalanceFee.get() + refundOnlineFee.get() + refundOfflineFee.get());
        cmReturnedPurchase.setReturnedPurchaseFee(refundBalanceFee.get() + refundOnlineFee.get() + refundOfflineFee.get());
        cmReturnedPurchase.setDelFlag(0);
        refundMapper.updateReturnMoney(cmReturnedPurchase);
        /**
         * 为与旧数据兼容，保持cm_returned_purchase_product数据，当前版本退款为原路退回且只能全退，
         * cm_returned_purchase_product表数据无实际意义，可直接取cm_order_product，
         * cm_returned_purchase_product后续版本可删除
         * 看该子订单是否发货，发货则退货数量为子订单数量，未发货则为取消数量
         */
        Integer sendCount = refundMapper.findSendOut(shopOrderId);
        refundMapper.insertReturnProduct(cmReturnedPurchase.getId(), shopOrderId, sendCount);
        // 修改子订单信息
        refundMapper.updateShopOrder(shopOrderId);
        // 修改订单信息 refundType
        Integer count = refundMapper.findShopOrderRefundCount(order.getOrderId());
        Integer refundType = count > 0 ? 1 : 2;
        refundMapper.updateOrderRefund(order.getOrderId(), refundType);
        // 二手订单交易全退,标记未售
        if (1 == order.getSecondHandOrderFlag() && 7 == order.getStatus()) {
            refundMapper.updateSecondProduct(shopOrderId);
        }

        //采美豆退款相关
        if (7 == order.getStatus()) {
            /**  todo
             *   抵扣运费需退回，当前版本暂取消抵扣运费
             */


            /**
             *  1.下单赠送采美豆回收
             *  2.下单扣除采美豆返回
             */
            Integer num = refundMapper.findByBeansType(order.getOrderId());
            if (num != null && num > 0) {
                int userBeans = null == user.getUserBeans() ? 0 : user.getUserBeans();
                UserBeansHistoryPo beansHistory = new UserBeansHistoryPo();
                beansHistory.setBeansType(11);
                beansHistory.setType(2);
                beansHistory.setNum(num);
                beansHistory.setPushStatus(0);
                baseMapper.insertBeansHistory(beansHistory);
                userBeans = userBeans - num;
                user.setUserBeans(userBeans);
            }
        }
        //优惠券退回
        if (order.getCouponAmount() > 0 && null != order.getCouponClubId()) {
            CouponRecordPo couponClub = refundMapper.getCouponRecord(order.getCouponClubId());
            couponClub.setSource(3);
            couponClub.setStatus(1);
            refundMapper.insertCouponClub(couponClub);
        }
        // 修改user信息
        refundMapper.updateUser(user);
        return ResponseJson.success();
    }


    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public void returnOnline(OrderReceiptRelationPo relationPo) throws Exception {
        String splitCode = payOrderMapper.getSplitCodeByShopOrderId(relationPo.getShopOrderId());
        /**  网银
         *   快捷
         *   其他
         */
        if (12 == relationPo.getPayType() || 17 == relationPo.getPayType()) {
            /** todo
             *  网银退款此处只是发起手续费退回分账请求，如果本次退款中包含网银退款，应该直接return方法，在发起分账请求回调中
             *  继续退款方法中的其他操作，当前版本暂不考虑失败，默认转账成功
             */
            BigDecimal charge = 12 == relationPo.getPayType() ? BigDecimal.valueOf(10) : MathUtil.mul(relationPo.getAssociateAmount(), 0.002, 2);
            unionRefund(relationPo.getOrderRequestNo(), charge, relationPo.getShopOrderId(), splitCode);
        } else if (29 == relationPo.getPayType() || 30 == relationPo.getPayType()) {
            quickRefund(relationPo, splitCode);
        } else {
            onlinePay(relationPo, splitCode);
        }
    }

    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public void returnOffline(OrderReceiptRelationPo relationPo) {
        /**
         * todo 线下转账当前版本无操作
         */
    }

    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public void returnBalance(OrderReceiptRelationPo relationPo, CmReturnedPurchase cmReturnedPurchase, OrderVo order) {
        //添加余额收支记录
        BalanceRecordPo cmUserBalanceRecord = new BalanceRecordPo();
        cmUserBalanceRecord.setUserId(order.getUserId());
        cmUserBalanceRecord.setType(1);
        cmUserBalanceRecord.setBalanceType(3);
        cmUserBalanceRecord.setAddDate(new Date());
        cmUserBalanceRecord.setAmount(relationPo.getAssociateAmount());
        cmUserBalanceRecord.setOrderId(order.getOrderId());
        cmUserBalanceRecord.setShopOrderId(order.getShopOrderId());
        cmUserBalanceRecord.setReturnedId(cmReturnedPurchase.getId());
        cmUserBalanceRecord.setDelFlag(0);
        baseMapper.insertBalanceRecord(cmUserBalanceRecord);
    }


    /**
     * attention:网银退款不退手续费!
     * 本方法内转账手续费等待到账再发起退款请求
     * 本方法内仅发起向申请退款账户的分账请求，手续费分账给申请账户后，在回调接口中，再进行网银退款
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public void unionRefund(String orderRequestNo, BigDecimal money, Integer shopOrderId, String splitCode) {

        ArrayList<AccountPayOrder.AccountPayOrderExt.SplitBillRule> splitBillRules = new ArrayList<>();
        AccountPayOrder.AccountPayOrderExt.SplitBillRule splitBillRule = new AccountPayOrder.AccountPayOrderExt.SplitBillRule();
        splitBillRule.setSplitBillAmount(money);
        splitBillRule.setSplitBillMerchantNo(splitCode);
        splitBillRules.add(splitBillRule);

        AccountPayOrder accountPayOrder = new AccountPayOrder();
        accountPayOrder.setP1_bizType("AccountPaySub");
        accountPayOrder.setP2_signType("MD5");
        String format1 = new SimpleDateFormat("yyyy-MM-dd_HH:mm:ss.SSS").format(new Date());
        accountPayOrder.setP3_timestamp(format1);
        String substring = format1.substring(20);
        // fz+当前微秒时间+原唯一订单号
        accountPayOrder.setP4_orderId("WYTK" + substring + orderRequestNo);
        // 付款账户子订单绑定商户号
        accountPayOrder.setP5_customerNumber(Constant.CUSTOMERNUM2);
        AccountPayOrder.AccountPayOrderExt accountPayOrderExt = new AccountPayOrder.AccountPayOrderExt();
        accountPayOrderExt.setOrderType(AccountPayOrderType.TRANSFER);
        // 分账回调地址，此处重写一个接口，做网银退款专用，收到回调后，再接口中再进行网银的退款
        accountPayOrderExt.setServerCallbackUrl(Constant.prodWYREFUND);
        accountPayOrderExt.setGoodsName("网银手续费");
        accountPayOrderExt.setSplitBillRules(splitBillRules);

        String ext = JSON.toJSONString(accountPayOrderExt);
        log.info("网银退款手续费分账规则串json串：" + ext);
        accountPayOrder.setP6_ext(ext);
        // 生成签名
        StringBuilder builder = new StringBuilder();
        builder.append(Constant.SPLIT).append(accountPayOrder.getP1_bizType()).append(Constant.SPLIT).append(accountPayOrder.getP2_signType()).append(Constant.SPLIT).append(accountPayOrder.getP3_timestamp()).append(Constant.SPLIT).append(accountPayOrder.getP4_orderId()).append(Constant.SPLIT).append(accountPayOrder.getP5_customerNumber()).append(Constant.SPLIT).append(accountPayOrder.getP6_ext()).append(Constant.SPLIT).append(Constant.XUNI);
        String sign = Disguiser.disguiseMD5(builder.toString().trim());
        Map<String, String> bean = OnlinePostFormUtil.splitConvertBean(accountPayOrder);
        log.info("--------------------> 网银退款-手续费转账 参数：  " + bean);
        Map<String, String> map = OnlinePostFormUtil.splitPostForm(bean, Constant.FZ, sign, Map.class);
        log.info("----------------网银退款-手续费转账 返回数据: " + map.toString());
        if (map != null) {
            String code = map.get("rt5_retCode");
            if (!"0000".equals(code)) {
                String msg = map.get("rt6_retMsg");
                log.info("【网银退款-手续费转账】>>>>>>>>>>网银退款手续费转账失败>>>>>>>msg:" + msg);
            } else {
                refundMapper.insertTransfer(money, orderRequestNo, shopOrderId);
                log.info("【网银退款-手续费转账】>>>>>>>>>>网银退款手续费转账发起成功>>>>>>>");
            }
        }
    }

    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public void onlinePay(OrderReceiptRelationPo relationPo, String splitCode) throws Exception {
        AppPayRefundOrderVo orderVo = new AppPayRefundOrderVo();
        orderVo.setP1_bizType("AppPayRefund");
        orderVo.setP3_customerNumber(splitCode);
        //退款原单号
        orderVo.setP2_orderId(relationPo.getOrderRequestNo());
        //自定义退款单号
        String tk = new SimpleDateFormat("ddHHmmss").format(new Date());
        orderVo.setP4_refundOrderId("TK" + tk + relationPo.getOrderRequestNo());
        //退款金额
        orderVo.setP5_amount(relationPo.getAssociateAmount().toString());
        Map<String, String> map = MyBeanUtils.convertBean(orderVo, new LinkedHashMap());
        String oriMessage = MyBeanUtils.getSignedByPresetParameter(map, AppPayRefundOrderVo.NEED_SIGN_PARAMS) + Constant.SPLIT;
        oriMessage += Constant.SAOMA;
        String sign = Disguiser.disguiseMD5(oriMessage.trim());
        log.info("签名串：" + sign);
        map.put("sign", sign);
        log.info("发送参数：" + map);
        Map<String, Object> resultMap = HttpClientService.getHttpResp(map, Constant.REQUEST_URL);
        log.info("响应结果：" + resultMap);
        if ((Integer) resultMap.get("statusCode") == HttpStatus.SC_OK) {
            String resultMsg = (String) resultMap.get("response");
            AppPayRefundOrderResponseVo orderResponseVo = JSONObject.parseObject(resultMsg, AppPayRefundOrderResponseVo.class);
            String assemblyRespOriSign = MyBeanUtils.getSignedByPresetParameter(orderResponseVo, AppPayRefundOrderResponseVo.NEED_SIGN_PARAMS) + Constant.SPLIT;
            assemblyRespOriSign += Constant.SAOMA;
            log.info("组装返回结果签名串：" + assemblyRespOriSign);
            String responseSign = orderResponseVo.getSign();
            log.info("响应签名：" + responseSign);
            String checkSign = Disguiser.disguiseMD5(assemblyRespOriSign.trim());
            if (checkSign.equals(responseSign)) {
                if ("0001".equals(orderResponseVo.getRt2_retCode())) {
                    CmRefundRecord cmRefundRecord = new CmRefundRecord();
                    cmRefundRecord.setRefundAmount(relationPo.getAssociateAmount());
                    cmRefundRecord.setOrderRequestNo(relationPo.getOrderRequestNo());
                    cmRefundRecord.setShopOrderId(relationPo.getShopOrderId());
                    refundMapper.insertCmRefundRecord(cmRefundRecord);
                    log.info("第三方退款成功>>>>>>>shopOrderId>>>" + relationPo.getShopOrderId());
                } else {
                    log.error("退款失败========================" + orderResponseVo.getRt5_orderId() + "-" + orderResponseVo.getRt3_retMsg());
                    throw new RuntimeException("第三方退款失败！");
                }
            } else {
                log.error("退款验签失败");
                throw new RuntimeException("第三方退款验签失败！");
            }
        } else {
            log.error("退款请求失败");
            throw new RuntimeException("第三方退款请求失败！");
        }
    }


    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public void quickRefund(OrderReceiptRelationPo relationPo, String splitCode) throws Exception {
        //快捷支付
        QuickPaySpRefundVo quickPaySpRefundVo = new QuickPaySpRefundVo();
        quickPaySpRefundVo.setP1_bizType("QuickPaySpRefund");
        quickPaySpRefundVo.setP2_orderId(relationPo.getOrderRequestNo());
        quickPaySpRefundVo.setP3_customerNumber(splitCode);
        quickPaySpRefundVo.setP4_refundOrderId("ref" + relationPo.getOrderRequestNo());
        /**
         * [{ 'splitBillMerchantEmail': 'xun.zhang@caimei365.com','refundAmount': 0.12}]
         */
        String email = refundMapper.findSplitCodeEmail(splitCode);
        String refundJson = "[{ 'splitBillMerchantEmail': '" + email + "','refundAmount': " + relationPo.getAssociateAmount() + "}]";
        quickPaySpRefundVo.setP5_ruleJson(refundJson);
        quickPaySpRefundVo.setP6_callbackUrl("");
        quickPaySpRefundVo.setSignatureType("SM3WITHSM2");
        Map reqestMap = MessageHandle.getReqestMap(quickPaySpRefundVo);
        Map<String, Object> resultMap = HttpClientService.getHttpResp(reqestMap, Constant.REQUEST_URL_QUICKPAY);
        if ((Integer) resultMap.get("statusCode") != HttpStatus.SC_OK) {
            log.info("-------------------快捷支付退款请求失败---------------");
            throw new RuntimeException("快捷支付退款请求失败！");
        } else {
            String resultMsg = (String) resultMap.get("response");
            QuickPaySpRefundResponseVo responseVo = JSONObject.parseObject(resultMsg, QuickPaySpRefundResponseVo.class);
            log.info("--------------------->快捷支付退款json: " + JSONObject.toJSON(responseVo));
            if (!MessageHandle.checkSign(responseVo)) {
                log.info("---------------快捷支付退款验签失败------------------");
                throw new RuntimeException("快捷支付退款验签失败！");
            } else {
                if ("0000".equals(responseVo.getRt2_retCode())) {
                    CmRefundRecord cmRefundRecord = new CmRefundRecord();
                    cmRefundRecord.setRefundAmount(relationPo.getAssociateAmount());
                    cmRefundRecord.setOrderRequestNo(relationPo.getOrderRequestNo());
                    cmRefundRecord.setShopOrderId(relationPo.getShopOrderId());
                    refundMapper.insertCmRefundRecord(cmRefundRecord);
                    log.info("---------------快捷支付退款成功------------------");
                }
            }
        }
    }
}
