package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.components.WeChatService;
import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.mapper.OrderClubMapper;
import com.caimei365.order.mapper.OrderCommonMapper;
import com.caimei365.order.mapper.PayOrderMapper;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.bo.PayParamBo;
import com.caimei365.order.model.bo.VipRecordBo;
import com.caimei365.order.model.dto.PayCouponDto;
import com.caimei365.order.model.dto.PaySecondDto;
import com.caimei365.order.model.dto.PayVipDto;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.DiscernReceiptVo;
import com.caimei365.order.model.vo.OrderPayLinkVo;
import com.caimei365.order.service.PayNonOrderService;
import com.caimei365.order.service.RemoteCallService;
import com.caimei365.order.utils.CodeUtil;
import com.caimei365.order.utils.MathUtil;
import com.caimei365.order.utils.PayUtil;
import com.caimei365.order.utils.pay.RSAUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/9/27
 */
@Slf4j
@Service
public class PayNonOrderServiceImpl implements PayNonOrderService {
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private PayOrderMapper payOrderMapper;
    @Resource
    private OrderCommonMapper orderCommonMapper;
    @Resource
    private OrderClubMapper orderClubMapper;
    @Resource
    private WeChatService weChatService;
    @Value("${pay.second-notify-url}")
    private String secondHandUrl;
    @Value("${pay.vip-notify-url}")
    private String superVipUrl;
    @Value("${pay.auth-vip-notify-url}")
    private String authVipUrl;
    @Value("${pay.coupon-notify-url}")
    private String couponUrl;
    @Resource
    private RemoteCallService remoteCallService;

    /**
     * 二手发布微信线上支付
     *
     * @param paySecondDto {
     *                     productId      二手发布商品id
     *                     returnUrl     页面回调地址
     *                     code          微信小程序code
     *                     state         微信公众号state参数
     *                     }
     * @param headers
     */
    @Override
    public ResponseJson<JSONObject> paySecondByWeChat(PaySecondDto paySecondDto, HttpHeaders headers) {
        PayParamBo tempParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(paySecondDto, tempParam);
        ResponseJson<PayParamBo> jsonParam = getWeChatResponseJson(tempParam, headers);
        if (-1 == jsonParam.getCode()) {
            return ResponseJson.error(jsonParam.getMsg(), null);
        }
        PayParamBo payParam = jsonParam.getData();
        // 二手发布默认100元
        payParam.setPayAmount(10000);
        payParam.setNotifyUrl(secondHandUrl);
        log.info("【二手发布微信支付】>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>获取openId:" + payParam.getOpenId());
        return toPaySecond(payParam, headers);
    }

    /**
     * 微信线上支付-获取微信支付信息
     *
     * @param payParam 支付参数
     * @param headers  HttpHeaders
     * @return payParam获取微信支付信息
     */
    private ResponseJson<PayParamBo> getWeChatResponseJson(PayParamBo payParam, HttpHeaders headers) {
        // 微信线上支付
        payParam.setPayWay("WEIXIN");
        String openId = null;
        if (null == payParam.getState()) {
            // 小程序微信授权获取登录信息
            ResponseJson<Map<String, Object>> appletsInfo = weChatService.getInfoMapByApplets(payParam.getCode(), headers, 1);
            if (appletsInfo.getCode() == -1) {
                return ResponseJson.error(appletsInfo.getMsg(), null);
            }
            Map<String, Object> infoData = appletsInfo.getData();
            openId = (String) infoData.get(WeChatService.Keys.OPEN_ID);
            if (StringUtils.isEmpty(openId)) {
                return ResponseJson.error("微信小程序获取openId失败！", null);
            }
            //小程序微信快捷支付
            payParam.setPayType("MINIAPP_WEIXIN");
        } else {
            try {
                // 微信公众号，通过code获取微信用户信息
                Map<String, Object> map = weChatService.getInfoMapByWeb(payParam.getCode(), "crm");
                openId = (String) map.get(WeChatService.Keys.OPEN_ID);
            } catch (Exception e) {
                log.error("try-catch:", e);
            }
            if (StringUtils.isEmpty(openId)) {
                return ResponseJson.error("微信公众号获取openId失败！", null);
            }
            //pc微信扫码支付,微信公众号支付
            payParam.setPayType("JSAPI_WEIXIN");
        }
        payParam.setOpenId(openId);
        return ResponseJson.success(payParam);
    }

    /**
     * 二手发布支付宝线上支付
     *
     * @param paySecondDto {
     *                     productId      二手发布商品id
     *                     returnUrl     页面回调地址
     *                     }
     * @param headers      HttpHeaders
     */
    @Override
    public ResponseJson<JSONObject> paySecondByAlipay(PaySecondDto paySecondDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(paySecondDto, payParam);
        //支付宝支付
        payParam.setPayWay("ALIPAY");
        payParam.setPayType("ALIPAY_H5");
        // 二手发布默认100元
        payParam.setPayAmount(10000);
        payParam.setNotifyUrl(secondHandUrl);
        log.info("【二手发布支付宝支付】>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>productId:" + payParam.getProductId());
        return toPaySecond(payParam, headers);
    }

    /**
     * 二手发布银联线上支付
     *
     * @param paySecondDto {
     *                     productId       二手发布商品id
     *                     returnUrl     页面回调地址
     *                     bankCode      银行编码(银联支付使用)
     *                     userType      用户类型(银联支付使用)企业:ENTERPRISE，个人:USER
     *                     }
     * @param headers      HttpHeaders
     */
    @Override
    public ResponseJson<JSONObject> paySecondByUnionPay(PaySecondDto paySecondDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(paySecondDto, payParam);
        // 银联支付
        payParam.setPayWay("UNIONPAY");
        payParam.setPayType("GATEWAY_UNIONPAY");
        // 二手发布默认100元
        payParam.setPayAmount(10000);
        payParam.setNotifyUrl(secondHandUrl);
        log.info("【二手发布银联支付】>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>productId:" + payParam.getProductId());
        return toPaySecond(payParam, headers);
    }

    /**
     * 二手发布支付
     */
    private ResponseJson<JSONObject> toPaySecond(PayParamBo payParam, HttpHeaders headers) {
        // 定义支付请求结果
        JSONObject result = null;
        try {
            // 时间戳
            long time = System.currentTimeMillis() / 1000;
            // 用户IP地址
            JSONObject json = getPayJsonObject(payParam, headers, time);
            String environment = "";
            if (payParam.getNotifyUrl().contains("18002")) {
                environment = "DEV";
            } else if (payParam.getNotifyUrl().contains("core-b")) {
                environment = "BETA";
            }
            // 商户订单号
            String orderId = payParam.getProductId() + "T" + time + environment;
            json.put("orderId", orderId);
            //商品名称
            String product = "采美订单" + orderId;
            json.put("product", product);
            String attach = payParam.getProductId() + "," + payParam.getPayType() + "," + payParam.getUserType();
            json.put("attach", attach);
            // 二手发布默认100元,默认公账专票
            List<Map<String, String>> list = new ArrayList<>();
            Map<String, String> map = new HashMap<>(3);
            map.put("subUserNo", PayUtil.publicAccountNo);
            map.put("splitBillType", "1");
            map.put("splitBillValue", "1");
            list.add(map);
            String splitBillDetail = JSONObject.toJSONString(list);
            json.put("splitBillDetail", splitBillDetail);
            String sign = PayUtil.getPaySign(json, PayUtil.merKey);
            json.put("sign", sign);
            // 私钥加密
            log.info("发送参数 json：" + json);
            String data = RSAUtil.privateEncrypt(json.toJSONString(), PayUtil.merKey);
            // 提交
            result = PayUtil.httpGet("https://platform.mhxxkj.com/paygateway/mbpay/order/v1", PayUtil.merAccount, data);
        } catch (Exception e) {
            log.error("错误信息", e);
            return ResponseJson.error("支付失败！", null);
        }
        String code = result.getString("code");
        if (!"000000".equals(code)) {
            String msg = result.getString("msg");
            log.info("第三方支付失败>>>>>>>msg:" + msg);
            return ResponseJson.error(msg, null);
        }
        // 保存二手发布分账参数
        SplitAccountPo splitAccount = new SplitAccountPo();
        splitAccount.setOrderId(payParam.getOrderId());
        splitAccount.setProductId(payParam.getProductId());
        // 二手交易供应商
        splitAccount.setShopId(1252);
        splitAccount.setProductType(4);
        splitAccount.setType(2);
        if ("UNIONPAY".equals(payParam.getPayWay())) {
            splitAccount.setSplitAccount(92.00);
        } else {
            splitAccount.setSplitAccount(99.62);
        }
        JSONObject data = result.getJSONObject("data");
        splitAccount.setMbOrderId(data.getString("mbOrderId"));
        splitAccount.setOrderRequestNo(data.getString("orderId"));
        splitAccount.setPayStatus(0);
        // 保存分账详情
        payOrderMapper.insertSplitAccount(splitAccount);

        return ResponseJson.success(result);
    }

    /**
     * 二手发布支付回调
     */
    @Override
    public String paymentSecondCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException {
        log.info("******************** 二手发布支付异步回调 start *******************");
        // 公钥解密
        JSONObject json = PayUtil.publicKeyDecrypt(data, PayUtil.publicKey);
        log.info("【二手发布支付异步回调】>>>>>>>>>>>>>>公钥解密:" + json);
        // 公钥验签
        String signaa = json.getString("sign");
        json.remove("sign");
        String signbb = PayUtil.getPaySign(json, PayUtil.publicKey);
        if (!signaa.equals(signbb)) {
            return "验签失败";
        }
        // 订单状态
        String orderStatus = json.getString("orderStatus");
        // 平台唯一流水号
        String mbOrderId = json.getString("mbOrderId");
        // 附加数据，下单时若有传输则原样返回，下单时为空，则不返回该数据
        String attach = json.getString("attach");
        // 订单金额，以元为单位
        Double amount = json.getDouble("amount");
        String payFormData = json.toJSONString();
        log.info("【二手发布支付异步回调】>>>>>>>>>>>>>>支付订单状态:" + orderStatus);
        if ("FAILED".equals(orderStatus)) {
            return "支付失败";
        }
        log.info("【二手发布支付异步回调】>>>>>>>>>>>>>>本次支付金额：" + amount);
        String[] split = attach.split(",");
        // 订单id
        Integer productId = Integer.valueOf(split[0]);
        // 支付类型
        String payType = PayUtil.getPayTypeId(split[1], split[2]);
        payOrderMapper.updateSeconHandDetail(productId, payType, amount, payFormData);
        //修改分账付款状态
        payOrderMapper.updateSplitAccountByPay(mbOrderId);
        //存收款列表
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        // 保存收款记录
        DiscernReceiptPo discernReceipt = new DiscernReceiptPo();
        discernReceipt.setPayWay(1);
        discernReceipt.setPayType(Integer.valueOf(payType));
        discernReceipt.setReceiptType(7);
        discernReceipt.setReceiptStatus(3);
        discernReceipt.setReceiptAmount(amount);
        discernReceipt.setConfirmType(4);
        discernReceipt.setRePayFlag(1);
        discernReceipt.setFormData(payFormData);
        discernReceipt.setReceiptDate(curDateStr);
        discernReceipt.setConfirmDate(curDateStr);
        discernReceipt.setReviewDate(curDateStr);
        discernReceipt.setUpdateDate(curDateStr);
        discernReceipt.setDelFlag(0);
        // 保存 收款记录
        baseMapper.insertDiscernReceipt(discernReceipt);
        // 收款项和订单关系表
        OrderReceiptRelationPo relation = new OrderReceiptRelationPo();
        relation.setReceiptId(discernReceipt.getId());
        relation.setAssociateAmount(amount);
        relation.setMbOrderId(mbOrderId);
        relation.setSplitStatus(1);
        relation.setRelationType(4);
        relation.setDelFlag(0);
        //二手商品id
        relation.setProductId(productId);
        // 保存 收款项和订单关系
        baseMapper.insertOrderReceiptRelation(relation);
        return "SUCCESS";
    }

    /**
     * 设置支付基础参数
     */
    private JSONObject getPayJsonObject(PayParamBo payParam, HttpHeaders headers, long time) {
        // 用户IP地址
        String userIp = headers.getFirst("X-CLIENT-IP");
        JSONObject json = new JSONObject();
        json.put("merAccount", PayUtil.merAccount);
        json.put("merNo", PayUtil.merNo);
        json.put("time", time);
        //支付金额
        json.put("amount", payParam.getPayAmount());
        json.put("payWay", payParam.getPayWay());
        json.put("payType", payParam.getPayType());
        json.put("userIp", userIp);
        json.put("returnUrl", payParam.getReturnUrl());
        json.put("notifyUrl", payParam.getNotifyUrl());
        if (StringUtils.isNotBlank(payParam.getBankCode())) {
            json.put("bankCode", payParam.getBankCode());
        }
        if (StringUtils.isNotBlank(payParam.getUserType())) {
            json.put("userType", payParam.getUserType());
        }
        if (StringUtils.isNotBlank(payParam.getOpenId())) {
            json.put("openId", payParam.getOpenId());
        }
        return json;
    }

    /**
     * 升级超级会员-微信线上支付
     *
     * @param payVipDto {
     *                  vipRecordId   会员购买记录Id
     *                  returnUrl     页面回调地址
     *                  code          微信小程序code
     *                  state         微信公众号state参数
     *                  }
     * @param headers
     */
    @Override
    public ResponseJson<JSONObject> paySuperVipByWeChat(PayVipDto payVipDto, HttpHeaders headers) {
        PayParamBo tempParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payVipDto, tempParam);
        ResponseJson<PayParamBo> jsonParam = getWeChatResponseJson(tempParam, headers);
        if (-1 == jsonParam.getCode()) {
            return ResponseJson.error(jsonParam.getMsg(), null);
        }
        PayParamBo payParam = jsonParam.getData();
        // 获取会员套餐记录
        VipRecordBo record = payOrderMapper.getVipPackageRecord(payParam.getVipRecordId());
        if (null == record) {
            log.info("【升级超级会员-微信线上支付】该笔记录异常,记录Id:" + payParam.getVipRecordId());
            return ResponseJson.error("该笔记录异常！", null);
        }
        if (null != record.getPayStatus() && 1 == record.getPayStatus()) {
            return ResponseJson.error("该笔记录已支付，请勿重复支付！", null);
        }
        payParam.setVipId(record.getPackageId());
        payParam.setUserId(record.getUserId());
        // 获取会员套餐价格
        Double price = payOrderMapper.getVipPackagePriceById(record.getPackageId());
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(superVipUrl);
        log.info("【升级超级会员微信支付】>>>>>>>>>>>vipRecordId:" + payParam.getVipRecordId() + "，获取openId:" + payParam.getOpenId());
        return toPaySuperVip(payParam, headers);
    }

    /**
     * 升级超级会员-支付宝线上支付
     *
     * @param payVipDto {
     *                  vipRecordId   会员购买记录Id
     *                  returnUrl     页面回调地址
     *                  }
     * @param headers
     */
    @Override
    public ResponseJson<JSONObject> paySuperVipByAlipay(PayVipDto payVipDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payVipDto, payParam);
        //支付宝支付
        payParam.setPayWay("ALIPAY");
        payParam.setPayType("ALIPAY_H5");
        // 获取会员套餐记录
        VipRecordBo record = payOrderMapper.getVipPackageRecord(payParam.getVipRecordId());
        if (null != record.getPayStatus() && 1 == record.getPayStatus()) {
            return ResponseJson.error("该笔记录已支付，请勿重复支付！", null);
        }
        payParam.setVipId(record.getPackageId());
        payParam.setUserId(record.getUserId());
        // 获取会员套餐价格
        Double price = payOrderMapper.getVipPackagePriceById(record.getPackageId());
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(superVipUrl);
        log.info("【升级超级会员支付宝支付】>>>>>>>>>>>vipRecordId:" + payParam.getVipRecordId());
        return toPaySuperVip(payParam, headers);
    }

    /**
     * 升级超级会员-银联线上支付
     *
     * @param payVipDto {
     *                  vipRecordId   会员购买记录Id
     *                  returnUrl     页面回调地址
     *                  bankCode      银行编码(银联支付使用)
     *                  userType      用户类型(银联支付使用)企业:ENTERPRISE，个人:USER
     *                  }
     * @param headers
     */
    @Override
    public ResponseJson<JSONObject> paySuperVipByUnionPay(PayVipDto payVipDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payVipDto, payParam);
        // 银联支付
        payParam.setPayWay("UNIONPAY");
        payParam.setPayType("GATEWAY_UNIONPAY");
        // 获取会员套餐记录
        VipRecordBo record = payOrderMapper.getVipPackageRecord(payParam.getVipRecordId());
        if (null != record.getPayStatus() && 1 == record.getPayStatus()) {
            return ResponseJson.error("该笔记录已支付，请勿重复支付！", null);
        }
        payParam.setVipId(record.getPackageId());
        payParam.setUserId(record.getUserId());
        // 获取会员套餐价格
        Double price = payOrderMapper.getVipPackagePriceById(record.getPackageId());
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(superVipUrl);
        log.info("【升级超级会员银联线上支付】>>>>>>>>>>>vipRecordId:" + payParam.getVipRecordId());
        return toPaySuperVip(payParam, headers);
    }

    /**
     * 升级超级会员-银联转账支付
     *
     * @param payVipDto {
     *                  vipRecordId   会员购买记录Id
     *                  }
     * @param headers
     */
    @Override
    public ResponseJson<JSONObject> paySuperVipByTransfer(PayVipDto payVipDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payVipDto, payParam);
        // 银联支付
        payParam.setPayWay("UNIONPAY");
        payParam.setPayType("TRANSFER_UNIONPAY");
        // 获取会员套餐记录
        VipRecordBo record = payOrderMapper.getVipPackageRecord(payParam.getVipRecordId());
        if (null != record.getPayStatus() && 1 == record.getPayStatus()) {
            return ResponseJson.error("该笔记录已支付，请勿重复支付！", null);
        }
        payParam.setVipId(record.getPackageId());
        payParam.setUserId(record.getUserId());
        // 获取会员套餐价格
        Double price = payOrderMapper.getVipPackagePriceById(record.getPackageId());
        if (price < 12) {
            return ResponseJson.error("会员价格小于银联转账手续费！");
        }
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(superVipUrl);
        log.info("【升级超级会员银联线上支付】>>>>>>>>>>>vipRecordId:" + payParam.getVipRecordId());
        return toPaySuperVip(payParam, headers);
    }

    /**
     * 升级超级会员支付
     */
    private ResponseJson<JSONObject> toPaySuperVip(PayParamBo payParam, HttpHeaders headers) {
        // 定义支付请求结果
        JSONObject result = null;
        try {
            // 时间戳
            long time = System.currentTimeMillis() / 1000;
            // 设置支付基础参数
            JSONObject json = getPayJsonObject(payParam, headers, time);
            // 支付环境
            String environment = "";
            if (payParam.getNotifyUrl().contains("18002")) {
                environment = "DEV";
            } else if (payParam.getNotifyUrl().contains("core-b")) {
                environment = "BETA";
            }
            // 商户订单号
            String orderId = payParam.getUserId() + "R" + payParam.getVipRecordId() + "V" + payParam.getVipId() + "T" + time + environment;
            json.put("orderId", orderId);
            //商品名称
            String product = "采美订单" + orderId;
            json.put("product", product);
            //支付类型
            String payType = PayUtil.getPayTypeId(payParam.getPayType(), payParam.getUserType());
            String attach = payParam.getVipId() + "," + payParam.getUserId() + "," + payParam.getVipRecordId() + "," + payType;
            json.put("attach", attach);
            // 升级超级会员默认公账
            List<Map<String, String>> list = new ArrayList<>();
            Map<String, String> map = new HashMap<>(3);
            map.put("subUserNo", PayUtil.publicAccountNo);
            // 1按比例分账
            map.put("splitBillType", "1");
            // 1比例值总额
            map.put("splitBillValue", "1");
            list.add(map);
            String splitBillDetail = JSONObject.toJSONString(list);
            json.put("splitBillDetail", splitBillDetail);
            String sign = PayUtil.getPaySign(json, PayUtil.merKey);
            json.put("sign", sign);
            // 私钥加密
            log.info("发送参数 json：" + json);
            String data = RSAUtil.privateEncrypt(json.toJSONString(), PayUtil.merKey);
            // 提交
            result = PayUtil.httpGet("https://platform.mhxxkj.com/paygateway/mbpay/order/v1", PayUtil.merAccount, data);
        } catch (Exception e) {
            log.error("错误信息", e);
            return ResponseJson.error("支付失败！", null);
        }
        String code = result.getString("code");
        if (!"000000".equals(code)) {
            String msg = result.getString("msg");
            log.info("第三方支付失败>>>>>>>msg:" + msg);
            return ResponseJson.error(msg, null);
        }
        // 保存升级超级会员分账参数
        JSONObject data = result.getJSONObject("data");
        SplitAccountPo splitAccount = new SplitAccountPo();
        splitAccount.setVipRecordId(payParam.getVipRecordId());
        // 超级会员 公账-专票
        splitAccount.setType(1);
        // 待分账总金额
        double splitAmount = MathUtil.div(payParam.getPayAmount(), 100).doubleValue();
        // 总手续费
        double procedureFee;
        if ("UNIONPAY".equals(payParam.getPayWay())) {
            procedureFee = 8.00;
        } else {
            //手续费
            procedureFee = MathUtil.mul(splitAmount, 0.0038, 2).doubleValue();
            if (MathUtil.compare(procedureFee, 0) == 0) {
                procedureFee = 0.01;
            }
        }
        splitAmount = MathUtil.sub(splitAmount, procedureFee).doubleValue();
        splitAccount.setSplitAccount(splitAmount);
        splitAccount.setMbOrderId(data.getString("mbOrderId"));
        splitAccount.setOrderRequestNo(data.getString("orderId"));
        splitAccount.setPayStatus(0);
        // 保存分账详情
        payOrderMapper.insertSplitAccount(splitAccount);
        log.info("【升级超级会员支付成功！】>>>>>>>>>>>vipRecordId:" + payParam.getVipRecordId() + "，data:" + data.toJSONString());
        return ResponseJson.success(result);
    }

    /**
     * 升级超级会员-支付回调
     *
     * @param data
     */
    @Override
    public String paymentSuperVipCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException {
        log.info("******************** 升级超级会员异步回调 start *******************");
        // 公钥解密
        JSONObject json = PayUtil.publicKeyDecrypt(data, PayUtil.publicKey);
        log.info("【升级超级会员异步回调】>>>>>>>>>>>公钥解密：" + json);
        // 公钥验签
        String signaa = json.getString("sign");
        json.remove("sign");
        String signbb = PayUtil.getPaySign(json, PayUtil.publicKey);
        if (!signaa.equals(signbb)) {
            return "验签失败";
        }
        // 订单状态
        String orderStatus = json.getString("orderStatus");
        // 平台唯一流水号
        String mbOrderId = json.getString("mbOrderId");
        // 商户唯一订单号
        String orderRequestNo = json.getString("orderId");
        // 金额，以元为单位
        Double amount = json.getDouble("amount");
        String payFormData = json.toJSONString();
        log.info("【升级超级会员异步回调】>>>>>>>>>>>支付订单状态：" + orderStatus);
        if ("FAILED".equals(orderStatus)) {
            return "支付失败";
        }
        log.info("【升级超级会员异步回调】>>>>>>>>>>>>>>本次支付金额：" + amount);
        // 附加数据，支付时若有传输则原样返回(vipId,userId,recordId,payType)，下单时为空，则不返回该数据
        String attach = json.getString("attach");
        String[] split = attach.split(",");
        int vipId = 0;
        int userId = 0;
        int recordId = 0;
        int payType = 0;
        try {
            // 会员套餐Id
            vipId = Integer.parseInt(split[0]);
            // 用户Id
            userId = Integer.parseInt(split[1]);
            // 购买历史记录Id
            recordId = Integer.parseInt(split[2]);
            // 支付方式
            payType = Integer.parseInt(split[3]);
        } catch (NumberFormatException e) {
            log.info("【升级超级会员异步回调】>>>>>>>>>>>>>>获取回调参数解析失败！");
        }
        //超级会员数据库修改
        UserVipPo userVip = new UserVipPo();
        UserVipPo dbUserVip = payOrderMapper.getUserVipInfo(userId);
        Date beginTime = new Date();
        if (null != dbUserVip && null != dbUserVip.getEndTime()) {
            // 有效期内续费
            beginTime = dbUserVip.getEndTime();
            userVip.setId(dbUserVip.getId());
        }
        userVip.setUserId(userId);
        userVip.setUpdateTime(new Date());
        userVip.setBeginTime(beginTime);
        Calendar cal = Calendar.getInstance();
        cal.setTime(beginTime);
        // 超级会员套餐时长（月）
        Integer mouth = payOrderMapper.getVipPackageDurationById(vipId);
        cal.add(Calendar.MONTH, mouth);
        userVip.setEndTime(cal.getTime());
        userVip.setDelFlag(0);
        if (null != userVip.getId()) {
            // 更新会员信息
            payOrderMapper.updateUserVipInfo(userVip);
        } else {
            // 新增会员信息
            payOrderMapper.insertUserVipInfo(userVip);
        }
        // 修改会员历史记录支付状态
        VipRecordBo record = payOrderMapper.getVipPackageRecord(recordId);
        record.setPayStatus(1);
        record.setBeginTime(beginTime);
        record.setEndTime(cal.getTime());
        record.setPayWay(1);
        record.setPayType(payType);
        record.setPayTime(new Date());
        payOrderMapper.updateVipPackageRecord(record);
        // 修改支付链接状态
        OrderPayLinkVo orderPayLink = payOrderMapper.getVipPayLink(recordId, amount);
        if (null != orderPayLink && (12 == payType || 17 == payType)) {
            orderPayLink.setPayStatus(1);
            payOrderMapper.updateOrderPayLinkStatus(orderPayLink);
        }
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        // 保存收款记录
        DiscernReceiptPo discernReceipt = new DiscernReceiptPo();
        discernReceipt.setPayWay(1);
        discernReceipt.setPayType(record.getPayType());
        discernReceipt.setReceiptType(6);
        discernReceipt.setReceiptStatus(3);
        discernReceipt.setReceiptAmount(amount);
        discernReceipt.setConfirmType(4);
        discernReceipt.setRePayFlag(1);
        discernReceipt.setFormData(payFormData);
        discernReceipt.setReceiptDate(curDateStr);
        discernReceipt.setConfirmDate(curDateStr);
        discernReceipt.setReviewDate(curDateStr);
        discernReceipt.setUpdateDate(curDateStr);
        discernReceipt.setDelFlag(0);
        // 保存 收款记录
        baseMapper.insertDiscernReceipt(discernReceipt);
        log.info("【升级超级会员异步回调】>>>>>>>>>>>>>>保存识别款项(insert[cm_discern_receipt])id：" + discernReceipt.getId() + "，vipRecordId：" + recordId);
        // 收款项和订单关系表
        OrderReceiptRelationPo relation = new OrderReceiptRelationPo();
        relation.setReceiptId(discernReceipt.getId());
        relation.setVipRecordId(recordId);
        relation.setAssociateAmount(amount);
        relation.setMbOrderId(mbOrderId);
        relation.setOrderRequestNo(orderRequestNo);
        relation.setSplitStatus(1);
        relation.setRelationType(3);
        relation.setDelFlag(0);
        // 保存 收款项和订单关系
        baseMapper.insertOrderReceiptRelation(relation);
        log.info("【升级超级会员异步回调】>>>>>>>>>>>收款项和购买历史关系(insert[cm_receipt_order_relation])id：" + relation.getId() + "，vipRecordId：" + recordId);
        //修改分账付款状态
        payOrderMapper.updateSplitAccountByPay(mbOrderId);
        //已支付推送
        boolean smsPushFlag = !orderRequestNo.contains("BETA") && !orderRequestNo.contains("DEV");
        String bindMobile = baseMapper.getBindMobileByUserId(userId);
        if (smsPushFlag && StringUtils.isNotBlank(bindMobile)) {
            String shortLink = remoteCallService.getShortLink(8, 2, "https://www.caimei365.com/user/member.html?userId=" + userId);
            String content = "恭喜您成功开通采美平台超级会员，为期" + mouth + "个月，快戳采美网站链接www.caimei365.com/t/" + shortLink + "或微信搜索“采美采购商城”小程序登录采美平台畅享会员特权吧。关注公众号“采美365网”可获取更多优惠和精彩资讯。退订回T";
            boolean sendSms = remoteCallService.getSendSms(3, 2, bindMobile, content);
            if (!sendSms) {
                log.info("购买会员短信推送失败");
            }
        }
        return "SUCCESS";
    }

    /**
     * 升级超级会员-线上退款
     *
     * @param payVipDto {
     *                  vipRecordId   会员购买记录Id
     *                  code          退款口令
     *                  }
     */
    @Override
    public ResponseJson<JSONObject> superVipOnlineRefund(PayVipDto payVipDto) throws NoSuchAlgorithmException, InvalidKeySpecException {
        // 超级会员支付记录
        DiscernReceiptVo discernReceipt = orderCommonMapper.getVipDiscernReceipt(payVipDto.getVipRecordId());
        JSONObject json = new JSONObject();
        json.put("merAccount", PayUtil.merAccount);
        // 当前时间戳
        long time = System.currentTimeMillis() / 1000;
        json.put("time", time);
        //商户退款流水号，由商户自行生成，必须唯一
        String randomLetter = CodeUtil.randomLetter(6);
        String merchantRefundNo = payVipDto.getVipRecordId() + payVipDto.getCode() + randomLetter + "##" + discernReceipt.getId();
        json.put("merchantRefundNo", merchantRefundNo);
        //退款金额，单位分，必须大于0
        double refundAmt = MathUtil.mul(discernReceipt.getAssociateAmount(), 100).doubleValue();
        json.put("refundAmt", refundAmt);
        //退款原因
        String refundCause = "用户主动退款";
        json.put("refundCause", refundCause);
        //平台交易订单号
        json.put("mbOrderId", discernReceipt.getMbOrderId());
        String sign = PayUtil.getPaySign(json, PayUtil.merKey);
        json.put("sign", sign);
        String data = PayUtil.privateKeyEncrypt(json, PayUtil.merKey);
        JSONObject result = PayUtil.httpPost("https://platform.mhxxkj.com/paygateway/mbrefund/orderRefund/v1", PayUtil.merAccount, data);
        String code = result.getString("code");
        if (!"000000".equals(code)) {
            String msg = result.getString("msg");
            log.info("【线上退款】失败>>>>>>>msg:" + msg);
            return ResponseJson.error(result);
        }
        log.info("【线上退款】成功>>>>>>>vipRecordId：" + payVipDto.getVipRecordId());
        return ResponseJson.success(result);
    }

    /**
     * 微信买优惠券
     *
     * @param payCouponDto
     * @param headers
     * @return
     */
    @Override
    public ResponseJson<JSONObject> payCouponByWeChat(PayCouponDto payCouponDto, HttpHeaders headers) {
        Integer status = payOrderMapper.findPayStatus(payCouponDto.getCouponRecordId());
        if (null != status && 1 == status) {
            return ResponseJson.error("该笔记录已支付，请勿重复支付！", null);
        }
        PayParamBo tempParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payCouponDto, tempParam);
        ResponseJson<PayParamBo> jsonParam = getWeChatResponseJson(tempParam, headers);
        if (-1 == jsonParam.getCode()) {
            return ResponseJson.error(jsonParam.getMsg(), null);
        }
        PayParamBo payParam = jsonParam.getData();
        // 获取优惠券价格
        Double price = payOrderMapper.getCouponPrice(payCouponDto.getCouponId());
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(couponUrl);
        payParam.setCouponRecordId(payCouponDto.getCouponRecordId());
        log.info("【购买价值优惠券微信支付】>>>>>>>>>>>couponId:" + payParam.getCouponId() + "，获取openId:" + payParam.getOpenId());
        return toPayCoupon(payParam, headers);
    }

    /**
     * 支付宝买优惠券
     *
     * @param headers
     */
    @Override
    public ResponseJson<JSONObject> payCouponByAlipay(PayCouponDto payCouponDto, HttpHeaders headers) {
        Integer status = payOrderMapper.findPayStatus(payCouponDto.getCouponRecordId());
        if (null != status && 1 == status) {
            return ResponseJson.error("该笔记录已支付，请勿重复支付！", null);
        }
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payCouponDto, payParam);
        //支付宝支付
        payParam.setPayWay("ALIPAY");
        payParam.setPayType("ALIPAY_H5");
        // 获取优惠券价格
        Double price = payOrderMapper.getCouponPrice(payCouponDto.getCouponId());
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(couponUrl);
        payParam.setCouponRecordId(payCouponDto.getCouponRecordId());
        log.info("【购买价值优惠券支付宝支付】>>>>>>>>>>>couponId:" + payParam.getCouponId());
        return toPayCoupon(payParam, headers);
    }

    @Override
    public ResponseJson<JSONObject> payCouponUnionPay(PayCouponDto payCouponDto, HttpHeaders headers) {
        Integer status = payOrderMapper.findPayStatus(payCouponDto.getCouponRecordId());
        if (null != status && 1 == status) {
            return ResponseJson.error("该笔记录已支付，请勿重复支付！", null);
        }
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payCouponDto, payParam);
        // 银联支付
        payParam.setPayWay("UNIONPAY");
        payParam.setPayType("GATEWAY_UNIONPAY");
        // 获取会员套餐价格
        Double price = payOrderMapper.getCouponPrice(payCouponDto.getCouponId());
        if ("ENTERPRISE".equals(payCouponDto.getUserType()) && price <= 8) {
            return ResponseJson.error("企业网银支付金额低于手续费！", null);
        }
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(couponUrl);
        payParam.setCouponRecordId(payCouponDto.getCouponRecordId());
        log.info("【购买价值优惠券银联线上支付】>>>>>>>>>>>RecordId:" + payCouponDto.getCouponRecordId());
        return toPayCoupon(payParam, headers);
    }

    @Override
    public ResponseJson<JSONObject> payCouponByTransfer(PayCouponDto payCouponDto, HttpHeaders headers) {
        Integer status = payOrderMapper.findPayStatus(payCouponDto.getCouponRecordId());
        if (null != status && 1 == status) {
            return ResponseJson.error("该笔记录已支付，请勿重复支付！", null);
        }
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payCouponDto, payParam);
        // 银联支付
        payParam.setPayWay("UNIONPAY");
        payParam.setPayType("TRANSFER_UNIONPAY");
        // 获取会员套餐价格
        Double price = payOrderMapper.getCouponPrice(payCouponDto.getCouponId());
        if (price < 12) {
            return ResponseJson.error("银联转账金额低于手续费！", null);
        }
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(couponUrl);
        payParam.setCouponRecordId(payCouponDto.getCouponRecordId());
        log.info("【购买价值优惠券银联转账支付】>>>>>>>>>>>RecordId:" + payCouponDto.getCouponRecordId());
        return toPayCoupon(payParam, headers);
    }

    @Override
    public ResponseJson<PayCouponDto> getCouponRecord(PayCouponDto payCouponDto, HttpHeaders headers) {
        //个人机构返回状态码-1  不是医美 -2
        if (49 == payCouponDto.getCouponId()) {
            //正式id49优惠券 普通机构和非医美机构无法购买
            //先判断是否是普通机构，是就打回
            Integer userIdentity = baseMapper.getIdentityByUserId(payCouponDto.getUserId());
            if (userIdentity != null && 4 == userIdentity) {
                return ResponseJson.error(-1, "仅供医美机构购买！", null);
            }
            Integer clubType = payOrderMapper.findClubType(payCouponDto.getUserId());
            //为空或者不等于1    医美=1和生美=2
            if (null == clubType || 1 != clubType) {
                return ResponseJson.error(-2, "仅供医美机构购买！", null);
            }
        }
        CouponRecordPo cr = new CouponRecordPo();
        cr.setCouponId(payCouponDto.getCouponId());
        cr.setUserId(payCouponDto.getUserId());
        payOrderMapper.insertCouponRecord(cr);
        payCouponDto.setCouponRecordId(cr.getId());
        return ResponseJson.success(payCouponDto);
    }

    @Override
    public ResponseJson couponCheck(Integer recordId) {
        Integer status = payOrderMapper.findPayStatus(recordId);
        if (null != status && 1 == status) {
            return ResponseJson.success("支付成功", null);
        }
        return ResponseJson.error("支付失败", null);
    }

    @Override
    public ResponseJson vipCheck(Integer recordId) {
        // 获取会员套餐记录
        VipRecordBo record = payOrderMapper.getVipPackageRecord(recordId);
        if (null != record.getPayStatus() && 1 == record.getPayStatus()) {
            return ResponseJson.success("支付成功！", null);
        }
        return ResponseJson.error("支付失败", null);
    }


    private ResponseJson<JSONObject> toPayCoupon(PayParamBo payParam, HttpHeaders headers) {
        // 定义支付请求结果
        JSONObject result = null;
        try {
            // 时间戳
            long time = System.currentTimeMillis() / 1000;
            // 设置支付基础参数
            JSONObject json = getPayJsonObject(payParam, headers, time);
            // 支付环境
            String environment = "";
            if (payParam.getNotifyUrl().contains("18002")) {
                environment = "DEV";
            } else if (payParam.getNotifyUrl().contains("core-b")) {
                environment = "BETA";
            }
            // 商户订单号
            String orderId = payParam.getUserId() + "C" + payParam.getCouponId() + "R" + payParam.getCouponRecordId() + "T" + time + environment;
            json.put("orderId", orderId);
            //商品名称
            String product = "采美订单" + orderId;
            json.put("product", product);
            //支付类型
            String payType = PayUtil.getPayTypeId(payParam.getPayType(), payParam.getUserType());
            String attach = payParam.getCouponId() + "," + payParam.getUserId() + "," + payParam.getCouponRecordId() + "," + payType + "," + payParam.getSource();
            json.put("attach", attach);
            // 价值优惠券默认公账专票，彩美信息
            List<Map<String, String>> list = new ArrayList<>();
            Map<String, String> map = new HashMap<>(3);
            map.put("subUserNo", PayUtil.publicAccountNo);
            // 1按比例分账
            map.put("splitBillType", "1");
            // 1比例值总额
            map.put("splitBillValue", "1");
            list.add(map);
            String splitBillDetail = JSONObject.toJSONString(list);
            json.put("splitBillDetail", splitBillDetail);
            String sign = PayUtil.getPaySign(json, PayUtil.merKey);
            json.put("sign", sign);
            log.info("发送参数 json：" + json);
            // 私钥加密
            String data = RSAUtil.privateEncrypt(json.toJSONString(), PayUtil.merKey);
            // 提交
            result = PayUtil.httpGet("https://platform.mhxxkj.com/paygateway/mbpay/order/v1", PayUtil.merAccount, data);
        } catch (Exception e) {
            log.error("错误信息", e);
            return ResponseJson.error("支付失败！", null);
        }
        String code = result.getString("code");
        if (!"000000".equals(code)) {
            String msg = result.getString("msg");
            log.info("第三方支付失败>>>>>>>msg:" + msg);
            return ResponseJson.error(msg, null);
        }
        // 保存购买优惠券分账参数
        JSONObject data = result.getJSONObject("data");
        SplitAccountPo splitAccount = new SplitAccountPo();
        splitAccount.setCouponRecordId(payParam.getCouponRecordId());
        // 优惠券 公账-专票
        splitAccount.setType(1);
        // 待分账总金额
        double splitAmount = MathUtil.div(payParam.getPayAmount(), 100).doubleValue();
        // 总手续费
        double procedureFee;
        if ("UNIONPAY".equals(payParam.getPayWay())) {
            procedureFee = 8.00;
        } else {
            //手续费
            procedureFee = MathUtil.mul(splitAmount, 0.0038, 2).doubleValue();
            if (MathUtil.compare(procedureFee, 0) == 0) {
                procedureFee = 0.01;
            }
        }
        splitAmount = MathUtil.sub(splitAmount, procedureFee).doubleValue();
        splitAccount.setSplitAccount(splitAmount);
        splitAccount.setMbOrderId(data.getString("mbOrderId"));
        splitAccount.setOrderRequestNo(data.getString("orderId"));
        splitAccount.setPayStatus(0);
        // 保存分账详情
        payOrderMapper.insertSplitAccount(splitAccount);
        log.info("【购买优惠券支付成功！】>>>>>>>>>>>couponRecordId:" + payParam.getCouponRecordId() + "，data:" + data.toJSONString());
        return ResponseJson.success(result);
    }

    @Override
    public String couponCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException {
        log.info("******************** 购买优惠券异步回调 start *******************");
        // 公钥解密
        JSONObject json = PayUtil.publicKeyDecrypt(data, PayUtil.publicKey);
        log.info("【购买优惠券异步回调】>>>>>>>>>>>公钥解密：" + json);
        // 公钥验签
        String signaa = json.getString("sign");
        json.remove("sign");
        String signbb = PayUtil.getPaySign(json, PayUtil.publicKey);
        if (!signaa.equals(signbb)) {
            return "验签失败";
        }
        // 订单状态
        String orderStatus = json.getString("orderStatus");
        // 平台唯一流水号
        String mbOrderId = json.getString("mbOrderId");
        // 商户唯一订单号
        String orderRequestNo = json.getString("orderId");
        // 金额，以元为单位
        Double amount = json.getDouble("amount");
        String payFormData = json.toJSONString();
        log.info("【购买优惠券异步回调】>>>>>>>>>>>支付订单状态：" + orderStatus);
        if ("FAILED".equals(orderStatus)) {
            return "支付失败";
        }
        log.info("【购买优惠券异步回调】>>>>>>>>>>>>>>本次支付金额：" + amount);
        // 附加数据，支付时若有传输则原样返回(vipId,userId,recordId,payType)，下单时为空，则不返回该数据
        String attach = json.getString("attach");
        String[] split = attach.split(",");
        int couponId = 0;
        int userId = 0;
        int recordId = 0;
        int payType = 0;
        int source = 0;
        try {
            // 优惠券Id
            couponId = Integer.parseInt(split[0]);
            // 用户Id
            userId = Integer.parseInt(split[1]);
            // 购买历史记录Id
            recordId = Integer.parseInt(split[2]);
            // 支付方式
            payType = Integer.parseInt(split[3]);
            //购买渠道
            source = Integer.parseInt(split[4]);
        } catch (NumberFormatException e) {
            log.info("【购买优惠券异步回调】>>>>>>>>>>>>>>获取回调参数解析失败！");
        }
        //cm_coupon_club数据库修改
        CouponClubPo couponClubPo = new CouponClubPo();
        couponClubPo.setSource(source);
        couponClubPo.setUserId(userId);
        couponClubPo.setCouponId(couponId);
        couponClubPo.setStatus("1");
        couponClubPo.setDelFlag("0");
        couponClubPo.setCreateDate(new Date());
        orderClubMapper.insertCouponClub(couponClubPo);
        // 改历史记录支付状态
        payOrderMapper.updateCouponRecord(recordId);
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        // 保存收款记录
        DiscernReceiptPo discernReceipt = new DiscernReceiptPo();
        discernReceipt.setPayWay(1);
        discernReceipt.setPayType(payType);
        discernReceipt.setReceiptType(8);
        discernReceipt.setReceiptStatus(3);
        discernReceipt.setReceiptAmount(amount);
        discernReceipt.setConfirmType(4);
        discernReceipt.setRePayFlag(1);
        discernReceipt.setFormData(payFormData);
        discernReceipt.setReceiptDate(curDateStr);
        discernReceipt.setConfirmDate(curDateStr);
        discernReceipt.setReviewDate(curDateStr);
        discernReceipt.setUpdateDate(curDateStr);
        discernReceipt.setDelFlag(0);
        // 保存 收款记录
        baseMapper.insertDiscernReceipt(discernReceipt);
        log.info("【购买优惠券异步回调】>>>>>>>>>>>>>>保存识别款项(insert[cm_discern_receipt])id：" + discernReceipt.getId() + "，couponRecordId：" + recordId);
        // 收款项和订单关系表
        OrderReceiptRelationPo relation = new OrderReceiptRelationPo();
        relation.setReceiptId(discernReceipt.getId());
        relation.setCouponRecordId(recordId);
        relation.setAssociateAmount(amount);
        relation.setMbOrderId(mbOrderId);
        relation.setOrderRequestNo(orderRequestNo);
        relation.setSplitStatus(1);
        relation.setRelationType(6);
        relation.setDelFlag(0);
        // 保存 收款项和订单关系
        baseMapper.insertOrderReceiptRelation(relation);
        log.info("【购买优惠券异步回调】>>>>>>>>>>>收款项和购买历史关系(insert[cm_receipt_order_relation])id：" + relation.getId() + "，couponRecordId：" + recordId);
        //修改分账付款状态
        payOrderMapper.updateSplitAccountByPay(mbOrderId);
        return "SUCCESS";
    }

    /**
     * 开通认证通会员-微信线上支付
     *
     * @param payVipDto {
     *                  vipRecordId   会员购买记录Id
     *                  returnUrl     页面回调地址
     *                  code          微信小程序code
     *                  state         微信公众号state参数
     *                  }
     * @param headers
     */
    @Override
    public ResponseJson<JSONObject> payAuthVipByWeChat(PayVipDto payVipDto, HttpHeaders headers) {
        PayParamBo tempParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payVipDto, tempParam);
        ResponseJson<PayParamBo> jsonParam = getWeChatResponseJson(tempParam, headers);
        if (-1 == jsonParam.getCode()) {
            return ResponseJson.error(jsonParam.getMsg(), null);
        }
        PayParamBo payParam = jsonParam.getData();
        // 获取会员套餐记录
        VipRecordBo record = payOrderMapper.getAuthVipRecord(payParam.getVipRecordId());
        if (null == record) {
            log.info("【开通认证通会员-微信线上支付】该笔记录异常,记录Id:" + payParam.getVipRecordId());
            return ResponseJson.error("该笔记录异常！", null);
        }
        if (null != record.getPayStatus() && 1 == record.getPayStatus()) {
            return ResponseJson.error("该笔记录已支付，请勿重复支付！", null);
        }
        payParam.setVipId(record.getPackageId());
        payParam.setUserId(record.getUserId());
        // 获取会员套餐价格
        Double price = payOrderMapper.getAuthVipPackagePrice(record.getPackageId());
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(authVipUrl);
        log.info("【开通认证通会员微信支付】>>>>>>>>>>>vipRecordId:" + payParam.getVipRecordId() + "，获取openId:" + payParam.getOpenId());
        return toPayAuthVip(payParam, headers);
    }

    /**
     * 开通认证通会员-支付宝线上支付
     *
     * @param payVipDto {
     *                  vipRecordId   会员购买记录Id
     *                  returnUrl     页面回调地址
     *                  }
     * @param headers
     */
    @Override
    public ResponseJson<JSONObject> payAuthVipByAlipay(PayVipDto payVipDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payVipDto, payParam);
        //支付宝支付
        payParam.setPayWay("ALIPAY");
        payParam.setPayType("ALIPAY_H5");
        // 获取会员套餐记录
        VipRecordBo record = payOrderMapper.getAuthVipRecord(payParam.getVipRecordId());
        if (null != record.getPayStatus() && 1 == record.getPayStatus()) {
            return ResponseJson.error("该笔记录已支付，请勿重复支付！", null);
        }
        payParam.setVipId(record.getPackageId());
        payParam.setUserId(record.getUserId());
        // 获取会员套餐价格
        Double price = payOrderMapper.getAuthVipPackagePrice(record.getPackageId());
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(authVipUrl);
        log.info("【开通认证通会员支付宝支付】>>>>>>>>>>>vipRecordId:" + payParam.getVipRecordId());
        return toPayAuthVip(payParam, headers);
    }

    /**
     * 开通认证通会员-银联线上支付
     *
     * @param payVipDto {
     *                  vipRecordId   会员购买记录Id
     *                  returnUrl     页面回调地址
     *                  bankCode      银行编码(银联支付使用)
     *                  userType      用户类型(银联支付使用)企业:ENTERPRISE，个人:USER
     *                  }
     * @param headers
     */
    @Override
    public ResponseJson<JSONObject> payAuthVipByUnionPay(PayVipDto payVipDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payVipDto, payParam);
        // 银联支付
        payParam.setPayWay("UNIONPAY");
        payParam.setPayType("GATEWAY_UNIONPAY");
        // 获取会员套餐记录
        VipRecordBo record = payOrderMapper.getAuthVipRecord(payParam.getVipRecordId());
        if (null != record.getPayStatus() && 1 == record.getPayStatus()) {
            return ResponseJson.error("该笔记录已支付，请勿重复支付！", null);
        }
        payParam.setVipId(record.getPackageId());
        payParam.setUserId(record.getUserId());
        // 获取会员套餐价格
        Double price = payOrderMapper.getAuthVipPackagePrice(record.getPackageId());
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(authVipUrl);
        log.info("【开通认证通会员银联线上支付】>>>>>>>>>>>vipRecordId:" + payParam.getVipRecordId());
        return toPayAuthVip(payParam, headers);
    }


    /**
     * 开通认证通会员支付
     */
    private ResponseJson<JSONObject> toPayAuthVip(PayParamBo payParam, HttpHeaders headers) {
        // 定义支付请求结果
        JSONObject result = null;
        try {
            // 时间戳
            long time = System.currentTimeMillis() / 1000;
            // 设置支付基础参数
            JSONObject json = getPayJsonObject(payParam, headers, time);
            // 支付环境
            String environment = "";
            if (payParam.getNotifyUrl().contains("18002")) {
                environment = "DEV";
            } else if (payParam.getNotifyUrl().contains("core-b")) {
                environment = "BETA";
            }
            // 商户订单号
            String orderId = payParam.getUserId() + "R" + payParam.getVipRecordId() + "V" + payParam.getVipId() + "T" + time + environment;
            json.put("orderId", orderId);
            //商品名称
            String product = "采美订单" + orderId;
            json.put("product", product);
            //支付类型
            String payType = PayUtil.getPayTypeId(payParam.getPayType(), payParam.getUserType());
            String attach = payParam.getVipId() + "," + payParam.getUserId() + "," + payParam.getVipRecordId() + "," + payType;
            json.put("attach", attach);
            // 开通认证通会员默认公账
            List<Map<String, String>> list = new ArrayList<>();
            Map<String, String> map = new HashMap<>(3);
            map.put("subUserNo", PayUtil.publicAccountNo);
            // 1按比例分账
            map.put("splitBillType", "1");
            // 1比例值总额
            map.put("splitBillValue", "1");
            list.add(map);
            String splitBillDetail = JSONObject.toJSONString(list);
            json.put("splitBillDetail", splitBillDetail);
            String sign = PayUtil.getPaySign(json, PayUtil.merKey);
            json.put("sign", sign);
            // 私钥加密
            String data = RSAUtil.privateEncrypt(json.toJSONString(), PayUtil.merKey);
            // 提交
            result = PayUtil.httpGet("https://platform.mhxxkj.com/paygateway/mbpay/order/v1", PayUtil.merAccount, data);
        } catch (Exception e) {
            log.error("错误信息", e);
            return ResponseJson.error("支付失败！", null);
        }
        String code = result.getString("code");
        if (!"000000".equals(code)) {
            String msg = result.getString("msg");
            log.info("第三方支付失败>>>>>>>msg:" + msg);
            return ResponseJson.error(msg, null);
        }
        // 保存开通认证通会员分账参数
        JSONObject data = result.getJSONObject("data");
        SplitAccountPo splitAccount = new SplitAccountPo();
        splitAccount.setAuthVipRecordId(payParam.getVipRecordId());
        // 认证通会员 公账-专票
        splitAccount.setType(1);
        // 待分账总金额
        double splitAmount = MathUtil.div(payParam.getPayAmount(), 100).doubleValue();
        // 总手续费
        double procedureFee;
        if ("UNIONPAY".equals(payParam.getPayWay())) {
            procedureFee = 8.00;
        } else {
            //手续费
            procedureFee = MathUtil.mul(splitAmount, 0.0038, 2).doubleValue();
            if (MathUtil.compare(procedureFee, 0) == 0) {
                procedureFee = 0.01;
            }
        }
        splitAmount = MathUtil.sub(splitAmount, procedureFee).doubleValue();
        splitAccount.setSplitAccount(splitAmount);
        splitAccount.setMbOrderId(data.getString("mbOrderId"));
        splitAccount.setOrderRequestNo(data.getString("orderId"));
        splitAccount.setPayStatus(0);
        // 保存分账详情
        payOrderMapper.insertSplitAccount(splitAccount);
        log.info("【开通认证通会员支付成功！】>>>>>>>>>>>vipRecordId:" + payParam.getVipRecordId() + "，data:" + data.toJSONString());
        return ResponseJson.success(result);
    }

    /**
     * 开通认证通会员-支付回调
     *
     * @param data
     */
    @Override
    public String paymentAuthVipCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException {
        log.info("******************** 开通认证通会员异步回调 start *******************");
        // 公钥解密
        JSONObject json = PayUtil.publicKeyDecrypt(data, PayUtil.publicKey);
        log.info("【开通认证通会员异步回调】>>>>>>>>>>>公钥解密：" + json);
        // 公钥验签
        String signaa = json.getString("sign");
        json.remove("sign");
        String signbb = PayUtil.getPaySign(json, PayUtil.publicKey);
        if (!signaa.equals(signbb)) {
            return "验签失败";
        }
        // 订单状态
        String orderStatus = json.getString("orderStatus");
        // 平台唯一流水号
        String mbOrderId = json.getString("mbOrderId");
        // 商户唯一订单号
        String orderRequestNo = json.getString("orderId");
        // 金额，以元为单位
        Double amount = json.getDouble("amount");
        String payFormData = json.toJSONString();
        log.info("【开通认证通会员异步回调】>>>>>>>>>>>支付订单状态：" + orderStatus);
        if ("FAILED".equals(orderStatus)) {
            return "支付失败";
        }
        log.info("【开通认证通会员异步回调】>>>>>>>>>>>>>>本次支付金额：" + amount);
        // 附加数据，支付时若有传输则原样返回(vipId,userId,recordId,payType)，下单时为空，则不返回该数据
        String attach = json.getString("attach");
        String[] split = attach.split(",");
        int vipId = 0;
        int userId = 0;
        int recordId = 0;
        int payType = 0;
        try {
            // 会员套餐Id
            vipId = Integer.parseInt(split[0]);
            // 用户Id
            userId = Integer.parseInt(split[1]);
            // 购买历史记录Id
            recordId = Integer.parseInt(split[2]);
            // 支付方式
            payType = Integer.parseInt(split[3]);
        } catch (NumberFormatException e) {
            log.info("【开通认证通会员异步回调】>>>>>>>>>>>>>>获取回调参数解析失败！");
        }
        //认证通会员数据库修改
        UserVipPo authVip = new UserVipPo();
        UserVipPo dbAuthVip = payOrderMapper.getAuthVipInfo(userId);
        Date beginTime = new Date();
        if (null != dbAuthVip) {
            authVip.setId(dbAuthVip.getId());
            if (dbAuthVip.getEndTime().compareTo(beginTime) > 0) {
                // 有效期内续费,用原结束时间作为开始时间来计算结束时间
                beginTime = dbAuthVip.getEndTime();
                authVip.setBeginTime(beginTime);
            } else {
                // 过期重新续费
                authVip.setBeginTime(beginTime);
            }
        } else {
            authVip.setBeginTime(beginTime);
        }
        authVip.setUserId(userId);
        authVip.setUpdateTime(new Date());
        Calendar cal = Calendar.getInstance();
        cal.setTime(beginTime);
        // 查询认证通会员套餐信息
        AuthVipPackagePo packagePo = payOrderMapper.getAuthVipPackageById(vipId);
        cal.add(1 == packagePo.getUnit() ? Calendar.MONTH : Calendar.YEAR, packagePo.getDuration());
        authVip.setEndTime(cal.getTime());
        authVip.setDelFlag(0);
        if (null != authVip.getId()) {
            // 更新会员信息
            payOrderMapper.updateAuthVipInfo(authVip);
            if (authVip.getBeginTime().compareTo(beginTime) >= 0) {
                // 删除原有角色关联
                payOrderMapper.deleteAuthVipRoleRelation(userId);
                // 会员过期重新分配角色
                saveRoleRelation(userId, "1");
            }
        } else {
            // 新增会员信息
            payOrderMapper.insertAuthVipInfo(authVip);
            // 删除原有角色关联
            payOrderMapper.deleteAuthVipRoleRelation(userId);
            // 分配角色
            saveRoleRelation(userId, "1");
        }
        // 修改会员历史记录支付状态
        VipRecordBo record = payOrderMapper.getAuthVipRecord(recordId);
        record.setPayStatus(1);
        record.setBeginTime(beginTime);
        record.setEndTime(cal.getTime());
        record.setPayWay(1);
        record.setPayType(payType);
        record.setPayTime(new Date());
        payOrderMapper.updateAuthVipRecord(record);
        // 修改支付链接状态
        OrderPayLinkVo orderPayLink = payOrderMapper.getAuthVipPayLink(recordId, amount);
        if (null != orderPayLink && (12 == payType || 17 == payType)) {
            orderPayLink.setPayStatus(1);
            payOrderMapper.updateOrderPayLinkStatus(orderPayLink);
        }
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        // 保存收款记录
        DiscernReceiptPo discernReceipt = new DiscernReceiptPo();
        discernReceipt.setPayWay(1);
        discernReceipt.setPayType(record.getPayType());
        discernReceipt.setReceiptType(9);
        discernReceipt.setReceiptStatus(3);
        discernReceipt.setReceiptAmount(amount);
        discernReceipt.setConfirmType(4);
        discernReceipt.setRePayFlag(1);
        discernReceipt.setFormData(payFormData);
        discernReceipt.setReceiptDate(curDateStr);
        discernReceipt.setConfirmDate(curDateStr);
        discernReceipt.setReviewDate(curDateStr);
        discernReceipt.setUpdateDate(curDateStr);
        discernReceipt.setDelFlag(0);
        // 保存 收款记录
        baseMapper.insertDiscernReceipt(discernReceipt);
        log.info("【开通认证通会员异步回调】>>>>>>>>>>>>>>保存识别款项(insert[cm_discern_receipt])id：" + discernReceipt.getId() + "，vipRecordId：" + recordId);
        // 收款项和订单关系表
        OrderReceiptRelationPo relation = new OrderReceiptRelationPo();
        relation.setReceiptId(discernReceipt.getId());
        relation.setAuthVipRecordId(recordId);
        relation.setAssociateAmount(amount);
        relation.setMbOrderId(mbOrderId);
        relation.setOrderRequestNo(orderRequestNo);
        relation.setSplitStatus(1);
        relation.setRelationType(3);
        relation.setDelFlag(0);
        // 保存 收款项和订单关系
        baseMapper.insertOrderReceiptRelation(relation);
        log.info("【开通认证通会员异步回调】>>>>>>>>>>>收款项和购买历史关系(insert[cm_receipt_order_relation])id：" + relation.getId() + "，vipRecordId：" + recordId);
        //修改分账付款状态
        payOrderMapper.updateSplitAccountByPay(mbOrderId);
        return "SUCCESS";
    }

    /**
     * 开通认证通会员-线上退款
     *
     * @param payVipDto {
     *                  vipRecordId   会员购买记录Id
     *                  code          退款口令
     *                  }
     */
    @Override
    public ResponseJson<JSONObject> authVipOnlineRefund(PayVipDto payVipDto) throws NoSuchAlgorithmException, InvalidKeySpecException {
        // 认证通会员支付记录
        DiscernReceiptVo discernReceipt = orderCommonMapper.getVipDiscernReceipt(payVipDto.getVipRecordId());
        JSONObject json = new JSONObject();
        json.put("merAccount", PayUtil.merAccount);
        // 当前时间戳
        long time = System.currentTimeMillis() / 1000;
        json.put("time", time);
        //商户退款流水号，由商户自行生成，必须唯一
        String randomLetter = CodeUtil.randomLetter(6);
        String merchantRefundNo = payVipDto.getVipRecordId() + payVipDto.getCode() + randomLetter + "##" + discernReceipt.getId();
        json.put("merchantRefundNo", merchantRefundNo);
        //退款金额，单位分，必须大于0
        double refundAmt = MathUtil.mul(discernReceipt.getAssociateAmount(), 100).doubleValue();
        json.put("refundAmt", refundAmt);
        //退款原因
        String refundCause = "用户主动退款";
        json.put("refundCause", refundCause);
        //平台交易订单号
        json.put("mbOrderId", discernReceipt.getMbOrderId());
        String sign = PayUtil.getPaySign(json, PayUtil.merKey);
        json.put("sign", sign);
        String data = PayUtil.privateKeyEncrypt(json, PayUtil.merKey);
        JSONObject result = PayUtil.httpPost("https://platform.mhxxkj.com/paygateway/mbrefund/orderRefund/v1", PayUtil.merAccount, data);
        String code = result.getString("code");
        if (!"000000".equals(code)) {
            String msg = result.getString("msg");
            log.info("【线上退款】失败>>>>>>>msg:" + msg);
            return ResponseJson.error(result);
        }
        log.info("【线上退款】成功>>>>>>>vipRecordId：" + payVipDto.getVipRecordId());
        return ResponseJson.success(result);
    }

    @Override
    public ResponseJson<Map<String, Map<String, String>>> authVipCheck(Integer vipRecordId) {
        // 获取会员套餐记录
        VipRecordBo record = payOrderMapper.getAuthVipRecord(vipRecordId);
        Map<String, Map<String, String>> map1 = new HashMap<>();
        Map<String, String> map2 = new HashMap<>();
        if (null != record.getPayStatus() && 1 == record.getPayStatus()) {
            map2.put("status", "1");
            map1.put("data", map2);
            return ResponseJson.success("支付成功！", map1);
        }
        map2.put("status", "0");
        map1.put("data", map2);
        return ResponseJson.success("支付失败", map1);
    }

    public void saveRoleRelation(Integer userId, String roleIds) {
        if (StringUtils.isNotEmpty(roleIds)) {
            // 设置用户角色
            String[] split = roleIds.split(",");
            for (String s : split) {
                if (org.springframework.util.StringUtils.hasLength(s)) {
                    // 保存用户角色关系
                    payOrderMapper.insertAuthVipRoleRelation(userId, Integer.valueOf(s));
                }
            }
        }
    }
}
