package com.caimei365.order.utils.helipay;


import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.annotation.FieldEncrypt;
import com.caimei365.order.annotation.SignExclude;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;


public class MessageHandle {

	private static Logger log = LoggerFactory.getLogger(MessageHandle.class);

	private static final String ENCRYPTION_KEY = "encryptionKey";

	private static final String SPLIT = "&";

	private static final String SIGN = "sign";

	public static boolean isJSON(String test) {
		try {
			JSONObject.parseObject(test);
		} catch (JSONException ex) {
			return false;
		}
		return true;
	}

	/**
	 * 获取map
	 */
	public static Map getReqestMap(Object bean) throws Exception {

		Map retMap = new HashMap();

		boolean isEncrypt = false;
		String sm4Key = SM4Utils.generateRandomKey();
		StringBuilder sb = new StringBuilder();

		Class clazz = bean.getClass();
		Field[] fields = clazz.getDeclaredFields();
		for (Field field : fields) {
			field.setAccessible(true);
			String key = field.toString().substring(field.toString().lastIndexOf(".") + 1);
			String value = (String) field.get(bean);
			if (value == null) {
				value = "";
			}
			//查看是否有需要SM4加密字段的注解,有则加密
			//这部分是将需要SM4加密的字段先进行加密
			if (field.isAnnotationPresent(FieldEncrypt.class) && StringUtils.isNotEmpty(value)) {
				isEncrypt = true;
				value = SM4Utils.encryptBase64(value , sm4Key);
			}

			//字段没有@SignExclude注解的拼签名串
			//这部分是把需要参与签名的字段拼成一个待签名的字符串
			if (!field.isAnnotationPresent(SignExclude.class)) {
				sb.append(SPLIT);
				sb.append(value);
			}

			retMap.put(key, value);
		}

		//请求参数中有需要加密的字段，使用合利宝的公钥的对SM4密钥KEY进行加密处理
		if (isEncrypt) {
			String encrytionKey = SM2Utils.encryptToBase64(HeliPayCertConfig.getHelipayPublicKey(), sm4Key);
			retMap.put(ENCRYPTION_KEY, encrytionKey);
		}

		log.info("原签名串：" + sb.toString());
		//使用商户的私钥进行签名
		String sign = SM2Utils.sign(HeliPayCertConfig.getMerchantPrivateKey(), sb.toString());
		retMap.put(SIGN, sign);
		log.info("签名sign：" + sign);
		return retMap;
	}


	public static boolean checkSign(Object bean) throws Exception {

		boolean flag = false;
		StringBuilder sb = new StringBuilder();

		Class clazz = bean.getClass();
		Field[] fields = clazz.getDeclaredFields();
		String sign = "";
		for (Field field : fields) {
			field.setAccessible(true);
			String key = field.toString().substring(field.toString().lastIndexOf(".") + 1);
			String value = (String) field.get(bean);
			if (value == null) {
				value = "";
			}

			if (SIGN.equals(key)) {
				sign = value;
			}

			//字段没有@SignExclude注解的拼签名串
			//这部分是把需要参与签名的字段拼成一个待签名的字符串
			if (!field.isAnnotationPresent(SignExclude.class)) {
				sb.append(SPLIT);
				sb.append(value);
			}

		}
		log.info("response验签原签名串：" + sb.toString());

		//使用合利宝的公钥进行验签
		flag = SM2Utils.verify(HeliPayCertConfig.getHelipayPublicKey(), sb.toString(), sign);
		if (flag) {
			log.info("验签成功");
		} else {
			log.info("验签失败");
		}
		return flag;

	}
}
